// Project: Rocket Commander, File: BaseAsteroidManager.cs
// Namespace: RocketCommander.Game, Class: BaseAsteroidManager
// Path: C:\code\Rocket Commander\Game, Author: Abi
// Code lines: 1551, Size of file: 55,17 KB
// Creation date: 19.01.2006 18:55
// Last modified: 20.01.2006 04:11
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Effect = Microsoft.DirectX.Direct3D.Effect;
using RocketCommander.Graphics;
using RocketCommander.Helpers;
using RocketCommander.Shaders;
using RocketCommander.Input;
using RocketCommander.Properties;
using RocketCommander.Sounds;
using Texture = RocketCommander.Graphics.Texture;
using Font = RocketCommander.Graphics.Font;
using Material = RocketCommander.Graphics.Material;
#endregion

namespace RocketCommander.Game
{
	/// <summary>
	/// Base asteroid manager.
	/// Handles all the base variables we need for our asteroid field.
	/// Also generates sectors and handles smaller asteroids.
	/// Provides some basic render methods to show the asteroid field.
	/// This class is nothing we can use for the game, it just provides us
	/// the very basic features for the asteroid field.
	/// <para/>
	/// Derived from this is the PhysicsAsteroidManager, which adds all Physics
	/// (handle player collision, handle sector Physics, etc.).
	/// <para/>
	/// And derived from that we finally get to the GameAsteroidManager,
	/// which calculates sector movement, does visibility checks, etc.
	/// The GameAsteroidManager is the class we use in the game, it provides
	/// a much richer Render method and handles also additional effects like
	/// the Hit effects we see on screen.
	/// </summary>
	public class BaseAsteroidManager
	{
		#region Constants
		/// <summary>
		/// Equally distribute all sectors in 3d space (7x7x7 = 343 sectors)
		/// This are only visible sectors. If we are moving new sectors
		/// are added in that direction and all other ones are shifted then.
		/// </summary>
		protected const int NumberOfSectors = 13,//3,//1,
			MiddleSector = NumberOfSectors / 2,
			MinSector = -MiddleSector,
			MaxSector = +MiddleSector;

		/// <summary>
		/// For smaller asteroids we only need a very small inner sector area.
		/// </summary>
		protected const int NumberOfSmallSectors = 5,
			SmallMiddleSector = NumberOfSmallSectors / 2,
			SmallSectorAdd = MiddleSector - SmallMiddleSector,
			MinSmallSector = -SmallMiddleSector,
			MaxSmallSector = +SmallMiddleSector;

		/// <summary>
		/// Sector size is 100*100*100, each component must be at least
		/// MaxAsteroidSize/2.0f for correct collision checking.
		/// For physics we use SectorWidth+MaxAsteroidSize, and asteroids
		/// could be in more than 1 sector for more accurate physics calculations,
		/// but this is to complicated right now, maybe we add that later.
		/// </summary>
		public const float SectorWidth = 100.0f,//25.0f,
			SectorHeight = 100.0f,//25.0f,
			SectorDepth = 100.0f;//25.0f;

		/// <summary>
		/// Minimum and maximum asteroid size.
		/// If this is increased the sector depth may have to be increased as well.
		/// Smaller asteroid sizes look also nice but they don't serve much of
		/// a game purpose and we would need a lot more of them, which would
		/// slow down rendering.
		/// </summary>
		public const float MinAsteroidSize = 15.0f,//12.5f,//50.0f,//15.0f,//3.5f,
			MaxAsteroidSize = 32.5f;//25.0f;//50.0f;//30.0f;//10.0f;

		/// <summary>
		/// Smaller asteroid size
		/// </summary>
		public const float SmallAsteroidSize = 30.0f;

		/// <summary>
		/// Max view depth for our camera!
		/// Everything is faded out at FadeOutDepth.
		/// HalfViewDepth is used to switch to faster lod model type.
		/// SmallAsteroidFadeoutDepth is used to fade out small asteroids,
		/// which use HalfViewDepth for the max view range.
		/// </summary>
		public const float MinViewDepth = 0.33f,//1.0f,
			MaxViewDepth = SectorDepth * MiddleSector - SectorDepth / 3.0f,
			FadeOutDepth = MaxViewDepth * 0.8f,
			HalfViewDepth = MaxViewDepth * 0.4f,
			SmallAsteroidFadeOutDepth = HalfViewDepth * 0.6f;

		/// <summary>
		/// Viewable field of view, used to check if a sector is visible or not.
		/// We can't use FieldOfView/2 because most sectors at the screen borders
		/// would pop in and out. FieldOfView/1.5 is very good, but some popping
		/// sometimes occur when rotating. For that reason we use FieldOfView/1.4,
		/// which works very good!
		/// </summary>
		/// <returns>Float</returns>
		public static float ViewableFieldOfView
		{
			get
			{
				// In game / 1.4f is fine, but for GameOver screen we need more fov.
				if (Player.GameOver)
					return GraphicForm.FieldOfView;
				else
					return GraphicForm.FieldOfView / 1.4f;// 1.4f; //1.5f; // 1.33f;
			} // get
		} // ViewableFieldOfView
		#endregion

		#region Variables
		/// <summary>
		/// Current level, change it with SetLevel(.)
		/// </summary>
		protected internal Level level = null;

		/// <summary>
		/// Current level
		/// </summary>
		/// <returns>Level</returns>
		public virtual Level CurrentLevel
		{
			get
			{
				return level;
			} // get
			set
			{
				level = value;

				// Update target position
				targetPos = new Vector3(0, 0, (level.Length + 10) * SectorDepth);
			} // set
		} // CurrentLevel

		/// <summary>
		/// Helpers to remember target position. Updated in LoadLevel.
		/// </summary>
		static private Vector3 targetPos = new Vector3(0, 0, 10000);

		/// <summary>
		/// High detail asteroid models. All of them are named
		/// "Asteroid1.x", "Asteroid2.x", etc.
		/// If the model files "Asteroid1Low.x", Asteroid2Low.x" do exists,
		/// they will be used as a lower LOD (level of detail) rendering with
		/// the same textures. These lower polygon models only have 200-300
		/// polygons instead of 1200-1300. Rendering is a little bit faster,
		/// when using instancing up to 200% faster. It will be used for
		/// everything farther away than HalfViewDepth.
		/// </summary>
		protected List<Model> asteroidModels = new List<Model>(),
			asteroidModelsLow = new List<Model>();

		/// <summary>
		/// For the near view distance we also add smaller asteroid parts
		/// to increase realism and the velocity feeling.
		/// We won't collide with any of these objects and each model contains
		/// a group of small particles.
		/// </summary>
		protected List<Model> smallAsteroidModels = new List<Model>();

		/// <summary>
		/// Devide all asteroids into sectors to speed up physics, rendering
		/// and all other kind of checks (visibility, collision).
		/// This improves performance by a couple of thousand percent!
		/// If an asteroid sector leaves a sector, we have to remove it
		/// from the old sector and add it to the new sector. This
		/// would involve a lot of checks per frame and be very slow,
		/// to optimize this we only perform this check for highSpeedAsteroids.
		/// 
		/// Note: This wasn't the first approach. First I used a simple 1d list,
		/// then a queue, then a multidimensional (2d) list of queues and finally
		/// this 3 dimensional list of lists. One list is used for each sector.
		/// Still a bit complicated, but required for sector handling!
		/// </summary>
		protected List<Asteroid>[, ,] sectorAsteroids =
			new List<Asteroid>[NumberOfSectors, NumberOfSectors, NumberOfSectors];

		/// <summary>
		/// Sector small asteroids, only use inner rect of full sector grid.
		/// </summary>
		protected List<SmallAsteroids>[, ,] sectorSmallerAsteroids =
			new List<SmallAsteroids>[
			NumberOfSmallSectors, NumberOfSmallSectors, NumberOfSmallSectors];

		/// <summary>
		/// To check if we are moving to another sector with our camera.
		/// </summary>
		private int centerSectorPositionX = 0,
			centerSectorPositionY = 0,
			centerSectorPositionZ = 0;

		#region Sector visibility
		/// <summary>
		/// Is a specific sector in range to be visible? This is a boolean
		/// precalculated list. We only use it for the isVisible calculation.
		/// This alone will reduce the number of sectors we have to render by
		/// almost 50%, which is a great performance boost!
		/// </summary>
		protected bool[, ,] sectorVisibleInRange =
			new bool[NumberOfSectors, NumberOfSectors, NumberOfSectors];

		/// <summary>
		/// Sector direction, used for calculating if this sector is visible 
		/// or not. To do that we get the angel between the camera looking vector
		/// and the sector vector and check if sector is in field of view.
		/// </summary>
		protected Vector3[, ,] sectorDirection =
			new Vector3[NumberOfSectors, NumberOfSectors, NumberOfSectors];

		/// <summary>
		/// Is sector visible? This is precalculated and only updated when
		/// we rotate or move into a new sector.
		/// This is the biggest performance gain of this whole game, instead
		/// of rendering over 10000 possible asteroids in our view range we
		/// only have to render 500-1000 asteroids and can skip any visibility
		/// test because we only render sectors that are somehow visible anyway.
		/// </summary>
		protected bool[, ,] sectorIsVisible =
			new bool[NumberOfSectors, NumberOfSectors, NumberOfSectors];

		/// <summary>
		/// Find out which asteroids are visble. List is cleared each frame!
		/// We can't just render all asteroids, that would be way to slow.
		/// While we are at it, we can also put each type in a seperate list,
		/// this way we can speed up the rendering in the shader below.
		/// </summary>
		protected List<Asteroid>[] visibleAsteroids = null;
		#endregion
		#endregion

		#region Properties
		/// <summary>
		/// Number of asteroid types
		/// </summary>
		/// <returns>Int</returns>
		public int NumberOfAsteroidTypes
		{
			get
			{
				return asteroidModels.Count;
			} // get
		} // NumOfAsteroidTypes

		/// <summary>
		/// Total number of asteroids. Not really used, just for debugging.
		/// </summary>
		/// <returns>Int</returns>
		public int TotalNumberOfAsteroids
		{
			get
			{
				int num = 0;
				for (int z = 0; z < NumberOfSectors; z++)
					for (int y = 0; y < NumberOfSectors; y++)
						for (int x = 0; x < NumberOfSectors; x++)
							foreach (Asteroid asteroid in sectorAsteroids[z, y, x])
								num++;
				return num;
			} // get
		} // TotalNumberOfAsteroids

		/// <summary>
		/// Visible number of asteroids. Not really used, just for debugging.
		/// </summary>
		/// <returns>Int</returns>
		public int VisibleNumberOfAsteroids
		{
			get
			{
				int num = 0;
				for (int z = 0; z < NumberOfSectors; z++)
					for (int y = 0; y < NumberOfSectors; y++)
						for (int x = 0; x < NumberOfSectors; x++)
							if (sectorIsVisible[z, y, x])
								foreach (Asteroid asteroid in sectorAsteroids[z, y, x])
									num++;
				return num;
			} // get
		} // VisibleNumberOfAsteroids

		/// <summary>
		/// Target position
		/// </summary>
		/// <returns>Vector 3</returns>
		static public Vector3 TargetPosition
		{
			get
			{
				return targetPos;
			} // get
		} // TargetPosition
		#endregion

		#region Constructor
		/// <summary>
		/// Create asteroid manager
		/// </summary>
		public BaseAsteroidManager(Level setLevel)
		{
			CurrentLevel = setLevel;

			// Load all available asteroids
			int asteroidNum = 1;
			while (File.Exists(Directories.ModelsDirectory + "\\" +
				"Asteroid" + asteroidNum + "." + Model.Extension))
			{
				Model asteroidModel = new Model("Asteroid" + asteroidNum);
				asteroidModels.Add(asteroidModel);

				// Try to use low version of asteroid
				if (File.Exists(Directories.ModelsDirectory + "\\" +
					"Asteroid" + asteroidNum + "Low." + Model.Extension))
					asteroidModelsLow.Add(new Model("Asteroid" + asteroidNum + "Low"));
				else
					// If none is found, use normal one.
					asteroidModelsLow.Add(asteroidModel);

				asteroidNum++;
			} // while (File.Exists)

			if (asteroidModels.Count == 0)
				throw new Exception("Unable to start game, no asteroid models were " +
					"found, please check the Models directory!");

			// Load all small asteroids
			int smallAsteroidNum = 1;
			while (File.Exists(Directories.ModelsDirectory + "\\" +
				"SmallAsteroid" + smallAsteroidNum + "." + Model.Extension))
			{
				smallAsteroidModels.Add(new Model("SmallAsteroid" + smallAsteroidNum));
				smallAsteroidNum++;
			} // while (File.Exists)

			// Create all asteroids
			for (int z = MinSector; z <= MaxSector; z++)
				for (int y = MinSector; y <= MaxSector; y++)
					for (int x = MinSector; x <= MaxSector; x++)
					{
						int iz = z + MiddleSector,
							iy = y + MiddleSector,
							ix = x + MiddleSector;
						sectorAsteroids[iz, iy, ix] = new List<Asteroid>();
						//unused: helperCopyAsteroids[iz, iy, ix] = new List<Asteroid>();

						GenerateSector(sectorAsteroids[iz, iy, ix], x, y, z);
					} // for for for (int)

			// Create smaller asteroids
			for (int z = MinSmallSector; z <= MaxSmallSector; z++)
				for (int y = MinSmallSector; y <= MaxSmallSector; y++)
					for (int x = MinSmallSector; x <= MaxSmallSector; x++)
					{
						int iz = z + SmallMiddleSector,
							iy = y + SmallMiddleSector,
							ix = x + SmallMiddleSector;
						sectorSmallerAsteroids[iz, iy, ix] = new List<SmallAsteroids>();

						GenerateSmallerAsteroidsSector(
							sectorSmallerAsteroids[iz, iy, ix],
							sectorAsteroids[iz + SmallSectorAdd, iy + SmallSectorAdd,
							ix + SmallSectorAdd].Count, x, y, z);
					} // for for for (int)

			// Precalculate visible sector stuff
			for (int z = MinSector; z <= MaxSector; z++)
				for (int y = MinSector; y <= MaxSector; y++)
					for (int x = MinSector; x <= MaxSector; x++)
					{
						int iz = z + MiddleSector,
							iy = y + MiddleSector,
							ix = x + MiddleSector;

						// Check if distance (that sqrt thingy) is smaller than
						// the max view depth (in int this is MiddleSector) and add
						// a small offset (0.25) to include nearly visible sectors.
						sectorVisibleInRange[iz, iy, ix] =
							(float)Math.Sqrt(x * x + y * y + z * z) <
							MiddleSector + 0.25f;

						// Calculate direction (just normalize relative position)
						sectorDirection[iz, iy, ix] = new Vector3(x, y, z);
						sectorDirection[iz, iy, ix].Normalize();
					} // for for for (int)

			// Create lists for visibleAsteroids
			visibleAsteroids = new List<Asteroid>[NumberOfAsteroidTypes];
			for (int num = 0; num < NumberOfAsteroidTypes; num++)
				visibleAsteroids[num] = new List<Asteroid>();

			// Calculate sectors and visibility
			CalculateSectors();
		} // BaseAsteroidManager(setLevel)
		#endregion

		#region Generate sector
		/// <summary>
		/// Generate sector
		/// </summary>
		/// <param name="list">List</param>
		/// <param name="x">X</param>
		/// <param name="y">Y</param>
		/// <param name="z">Z</param>
		private void GenerateSector(List<Asteroid> list, int x, int y, int z)
		{
			// Make sure there are no asteroids in sector 0, 0, 0
			if (x == 0 && y == 0 && z == 0)
				return;

			// Check out how much asteroids we got in this sector.
			// If this is not a valid sector, use a medium density.
			float density = 0.3f;
			int levelX = x + level.Width / 2;
			if (levelX >= 0 && levelX < level.Width &&
				y >= -level.Width / 2 && y < level.Width / 2 &&
				z >= 0 && z < level.Length)
			{
				density = level.density[levelX, z];
				float zDistance = Math.Abs(y) < 2.0f ? 0.0f : Math.Abs(y);
				// For z distance < 12, increase asteroid depth up to at least 0.65f
				if (zDistance > 12)
					zDistance = 12;
				if (zDistance >= 1 && density < 0.65f)
					density = Math.Max(density, 0.65f * zDistance / 12);
				if (density < 0)
					density = 0;
				if (density > 1)
					density = 1;
			} // if (levelX)

			// Alrighty, the more we move away from the center line, the less
			// asteroids we need. But only apply this for asteroids farer away than
			// 12 sectors from the center (and use 16 for width)
			float distanceFromCenter = (float)Math.Sqrt(2 * x * x + y * y);
			float distancePercent = 1.0f;
			if (distanceFromCenter > 12.0f)
			{
				distancePercent = 1.0f - (distanceFromCenter - 12.0f) / 32.0f;
				if (distancePercent < 0.0f)
					distancePercent = 0.0f;
			} // if (distanceFromCenter)

			//Console.WriteLine("density=" + density);
			int numOfAsteroids = RandomHelper.GetRandomInt(
				(int)(distancePercent * (1 + density * 6)));
			//(int)(density * 8);

			// If empty, create 1 asteroid in 25% of the cases
			if (numOfAsteroids == 0 &&
				distancePercent > 0 &&
				RandomHelper.GetRandomInt(4) == 1)
				numOfAsteroids = 1;

			for (int num = 0; num < numOfAsteroids; num++)
			{
				int type = RandomHelper.GetRandomInt(asteroidModels.Count);
				list.Add(new Asteroid(type,
					new Vector3(x * SectorWidth, y * SectorHeight, z * SectorDepth) +
					// Keep slow moving asteroids in the middle of sectors,
					// don't let them move out to quickly.
					//obs: RandomHelper.GetRandomVector3(
					//-SectorWidth * 0.25f, +SectorWidth * 0.25f),
					RandomHelper.GetRandomVector3(
					-SectorWidth * 0.4f, +SectorWidth * 0.4f),
					asteroidModels[type].ObjectSize));
			} // for (num)
		} // GenerateSector(list, x, y)
		#endregion

		#region Generate smaller asteroids sector
		/// <summary>
		/// Generate smaller asteroids sector
		/// </summary>
		/// <param name="list">List</param>
		/// <param name="numOfAsteroids">Number of asteroids in this sector</param>
		/// <param name="x">X coordinate</param>
		/// <param name="y">Y coordinate</param>
		/// <param name="z">Z coordinate</param>
		private void GenerateSmallerAsteroidsSector(
			List<SmallAsteroids> list, int numOfAsteroids, int x, int y, int z)
		{
			// Always create at least 1 smaller asteroid instance per sector.
			int numOfSmallerAsteroids = 1 + RandomHelper.GetRandomInt(
				(int)(2 + numOfAsteroids));

			for (int num = 0; num < numOfSmallerAsteroids; num++)
			{
				int type = RandomHelper.GetRandomInt(smallAsteroidModels.Count);
				list.Add(new SmallAsteroids(type,
					new Vector3(x * SectorWidth, y * SectorHeight, z * SectorDepth) +
					RandomHelper.GetRandomVector3(-SectorWidth / 2, +SectorWidth / 2),
					smallAsteroidModels[type].ObjectSize));
			} // for (num)
		} // GenerateSmallerAsteroidsSector(list, numOfAsteroids, x)
		#endregion

		#region Calculate sectors
		/// <summary>
		/// Calculate sectors, all asteroids we need to update if we are moving
		/// and checks the visibility of all sectors.
		/// This method add sectors if moving happend and calls
		/// CalculateSectorVisibility to update the visibility of each sector.
		/// For more help please check out the unit tests below.
		/// </summary>
		private void CalculateSectors()
		{
			// Step 1: If moving into a new sector, copy all sectors over
			// and add a new sector at the side we moving into.
			CalculateSectorMovement();

			// Step 2: Visiblity test for all sectors in range
			CalculateSectorVisibility();
		} // CalculateSectors()
		#endregion

		#region Calculate sector movement
		/// <summary>
		/// Calculate sector movement, will add new sectors at one end if
		/// we move at least by one sector in any direction.
		/// </summary>
		private void CalculateSectorMovement()
		{
			Vector3 cameraPos = GraphicForm.CameraPos;
			int cameraSectorPosX = (int)Math.Round(cameraPos.X / SectorWidth);
			int cameraSectorPosY = (int)Math.Round(cameraPos.Y / SectorHeight);
			int cameraSectorPosZ = (int)Math.Round(cameraPos.Z / SectorDepth);

			// Changed sector?
			if (centerSectorPositionX != cameraSectorPosX ||
				centerSectorPositionY != cameraSectorPosY ||
				centerSectorPositionZ != cameraSectorPosZ)
			{
				// Move sectors over, check out how much we are moving
				int movedXSectors = cameraSectorPosX - centerSectorPositionX;
				int movedYSectors = cameraSectorPosY - centerSectorPositionY;
				int movedZSectors = cameraSectorPosZ - centerSectorPositionZ;

				// Helpers to modulate sector position to presever helper lists.
				int modulatedXSectors = movedXSectors;
				int modulatedYSectors = movedYSectors;
				int modulatedZSectors = movedZSectors;
				if (modulatedXSectors < 0)
					modulatedXSectors += NumberOfSectors +
						(-modulatedXSectors / NumberOfSectors) * NumberOfSectors;
				if (modulatedYSectors < 0)
					modulatedYSectors += NumberOfSectors +
						(-modulatedYSectors / NumberOfSectors) * NumberOfSectors;
				if (modulatedZSectors < 0)
					modulatedZSectors += NumberOfSectors +
						(-modulatedZSectors / NumberOfSectors) * NumberOfSectors;

				#region Normal asteroids
				// Helper list for CalculateSectors, which allows us to copy over
				// the whole sectorAsteroids list and add new elements as we need it
				// on any border.
				List<Asteroid>[, ,] helperCopyAsteroids =
					new List<Asteroid>[NumberOfSectors, NumberOfSectors, NumberOfSectors];

				for (int z = 0; z < NumberOfSectors; z++)
					for (int y = 0; y < NumberOfSectors; y++)
						for (int x = 0; x < NumberOfSectors; x++)
							helperCopyAsteroids[z, y, x] = new List<Asteroid>();

				// It is easier to just create a new list and copy everything over
				// that is the same, everything else gets created as we need it.
				// Note: This is complicated ^^ Better paint it on paper to understand
				// this optimized logic. Basically we are just copying all parts
				// that are still used over.
				for (int z = 0; z < NumberOfSectors; z++)
					for (int y = 0; y < NumberOfSectors; y++)
						for (int x = 0; x < NumberOfSectors; x++)
						{
							// Note: This all might look confusing with this temp variable.
							// The only reason we need it is to preserve the old list from
							// our helperCopyAsteroids list. If we would not do that we would
							// have to create a new array of over 1000 elements every time
							// we call this method, which would be too slow!
							//unused: List<Asteroid> temp = helperCopyAsteroids[z, y, x];

							// Can we copy the sector over from sectorAsteroids
							if (x >= -movedXSectors &&
								y >= -movedYSectors &&
								z >= -movedZSectors &&
								x < NumberOfSectors - movedXSectors &&
								y < NumberOfSectors - movedYSectors &&
								z < NumberOfSectors - movedZSectors)
							{
								// Swap lists, we have to do that do not lose the old list
								helperCopyAsteroids[z, y, x] =
									sectorAsteroids[z + movedZSectors,
										y + movedYSectors, x + movedXSectors];
							} // if (x)
							else
							{
								// Else we have to generate this sector!
								GenerateSector(helperCopyAsteroids[z, y, x],
									x - MiddleSector + cameraSectorPosX,
									y - MiddleSector + cameraSectorPosY,
									z - MiddleSector + cameraSectorPosZ);
							} // else
						} // for for for (int)

				// Swap helperCopyAsteroids and sectorAsteroids.
				// We can't just copy helperCopyAsteroids to sectorAsteroids because
				// next time we call MoveSectors helperCopyAsteroids and
				// sectorAsteroids would be the same array and we can't copy over then.
				sectorAsteroids = helperCopyAsteroids;
				#endregion

				#region Smaller asteroids
				// Same stuff for smaller asteroids
				// Helper list for CalculateSectors, which allows us to copy over
				// the whole sectorAsteroids list and add new elements as we need it
				// on any border.
				List<SmallAsteroids>[, ,] helperCopySmallerAsteroids =
					new List<SmallAsteroids>[
					NumberOfSmallSectors, NumberOfSmallSectors, NumberOfSmallSectors];

				for (int z = 0; z < NumberOfSmallSectors; z++)
					for (int y = 0; y < NumberOfSmallSectors; y++)
						for (int x = 0; x < NumberOfSmallSectors; x++)
							helperCopySmallerAsteroids[z, y, x] = new List<SmallAsteroids>();

				// It is easier to just create a new list and copy everything over
				// that is the same, everything else gets created as we need it.
				for (int z = 0; z < NumberOfSmallSectors; z++)
					for (int y = 0; y < NumberOfSmallSectors; y++)
						for (int x = 0; x < NumberOfSmallSectors; x++)
						{
							// Can we copy the sector over from sectorAsteroids
							if (x >= -movedXSectors &&
								y >= -movedYSectors &&
								z >= -movedZSectors &&
								x < NumberOfSmallSectors - movedXSectors &&
								y < NumberOfSmallSectors - movedYSectors &&
								z < NumberOfSmallSectors - movedZSectors)
							{
								// Swap lists, we have to do that do not lose the old list
								helperCopySmallerAsteroids[z, y, x] =
									sectorSmallerAsteroids[z + movedZSectors,
										y + movedYSectors, x + movedXSectors];
							} // if (x)
							else
							{
								// Else we have to generate this sector!
								GenerateSmallerAsteroidsSector(
									helperCopySmallerAsteroids[z, y, x],
									sectorAsteroids[z + SmallSectorAdd,
										y + SmallSectorAdd, x + SmallSectorAdd].Count,
									x - SmallMiddleSector + cameraSectorPosX,
									y - SmallMiddleSector + cameraSectorPosY,
									z - SmallMiddleSector + cameraSectorPosZ);
							} // else
						} // for for for (int)

				// Copy helperCopySmallerAsteroids to sectorSmallerAsteroids.
				sectorSmallerAsteroids = helperCopySmallerAsteroids;
				#endregion

				// Update current sector position
				centerSectorPositionX = cameraSectorPosX;
				centerSectorPositionY = cameraSectorPosY;
				centerSectorPositionZ = cameraSectorPosZ;
			} // if (centerSectorPositionX)
		} // CalculateSectorMovement()
		#endregion

		#region Calculate sector visibility
		/// <summary>
		/// Return angle between two vectors. Used for visbility testing.
		/// </summary>
		/// <param name="vec1">Vector 1</param>
		/// <param name="vec2">Vector 2</param>
		/// <returns>Float</returns>
		public static float GetAngleBetweenVectors(Vector3 vec1, Vector3 vec2)
		{
			// See http://en.wikipedia.org/wiki/Vector_(spatial)
			// for help and check out the Dot Product section ^^
			// Both vectors are normalized so we can save deviding through the
			// lengths.
			return (float)Math.Acos(Vector3.Dot(vec1, vec2));
		} // GetAngleBetweenVectors(vec1, vec2)

		/// <summary>
		/// Calculate sector visibility
		/// </summary>
		private void CalculateSectorVisibility()
		{
			Vector3 cameraRotation = new Vector3(0, 0, 1);
			cameraRotation.TransformNormal(GraphicForm.InverseViewMatrix);

			for (int z = MinSector; z <= MaxSector; z++)
				for (int y = MinSector; y <= MaxSector; y++)
					for (int x = MinSector; x <= MaxSector; x++)
					{
						int iz = z + MiddleSector,
							iy = y + MiddleSector,
							ix = x + MiddleSector;

						// Pre check by distance, sector must be in range.
						bool isVisible = sectorVisibleInRange[iz, iy, ix];

						// Only continue checking if sector is visible at all.
						if (isVisible)
						{
							// Half field of view should be fov / 2, but because of
							// the aspect ratio (1.33) and an additional offset we need
							// to include to see asteroids at the borders.
							isVisible = GetAngleBetweenVectors(
								cameraRotation, sectorDirection[iz, iy, ix]) <
								ViewableFieldOfView;
						} // if (isVisible)

						// Always show inner 3x3 sectors
						if (x >= -1 && x <= +1 &&
							y >= -1 && y <= +1 &&
							z >= -1 && z <= +1)
							isVisible = true;

						// Assign our result
						sectorIsVisible[iz, iy, ix] = isVisible;
					} // for for for (int)
		} // CalculateSectorVisibility()
		#endregion

		#region Abstract methods, not implemented here, but in derived classes
		/// <summary>
		/// Handle sector physics. Not done here, but in derived class!
		/// </summary>
		protected virtual void HandleSectorPhysics(
			int checkX, int checkY, int checkZ,
			int cameraSectorPosX, int cameraSectorPosY, int cameraSectorPosZ)
		{
			// Implement in derived class!
		} // HandleSectorPhysics(checkX, checkY, checkZ)

		/// <summary>
		/// Show all items
		/// </summary>
		/// <param name="shader">Shader</param>
		/// <param name="objectColor">Object color</param>
		/// <param name="lastUpdatedAlpha">Last updated alpha</param>
		/// <param name="cameraPos">Camera position</param>
		protected virtual void ShowAllItems(ParallaxShader shader,
			ref Color objectColor, ref float lastUpdatedAlpha, ref Vector3 cameraPos)
		{
			// Implement in derived class!
		} // ShowAllItems(shader, objectColor, lastUpdatedAlpha)
		#endregion

		#region Render
		/// <summary>
		/// Render, most performance critical method of the game.
		/// Renders just all asteroids, everything else (physics, items, etc.)
		/// is done in the derived classes.
		/// We use (shader) instancing to speed up asteroid rendering!
		/// </summary>
		public void Render()
		{
			#region Initialize and calculate sectors
			// Use alpha blending for blending out asteroids
			GraphicForm.EnableAlphaBlending();

			// Get current sector we are in.
			Vector3 cameraPos = GraphicForm.CameraPos;
			int cameraSectorPosX = (int)Math.Round(cameraPos.X / SectorWidth);
			int cameraSectorPosY = (int)Math.Round(cameraPos.Y / SectorHeight);
			int cameraSectorPosZ = (int)Math.Round(cameraPos.Z / SectorDepth);

			// Note: This method only eats up 2% of the performance in this method,
			// optimizing would still be good, but I've got better things to do
			// right now. Maybe later. Visibility test could also be improved
			// and give us more speed if we skip more sectors!
			CalculateSectors();

			// Clear all visibleAsteroids
			for (int num = 0; num < NumberOfAsteroidTypes; num++)
				visibleAsteroids[num].Clear();
			#endregion

			#region Add visible sectors and handle physics
			for (int z = 0; z < NumberOfSectors; z++)
				for (int y = 0; y < NumberOfSectors; y++)
					for (int x = 0; x < NumberOfSectors; x++)
						if (sectorIsVisible[z, y, x])
						{
							// Update rotation and movement for all asteroids
							foreach (Asteroid asteroid in sectorAsteroids[z, y, x])
							{
								// Update rotation and position.
								asteroid.UpdateMovement(ElapsedTime.MoveFactorPerSecond);

								// Add it to visibility list
								visibleAsteroids[asteroid.Type].Add(asteroid);
							} // foreach (asteroid)

							// Warning: Physics are REALLY slow because we check everything
							// with everything and it involves a lot of math.
							// Only perform physics when we really need it
							// and only apply physics inside of a sector (which might change
							// later in this loop when asteroids are moving outside).
							// See HandleSectorPhysics for more details.

							// Handle sector physics (only calculate 50% per frame).
							if ((z + y + x) % 2 == ElapsedTime.TotalFrames % 2)
								HandleSectorPhysics(x, y, z,
									cameraSectorPosX, cameraSectorPosY, cameraSectorPosZ);
						} // for for for if (sectorIsVisible[z,)
			#endregion

			#region Prepare level position and colors
			// Get level position for sun color
			int levelPos = cameraSectorPosZ;
			if (levelPos < 0)
				levelPos = 0;
			if (levelPos >= level.Length)
				levelPos = level.Length - 1;

			// Only use brown color in the beginning.
			Color brownAsteroidColor = Color.White;
			// Update sun color, also for lighting and lens flare!
			Color sunColor = ColorHelper.InterpolateColor(
				//Color.FromArgb(255, 255, 255),
				brownAsteroidColor,
				level.sunColor[levelPos],
				0.3f);//0.2f);
			GraphicForm.SkyBackgroundColor = ColorHelper.InterpolateColor(
				Color.FromArgb(255, 255, 255),
				level.sunColor[levelPos],
				0.45f);//0.33f);
			GraphicForm.LensFlareColor = ColorHelper.InterpolateColor(
				Color.FromArgb(255, 255, 255),
				level.sunColor[levelPos],
				0.75f);//0.6f);

			// Set color to lighting.
			GraphicForm.LightColor = sunColor;

			// Also update sun position!
			Vector3 sunPos = LensFlare.RotateSun(
				(float)Math.PI / 4.0f +
				Player.gameTimeMs / 50000.0f);
			LensFlare.Origin3D = sunPos;
			GraphicForm.LightDir = -sunPos;

			// Apply sun color for object colors (to 25%)
			Color objectColor = ColorHelper.InterpolateColor(
				brownAsteroidColor,
				sunColor,
				0.33f);//0.2f);
			Color ambientObjectColor =
				ColorHelper.InterpolateColor(
				Material.DefaultAmbientColor,
				level.sunColor[levelPos],
				0.15f);//0.1f);

			float lastUpdatedAlpha = 1.0f;
			#endregion

			// Start up shader
			if (GraphicForm.ParallaxShader.Valid)
			//tst:&& GameForm.Keyboard.Up == false)
			{
				#region Render with shader
				try
				{
					// Prepare shader rendering
					Matrix worldMatrix = Matrix.Identity;
					ParallaxShader shader = GraphicForm.ParallaxShader;
					shader.UpdateMatricesAndEffectParameters();
					Effect effect = shader.D3DEffect;
					effect.Begin(FX.None);

					// We only support 1 pass, more isn't used anyway.
					effect.BeginPass(0);

					// Just render everything with the selected shader
					for (int asteroidType = 0; asteroidType < visibleAsteroids.Length;
						asteroidType++)
					{
						Material mat = asteroidModels[asteroidType].FirstMaterial;
						shader.UpdateMaterialParameters(mat);

						shader.AmbientColorValue = ambientObjectColor;
						shader.DiffuseColorValue = objectColor;

						foreach (Asteroid asteroid in visibleAsteroids[asteroidType])
						{
							// Get distance to viewer
							float distance = (asteroid.position - cameraPos).Length();

							// Skip if out of visible range
							if (distance > MaxViewDepth)
							{
								continue;
							} // if (distance)

							// Update world matrix and commit
							shader.WorldMatrix = asteroid.RenderMatrix;

							// Alpha by distance! Fade out at FadeOutDepth
							float alpha = 1.0f;
							if (distance > FadeOutDepth)
							{
								alpha = 1.0f -
									 ((distance - FadeOutDepth) / (MaxViewDepth - FadeOutDepth));
							} // if (distance)
							if (alpha != lastUpdatedAlpha)
							{
								lastUpdatedAlpha = alpha;
								objectColor = //alpha < 1 ? Color.Red : Color.Green;
									Color.FromArgb((byte)(alpha * 255),
									objectColor);
								shader.DiffuseColorValue = objectColor;
							} // if (alpha)

							// Apply all shader changes
							effect.CommitChanges();

							// Choose second lod level at 50% distance!
							if (distance > HalfViewDepth)
								asteroidModelsLow[asteroidType].D3DMesh.DrawSubset(0);
							else
								asteroidModels[asteroidType].D3DMesh.DrawSubset(0);
						} // foreach (asteroid)
					} // for (asteroidType)

					// Show all items with shader
					ShowAllItems(shader,
						ref objectColor, ref lastUpdatedAlpha, ref cameraPos);

					// End pass and shader. Puh, that was a lot of rendering ^^
					effect.EndPass();
					effect.End();
				} // try
				catch (Exception ex)
				{
					Log.Write("Failed to render asteroids with shader, " +
						"falling back to fixed function pipeline rendering: " +
						ex.ToString());
					GraphicForm.ParallaxShader.SetEffectInvalid();
				} // catch

				// Own profiler results 2005-11-22 (1331 asteroids, 161 visible):
				// [11:58:08] AsteroidManager (Total time: 6,56ms)
				// Section: Render asteroids (6,56ms, 100%): 
				//	calc visiblity: 151ns (2,3%)
				//	clearing visibleAsteroids list: 8ns (0,1%)
				//	adding to visibleAsteroids list: 67ns (1%)
				//	Prepare: 85ns (1,3%)
				//	UpdateMaterialParameters: 105ns (1,6%, Times called: 4,
				//    Average time per call: 26ns)
				//	UpdateRenderMatrix: 1,77ms (26,9%, Times called: 161,
				//    Average time per call: 10ns)
				//	Update world matrix and commit: 2,51ms (38,2%, Times called: 161,
				//    Average time per call: 15ns)
				//	Render mesh: 1,84ms (28,1%, Times called: 161,
				//    Average time per call: 11ns)
				//	End shader stuff: 23ns (0,4%)
				#endregion
			} // if (GraphicForm.ParallaxShader.Valid)
			else
			{
				#region Render without shader
				// Restore all render states we need here
				GraphicForm.DirectXDevice.RenderState.Lighting = true;
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = true;
				GraphicForm.DirectXDevice.RenderState.CullMode = Cull.CounterClockwise;

				// Use alpha blending for blending out asteroids
				GraphicForm.EnableAlphaBlending();

				// Just render normally without shader
				for (int asteroidType = 0; asteroidType < visibleAsteroids.Length;
					asteroidType++)
				{
					asteroidModels[asteroidType].FirstMaterial.SelectWithoutShader();
					Microsoft.DirectX.Direct3D.Material d3dMaterial =
						asteroidModels[asteroidType].FirstMaterial.D3DMaterial;

					// We don't have to set ambient or diffuse colors, lighting
					// color is already set above and will be used.
					d3dMaterial.Ambient = ambientObjectColor;

					foreach (Asteroid asteroid in visibleAsteroids[asteroidType])
					{
						// Get distance to viewer
						float distance = (asteroid.position - cameraPos).Length();

						// Skip if out of visible range
						if (distance > MaxViewDepth)
						{
							continue;
						} // if (distance)

						GraphicForm.DirectXDevice.Transform.World =
							asteroid.RenderMatrix;

						// Alpha by distance! Fade out at FadeOutDepth
						float alpha = 1.0f;
						if (distance > FadeOutDepth)
						{
							alpha = 1.0f -
								 ((distance - FadeOutDepth) / (MaxViewDepth - FadeOutDepth));
						} // if (distance)
						if (alpha != lastUpdatedAlpha)
						{
							lastUpdatedAlpha = alpha;

							// Fade in/out with alpha
							d3dMaterial.Diffuse = Color.FromArgb(
								(byte)(alpha * 255), d3dMaterial.Diffuse);

							// Update material. Note: This is slow, maybe it is
							// faster just to update a special color like ConstantColor
							// and improving performance this way.
							GraphicForm.DirectXDevice.Material = d3dMaterial;
						} // if (alpha)

						// Choose second lod level at 50% distance!
						if (distance > HalfViewDepth)
							asteroidModelsLow[asteroidType].D3DMesh.DrawSubset(0);
						else
							asteroidModels[asteroidType].D3DMesh.DrawSubset(0);
					} // foreach (asteroid)
				} // for (asteroidType)

				// Show all items, this time without shader
				ShowAllItems(null,
					ref objectColor, ref lastUpdatedAlpha, ref cameraPos);
				#endregion
			} // else

			#region Small asteroids
			// Show all small asteroids, all of them use the same texture!
			Material smallAsteroidMat = smallAsteroidModels[0].FirstMaterial;
			Microsoft.DirectX.Direct3D.Material d3dMat =
				smallAsteroidMat.D3DMaterial;
			smallAsteroidMat.SelectWithoutShader();
			GraphicForm.DirectXDevice.RenderState.Lighting = true;
			float lastAlphaValue = 1.0f;

			// Go through all visible smaller asteroids
			for (int z = SmallSectorAdd; z < SmallSectorAdd + NumberOfSmallSectors;
				z++)
				for (int y = SmallSectorAdd; y < SmallSectorAdd + NumberOfSmallSectors;
					y++)
					for (int x = SmallSectorAdd; x < SmallSectorAdd + NumberOfSmallSectors;
						x++)
						if (sectorIsVisible[z, y, x])
						{
							foreach (SmallAsteroids smallAsteroids in
								sectorSmallerAsteroids[z - SmallSectorAdd,
								y - SmallSectorAdd, x - SmallSectorAdd])
							{
								// Get distance to viewer
								float distance =
									(smallAsteroids.Position - cameraPos).Length();

								// Skip if out of visible range
								if (distance > HalfViewDepth)
									continue;

								GraphicForm.WorldMatrix = smallAsteroids.RenderMatrix;

								// Alpha by distance! Fade out at SmallAsteroidFadeOutDepth
								float alpha = 1.0f;

								if (distance > SmallAsteroidFadeOutDepth)
								{
									alpha = 1.0f -
										 ((distance - SmallAsteroidFadeOutDepth) /
										 (HalfViewDepth - SmallAsteroidFadeOutDepth));
								} // if (distance)
								if (alpha != lastAlphaValue)
								{
									lastAlphaValue = alpha;

									// Fade in/out with alpha
									d3dMat.Diffuse = Color.FromArgb(
										(byte)(alpha * 255), d3dMat.Diffuse);

									// Update material. Note: This is slow, maybe it is
									// faster just to update a special color like ConstantColor
									// and improving performance this way.
									GraphicForm.DirectXDevice.Material = d3dMat;
								} // if (alpha)

								// And render
								smallAsteroidModels[smallAsteroids.Type].D3DMesh.DrawSubset(0);
							} // foreach (smallAsteroids)
						} // for for for if (sectorIsVisible[z,)
			#endregion
		} // Render()
		#endregion

		#region Unit Testing
#if DEBUG
		#region Asteroid manager tests
		/// <summary>
		/// Asteroid manager tests
		/// </summary>
		[TestFixture]
		public class AsteroidManagerTests
		{
			/// <summary>
			/// For simplicity we just use a simple 2d array
			/// Note: We use the same variable names as above for easy copy+paste ^^
			/// </summary>
			// Note 2005-11-24: We no longer use helperCopyAsteroids array above
			// anymore because it only makes things more complicated and doesn't
			// give us any performance benefit. Maybe we need this logic later on.
			const int NumOfSectors = 3;
			/// <summary>
			/// Sector asteroids
			/// </summary>
			List<int>[,] sectorAsteroids =
				new List<int>[NumOfSectors, NumOfSectors];
			List<int>[,] helperCopyAsteroids =
				new List<int>[NumOfSectors, NumOfSectors];
			/// <summary>
			/// Camera sector position x coordinate
			/// </summary>
				/// <summary>
				/// Camera sector position y coordinate
				/// </summary>
			int cameraSectorPosX = 0,
				cameraSectorPosY = 0;

			/// <summary>
			/// Our crazy value formular, used for TestCalculateSectorsMovement.
			/// </summary>
			/// <param name="x">X coordinate</param>
			/// <param name="y">Y coordinate</param>
			/// <returns>Int</returns>
			private void GenerateSector(List<int> list, int x, int y)
			{
				list.Clear();
				list.Add(x + y * 2);
			} // GenerateSector(list, x, y)

			/// <summary>
			/// Move sectors, helper method for TestCalculateSectorsMovement.
			/// </summary>
			/// <param name="moveX">Move x coordinate</param>
			/// <param name="moveY">Move y coordinate</param>
			private void MoveSectors(int movedXSectors, int movedYSectors)
			{
				cameraSectorPosX += movedXSectors;
				cameraSectorPosY += movedYSectors;
				/*I think we don't need that!
				// Bound to NumOfSectors
				if (movedXSectors > NumOfSectors)
					movedXSectors = NumOfSectors;
				if (movedXSectors < -NumOfSectors)
					movedXSectors = -NumOfSectors;
				if (movedYSectors > NumOfSectors)
					movedYSectors = NumOfSectors;
				if (movedYSectors < -NumOfSectors)
					movedYSectors = -NumOfSectors;
				 */
				// Helpers to modulate sector position to presever helper lists.
				int modulatedXSectors = movedXSectors;
				int modulatedYSectors = movedYSectors;
				if (modulatedXSectors < 0)
					modulatedXSectors += NumOfSectors +
						(-modulatedXSectors / NumOfSectors) * NumOfSectors;
				if (modulatedYSectors < 0)
					modulatedYSectors += NumOfSectors +
						(-modulatedYSectors / NumOfSectors) * NumOfSectors;
				//Console.WriteLine("movedXSectors=" + movedXSectors +
				//  ", movedYSectors=" + movedYSectors +
				//  ", modulatedXSectors=" + modulatedXSectors +
				//  ", modulatedYSectors=" + modulatedYSectors);

				// It is easier to just create a new list and copy everything over
				// that is the same, everything else gets created as we need it.
				// Note: This is complicated ^^ Better paint it on paper to understand
				// this optimized logic. Basically we are just copying all parts
				// that are still used over.
				for (int y = 0; y < NumOfSectors; y++)
					for (int x = 0; x < NumOfSectors; x++)
					{
						// Note: This all might look confusing with this temp variable.
						// The only reason we need it is to preserve the old list from
						// our helperCopyAsteroids list. If we would not do that we would
						// have to create a new array of over 1000 elements every time
						// we call this method, which would be too slow!
						List<int> temp = helperCopyAsteroids[y, x];

						// Can we copy the sector over from sectorAsteroids
						if (x >= -movedXSectors &&
							y >= -movedYSectors &&
							x < NumOfSectors - movedXSectors &&
							y < NumOfSectors - movedYSectors)
						{
							// Swap lists, we have to do that do not lose the old list
							helperCopyAsteroids[y, x] =
								sectorAsteroids[y + movedYSectors, x + movedXSectors];
							sectorAsteroids[y + movedYSectors, x + movedXSectors] = temp;
							//Console.WriteLine("x=" + x + ", y=" + y +
							//  ", movedXSectors=" + movedXSectors +
							//  ", movedYSectors=" + movedYSectors +
							//  ", value=" + helperCopyAsteroids[y, x][0]);
						} // if (x)
						else
						{
							// Else we have to generate this sector!
							GenerateSector(helperCopyAsteroids[y, x],
								x + cameraSectorPosX, y + cameraSectorPosY);
							sectorAsteroids[(y + modulatedYSectors) % NumOfSectors,
								(x + modulatedXSectors) % NumOfSectors] = temp;
							//Console.WriteLine("x=" + x + ", y=" + y +
							//  ", value=" + helperCopyAsteroids[y, x][0]);
						} // else
					} // for for (int)

				/*not longer required
				// Now we generate the newly generated sectors.
				for (int y = 0; y < NumOfSectors; y++)
					for (int x = 0; x < NumOfSectors; x++)
						// This is the inverse test that aboveUff, TODO: help!
						if ((movedXSectors > 0 &&
							x >= NumOfSectors - movedXSectors) ||
							(movedXSectors < 0 &&
							x < -movedXSectors) ||
							(movedYSectors > 0 &&
							y >= NumOfSectors - movedYSectors) ||
							(movedYSectors < 0 &&
							y < -movedYSectors))
						{
							helperCopyAsteroids[y, x] =
								OurCrazyValueFormular(x + movedXSectors, y + movedYSectors);
						} // for for if
				 */

				// Swap helperCopyAsteroids and sectorAsteroids.
				// We can't just copy helperCopyAsteroids to sectorAsteroids because
				// next time we call MoveSectors helperCopyAsteroids and
				// sectorAsteroids would be the same array and we can't copy over then.
				List<int>[,] tempList = sectorAsteroids;
				sectorAsteroids = helperCopyAsteroids;
				helperCopyAsteroids = tempList;
			} // MoveSectors(movedXSectors, movedYSectors)

			/// <summary>
			/// Get first sector value
			/// </summary>
			/// <param name="x">X coordinate</param>
			/// <param name="y">Y coordinate</param>
			/// <returns>Int</returns>
			public int GetFirstSectorValue(int x, int y)
			{
				return sectorAsteroids[y, x][0];
			} // GetFirstSectorValue(x, y)

			/// <summary>
			/// Test calculate sectors movement.
			/// Haha, good luck to understand this unit test without my
			/// graphics on paper ^^ anyway, it is a good way to check the logic
			/// above. Check out the CalculateSectors method in AsteroidManager.
			/// </summary>
			[Test]
			public void TestCalculateSectorsMovement()
			{
				// Fill just some numbers (x+y)
				for (int y = 0; y < NumOfSectors; y++)
					for (int x = 0; x < NumOfSectors; x++)
					{
						sectorAsteroids[y, x] = new List<int>();
						helperCopyAsteroids[y, x] = new List<int>();

						GenerateSector(sectorAsteroids[y, x], x, y);
					} // for for (int)

				// Check if sector values are as expected
				Assert.AreEqual(3, GetFirstSectorValue(1, 1));
				Assert.AreEqual(4, GetFirstSectorValue(0, 2));
				Assert.AreEqual(5, GetFirstSectorValue(1, 2));

				// Move sectors
				MoveSectors(1, 0);
				//for (int y = 0; y < NumOfSectors; y++)
				//	for (int x = 0; x < NumOfSectors; x++)
				//		helperCopyAsteroids[y, x] = new List<int>();

				// Check if sector values are as expected
				Assert.AreEqual(1, GetFirstSectorValue(0, 0));
				Assert.AreEqual(3, GetFirstSectorValue(0, 1));
				Assert.AreEqual(4, GetFirstSectorValue(1, 1));
				Assert.AreEqual(7, GetFirstSectorValue(2, 2));

				// Restore
				MoveSectors(-1, 0);
				//for (int y = 0; y < NumOfSectors; y++)
				//	for (int x = 0; x < NumOfSectors; x++)
				//		helperCopyAsteroids[y, x] = new List<int>();

				// Check if sector values are as expected
				Assert.AreEqual(3, GetFirstSectorValue(1, 1));
				Assert.AreEqual(4, GetFirstSectorValue(0, 2));
				Assert.AreEqual(5, GetFirstSectorValue(1, 2));
				/*
				for (int y = 0; y < NumOfSectors; y++)
					for (int x = 0; x < NumOfSectors; x++)
					{
						GenerateSector(sectorAsteroids[y, x], x, y);
						//helperCopyAsteroids[y, x] = new List<int>();
					} // for for
				 */

				//Console.WriteLine(
				//	", value 0 0=" + sectorAsteroids[0, 0] +
				//	", value 1 1=" + sectorAsteroids[1, 1]);

				// Move sectors
				MoveSectors(-1, 0);
				//for (int y = 0; y < NumOfSectors; y++)
				//	for (int x = 0; x < NumOfSectors; x++)
				//		helperCopyAsteroids[y, x] = new List<int>();

				// Check if sector values are as expected
				Assert.AreEqual(-1, GetFirstSectorValue(0, 0));
				Assert.AreEqual(1, GetFirstSectorValue(0, 1));
				Assert.AreEqual(2, GetFirstSectorValue(1, 1));
				Assert.AreEqual(5, GetFirstSectorValue(2, 2));

				// Restore
				MoveSectors(+1, 0);
				//for (int y = 0; y < NumOfSectors; y++)
				//	for (int x = 0; x < NumOfSectors; x++)
				//		helperCopyAsteroids[y, x] = new List<int>();

				// Move sectors
				MoveSectors(-1, +1);

				// Check if sector values are as expected
				Assert.AreEqual(1, GetFirstSectorValue(0, 0));
				Assert.AreEqual(2, GetFirstSectorValue(1, 0));
				Assert.AreEqual(3, GetFirstSectorValue(2, 0));
				Assert.AreEqual(3, GetFirstSectorValue(0, 1));
				Assert.AreEqual(7, GetFirstSectorValue(2, 2));
				Assert.AreEqual(5, GetFirstSectorValue(0, 2));

				// Restore
				MoveSectors(+1, -1);
				//for (int y = 0; y < NumOfSectors; y++)
				//	for (int x = 0; x < NumOfSectors; x++)
				//		helperCopyAsteroids[y, x] = new List<int>();

				// Move sectors
				MoveSectors(-5, -2);
				//for (int y = 0; y < NumOfSectors; y++)
				//	for (int x = 0; x < NumOfSectors; x++)
				//		helperCopyAsteroids[y, x] = new List<int>();

				// Check if sector values are as expected
				Assert.AreEqual(-9, GetFirstSectorValue(0, 0));
				Assert.AreEqual(-6, GetFirstSectorValue(1, 1));
				Assert.AreEqual(-7, GetFirstSectorValue(2, 0));
				Assert.AreEqual(-3, GetFirstSectorValue(2, 2));
				Assert.AreEqual(-5, GetFirstSectorValue(0, 2));

				// Move a little around like crazy and make sure we don't crash
				for (int i = 0; i < 11; i++)
					for (int j = 0; j < 14; j++)
						MoveSectors(-i, +j);
			} // TestCalculateSectorsMovement()
		} // class AsteroidManagerTests
		#endregion

		#region Fade in asteroids without shader
		/// <summary>
		/// Fade in asteroids tests
		/// </summary>
		class FadeInAsteroidsTests
		{
			/// <summary>
			/// Test fade in asteroids without shader
			/// </summary>
			[Test]
			public static void TestFadeInAsteroidsWithoutShader()
			{
				Model smallAsteroid = null;
				GameForm.StartTest("TestFadeInAsteroidsWithoutShader",
					delegate
					{
						smallAsteroid = new Model("SmallAsteroid1.X");
					},
					delegate
					{
						GameForm.TestGameForm.camera.FreeCamera = true;
						smallAsteroid.FirstMaterial.SelectWithoutShader();
						GraphicForm.DirectXDevice.RenderState.Lighting = true;
						GraphicForm.WorldMatrix = Matrix.Scaling(0.1f, 0.1f, 0.1f);
						GraphicForm.EnableAlphaBlending();

						Microsoft.DirectX.Direct3D.Material d3dMat =
							smallAsteroid.FirstMaterial.D3DMaterial;
						d3dMat.Diffuse = Color.FromArgb(
							(ElapsedTime.MsTotal / 10) % 256, Color.Green);
						GraphicForm.DirectXDevice.Material = d3dMat;

						/*
						Color testColor = Color.FromArgb(
							(ElapsedTime.MsTotal/10)%256, Color.Green);
						
						// Enable constant color mode
						TextureStateManager firstTextureState =
							GraphicForm.DirectXDevice.TextureState[0];
						firstTextureState.ConstantColorValue = testColor.ToArgb();
						firstTextureState.ColorOperation = TextureOperation.Modulate;
						firstTextureState.ColorArgument2 = TextureArgument.Constant;
						firstTextureState.AlphaOperation = TextureOperation.Modulate;
						firstTextureState.AlphaArgument2 = TextureArgument.Constant;
						 */

						smallAsteroid.D3DMesh.DrawSubset(0);
					});
			} // TestFadeInAsteroidsWithoutShader()
		} // class FadeInAsteroidsTests
		#endregion

		#region Test rendering asteroids
		/// <summary>
		/// Test rendering asteroids for Tutorial 2.
		/// We want just to display a bunch of asteroids in 3d space.
		/// </summary>
		[Test]
		public static void TestRenderingAsteroids()
		{
			List<Asteroid> heinoAsteroids = new List<Asteroid>();
			heinoAsteroids.Add(new Asteroid(
				4, Vector3.Empty, 1.0f));
			heinoAsteroids.Add(new Asteroid(
				4, new Vector3(100, 0, 0), 3.0f));
			heinoAsteroids.Add(new Asteroid(
				4, new Vector3(0, 100, 0), 0.5f));
			Model asteroidModel = null;

			GameForm.StartTest("Test rendering asteroids",
				delegate
				{
					asteroidModel = new Model("Asteroid5.x");
				},
				delegate
				{
					foreach (Asteroid asteroid in heinoAsteroids)
						asteroidModel.Render(asteroid.RenderMatrix);
				});
		} // TestRenderingAsteroids()
		#endregion

		#region Test asteroid manager
		/// <summary>
		/// Basically test the BaseAsteroidManager class.
		/// </summary>
		[Test]
		public static void TestAsteroidManager()
		{
			BaseAsteroidManager asteroidManager = null;

			GameForm.StartTest("Test asteroid manager",
				delegate
				{
					asteroidManager = new BaseAsteroidManager(
						Level.LoadAllLevels()[0]);
				},
				delegate
				{
					asteroidManager.Render();
				});
		} // TestAsteroidManager()
		#endregion
#endif
		#endregion
	} // class BaseAsteroidManager
} // namespace RocketCommander.Game
