// Project: Rocket Commander, File: GameAsteroidManager.cs
// Namespace: RocketCommander.Game, Class: GameAsteroidManager
// Path: C:\code\Rocket Commander\Game, Author: Abi
// Code lines: 4019, Size of file: 148423 Bytes
// Creation date: 07.11.2005 19:38
// Last modified: 27.02.2006 19:59
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Drawing;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;
using Effect = Microsoft.DirectX.Direct3D.Effect;
using RocketCommander.Graphics;
using RocketCommander.Helpers;
using RocketCommander.Shaders;
using RocketCommander.Input;
using RocketCommander.Properties;
using RocketCommander.Sounds;
using Texture = RocketCommander.Graphics.Texture;
using Font = RocketCommander.Graphics.Font;
using Material = RocketCommander.Graphics.Material;
#endregion

namespace RocketCommander.Game
{
	/// <summary>
	/// Game asteroid manager, derived from BaseAsteroidManager over the
	/// PhysicsAsteroidManager.
	/// 
	/// The BaseAsteroidManager handles all the base variables we need for our
	/// asteroid field. Also generates sectors and handles smaller asteroids.
	/// Provides some basic render methods to show the asteroid field.
	/// This class is nothing we can use for the game, it just provides us
	/// the very basic features for the asteroid field.
	/// <para/>
	/// Derived from this is the PhysicsAsteroidManager, which adds all Physics
	/// (handle player collision, handle sector Physics, etc.).
	/// <para/>
	/// And derived from that we finally get to the GameAsteroidManager,
	/// which calculates sector movement, does visibility checks, etc.
	/// The GameAsteroidManager is the class we use in the game, it provides
	/// a much richer Render method and handles also additional effects like
	/// the Hit effects we see on screen.
	/// </summary>
	public class GameAsteroidManager : PhysicsAsteroidManager, IDisposable
	{
		#region Constants
		/// <summary>
		/// All item colors in an array, used for glow effects in Render(..)
		/// </summary>
		static readonly Color[] ItemColors =
			new Color[Level.NumOfItemTypes]
			{
				Level.FuelItemColor,
				Level.HealthItemColor,
				Level.ExtraLifeItemColor,
				Level.SpeedItemColor,
				Level.BombItemColor,
			};

		/// <summary>
		/// Filenames for the item models.
		/// </summary>
		public static readonly string[] ItemModelFilenames =
			new string[Level.NumOfItemTypes]
			{
				"Fuel",
				"Health",
				"ExtraLife",
				"Speed",
				"Bomb",
			};

		public static readonly string[] ItemSoundFilename =
			new string[Level.NumOfItemTypes]
			{
				"Fuel.wav",
				"Health.wav",
				"ExtraLife.wav",
				"Speed.wav",
				"Bomb.wav",
			};

		/// <summary>
		/// Size for items.
		/// </summary>
		public const float ItemSize = 33.33f;
		#endregion

		#region Variables
		/// <summary>
		/// Current items in our current level,
		/// rendered at the end of the Render method.
		/// Will be copied over from level.items when calling SetLevel!
		/// </summary>
		List<Vector3>[] items = new List<Vector3>[]
			{
				// Fuel
				new List<Vector3>(),
				// Health
				new List<Vector3>(),
				// ExtraLife
				new List<Vector3>(),
				// Speed
				new List<Vector3>(),
				// Bomb
				new List<Vector3>(),
			};

		/// <summary>
		/// Item sounds, played when collecting a item.
		/// The reason why the sounds are located here in the AsteroidManager
		/// instead of the Mission class is to improve performance.
		/// We can check in the render loop if any item is near to us for free.
		/// The distance check is performed anyway for visibility and alpha
		/// blending operations.
		/// </summary>
		Sample[] itemSounds = new Sample[Level.NumOfItemTypes];

		/// <summary>
		/// And some more sound samples for the game. Used in Player.cs.
		/// </summary>
		private Sample rocketMotorSound = null,
			explosionSound = null,
			victorySound = null,
			defeatSound = null;

		/// <summary>
		/// Play rocket motor sound
		/// </summary>
		/// <param name="volume">Volume</param>
		public void PlayRocketMotorSound(float volume)
		{
			rocketMotorSound.PlayLooped();
			rocketMotorSound.Volume = 0.86f;
		} // PlayRocketMotorSound(volume)

		/// <summary>
		/// Change rocket motor pitch effect
		/// </summary>
		/// <param name="pitchFactor">Pitch factor</param>
		public void ChangeRocketMotorPitchEffect(float pitchFactor)
		{
			rocketMotorSound.PitchFactor = pitchFactor;
		} // ChangeRocketMotorPitchEffect(pitchFactor)

		/// <summary>
		/// Stop rocket motor sound
		/// </summary>
		public void StopRocketMotorSound()
		{
			rocketMotorSound.Stop();
		} // StopRocketMotorSound()

		/// <summary>
		/// Play defeat sound
		/// </summary>
		public void PlayDefeatSound()
		{
			defeatSound.Play();
		} // PlayDefeatSound()

		/// <summary>
		/// Play victory sound
		/// </summary>
		public void PlayVictorySound()
		{
			victorySound.Play();
		} // PlayVictorySound()

		/// <summary>
		/// Play explosion sound
		/// </summary>
		public void PlayExplosionSound()
		{
			explosionSound.Play();
		} // PlayExplosionSound()

		/// <summary>
		/// Item models for all 5 item types: Fuel, Health, ExtraLife, Speed, Bomb.
		/// </summary>
		private AnimatedModel[] itemModels =
			new AnimatedModel[Level.NumOfItemTypes];

		/// <summary>
		/// Goal texture, displayed at the end of each level.
		/// </summary>
		private Texture goalTexture = null;

		/// <summary>
		/// Product logos texture. Requested by microsoft to display
		/// the ascend and touchdown product logos in the game while playing.
		/// </summary>
		private Texture productLogosTexture = null;

		/// <summary>
		/// Helper texture to show direction where we hit some asteroid.
		/// This greatly helps the player to understand why he loses hitpoints
		/// and that he loses hitpoints at all (together with the sound effect).
		/// </summary>
		private Texture hitDirectionTexture = null;

		/// <summary>
		/// Helper mesh for the goal.
		/// </summary>
		Mesh goalMesh = null;
		#endregion

		#region Constructor
		/// <summary>
		/// Create asteroid manager
		/// </summary>
		public GameAsteroidManager(Level setLevel)
			: base(setLevel)
		{
			// Load all item sounds
			for (int num = 0; num < Level.NumOfItemTypes; num++)
			{
   			itemSounds[num] = new Sample(ItemSoundFilename[num]);
			} // for (num)

			rocketMotorSound = new Sample("RocketMotor.wav", 1);
			explosionSound = new Sample("Explosion.wav");
			victorySound = new Sample("Victory.wav", 1);
			defeatSound = new Sample("Defeat.wav", 1);

			// Load all items
			for (int num = 0; num < Level.NumOfItemTypes; num++)
			{
				// All items are animated, load with help of AnimatedModel.
				itemModels[num] = new AnimatedModel(ItemModelFilenames[num]);
			} // for (num)

			// Load product logos texture
			productLogosTexture = new Texture("ProductLogos.png");

			// Load hit direction texture
			hitDirectionTexture = new Texture("HitDirection.dds");

			// Load goal texture and build sphere mesh for it.
			goalTexture = new Texture("Goal.png");
			goalMesh = Mesh.Sphere(GraphicForm.DirectXDevice, 10.0f, 16, 16);
			Model.GenerateNormalsAndTangentsIfNotPresent(ref goalMesh, false);
		} // AsteroidManager()

		/// <summary>
		/// Current level
		/// </summary>
		/// <returns>Level</returns>
		public override Level CurrentLevel
		{
			get
			{
				return level;
			} // get
			set
			{
				base.CurrentLevel = value;

				// Copy over all items
				for (int num = 0; num < Level.NumOfItemTypes; num++)
				{
					items[num] = new List<Vector3>(level.items[num]);
				} // for (num)
			} // set
		} // CurrentLevel
		#endregion

		#region Dispose
		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			for (int num = 0; num < Level.NumOfItemTypes; num++)
				if (itemSounds[num] != null)
					itemSounds[num].Dispose();

			if (goalTexture != null)
				goalTexture.Dispose();
			if (productLogosTexture != null)
				productLogosTexture.Dispose();
			if (hitDirectionTexture != null)
				hitDirectionTexture.Dispose();
		} // Dispose()
		#endregion

		#region Add distance to be displayed
		/// <summary>
		/// Helper list to hold all screen positions and distances we want
		/// to display later on (after all the 3d rendering).
		/// </summary>
		static List<PosAndDistance> remToDisplayDistance =
			new List<PosAndDistance>();

		/// <summary>
		/// Product logos to display below the distance.
		/// </summary>
		public enum ProductLogo
		{
			None,
			Ascend,
			Touchdown,
		} // enum ProductLogo

		/// <summary>
		/// Position and distance
		/// </summary>
		class PosAndDistance
		{
			/// <summary>
			/// Screen position
			/// </summary>
			public Point pos;
			/// <summary>
			/// Distance
			/// </summary>
			public float distance;
			/// <summary>
			/// Alpha
			/// </summary>
			public float alpha;
			/// <summary>
			/// Product logo
			/// </summary>
			public ProductLogo productLogo;

			/// <summary>
			/// Create position and distance
			/// </summary>
			/// <param name="setPos">Set position</param>
			/// <param name="setDistance">Set distance</param>
			/// <param name="setAlpha">Set alpha</param>
			public PosAndDistance(Point setPos, float setDistance,
				float setAlpha, ProductLogo setProductLogo)
			{
				pos = setPos;
				distance = setDistance;
				alpha = setAlpha;
				productLogo = setProductLogo;
			} // PosAndDistance(setPos, setDistance, setAlpha)
		} // class PosAndDistance

		/// <summary>
		/// Add distance to be displayed
		/// </summary>
		/// <param name="screenPos">Screen position</param>
		/// <param name="distance">Distance</param>
		public static void AddDistanceToBeDisplayed(
			Point screenPos, float distance, float alpha,
			ProductLogo productLogo)
		{
			// Max. display 10 distances, else the screen will get very confusing!
			if (remToDisplayDistance.Count < 10)
				remToDisplayDistance.Add(new PosAndDistance(
					screenPos, distance, alpha, productLogo));
			else
			{
				// If we have more distances, check if we can replace the farest
				float farestDistance = float.MinValue;
				PosAndDistance remFarestItem = null;
				foreach (PosAndDistance pad in remToDisplayDistance)
					if (pad.distance > farestDistance)
					{
						farestDistance = pad.distance;
						remFarestItem = pad;
					} // if

				// Is this better than farest distance?
				if (distance < farestDistance)
				{
					// Then replace
					remFarestItem.pos = screenPos;
					remFarestItem.distance = distance;
					remFarestItem.alpha = alpha;
					remFarestItem.productLogo = productLogo;
				} // if
			} // else
		} // AddDistanceToBeDisplayed(screenPos, distance, alpha)

		/// <summary>
		/// Show all distances
		/// </summary>
		public void ShowAllDistances(Font smallFont)
		{
			foreach (PosAndDistance posDistance in remToDisplayDistance)
			{
				smallFont.WriteSmallTextCentered(
					posDistance.pos.X, posDistance.pos.Y + 40,
					// Only display full numbers (round them up)
					Texts.Distance + ": " +
					(int)Math.Round(posDistance.distance / SectorDepth - 0.26f),
					// Only display with half the alpha!
					posDistance.alpha * 0.5f);

				// For the health item display the ascend logo and
				// for the speed item display the touchdown logo!
				if (posDistance.productLogo == ProductLogo.Ascend ||
					posDistance.productLogo == ProductLogo.Touchdown)
				{
					float logoAlpha = 1.0f -
						(posDistance.distance / (SectorDepth * 22.0f));
					if (posDistance.distance < SectorDepth * 3.5f)
						logoAlpha = -0.2f + posDistance.distance / (SectorDepth * 3.5f);
					if (logoAlpha < 0.0f)
						continue;
					if (logoAlpha > 1.0f)
						logoAlpha = 1.0f;

					if (posDistance.productLogo == ProductLogo.Ascend)
						productLogosTexture.RenderOnScreen(
							new Rectangle(posDistance.pos.X - 64,
							posDistance.pos.Y + 70 - 32,
							128, 64), new Rectangle(0, 64, 128, 64),
							Color.White, logoAlpha * 0.85f);
					else
						productLogosTexture.RenderOnScreen(
							new Rectangle(posDistance.pos.X - 64,
							posDistance.pos.Y + 70 - 32,
							128, 64), new Rectangle(0, 0, 128, 64),
							Color.White, logoAlpha * 0.85f);
				} // if
			} // foreach  (posDistance)

			// Clear
			remToDisplayDistance.Clear();
		} // ShowAllDistances()
		#endregion

		#region Show hit direction effect
		const int MaxHitDirectionTimeoutMs = 2000;
		int hitDirectionEffectTimeoutMs = 0;
		float hitDirection = 0.0f;
		/// <summary>
		/// Set hit direction effect
		/// </summary>
		/// <param name="setDirection">Direction</param>
		protected override void SetHitDirectionEffect(float setDirection)
		{
			hitDirectionEffectTimeoutMs = MaxHitDirectionTimeoutMs;
			hitDirection = setDirection;
		} // SetHitDirectionEffect(direction)

		/// <summary>
		/// Show hit direction effect
		/// </summary>
		public void ShowHitDirectionEffect()
		{
			if (hitDirectionEffectTimeoutMs > 0)
			{
				hitDirectionEffectTimeoutMs -= ElapsedTime.MsLastFrame;

				hitDirectionTexture.RenderOnScreenWithRotation(
					new Point(GraphicForm.Resolution.Width / 2,
					GraphicForm.Resolution.Height / 2),
					GraphicForm.Resolution.Height / 2,
					(float)Math.PI + hitDirection,
					0, 0, 1, 1,
					Color.White,
					// Use 1.0f for first second, then fade out.
					Math.Min(1.0f,
					(hitDirectionEffectTimeoutMs /
					((float)MaxHitDirectionTimeoutMs / 2.0f))));
			} // if
		} // ShowHitDirectionEffect()
		#endregion

		#region Show all items helper method
		/// <summary>
		/// Show all items
		/// </summary>
		/// <param name="useShader">Use shader</param>
		protected override void ShowAllItems(ParallaxShader shader,
			ref Color objectColor, ref float lastUpdatedAlpha, ref Vector3 cameraPos)
		{
			try
			{
				// Shader version?
				if (shader != null)
				{
					// Show all items
					for (int num = 0; num < Level.NumOfItemTypes; num++)
					{
						// Animate item type
						itemModels[num].Animate();
						shader.UpdateMaterialParameters(itemModels[num].FirstMaterial);
						objectColor = Color.White;
						lastUpdatedAlpha = 1.0f;
						shader.DiffuseColorValue = objectColor;
						shader.AmbientColorValue = Color.DarkGray;

						// Go through all items of this type
						for (int itemNum = 0; itemNum < items[num].Count; itemNum++)
						{
							Vector3 objPos = items[num][itemNum];

							// Get distance to viewer
							float distance = (objPos - cameraPos).Length();

							// Skip if out of visible range
							// Use trick to display even items 6 times out of visible area!
							if (distance > MaxViewDepth * 6)
								continue;

							// If very close to item, collect it and play sound
							if (distance <= ItemSize * 1.45f)
							{
								Player.HandleItem(itemSounds, num);
								// Remove item!
								items[num].RemoveAt(itemNum--);
								continue;
							} // if (distance)

							// Update world matrix
							float size = ItemSize;

							// Scale down if out of visible area!
							if (distance > MaxViewDepth)
							{
								size = size / (distance / MaxViewDepth);
								// And reposition to visible distance!
								Vector3 distVector = (objPos - cameraPos);
								distVector.Normalize();
								objPos = cameraPos + distVector * (MaxViewDepth - 25.0f);
							} // if (distance)

							// Apply to matrix
							Matrix matrix = Matrix.Scaling(size, size, size);

							// And set translation (faster than using Matrix.Translate!)
							matrix.M41 = objPos.X;
							matrix.M42 = objPos.Y;
							matrix.M43 = objPos.Z;

							// Alpha by distance! Fade out at FadeOutDepth
							float alpha = 1.0f;

							if (distance > MaxViewDepth * 2)
							{
								alpha = 1.0f -
									 ((distance - MaxViewDepth * 2) /
									 (MaxViewDepth * (6 - 2)));
							} // if (distance)
							if (alpha != lastUpdatedAlpha)
							{
								lastUpdatedAlpha = alpha;
								objectColor = //alpha < 1 ? Color.Red : Color.Green;
									Color.FromArgb((byte)(alpha * 255), objectColor);
								shader.DiffuseColorValue = objectColor;
							} // if (alpha)

							// And finally render.
							itemModels[num].RenderWithShader(matrix, shader);
						} // for (itemNum)
					} // for (num)
				} // if (shader)
				else
				{
					// Non shader version

					// Show all items
					for (int num = 0; num < Level.NumOfItemTypes; num++)
					{
						// Animate item type
						itemModels[num].Animate();
						itemModels[num].FirstMaterial.SelectWithoutShader();
						Microsoft.DirectX.Direct3D.Material d3dMaterial =
							itemModels[num].FirstMaterial.D3DMaterial;
						d3dMaterial.Ambient = Color.LightGray;//.Gray;//.DarkGray;
						objectColor = Color.White;
						lastUpdatedAlpha = 1.0f;

						// Go through all items of this type
						for (int itemNum = 0; itemNum < items[num].Count; itemNum++)
						{
							Vector3 objPos = items[num][itemNum];

							// Get distance to viewer
							float distance = (objPos - cameraPos).Length();

							// Skip if out of visible range
							// Use trick to display even items 6 times out of visible area!
							if (distance > MaxViewDepth * 6)
								continue;

							// If very close to item, collect it and play sound
							if (distance <= ItemSize * 1.5f)
							{
								Player.HandleItem(itemSounds, num);
								// Remove item!
								items[num].RemoveAt(itemNum--);
								continue;
							} // if (distance)

							// Update world matrix
							float size = ItemSize;

							// Scale down if out of visible area!
							if (distance > MaxViewDepth)
							{
								size = size / (distance / MaxViewDepth);
								// And reposition to visible distance!
								Vector3 distVector = (objPos - cameraPos);
								distVector.Normalize();
								objPos = cameraPos + distVector * (MaxViewDepth - 25.0f);
							} // if (distance)

							// Apply to matrix
							Matrix matrix = Matrix.Scaling(size, size, size);

							// And set translation (faster than using Matrix.Translate!)
							matrix.M41 = objPos.X;
							matrix.M42 = objPos.Y;
							matrix.M43 = objPos.Z;

							// Alpha by distance! Fade out at FadeOutDepth
							float alpha = 1.0f;

							if (distance > MaxViewDepth * 2)
							{
								alpha = 1.0f -
									 ((distance - MaxViewDepth * 2) /
									 (MaxViewDepth * (6 - 2)));
							} // if (distance)
							if (alpha != lastUpdatedAlpha)
							{
								lastUpdatedAlpha = alpha;

								// Fade in/out with alpha
								d3dMaterial.Diffuse = Color.FromArgb(
									(byte)(alpha * 255), objectColor);

								// Update material. Note: This is slow, maybe it is
								// faster just to update a special color like ConstantColor
								// and improving performance this way.
								GraphicForm.DirectXDevice.Material = d3dMaterial;
							} // if (alpha)

							// And finally render.
							itemModels[num].RenderWithoutShader(matrix);
						} // for (itemNum)
					} // for (num)
				} // else
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to execute ShowAllItems: " + ex.ToString());
			} // catch (ex)
		} // ShowAllItems(useShader)
		#endregion

		#region Render
		/// <summary>
		/// Render, most performance critical method of the game.
		/// Renders all asteroids and handles all physics, updating, etc.
		/// We use instancing to speed up asteroid rendering!
		/// </summary>
		public void Render(Texture inGameTexture, Texture lightEffectTexture)
		{
			#region Initialize
			// Use alpha blending for blending out asteroids
			GraphicForm.EnableAlphaBlending();

			// Get camera position, no need to call the property 1 mio times.
			Vector3 cameraPos = GraphicForm.CameraPos;

			// Empty display distance list
			remToDisplayDistance.Clear();
			#endregion

			#region Show target behind asteroids
			// Show target icon behind all asteroids and items (only in game)
			if (inGameTexture != null)
			{
				// First find out where the target is.
				Point screenPos = GraphicForm.Convert3DPointTo2D(TargetPosition);
				bool isVisible = GraphicForm.IsInFrontOfCamera(TargetPosition);

				if (isVisible &&
					screenPos.X >= 0 && screenPos.X < GraphicForm.Resolution.Width &&
					screenPos.Y >= 0 && screenPos.Y < GraphicForm.Resolution.Height)
				{
					// From Mission.cs:70.
					Rectangle TargetIconRect = new Rectangle(106, 49, 61, 61);

					// Render target icon centered at screenPos
					// Note: This will be blurred because we render it before
					// the glow/bloom/motion blur shader is applied, but if we
					// render after that we can't render behind the asteroids.
					// See Mission.RenderTarget!
					inGameTexture.RenderOnScreen(new Rectangle(
						screenPos.X - TargetIconRect.Width / 2,
						screenPos.Y - TargetIconRect.Height / 2,
						TargetIconRect.Width, TargetIconRect.Height),
						TargetIconRect);
				} // if (isVisible)
			} // if (inGameTexture)
			#endregion

			#region Show glow behind items
			// Show glow behind all items
			for (int num = 0; num < Level.NumOfItemTypes; num++)
			{
				// Go through all items of this type
				foreach (Vector3 pos in items[num])
				{
					// Get distance to viewer
					float distance = (pos - cameraPos).Length();

					// Skip if out of visible range * 6
					if (distance > MaxViewDepth * 6 ||
						GraphicForm.IsInFrontOfCamera(pos) == false)
						continue;

					// Convert to screen coordinates
					Point screenPos = GraphicForm.Convert3DPointTo2D(pos);
					int glowSize = 36;

					// If not visible, skip!
					if (screenPos.X < -glowSize ||
						screenPos.Y < -glowSize ||
						screenPos.X > GraphicForm.Resolution.Width + glowSize ||
						screenPos.Y > GraphicForm.Resolution.Height + glowSize)
						continue;

					// Calculate alpha
					float alpha = 1.0f;
					if (distance > MaxViewDepth * 4)
						alpha = 1.0f -
							((distance - MaxViewDepth * 4) / (MaxViewDepth * (6 - 4)));

					// Show glow with help of light effect
					lightEffectTexture.RenderOnScreen(
						new Rectangle(screenPos.X - glowSize, screenPos.Y - glowSize,
						glowSize * 2, glowSize * 2),
						ItemColors[num],
						alpha * 0.5f);

					// And display distance to item below it (not here, see below).
					float textAlpha = alpha * 1.5f *
						(1.0f - (distance / (MaxViewDepth * 6)));
					AddDistanceToBeDisplayed(screenPos, distance,
						textAlpha < 1.0f ? textAlpha : 1.0f,
						num == Level.FuelItemType ? ProductLogo.Ascend :
						num == Level.SpeedItemType ? ProductLogo.Touchdown :
						ProductLogo.None);
				} // foreach (pos)
			} // for (num)
			#endregion

			// Call base render method, target and item glow is behind of the
			// asteroids.
			base.Render();

			#region Draw finish sphere
			// Get distance to viewer
			float distanceToTarget = (TargetPosition - cameraPos).Length();
			float maxDistanceToTarget = MaxViewDepth * 4;

			// If we are close to the end, show goal plane in 3d!
			if (distanceToTarget < maxDistanceToTarget)
			{
				float size = SectorWidth * 4.0f;
				Vector3 objPos = TargetPosition;

				// Scale down if out of visible area!
				if (distanceToTarget > MaxViewDepth - size / 2)
				{
					size = size / (distanceToTarget / (MaxViewDepth - size / 2));
					// And reposition to visible distance!
					Vector3 distVector = (TargetPosition - cameraPos);
					distVector.Normalize();
					objPos = cameraPos + distVector * (MaxViewDepth - size / 2);
				} // if (distance)

				// Alpha by distance! Fade out at FadeOutDepth
				float alpha = 1.0f;
				if (distanceToTarget > MaxViewDepth)
				{
					alpha = 1.0f -
							((distanceToTarget - MaxViewDepth) /
						(maxDistanceToTarget - MaxViewDepth));
					if (alpha < 0.0f)
						alpha = 0.0f;
					if (alpha > 1.0f)
						alpha = 1.0f;
				} // if

				// Select goal texture
				// Fade in/out with alpha (use only 1/3 alpha)
				new Material(
					Color.FromArgb((byte)(alpha * 150), Color.White),
					Color.FromArgb((byte)(alpha * 100), Color.White),
					goalTexture).SelectWithoutShader();

				// Downscale size, mesh has size of 10.
				size = size / 10.0f;

				// Draw target mesh
				GraphicForm.WorldMatrix =
					Matrix.Scaling(size, size, size) *
					Matrix.Translation(objPos);
				goalMesh.DrawSubset(0);
			} // if
			#endregion
		} // Render()
		#endregion
	} // class GameAsteroidManager
} // namespace RocketCommander.Game
