// Project: Rocket Commander, File: GameForm.cs
// Namespace: RocketCommander.Game, Class: GameForm
// Path: C:\code\Rocket Commander\Game, Author: Abi
// Code lines: 454, Size of file: 11,62 KB
// Creation date: 02.11.2005 21:24
// Last modified: 20.01.2006 04:23
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Collections.Generic;
using System.Text;
using RocketCommander.Graphics;
using RocketCommander.Sounds;
using RocketCommander.Input;
using Microsoft.DirectX.Direct3D;
using System.Windows.Forms;
using RocketCommander.Helpers;
using System.Drawing;
using System.Threading;
using Microsoft.DirectX.DirectInput;
using Microsoft.DirectX;
using RocketCommander.Shaders;
using System.IO;
using System.ComponentModel;
using RocketCommander.Properties;
#endregion

namespace RocketCommander.Game
{
	/// <summary>
	/// Game form
	/// </summary>
	public class GameForm : GraphicForm
	{
		#region Variables
		/// <summary>
		/// Music
		/// </summary>
		protected MusicManager music;
		/// <summary>
		/// Keyboard
		/// </summary>
		protected static KeyboardInput keyboardInput;
		/// <summary>
		/// Mouse
		/// </summary>
		protected static MouseInput mouseInput;
		/// <summary>
		/// XInput, added 2006-03-09 for XBox360 controllers :)
		/// </summary>
		protected static XInput xInput;

		/// <summary>
		/// Mouse position, intercepted from normal windows mouse events to
		/// this form.
		/// </summary>
		protected static Point mousePos;

		/// <summary>
		/// Camera for everything, used also for the rocket input.
		/// </summary>
		public SpaceCamera camera = new SpaceCamera(new Vector3(0, 0, -5));

		/// <summary>
		/// Post screen effect: Bloom and motion blur effect, every cool :)
		/// </summary>
		protected PostScreenGlow glowShader = null;
		#endregion

		#region Properties
		/// <summary>
		/// Keyboard
		/// </summary>
		/// <returns>Keyboard input</returns>
		public static KeyboardInput Keyboard
		{
			get
			{
				return keyboardInput;
			} // get
		} // Keyboard

		/// <summary>
		/// Mouse
		/// </summary>
		/// <returns>Mouse input</returns>
		public static MouseInput Mouse
		{
			get
			{
				return mouseInput;
			} // get
		} // Mouse

		/// <summary>
		/// Mouse position.
		/// Note: Each control has already a MousePosition property, but it
		/// is much faster to use this property, which directly returns mousePos!
		/// </summary>
		/// <returns>Point</returns>
		public new static Point MousePosition
		{
			get
			{
				return mousePos;
			} // get
		} // MousePosition
		#endregion

		#region Constructor, initializes DirectX, Sound, Music, etc.
		/// <summary>
		/// Create game form
		/// </summary>
		public GameForm(string title, Size resolution, bool fullscreen)
		{
			// Make sure all required directories exists, else we can't continue!
			if (Directory.Exists(Directories.ModelsDirectory) == false ||
				Directory.Exists(Directories.LevelsDirectory) == false ||
				Directory.Exists(Directories.ShadersDirectory) == false ||
				Directory.Exists(Directories.TexturesDirectory) == false)
				throw new DirectoryNotFoundException(
					"Game directories not found (Models, Levels, Shaders, Textures, " +
					"etc.), unable to start game.");

			ComponentResourceManager resources =
				new ComponentResourceManager(typeof(RocketCommanderForm));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "RocketCommanderForm";
			this.Text = "Rocket Commander";
			this.BackColor = Color.Black;

			// Check if autodetect resolution is selected
			if (resolution == Size.Empty)
			{
				// Just use desktop resolution, this is usually the
				// best resolution anyway.
				resolution = new Size(
					Screen.PrimaryScreen.Bounds.Width,
					Screen.PrimaryScreen.Bounds.Height);
				// Check if this resolution is way to big for 1 monitor,
				// then assume this is a multimonitor setup and fix it.
				if (resolution.Width > resolution.Height * 2)
				{
					resolution.Width = resolution.Width / 2;
				} // if (resolution.Width)
			} // if (resolution)

			if (fullscreen)
			{
				// Change to non border type
				this.FormBorderStyle = FormBorderStyle.None;
				this.ClientSize = resolution;

				// Force on top, required for some older graphic cards (e.g. GeForce4),
				// else it will blink and suck big time.
				//obs: this.TopMost = true;
			} // if (fullscreen)

			Text = title;
			Size = resolution;

			// If in windowed mode, make sure ClientSize is correct.
			if (fullscreen == false)
				this.ClientSize = resolution;

			// Init DirectX
			InitializeDirectX(this, resolution, fullscreen);

			// Clip mouse to resolution in fullscreen mode,
			// we don't want it to move over to any other screen.
			if (fullscreen)
			{
				Cursor.Clip = ResolutionRect;
			} // if (fullscreen)

			//Log.Write("GameForm.Init music and stuff");
			// Initialize Music
			music = new MusicManager();

			// Initialize Sound for all samples we will use in this game
			Sound.Init(this);

			// Init keyboard and mouse with DirectInput
			keyboardInput = new KeyboardInput(this);
			mouseInput = new MouseInput(this);
			xInput = new XInput();

			// Init post screen glow
			glowShader = new PostScreenGlow();
		} // GameForm(title, resolution, fullscreen)

		/// <summary>
		/// Create game form
		/// </summary>
		public GameForm()
			: this(Application.ProductName,
			new Size(1024, 768), true)
		{
		} // GameForm()
		#endregion

		#region Dispose
		/// <summary>
		/// Dispose game form
		/// </summary>
		/// <param name="disposing">Currently disposing?</param>
		protected override void Dispose(bool disposing)
		{
			//Log.Write("GameForm.Dispose");
			// Stop music and stuff
			if (music != null)
				music.Dispose();
			music = null;
			Sound.Dispose();
			keyboardInput = null;
			mouseInput = null;
			xInput = null;

			base.Dispose(disposing);
		} // Dispose(disposing)
		#endregion

		#region Mouse events
		/// <summary>
		/// On mouse move
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnMouseMove(MouseEventArgs e)
		{
			mousePos = new Point(e.X, e.Y);
			base.OnMouseMove(e);
		} // OnMouseMove()
		#endregion

		#region On key up
		/// <summary>
		/// Show frames per second?
		/// True in debug mode by default, false otherwise.
		/// Press F1 to toggle showing fps.
		/// </summary>
		protected bool showFps =
#if DEBUG
			true;
#else
			false;
#endif
		/// <summary>
		/// Show postscreen glow and motion blur, toggle with F2.
		/// </summary>
		protected bool showGlow = true;

		/// <summary>
		/// On key up
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnKeyUp(KeyEventArgs e)
		{
			base.OnKeyUp(e);

			switch (e.KeyCode)
			{
				case Keys.F1:
					showFps = !showFps;
					e.Handled = true;
					break;

				case Keys.F2:
					showGlow = !showGlow;
					e.Handled = true;
					break;
			} // switch (KeyCode)
		} // OnKeyUp()
		#endregion

		#region On key press
		/// <summary>
		/// Helper to collect last pressed keys, only used to enter names.
		/// </summary>
		public static string lastPressedKeys = "";
		/// <summary>
		/// Delete pressed keys (backspace was pressed).
		/// </summary>
		public static int deletePressedKeys = 0;
		/// <summary>
		/// On key press
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnKeyPress(KeyPressEventArgs e)
		{
			base.OnKeyPress(e);
			if (e.Handled == false)
			{
				// No special key (max. remember 20 keys)
				if (e.KeyChar >= 32 &&
					lastPressedKeys.Length < 20)
					lastPressedKeys += e.KeyChar;

				// Backspace pressed?
				else if (e.KeyChar == (char)8)
					deletePressedKeys++;
			} // if (e.Handled)
		} // OnKeyPress()
		#endregion

		#region On got focus
		/// <summary>
		/// On got focus
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnGotFocus(EventArgs e)
		{
			// Make sure clip region is set again if in fullscreen mode
			if (fullscreen)
				Cursor.Clip = ResolutionRect;

			base.OnGotFocus(e);
		} // OnGotFocus()

		/// <summary>
		/// On lost focus
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnLostFocus(EventArgs e)
		{
			// Make sure clip region is set again if in fullscreen mode
			if (fullscreen)
				Cursor.Clip = Rectangle.Empty;

			base.OnGotFocus(e);
		} // OnLostFocus()
		#endregion

		#region Update, called each frame
		/// <summary>
		/// Update
		/// </summary>
		public virtual void Update(RenderDelegate renderDelegate)
		{
			// Can we render?
			if (HandleDevice() == false)
			{
				// Pause music if app is not active
				music.Pause();
				return;
			} // if (HandleDevice)

			// Update time and frame counters
			ElapsedTime.Update();

			// Update input
			keyboardInput.Update();
			mouseInput.Update();
			xInput.Update();

			// Update music (play next song, etc.)
			if (music != null)
			{
				music.Update();
			} // if (music)

			// Perform rendering only as long as our device is valid
			if (device != null &&
				device.Disposed == false &&
				isDeviceLost == false)
			{
				try
				{
					// Make sure clip region is set again if in fullscreen mode
					if (fullscreen)
						Cursor.Clip = ResolutionRect;

					// Update the camera (handle input and update matrices)
					camera.Update();

					// Clear background to black = 0
					device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, 0, 1.0f, 0);

					// Update the screen
					device.BeginScene();

					// Initialize common parameters for this frame!
					if (ParallaxShader != null)
						ParallaxShader.UpdateMatricesAndEffectParameters();

					// Render sky cube map as our background.
					skyCubeMap.RenderSky(
						// Slightly change background zooming when speeding up.
						//suxx big time: 1.0f + (Player.speedItemTimeout > 0 ? 0.2f : 0.0f),
						1.0f,
						// Use sky background color instead of lens flare color
						// or lighting color.
						remSkyBackgroundColor);

					// Everything uses our TangentVertex format.
					GraphicForm.DirectXDevice.VertexDeclaration =
						TangentVertex.VertexDeclaration;

					// Handle all the rendering
					if (renderDelegate != null)
						renderDelegate();
					else
						Render();

					// Show how much fps we got (press F1 to toggle)
					if (showFps)
						SmallFont.WriteText(1, 1, "Fps: " + ElapsedTime.FramesPerSecond);
				} // try
				catch (DirectXException)
				{
					// Device lost while rendering, should not happen!
					// Anyway, try to restore next frame.
					isDeviceLost = true;
				} // catch (DirectXException)
				catch (Exception ex)
				{
					Log.Write("Fatal error happened while rendering: " + ex.ToString());
					if (MessageBox.Show("Error happened while rendering. Error: " +
						ex.ToString() + "\nDo you want to quit?",
						"Fatal error", MessageBoxButtons.YesNo) == DialogResult.Yes)
						quit = true;
				} // catch
				finally
				{
					// Always end the scene
					device.EndScene();

					// Show everything to the user (if app was not quitted yet)
					try
					{
						if (isDeviceLost == false)
							device.Present();
					} // try
					catch (DirectXException)
					{
						// Device lost while calling device.Present.
						// Happens often if we switch from app to windows and back a lot.
						// Try to restore it in HandleDevice() next frame!
						isDeviceLost = true;
					} // catch (DirectXException)
					catch (Exception ex)
					{
						Log.Write("device.Present failed: " + ex.ToString());
					} // catch
				} // finally
			} // if (device)

			// Quit with escape
			//if (keyboard.Escape)
			//	quit = true;
			base.Update();
		} // Update(renderDelegate)

		/// <summary>
		/// Update
		/// </summary>
		public override void Update()
		{
			Update(null);
		} // Update()
		#endregion

		#region Unit Testing
		/// <summary>
		/// Delegate for rendering, only used for unit tests.
		/// Takes no parmeters and has no return value.
		/// </summary>
		public delegate void RenderDelegate();

#if DEBUG
		/// <summary>
		/// Access the form from StartTest.
		/// </summary>
		public static GameForm gameFormForTesting = null;
		/// <summary>
		/// Test game form
		/// </summary>
		/// <returns>Game form</returns>
		public static GameForm TestGameForm
		{
			get
			{
				return gameFormForTesting;
			} // get
		} // TestGameForm

		/// <summary>
		/// Start test
		/// </summary>
		[Test]
		public static void StartTest(string title,
			RenderDelegate initDelegate,
			RenderDelegate renderDelegate)
		{
			using (gameFormForTesting =
				new GameForm(title, new Size(640, 480), false))
			{
				if (initDelegate != null)
					initDelegate();

				WindowsHelper.ForceForegroundWindow(gameFormForTesting);
				gameFormForTesting.Show();
				while (TestGameForm.QuitApplication == false)
				{
					// Handle everything for our game
					gameFormForTesting.Update(renderDelegate);

					// Quit if user presses Escape
					if (keyboardInput.Escape)
						gameFormForTesting.quit = true;

					// Still handle default events
					if (gameFormForTesting.QuitApplication == false)
						Application.DoEvents();
				} // while (gameFormForTesting.Quit)
			} // using (form)

			// Disable access of form from outside
			gameFormForTesting = null;
		} // StartTest(title, initDelegate, renderDelegate)
#endif
		#endregion
	} // class GameForm
} // namespace RocketCommander.Game
