// Project: Rocket Commander, File: PhysicsAsteroidManager.cs
// Namespace: RocketCommander.Game, Class: PhysicsAsteroidManager
// Path: C:\code\Rocket Commander\Game, Author: Abi
// Code lines: 1362, Size of file: 50,96 KB
// Creation date: 19.01.2006 22:46
// Last modified: 19.01.2006 23:26
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;
using RocketCommander.Graphics;
using RocketCommander.Helpers;
using RocketCommander.Properties;
using RocketCommander.Sounds;
#endregion

namespace RocketCommander.Game
{
	/// <summary>
	/// Physics asteroid manager.
	/// Derived from BaseAsteroidManager and adds all Physics
	/// (handle player collision, handle sector Physics, etc.).
	/// </summary>
	public class PhysicsAsteroidManager : BaseAsteroidManager
	{
		#region Variables
		/// <summary>
		/// Number of whosh sounds used.
		/// </summary>
		const int NumOfWhoshSounds = 4;

		/// <summary>
		/// Whosh sounds are used when flying close near an asteroid.
		/// It will be played in 3d with effects and stuff.
		/// </summary>
		Sample[] whoshSounds = new Sample[NumOfWhoshSounds];

		/// <summary>
		/// Side hit, played when we only hid an asteroid, but don't die yet.
		/// </summary>
		Sample sideHitSound = null;
		#endregion

		#region Constructor
		/// <summary>
		/// Create asteroid manager
		/// </summary>
		public PhysicsAsteroidManager(Level setLevel)
			: base(setLevel)
		{
			// Load all whosh sounds
			for (int num = 0; num < NumOfWhoshSounds; num++)
			{
				whoshSounds[num] = new Sample("Whosh" + (num + 1) + ".wav", 10);
			} // for (num)
			sideHitSound = new Sample("SideHit.wav");
		} // PhysicsAsteroidManager(setLevel)
		#endregion

		#region Player asteroid collision
		/// <summary>
		/// Player asteroid collision, used to check if player collided with
		/// any asteroid nearby. Returns a value between 0 and 1, 0 means
		/// no collision has occurred and 1.0 means we have done a frontal crash
		/// and will lose a lot of hitpoints. Side hits do hurt do, but are not
		/// as fatal. This method will also play the side hit and whosh sounds.
		/// </summary>
		/// <returns>0 if no collision happened, 1 for a frontal collision.
		/// Everything in between is just some strafe collision (doesn't kill us).
		/// </returns>
		public float PlayerAsteroidCollision()
		{
			try
			{
				Vector3 camPos = GraphicForm.CameraPos;
				Vector3 camDir = new Vector3(0, 0, -1);
				camDir.TransformNormal(GraphicForm.ViewMatrix);
				Vector3 nextCamPos1 = camPos +
					camDir * 10.0f;
				Vector3 nextCamPos2 = camPos +
					camDir * (10.0f + Player.Speed *
					ElapsedTime.MoveFactorPerSecond *
					Player.MovementSpeedPerSecond);
				float damageFactor = 0.0f;

				// Helpers for intersection test
				Vector3 v = new Vector3(0, 0, 1);
				Matrix m = GraphicForm.InverseViewMatrix;
				// Transform the screen space pick ray into 3D space
				Vector3 rayDir = new Vector3(
					v.X * m.M11 + v.Y * m.M21 + v.Z * m.M31,
					v.X * m.M12 + v.Y * m.M22 + v.Z * m.M32,
					v.X * m.M13 + v.Y * m.M23 + v.Z * m.M33);
				Vector3 rayOrigin = new Vector3(
					m.M41, m.M42, m.M43);

				for (int z = -1; z <= +1; z++)
					for (int y = -1; y <= +1; y++)
						for (int x = -1; x <= +1; x++)
						{
							List<Asteroid> thisSector = sectorAsteroids[
								MiddleSector + z, MiddleSector + y, MiddleSector + x];
							for (int num = 0; num < thisSector.Count; num++)
							{
								Asteroid asteroid = thisSector[num];

								// Check distance to camera
								float distance = (asteroid.position - camPos).Length();

								// If distance is smaller than collision radius, we could have
								// a hit. Now do polygon based collision testing.
								if (distance <= asteroid.size * 1.25f)
								{
									// First of all do an intersection test to see if we
									// really hit something here.
									Matrix invWorldMatrix =
										Matrix.Invert(asteroid.RenderMatrix);
									Vector3 rayWorldPos =
										Vector3.TransformCoordinate(rayOrigin, invWorldMatrix);
									Vector3 rayWorldDir =
										Vector3.TransformNormal(rayDir, invWorldMatrix);
									rayWorldDir.Normalize();

									// Use D3DMesh.Intersect's capability to determinate if
									// our rocket (represented as a ray) hits the asteroid mesh.
									IntersectInformation closestHit;
									if (asteroidModels[asteroid.Type].D3DMesh.Intersect(
										rayWorldPos, rayWorldDir, out closestHit))
									{
										// Check if we hit near the camera, else discard.
										// Use speed to see how much we can still move to avoid
										// collision!
										if (closestHit.Dist > 15.0f + Player.Speed *
											Player.MovementSpeedPerSecond *
											ElapsedTime.MoveFactorPerSecond * 2.0f)
										{
											// To far away, skip this collision!
											continue;
										} // if (closestHit.Dist)
									} // if (asteroidModels[asteroid.Type].D3DMesh.Intersect)
									else
									{
										// Is this donut asteroid? Then check if we just fly
										// through it.
										if (asteroid.Type == 4 &&
											asteroid.movement.LengthSq() > 0 &&
											distance <= 25.0f + Player.Speed *
											Player.MovementSpeedPerSecond *
											ElapsedTime.MoveFactorPerSecond * 1.25f)
										{
											// Give player additional 4000 points,
											// thats really a lot!
											Player.score += 4000;
											// Add text message
											Player.SetItemMessage(Texts.FlyThroughBonus);
											// Mark this one as already handled, don't add score
											// anymore.
											asteroid.movement = Vector3.Empty;
										} // if (asteroid.Type)

										// No real collision here, just in bounding sphere
										continue;
									} // else

									// Find out how bad it is, add 25% to make collision hit
									// hurt much more than just flying by very closely.
									float thisDamageFactor =
										1.0f - (distance / (asteroid.collisionRadius * 1.25f));

									// Big asteroids to more damage than smaller ones
									thisDamageFactor *=
										0.5f + 0.5f * (asteroid.size / MaxAsteroidSize);

									// Check highest damage factor
									if (thisDamageFactor > damageFactor)
										damageFactor = thisDamageFactor;

									// Play side hit sound.
									// Play only once (asteroid is killed after that).
									sideHitSound.Play(1.0f);

									// Remember to show hit direction effect
									// Get direction from 3d data (similar to sound
									// calculation below).
									Vector3 distVector = asteroid.position - camPos;
									distVector.TransformNormal(GraphicForm.ViewMatrix);
									SetHitDirectionEffect(
										(float)Math.Atan2(distVector.X, distVector.Y));

									/*obs, see below
									// Make camera wobbel (use max. amount), damage must be high.
									if (damageFactor > 0.15f)
										Player.SetCameraWobbel(
											Math.Min(0.2f + damageFactor * damageFactor, 1.0f));
									 */
									// Lets rumble a bit if we are hurt
									XInput.Rumble(0.25f + damageFactor / 2.0f,
										0.45f + 0.55f * damageFactor);

									// In bomb mode, we have 1 free hit
									if (Player.numberOfBombItems > 0)
									{
										Player.numberOfBombItems--;
										damageFactor = 0.0f;
									} // if (Player.numberOfBombItems)

									// Always remove asteroid we hit!
									// This way we hit it only once.
									thisSector.RemoveAt(num--);
								} // if (distance)

								float distanceSoon =
									(asteroid.position - nextCamPos1).Length();
								// Check if some asteroids are getting really close
								if (distanceSoon < MaxAsteroidSize * 3.5f)
								{
									// Determinate whosh by checking if any asteroid comes nearer
									// than SectorDepth	then check if the asteroid is not longer
									// closing in (but moving away from us).
									// Now play sound, this should only happen once per asteroid!

									// Find out if the asteroid is not longer moving closer
									if (distanceSoon <
										(asteroid.position - nextCamPos2).Length())
									{
										// Play one of the whosh sounds
										float loudness = 1.0f -
											((distanceSoon - asteroid.collisionRadius * 1.5f) /
											(MaxAsteroidSize * 3.5f -
											asteroid.collisionRadius * 1.5f));
										loudness *= 1.25f;
										if (loudness > 1.0f)
											loudness = 1.0f;
										if (loudness > 0.1f)
										{
											// Lets rumble a bit to indicate flying near
											if (loudness > 0.8f)
												XInput.Rumble(0.05f, 0.15f);

											Sample sound = whoshSounds[
												RandomHelper.GetRandomInt(NumOfWhoshSounds)];

											// Get panning from 3d data
											Vector3 distVector = asteroid.position - nextCamPos1;
											distVector.TransformNormal(GraphicForm.ViewMatrix);
											float panning =
												distVector.X / MaxAsteroidSize * 4.0f;
											if (panning < -0.7f)
												panning = -0.7f;
											else if (panning > 0.7f)
												panning = 0.7f;

											// Play whosh sound
											sound.Play(0.25f + 0.75f * loudness, panning);

											if (distance <= asteroid.collisionRadius * 2.5f)
											{
												// Make camera wobbel (use smaller amounts)
												Player.SetCameraWobbel(0.33f * (1.0f -
													(distance / (asteroid.collisionRadius * 2.0f))));
											} // if (distance)

											// Play whosh a little slower, sounds better.
											sound.PitchFactor = 0.8f;

											// Add a little score for flying so close ^^
											if (ElapsedTime.TotalFrames % 20 == 0)
												Player.score += ElapsedTime.MsLastFrame;
										} // if (loudness)
									} // if (distanceSoon)
								} // if (distanceSoon)
							} // for (num)
						} // for for for (int)

				return damageFactor;
			} // try
			catch (Exception ex)
			{
				// Note: This happened a few times in the beta, no reason to crash
				// the whole game if anything simple here fails (physics or sound).
				Log.Write("Failed to execute PlayerAsteroidCollision: " +
					ex.ToString());
				return 0.0f;
			} // catch
		} // PlayerAsteroidCollision()
		
		/// <summary>
		/// Set hit direction effect
		/// </summary>
		/// <param name="setDirection">Direction</param>
		protected virtual void SetHitDirectionEffect(float setDirection)
		{
			// Implement in derived class
		} // SetHitDirectionEffect(setDirection)

		/// <summary>
		/// Kill all inner sector asteroids
		/// </summary>
		public void KillAllInnerSectorAsteroids()
		{
			// Kill all asteroids in inner sectors (3x3)
			for (int z = -1; z <= +1; z++)
				for (int y = -1; y <= +1; y++)
					for (int x = -1; x <= +1; x++)
					{
						// Kill all asteroids there
						sectorAsteroids[
							MiddleSector + z, MiddleSector + y, MiddleSector + x].Clear();

						// Also kill all inner small asteroids
						sectorSmallerAsteroids[SmallMiddleSector + z,
							SmallMiddleSector + y, SmallMiddleSector + x].Clear();
					} // for for for (int)
		} // KillAllInnerSectorAsteroids()
		#endregion

		#region Handle sector physics
		/// <summary>
		/// Handle sector physics. See below for more explanations and
		/// first simpler versions in the unit test region.
		/// 
		/// Will handle all collisions inside a given sector. We will not
		/// check against the whole universe, which is way to slow.
		/// Just checking this sector is very fast, but does not cover any
		/// collisions at the borders (which occur VERY often, the first
		/// version of this method ignored the borders and sucked big time).
		/// For this reason we will check all neighbouring sectors too!
		/// Note: All asteroids should have an updated position here.
		/// The position will be corrected if a collision occurs and next
		/// frame the asteroids will move into the reflected direction.
		/// </summary>
		protected override void HandleSectorPhysics(
			int checkX, int checkY, int checkZ,
			int cameraSectorPosX, int cameraSectorPosY, int cameraSectorPosZ)
		{
			List<Asteroid> thisSectorAsteroids =
				sectorAsteroids[checkZ, checkY, checkX];

			// No asteroids in this sector? Then no collision can occur.
			if (thisSectorAsteroids.Count == 0)
				return;

			// 2 modes: inside sector and at border.
			// If we are inside a simple inner check is totally fine.
			// If we are at any border of the current sector, we check
			// all surrounding sectors and include other asteroids in our search!
			// Note: I've also pfusched in the out of sector check inside this loop,
			// this is very ugly but saves a good amount of performance!
			for (int num = 0; num < thisSectorAsteroids.Count; num++)
			{
				Asteroid asteroid = thisSectorAsteroids[num];

				// Get position in percentage (we use this variable several times)
				float xp = asteroid.position.X / SectorWidth;
				float yp = asteroid.position.Y / SectorHeight;
				float zp = asteroid.position.Z / SectorDepth;

				// Get sector position, use asteroid pos and round it.
				int ix = (int)Math.Round(xp);
				int iy = (int)Math.Round(yp);
				int iz = (int)Math.Round(zp);

				// Calculate border distance -0.5 means left border, -0.4 to +0.4
				// means we are inside the sector and +0.5 means right border.
				float borderX = xp - ix;
				float borderY = yp - iy;
				float borderZ = zp - iz;

				// Substract current position from sector position
				ix = ix - cameraSectorPosX + MiddleSector;
				iy = iy - cameraSectorPosY + MiddleSector;
				iz = iz - cameraSectorPosZ + MiddleSector;

				// Bounding checks
				if (ix < 0)
					ix = 0;
				if (iy < 0)
					iy = 0;
				if (iz < 0)
					iz = 0;
				if (ix >= NumberOfSectors)
					ix = NumberOfSectors - 1;
				if (iy >= NumberOfSectors)
					iy = NumberOfSectors - 1;
				if (iz >= NumberOfSectors)
					iz = NumberOfSectors - 1;

				// Change asteroid sector if it is not longer in the correct sector.
				if (ix != checkX ||
					iy != checkY ||
					iz != checkZ)
				{
					// Remove from old list and reuse old num again.
					thisSectorAsteroids.RemoveAt(num--);

					// Add to new sector, which fits our current position.
					sectorAsteroids[iz, iy, ix].Add(asteroid);
				} // if (ix)

				// Full loop for all possible neighbour configurations is only used
				// if we are at any border. In most cases we don't ever check any
				// neighbour sectors. If we have to, that will be a lot of crazy
				// code and checks to optimize every last bit out of this loops!
				if (borderX > -0.4f && borderX < 0.4f &&
					borderY > -0.4f && borderY < 0.4f &&
					borderZ > -0.4f && borderZ < 0.4f)
				{
					// Only check this sector! Crosscheck with any other asteroid in
					// this sector.
					foreach (Asteroid otherAsteroid in thisSectorAsteroids)
						if (asteroid != otherAsteroid)
						{
							float maxAllowedDistance =
								otherAsteroid.collisionRadius +
								asteroid.collisionRadius;

							// Distance smaller than max. allowed distance?
							if ((otherAsteroid.position -
								asteroid.position).LengthSq() <
								maxAllowedDistance * maxAllowedDistance)
							{
								HandleAsteroidCollision(asteroid, otherAsteroid);
							} // if (otherAsteroid.position)
						} // foreach if (asteroid)
				} // if (borderX)
				else
				{
					// Check neighbour sectors too.
					// Go through list of asteroids in surrounding sectors.
					// Please also note the order we go through the multidimensional
					// lists, by using z, y, x we can let the cpu cache memory
					// accesses better.
					for (int z = checkZ - 1; z <= checkZ + 1; z++)
						if (z >= 0 && z < NumberOfSectors)
							for (int y = checkY - 1; y <= checkY + 1; y++)
								if (y >= 0 && y < NumberOfSectors)
									for (int x = checkX - 1; x <= checkX + 1; x++)
										if (x >= 0 && x < NumberOfSectors &&
											sectorIsVisible[z, y, x])
										{
											// Check each of this other asteroids
											foreach (Asteroid otherAsteroid in
												sectorAsteroids[z, y, x])
												if (asteroid != otherAsteroid)
												{
													float maxAllowedDistance =
														otherAsteroid.collisionRadius +
														asteroid.collisionRadius;

													// Distance smaller than max. allowed distance?
													if ((otherAsteroid.position -
														asteroid.position).LengthSq() <
														maxAllowedDistance * maxAllowedDistance)
													{
														HandleAsteroidCollision(asteroid, otherAsteroid);
													} // if (otherAsteroid.position)
												} // foreach if (asteroid)
										} // for if for if for if (x)
				} // else
			} // for (num)
		} // HandleSectorPhysics(checkX, checkY, checkZ)

		/// <summary>
		/// Handle asteroid collision, called from HandleSectorPhysics.
		/// </summary>
		/// <param name="asteroid">Asteroid</param>
		/// <param name="otherAsteroid">Other asteroid</param>
		protected void HandleAsteroidCollision(
			Asteroid asteroid, Asteroid otherAsteroid)
		{
			float maxAllowedDistance =
				otherAsteroid.collisionRadius + asteroid.collisionRadius;

			// Calculate distance and max. allowed distance
			float distance =
				(otherAsteroid.position - asteroid.position).Length();

			// Collision detected, correct both asteroid positions first.
			// Middle is collision point (aprox. because we just use spheres)
			Vector3 middle =
				otherAsteroid.position *
				(asteroid.collisionRadius / maxAllowedDistance) +
				asteroid.position *
				(otherAsteroid.collisionRadius / maxAllowedDistance);

			// Calculate current relative positions to middle
			Vector3 positionRel = asteroid.position - middle;
			positionRel.Normalize();
			Vector3 otherPositionRel = otherAsteroid.position - middle;
			otherPositionRel.Normalize();

			// Put both circles outside of the collision
			// Add 1% to add a little distance between collided objects!
			otherAsteroid.position = middle +
				otherPositionRel * otherAsteroid.collisionRadius * 1.015f;
			asteroid.position = middle +
				positionRel * asteroid.collisionRadius * 1.015f;

			float asteroidSpeed = asteroid.movement.Length();
			float otherAsteroidSpeed = otherAsteroid.movement.Length();

			// Mass = size*size (size*size*size is too much)
			float asteroidMass =
				asteroid.size * asteroid.size;// *asteroid.size;
			float otherAsteroidMass =
				otherAsteroid.size * otherAsteroid.size;// *otherAsteroid.size;
			float bothMasses = asteroidMass + otherAsteroidMass;

			// Calculate force with speed * mass (mass=size)
			float asteroidForce = asteroidSpeed * asteroidMass;
			float otherAsteroidForce = otherAsteroidSpeed * otherAsteroidMass;
			float bothForces = asteroidForce + otherAsteroidForce;

			// Copy over normals
			Vector3 asteroidNormal = positionRel;
			Vector3 otherAsteroidNormal = otherPositionRel;

			// Normalize movement
			Vector3 asteroidDirection = asteroid.movement;
			asteroidDirection.Normalize();
			Vector3 otherAsteroidDirection = otherAsteroid.movement;
			otherAsteroidDirection.Normalize();

			// Get collision strenth (1 if pointing in same direction,
			// 0 if 90 degrees) for both asteroids.
			float asteroidCollisionStrength = Math.Abs(Vector3.Dot(
				asteroidDirection, asteroidNormal));
			float otherAsteroidCollisionStrength = Math.Abs(Vector3.Dot(
				otherAsteroidDirection, otherAsteroidNormal));

			Vector3 asteroidReflection =
				ReflectVector(asteroidDirection, asteroidNormal);
			Vector3 otherAsteroidReflection =
				ReflectVector(otherAsteroidDirection, otherAsteroidNormal);

			// Make sure the strength is calculated correctly
			// We have also to correct the reflection vector if the length was 0,
			// use the normal vector instead.
			if (asteroidDirection.Length() <= 0.01f)
			{
				asteroidCollisionStrength = otherAsteroidCollisionStrength;
				asteroidReflection = asteroidNormal;
			} // if (asteroidDirection.Length)
			if (otherAsteroidDirection.Length() <= 0.01f)
			{
				otherAsteroidCollisionStrength = asteroidCollisionStrength;
				otherAsteroidReflection = otherAsteroidNormal;
			} // if (otherAsteroidDirection.Length)

			// Ok, now the complicated part, everything above was really easy!
			asteroid.movement = asteroidReflection *
				// So, first we have to reflect our current movement speed.
				// This will be scaled to 1-strength to only account the reflection
				// amount (imagine a collision with a static wall). In most cases
				// Strength is close to 1 and this reflection will be very small.
				((1 - asteroidCollisionStrength) * asteroidSpeed +
				// And finally we have to add the impuls, which is calculated
				// by the formular ((m1-m2)*v1 + 2*m2*v2)/(m1+m2), see
				// http://de.wikipedia.org/wiki/Sto%C3%9F_%28Physik%29 for more help.
				(asteroidCollisionStrength *
				(Math.Abs(asteroidMass - otherAsteroidMass) * asteroidSpeed +
				(2 * otherAsteroidMass * otherAsteroidSpeed)) / bothMasses));

			// Same for other asteroid, just with asteroid and otherAsteroid
			// inverted.
			otherAsteroid.movement = otherAsteroidReflection *
				// Same as above.
				((1 - otherAsteroidCollisionStrength) * otherAsteroidSpeed +
				(otherAsteroidCollisionStrength *
				(Math.Abs(otherAsteroidMass - asteroidMass) * otherAsteroidSpeed +
				(2 * asteroidMass * asteroidSpeed)) / bothMasses));

			asteroid.rotationSpeed.Y = (asteroid.rotationSpeed.Y +
				RandomHelper.GetRandomFloat(-0.75f, +1.0f)) / 2.0f;
			otherAsteroid.rotationSpeed.Y = (otherAsteroid.rotationSpeed.Y +
				RandomHelper.GetRandomFloat(-0.75f, +1.0f)) / 2.0f;
		} // HandleAsteroidCollision(asteroid, otherAsteroid)

		/// <summary>
		/// Reflect first vector to a surface with a normal vector.
		/// Basically will move away from normal and invert the current direction.
		/// </summary>
		/// <param name="reflectVector">Vector we want to reflect.</param>
		/// <param name="normal">Normal we use for reflection.</param>
		/// <returns>Reflected vector (same length as vec)</returns>
		private static Vector3 ReflectVector(Vector3 vec, Vector3 normal)
		{
			// Reflect and rescale to old length (looks like shader code ^^)
			// Note: All vectors should be normalized.
			return (vec - normal * 2.0f * Vector3.Dot(vec, normal));
		} // ReflectVector(vec, normal)
		#endregion

		#region Unit Testing
#if DEBUG
		#region Asteroid physics tests
		/// <summary>
		/// Asteroid physics tests
		/// </summary>
		class AsteroidPhysicsTests
		{
			/// <summary>
			/// Asteroid model
			/// </summary>
			Model asteroidModel = null;
			const int NumOfSectors = 7, MiddleSector = NumOfSectors / 2;
			/// <summary>
			/// Asteroids
			/// </summary>
			List<Asteroid>[, ,] asteroids =
				new List<Asteroid>[NumOfSectors, NumOfSectors, NumOfSectors];

			#region Handle asteroid collision
			/// <summary>
			/// Handle asteroid collision, called from HandleSectorPhysics.
			/// </summary>
			/// <param name="asteroid">Asteroid</param>
			/// <param name="otherAsteroid">Other asteroid</param>
			private void HandleAsteroidCollision(
				Asteroid asteroid, Asteroid otherAsteroid)
			{
				float maxAllowedDistance =
					otherAsteroid.size + asteroid.size;

				// Calculate distance and max. allowed distance
				float distance =
					(otherAsteroid.position - asteroid.position).Length();

				// Collision detected, correct both asteroid positions first.
				// Middle is collision point (aprox. because we just use spheres)
				Vector3 middle =
					otherAsteroid.position * (asteroid.size / maxAllowedDistance) +
					asteroid.position * (otherAsteroid.size / maxAllowedDistance);

				// Calculate current relative positions to middle
				Vector3 otherPositionRel = otherAsteroid.position - middle;
				otherPositionRel.Normalize();
				Vector3 positionRel = asteroid.position - middle;
				positionRel.Normalize();

				//looks correct for unit tests below, even with 500+ asteroids,
				//but for some unknown reason pops really bad in sector mode:

				// Put both circles outside of the collision
				// Add 1% to add a little distance between collided objects!
				otherAsteroid.position = middle +
					otherPositionRel * otherAsteroid.size;// *1.01f;
				asteroid.position = middle +
					positionRel * asteroid.size;// *1.01f;

				// Increase speeds a bit for collision
				float bothSpeeds = 1.1f * (asteroid.movement.Length() +
					otherAsteroid.movement.Length());

				// And finally bounce back (depending on the mass=size)
				asteroid.movement = //asteroidForce *
					ReflectVector(asteroid.movement, positionRel) *
					(bothSpeeds * asteroid.size / maxAllowedDistance);
				otherAsteroid.movement = //otherAsteroidForce *
					ReflectVector(otherAsteroid.movement, otherPositionRel) *
					(bothSpeeds * otherAsteroid.size / maxAllowedDistance);

				asteroid.rotationSpeed.Y = (asteroid.rotationSpeed.Y +
					RandomHelper.GetRandomFloat(-0.75f, +1.0f)) / 2.0f;
				otherAsteroid.rotationSpeed.Y = (otherAsteroid.rotationSpeed.Y +
					RandomHelper.GetRandomFloat(-0.75f, +1.0f)) / 2.0f;
			} // HandleAsteroidCollision(asteroid, otherAsteroid)
			#endregion

			#region Handle sector physics
			/// <summary>
			/// Handle sector physics. See above for only implemented version.
			/// This unit tests contains all older versions too!
			/// 
			/// Will handle all collisions inside a given sector. We will not
			/// check against the whole universe, which is way to slow.
			/// Just checking this sector is very fast, but does not cover any
			/// collisions at the borders (which occur VERY often, the first
			/// version of this method ignored the borders and sucked big time).
			/// For this reason we will check all neighbouring sectors too!
			/// Note: All asteroids should have an updated position here.
			/// The position will be corrected if a collision occurs and next
			/// frame the asteroids will move into the reflected direction.
			/// </summary>
			private void HandleSectorPhysics(
				int checkX, int checkY, int checkZ,
				int cameraSectorPosX, int cameraSectorPosY, int cameraSectorPosZ)
			{
				List<Asteroid> thisSectorAsteroids = asteroids[checkZ, checkY, checkX];

				// No asteroids in this sector? Then no collision can occur.
				if (thisSectorAsteroids.Count == 0)
					return;

				// Note: Don't read anything of the next 2 regions if you don't want
				// to get confused (you are anyway, aren't you?). Maybe this helps
				// if you just want to test out a very simple physics system (see the
				// second try, it's very simple and easy to understand).
				#region First try, slow and way too incorrect for many sectors
				/*old
				// Go through all asteroids
				foreach (Asteroid asteroid in asteroids)
				{
					// Now crosscheck with any other asteroid in this sector
					foreach (Asteroid otherAsteroid in asteroids)
						if (asteroid != otherAsteroid)
						{
							float maxAllowedDistance = otherAsteroid.size + asteroid.size;
							/*optimized
							// Calculate distance and max. allowed distance
							float distance =
								(otherAsteroid.position - asteroid.position).Length();
							float maxAllowedDistance = otherAsteroid.size + asteroid.size;
						
							// Collision occurs if distance is smaller than maxAllowedDistance.
							if (distance < maxAllowedDistance)
							 */
				/*
								if ((otherAsteroid.position - asteroid.position).LengthSq() <
									maxAllowedDistance * maxAllowedDistance)
								{
									// Calculate distance and max. allowed distance
									float distance =
										(otherAsteroid.position - asteroid.position).Length();

									// Collision detected, correct both asteroid positions first.
									// Middle is collision point (aprox. because we just use spheres)
									Vector3 middle =
										otherAsteroid.position * (asteroid.size / maxAllowedDistance) +
										asteroid.position * (otherAsteroid.size / maxAllowedDistance);
									//bad: (asteroid.position + otherAsteroid.position)*0.5f;

									// Calculate current relative positions to middle
									Vector3 otherPositionRel = otherAsteroid.position - middle;
									otherPositionRel.Normalize();
									Vector3 positionRel = asteroid.position - middle;
									positionRel.Normalize();

									//looks correct for unit tests below, even with 500+ asteroids,
									//but for some unknown reason pops really bad in sector mode:

									// Put both circles outside of the collision
									// Add 1% to add a little distance between collided objects!
									otherAsteroid.position = middle +
										otherPositionRel * otherAsteroid.size * 1.01f;
									//otherPositionRel * (1.001f * maxAllowedDistance / 2.0f);
									asteroid.position = middle +
										positionRel * asteroid.size * 1.01f;
									//positionRel * (1.001f * maxAllowedDistance / 2.0f);

									/*doesn't work yet
									// Calc force for each asteroid
									float asteroidForce =
										//asteroid.movement.Length() *
										otherAsteroid.size / maxAllowedDistance;
									float otherAsteroidForce =
										//otherAsteroid.movement.Length() *
										asteroid.size / maxAllowedDistance;
									 */
				/*
								// Increase speeds a bit for collision
								float bothSpeeds = 1.1f * (asteroid.movement.Length() +
									otherAsteroid.movement.Length());

								// And finally bounce back (depending on the mass=size)
								asteroid.movement = //asteroidForce *
									ReflectVector(asteroid.movement, positionRel,
									bothSpeeds * asteroid.size / maxAllowedDistance);
								otherAsteroid.movement = //otherAsteroidForce *
									ReflectVector(otherAsteroid.movement, otherPositionRel,
									bothSpeeds * otherAsteroid.size / maxAllowedDistance);

								/*always
								// Give new rotation if was rotating really fast
								if (asteroid.rotationSpeed.Y > 0.9f)
									asteroid.rotationSpeed = new Vector2(
										RandomHelper.GetRandomFloat(-0.75f, +1.0f),
										RandomHelper.GetRandomFloat(-0.75f, +1.0f));
								if (otherAsteroid.rotationSpeed.Y > 0.9f)
									otherAsteroid.rotationSpeed = new Vector2(
										RandomHelper.GetRandomFloat(-0.75f, +1.0f),
										RandomHelper.GetRandomFloat(-0.75f, +1.0f));
								 */
				/*
									asteroid.rotationSpeed.Y =
										RandomHelper.GetRandomFloat(-0.75f, +1.0f);
									otherAsteroid.rotationSpeed.Y =
										RandomHelper.GetRandomFloat(-0.75f, +1.0f);
								} // if (distance)
							} // foreach if (asteroid)
					} // foreach (asteroid)
				 */
				#endregion

				#region Second try, much better, but not perfect
				/*ok and optimized nicely, but still too slow!
				// Go through list of asteroids in surrounding sectors, this is the
				// reverse logic, but saves us to enumerate all sectors as often
				// as we have thisSectorAsteroids.
				// Please also note the order we go through the multidimensional lists,
				// by using z, y, x we can let the cpu cache mem accesses better.
				for (int z = checkZ - 1; z <= checkZ + 1; z++)
					if (z >= 0 && z < NumOfSectors)
						for (int y = checkY - 1; y <= checkY + 1; y++)
							if (y >= 0 && y < NumOfSectors)
								for (int x = checkX - 1; x <= checkX + 1; x++)
									if (x >= 0 && x < NumOfSectors)// &&
										// Don't check this sector!
										//(x != checkX || y != checkY || z != checkZ))
									{
										bool sameSector = x == checkX && y == checkY && z == checkZ;
										// Check each of this other asteroids
										foreach (Asteroid otherAsteroid in asteroids[z, y, x])
											// Only include other asteroids if they have high speed
											if (otherAsteroid.HighSpeedAsteroid ||
												sameSector)
										{
											// Now go through all asteroids of our checking sector.
											foreach (Asteroid asteroid in thisSectorAsteroids)
												// Don't check against ourself
												if (asteroid != otherAsteroid)
											{
												float maxAllowedDistance =
													otherAsteroid.size + asteroid.size;
												
												// Distance smaller than max. allowed distance?
												if ((otherAsteroid.position -
													asteroid.position).LengthSq() <
													maxAllowedDistance * maxAllowedDistance)
												{
													HandleAsteroidCollision(asteroid, otherAsteroid);
												} // if
											} // foreach  (asteroid)
										} // foreach  (otherAsteroid)
									} // for if for if for if
				 */
				#endregion

				// Thrid try, 2 modes: inside sector and at border.
				// If we are inside a simple inner check is totally fine.
				// If we are at any border of the current sector, we check
				// all surrounding sectors and include other asteroids in our search!
				// Note: I've also pfusched the out of sector check inside this loop,
				// this is very ugly but saves a good amount of performance!
				for (int num = 0; num < thisSectorAsteroids.Count; num++)
				{
					Asteroid asteroid = thisSectorAsteroids[num];

					// Get position in percentage (we use this variable several times)
					float xp = asteroid.position.X / SectorWidth;
					float yp = asteroid.position.Y / SectorHeight;
					float zp = asteroid.position.Z / SectorDepth;

					// Get sector position, use asteroid pos and round it.
					int ix = (int)Math.Round(xp);
					int iy = (int)Math.Round(yp);
					int iz = (int)Math.Round(zp);

					// Calculate border distance -0.5 means left border, -0.4 to +0.4
					// means we are inside the sector and +0.5 means right border.
					float borderX = xp - ix;
					float borderY = yp - iy;
					float borderZ = zp - iz;

					// Substract current position from sector position
					ix = ix - cameraSectorPosX + MiddleSector;
					iy = iy - cameraSectorPosY + MiddleSector;
					iz = iz - cameraSectorPosZ + MiddleSector;

					if (ix < 0)
						ix = 0;
					if (iy < 0)
						iy = 0;
					if (iz < 0)
						iz = 0;
					if (ix >= NumOfSectors)
						ix = NumOfSectors - 1;
					if (iy >= NumOfSectors)
						iy = NumOfSectors - 1;
					if (iz >= NumOfSectors)
						iz = NumOfSectors - 1;

					// Change asteroid sector if it is not longer in the correct sector.
					if (ix != checkX ||
						iy != checkY ||
						iz != checkZ)
					{
						// Remove from old list and reuse old num again.
						thisSectorAsteroids.RemoveAt(num--);

						// Add to new sector, which fits our current position.
						asteroids[iz, iy, ix].Add(asteroid);
						//} // if
					} // if (ix)

					// Full loop for all possible neighbour configurations is only used
					// if we are at any border. In most cases we don't ever check any
					// neighbour sectors. If we have to, that will be a lot of crazy
					// code and checks to optimize every last bit out of this loops!
					if (borderX > -0.4f && borderX < 0.4f &&
						borderY > -0.4f && borderY < 0.4f &&
						borderZ > -0.4f && borderZ < 0.4f)
					{
						// Only check this sector! Crosscheck with any other asteroid in
						// this sector.
						foreach (Asteroid otherAsteroid in thisSectorAsteroids)
							if (asteroid != otherAsteroid)
							{
								float maxAllowedDistance =
									otherAsteroid.size + asteroid.size;

								// Distance smaller than max. allowed distance?
								if ((otherAsteroid.position -
									asteroid.position).LengthSq() <
									maxAllowedDistance * maxAllowedDistance)
								{
									HandleAsteroidCollision(asteroid, otherAsteroid);
								} // if (otherAsteroid.position)
							} // foreach if (asteroid)
					} // if (borderX)
					else
					{
						// Check neighbour sectors too.
						// Go through list of asteroids in surrounding sectors.
						// Please also note the order we go through the multidimensional
						// lists, by using z, y, x we can let the cpu cache memory
						// accesses better.
						for (int z = checkZ - 1; z <= checkZ + 1; z++)
							if (z >= 0 && z < NumOfSectors)
								for (int y = checkY - 1; y <= checkY + 1; y++)
									if (y >= 0 && y < NumOfSectors)
										for (int x = checkX - 1; x <= checkX + 1; x++)
											if (x >= 0 && x < NumOfSectors)
											{
												// Check each of this other asteroids
												foreach (Asteroid otherAsteroid in asteroids[z, y, x])
													if (asteroid != otherAsteroid)
													{
														float maxAllowedDistance =
															otherAsteroid.size + asteroid.size;

														// Distance smaller than max. allowed distance?
														if ((otherAsteroid.position -
															asteroid.position).LengthSq() <
															maxAllowedDistance * maxAllowedDistance)
														{
															HandleAsteroidCollision(asteroid, otherAsteroid);
														} // if (otherAsteroid.position)
													} // foreach if (asteroid)
											} // for if for if for if (x)
					} // else
				} // for (num)
			} // HandleSectorPhysics(checkX, checkY, checkZ)
			#endregion

			#region Test handle asteroid sector physics
			/// <summary>
			/// Test handle asteroid sector physics
			/// </summary>
			[Test]
			public void TestHandleSectorPhysics()
			{
				GameForm.StartTest("TestHandleSectorPhysics",
					delegate
					{
						asteroidModel = new Model("Asteroid1.X");

						// Add asteroids in all 5x5x5 sectors (5 in each)
						for (int z = 0; z < NumOfSectors; z++)
							for (int y = 0; y < NumOfSectors; y++)
								for (int x = 0; x < NumOfSectors; x++)
								{
									asteroids[z, y, x] = new List<Asteroid>();
									for (int num = 0; num < 3; num++)
									{
										asteroids[z, y, x].Add(new Asteroid(
											0,
											new Vector3((x - MiddleSector) * SectorWidth,
											(y - MiddleSector) * SectorHeight,
											(z - MiddleSector) * SectorDepth) +
											RandomHelper.GetRandomVector3(
											-SectorWidth * 0.4f, +SectorWidth * 0.4f),
											asteroidModel.ObjectSize));
									} // asteroidModel
								} // asteroidModel
					},
					delegate
					{
						GameForm.TestGameForm.camera.FreeCamera = true;
						asteroidModel.FirstMaterial.SelectWithoutShader();
						GraphicForm.DirectXDevice.RenderState.Lighting = true;
						GraphicForm.EnableAlphaBlending();

						for (int z = 0; z < NumOfSectors; z++)
							for (int y = 0; y < NumOfSectors; y++)
								for (int x = 0; x < NumOfSectors; x++)
								{
									// Update all positions for this sector
									foreach (Asteroid asteroid in asteroids[z, y, x])
										asteroid.UpdateMovement(ElapsedTime.MoveFactorPerSecond);

									// Handle physics (only calculate 50% per frame)
									if ((z + y + x) % 2 == ElapsedTime.TotalFrames % 2)
										HandleSectorPhysics(x, y, z,
											0, 0, 0);
								} // asteroidModel

						for (int z = 0; z < NumOfSectors; z++)
							for (int y = 0; y < NumOfSectors; y++)
								for (int x = 0; x < NumOfSectors; x++)
									//if (x <= 1 && y <= 1 && z <= 1)
									foreach (Asteroid asteroid in asteroids[z, y, x])
									{
										GraphicForm.WorldMatrix = asteroid.RenderMatrix;
										asteroidModel.D3DMesh.DrawSubset(0);
										//asteroidModel.Render(asteroid.RenderMatrix);
									} // asteroidModel
					});
			} // TestHandleSectorPhysics()
			#endregion

			#region Is position on border check
			/// <summary>
			/// Is position on border. Helper for TestAsteroidPositionOnBorder.
			/// </summary>
			/// <param name="pos">Position</param>
			/// <returns>Bool</returns>
			private bool IsPositionOnBorder(Vector3 pos)
			{
				// Check each dimension on its own
				float diffX = pos.X / SectorWidth -
					(float)Math.Round(pos.X / SectorWidth);
				float diffY = pos.Y / SectorHeight -
					(float)Math.Round(pos.Y / SectorHeight);
				float diffZ = pos.Z / SectorDepth -
					(float)Math.Round(pos.Z / SectorDepth);

				// Check if in -0.4 to +0.4 range
				return diffX < -0.4f || diffX > 0.4f ||
					diffY < -0.4f || diffY > 0.4f ||
					diffZ < -0.4f || diffZ > 0.4f;
			} // IsPositionOnBorder(pos)

			/// <summary>
			/// Test if asteroid position is in middle or on border,
			/// used for physics optimizations above. 0.4f is the inside range.
			/// </summary>
			[Test]
			public void TestAsteroidPositionOnBorder()
			{
				// Couple of positions
				Vector3 centerAsteroid1 = new Vector3(0, 0, 0);
				Vector3 centerAsteroid2 = new Vector3(135f, 237f, -20f);
				Vector3 centerAsteroid3 = new Vector3(10f, -525f, 1039f);
				Vector3 xBorderAsteroid1 = new Vector3(41f, 0.0f, 0.0f);
				Vector3 yBorderAsteroid1 = new Vector3(2021f, 50f, 0.0f);
				Vector3 zBorderAsteroid1 = new Vector3(21f, 1015f, -45f);
				Vector3 randomAsteroid1 = new Vector3(43965f, 4534f, -14592f);

				// Few simple tests
				Assert.IsFalse(IsPositionOnBorder(centerAsteroid1));
				Assert.IsFalse(IsPositionOnBorder(centerAsteroid2));
				Assert.IsFalse(IsPositionOnBorder(centerAsteroid3));
				Assert.IsTrue(IsPositionOnBorder(xBorderAsteroid1));
				Assert.IsTrue(IsPositionOnBorder(yBorderAsteroid1));
				Assert.IsTrue(IsPositionOnBorder(zBorderAsteroid1));
				Assert.IsFalse(IsPositionOnBorder(randomAsteroid1));
			} // TestAsteroidPositionOnBorder()
			#endregion
		} // class AsteroidPhysicsTests
		#endregion

		#region Asteroid physics tests using AsteroidManager
		/// <summary>
		/// Asteroid physics using asteroid manager tests
		/// </summary>
		public class AsteroidPhysicsUsingAsteroidManagerTests
		{
			/// <summary>
			/// Asteroid manager
			/// </summary>
			PhysicsAsteroidManager asteroidManager = null;
			/// <summary>
			/// Asteroid model
			/// </summary>
			Model asteroidModel = null;

			#region Test asteroid manager physics
			/// <summary>
			/// Test asteroid manager physics
			/// </summary>
			[Test]
			public void TestAsteroidManagerPhysics()
			{
				GameForm.StartTest("TestHandleSectorPhysics",
					delegate
					{
						asteroidManager = new GameAsteroidManager(
							Level.LoadAllLevels()[0]);
						asteroidModel = asteroidManager.asteroidModels[0];

						// Add asteroids in all 5x5x5 sectors (5 in each)
						for (int z = 0; z < NumberOfSectors; z++)
							for (int y = 0; y < NumberOfSectors; y++)
								for (int x = 0; x < NumberOfSectors; x++)
								{
									asteroidManager.sectorAsteroids[z, y, x] = new List<Asteroid>();
									for (int num = 0; num < 2; num++)
									{
										asteroidManager.sectorAsteroids[z, y, x].Add(new Asteroid(
											0,
											new Vector3((x - MiddleSector) * SectorWidth,
											(y - MiddleSector) * SectorHeight,
											(z - MiddleSector) * SectorDepth) +
											RandomHelper.GetRandomVector3(
											-SectorWidth * 0.4f, +SectorWidth * 0.4f),
											asteroidModel.ObjectSize));
									} // asteroidModel
								} // asteroidModel
					},
					delegate
					{
						GameForm.TestGameForm.camera.FreeCamera = true;
						asteroidModel.FirstMaterial.SelectWithoutShader();
						GraphicForm.DirectXDevice.RenderState.Lighting = true;
						GraphicForm.EnableAlphaBlending();

						for (int z = 0; z < NumberOfSectors; z++)
							for (int y = 0; y < NumberOfSectors; y++)
								for (int x = 0; x < NumberOfSectors; x++)
								{
									// Update all positions for this sector
									foreach (Asteroid asteroid in
										asteroidManager.sectorAsteroids[z, y, x])
										asteroid.UpdateMovement(ElapsedTime.MoveFactorPerSecond);

									// Handle physics (only calculate 50% per frame)
									if ((z + y + x) % 2 == ElapsedTime.TotalFrames % 2)
										asteroidManager.HandleSectorPhysics(x, y, z,
											0, 0, 0);
								} // asteroidModel

						for (int z = 0; z < NumberOfSectors; z++)
							for (int y = 0; y < NumberOfSectors; y++)
								for (int x = 0; x < NumberOfSectors; x++)
									//if (x <= 1 && y <= 1 && z <= 1)
									if (asteroidManager.sectorIsVisible[z, y, x])
										foreach (Asteroid asteroid in
											asteroidManager.sectorAsteroids[z, y, x])
										{
											GraphicForm.WorldMatrix = asteroid.RenderMatrix;
											asteroidModel.D3DMesh.DrawSubset(0);
										} // asteroidModel
					});
			} // TestAsteroidManagerPhysics()
			#endregion

			#region Add asteroidModel
			/// <summary>
			/// Add asteroid
			/// </summary>
			/// <param name="pos">Position</param>
			/// <param name="size">Size</param>
			/// <param name="objectSize">Object size</param>
			/// <param name="movement">Movement</param>
			private void AddAsteroid(Vector3 pos,
				float size, float objectSize, Vector3 movement)
			{
				Asteroid asteroid = new Asteroid(0, pos, objectSize);
				asteroid.movement = movement;
				asteroid.size = size;
				asteroid.collisionRadius = size *
					Asteroid.CollisionSizeCorrectionFactor;
				asteroid.renderSize = size * objectSize;
				asteroidManager.sectorAsteroids[MiddleSector,
					MiddleSector, MiddleSector].Add(asteroid);
			} // AddAsteroid(pos, size, objectSize)
			#endregion

			#region Setup scene
			/// <summary>
			/// Setup scene
			/// </summary>
			/// <param name="sceneNumber">Scene number</param>
			private void SetupScene(int sceneNumber)
			{
				// Kill all
				for (int z = 0; z < NumberOfSectors; z++)
					for (int y = 0; y < NumberOfSectors; y++)
						for (int x = 0; x < NumberOfSectors; x++)
						{
							asteroidManager.sectorAsteroids[z, y, x] = new List<Asteroid>();
						} // for for for (int)

				switch (sceneNumber)
				{
					case 0:
						// Just add 2 colliding asteroids
						AddAsteroid(new Vector3(-50, 0, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(20, 0, 0));
						AddAsteroid(new Vector3(+50, 0, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(-20, 0, 0));
						break;

					case 1:
						// Collide fast moving asteroid with still one
						// Use same sizes and masses for 100% impluse transfer.
						AddAsteroid(new Vector3(-50, 0, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(40, 0, 0));
						AddAsteroid(new Vector3(+50, 0, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(0, 0, 0));
						break;

					case 2:
						// Collide with crazy movement values
						AddAsteroid(new Vector3(-35, +19, 8) * (-2),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(-35, +19, 8));
						AddAsteroid(new Vector3(+15, 40, 14) * (-2),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(+15, 40, 14));
						break;

					case 3:
						// Collide with up/down values
						AddAsteroid(new Vector3(0, 0, -50),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(0, 0, 40));
						AddAsteroid(new Vector3(0, 0, +70),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(0, 0, -20));
						break;

					case 4:
						// Collide with up/down values
						AddAsteroid(new Vector3(0, -50, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(0, 40, 0));
						AddAsteroid(new Vector3(0, +70, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(0, -20, 0));
						break;

					case 5:
						// Add 2 colliding asteroids with different sizes
						AddAsteroid(new Vector3(-50, 0, 0),
							2.5f, asteroidModel.ObjectSize,
							new Vector3(20, 0, 0));
						AddAsteroid(new Vector3(+50, 0, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(-20, 0, 0));
						break;

					case 6:
						// Add 2 colliding asteroids with different sizes
						AddAsteroid(new Vector3(-50, +100, 0),
							2.5f, asteroidModel.ObjectSize,
							new Vector3(20, -40, 0));
						AddAsteroid(new Vector3(+50, +100, 0),
							12.5f, asteroidModel.ObjectSize,
							new Vector3(-20, -40, 0));
						break;
				} // switch
			} // SetupScene(sceneNumber)
			#endregion

			#region Test asteroid manager physics. Press 1-7 to start physics scenes.
			/// <summary>
			/// Test asteroid manager physics. Press 1-7 to start physics scenes.
			/// </summary>
			[Test]
			public void TestAsteroidPhysicsSmallScene()
			{
				GameForm.StartTest("TestAsteroidPhysicsSmallScene",
					delegate
					{
						asteroidManager = new GameAsteroidManager(
							Level.LoadAllLevels()[0]);
						asteroidModel = asteroidManager.asteroidModels[0];
						//new Model("Asteroid1.X");

						SetupScene(0);
					},
					delegate
					{
						GameForm.TestGameForm.camera.FreeCamera = true;
						asteroidModel.FirstMaterial.SelectWithoutShader();
						GraphicForm.DirectXDevice.RenderState.Lighting = true;
						GraphicForm.EnableAlphaBlending();

						// Press 1-7 to start physics scenes.
						if (GameForm.Keyboard.State[Key.D1])
							SetupScene(0);
						if (GameForm.Keyboard.State[Key.D2])
							SetupScene(1);
						if (GameForm.Keyboard.State[Key.D3])
							SetupScene(2);
						if (GameForm.Keyboard.State[Key.D4])
							SetupScene(3);
						if (GameForm.Keyboard.State[Key.D5])
							SetupScene(4);
						if (GameForm.Keyboard.State[Key.D6])
							SetupScene(5);
						if (GameForm.Keyboard.State[Key.D7])
							SetupScene(6);

						for (int z = 0; z < NumberOfSectors; z++)
							for (int y = 0; y < NumberOfSectors; y++)
								for (int x = 0; x < NumberOfSectors; x++)
								{
									// Update all positions for this sector
									foreach (Asteroid asteroid in
										asteroidManager.sectorAsteroids[z, y, x])
										asteroid.UpdateMovement(ElapsedTime.MoveFactorPerSecond);

									// Handle physics (only calculate 50% per frame)
									if ((z + y + x) % 2 == ElapsedTime.TotalFrames % 2)
										asteroidManager.HandleSectorPhysics(x, y, z,
											0, 0, 0);
								} // asteroidModel

						for (int z = 0; z < NumberOfSectors; z++)
							for (int y = 0; y < NumberOfSectors; y++)
								for (int x = 0; x < NumberOfSectors; x++)
									//if (x <= 1 && y <= 1 && z <= 1)
									if (asteroidManager.sectorIsVisible[z, y, x])
										foreach (Asteroid asteroid in
											asteroidManager.sectorAsteroids[z, y, x])
										{
											GraphicForm.WorldMatrix = asteroid.RenderMatrix;
											asteroidModel.D3DMesh.DrawSubset(0);
											//asteroidModel.Render(asteroid.RenderMatrix);
										} // asteroidModel
					});
			} // TestAsteroidPhysicsSmallScene()
			#endregion
		} // class AsteroidPhysicsUsingAsteroidManagerTests
		#endregion

		#region Collision test tests ^^
		/// <summary>
		/// Collision test
		/// </summary>
		class CollisionTest
		{
			#region Test donut collision
			/// <summary>
			/// Test donut collision
			/// </summary>
			[Test]
			public static void TestDonutCollision()
			{
				Model asteroidModel = null;
				GameForm.StartTest("TestDonutCollision",
					delegate
					{
						asteroidModel = new Model("Asteroid5.X");
					},
					delegate
					{
						GameForm.TestGameForm.camera.FreeCamera = true;
						asteroidModel.FirstMaterial.SelectWithoutShader();
						GraphicForm.DirectXDevice.RenderState.Lighting = true;
						GraphicForm.EnableAlphaBlending();

						// Init positions and sizes
						Vector3 camPos = GameForm.CameraPos;
						Vector3 objPos = Vector3.Empty;
						float objSize = 50.0f;

						// Setup world matrix
						GraphicForm.WorldMatrix =
							asteroidModel.ObjectMatrix *
							Matrix.Scaling(objSize, objSize, objSize);

						// Render asteroid
						asteroidModel.D3DMesh.DrawSubset(0);

						// Check if inside bounding sphere of asteroid
						// Add a little offset to include collisions at the border.
						bool insideBoundingSphere =
							(camPos - objPos).Length() < objSize * 1.25f;
						bool collisionWithCamera = false;
						float collisionDistance = 0.0f;
						if (insideBoundingSphere)
						{
							Vector3 v = new Vector3(0, 0, 1);
							Matrix m = GraphicForm.InverseViewMatrix;
							// Transform the screen space pick ray into 3D space
							Vector3 rayDir = new Vector3(
								v.X * m.M11 + v.Y * m.M21 + v.Z * m.M31,
								v.X * m.M12 + v.Y * m.M22 + v.Z * m.M32,
								v.X * m.M13 + v.Y * m.M23 + v.Z * m.M33);
							Vector3 rayOrigin = new Vector3(
								m.M41, m.M42, m.M43);

							Matrix invWorldMatrix =
								Matrix.Invert(GraphicForm.WorldMatrix);
							Vector3 rayWorldPos =
								Vector3.TransformCoordinate(rayOrigin, invWorldMatrix);
							//obs: Vector3 camDir = new Vector3(0, 0, 1);
							//camDir.TransformNormal(GraphicForm.ViewMatrix);
							Vector3 rayWorldDir = //10.0f *
								Vector3.TransformNormal(rayDir, invWorldMatrix);
							rayWorldDir.Normalize();

							IntersectInformation closestHit;
							if (asteroidModel.D3DMesh.Intersect(rayWorldPos, rayWorldDir,
								out closestHit))
							{
								// Check if we hit near the camera, else discard.
								if (closestHit.Dist < 50.0f) // TODO: use speed+offset here!
									collisionWithCamera = true;
								collisionDistance = closestHit.Dist;
							} // if
						} // if

						GameForm.SmallFont.WriteText(2, 20,
							"camPos=" + camPos.X + ", " + camPos.Y + ", " + camPos.Z + "\n" +
							"insideBoundingSphere=" + insideBoundingSphere + "\n" +
							"collisionDistance=" + collisionDistance + "\n" +
							"collisionWithCamera=" + collisionWithCamera);
					});
			} // TestDonutCollision()
			#endregion
		} // class CollisionTest
		#endregion
#endif
		#endregion
	} // class PhysicsAsteroidManager
} // namespace RocketCommander.Game
