// Project: Rocket Commander, File: SpaceCamera.cs
// Namespace: RocketCommander.Game, Class: SpaceCamera
// Path: C:\code\Rocket Commander\Game, Author: Abi
// Code lines: 389, Size of file: 11,15 KB
// Creation date: 06.11.2005 03:03
// Last modified: 11.03.2006 02:05
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Threading;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;
using RocketCommander.Game;
using RocketCommander.Helpers;
using RocketCommander.Properties;
using RocketCommander.Graphics;
using DirectXVector3 = Microsoft.DirectX.Vector3;
using Material = RocketCommander.Graphics.Material;
#endregion

namespace RocketCommander.Game
{
	/// <summary>
	/// Space camera like in space sims (e.g. descent) or shooters.
	/// Move around with aswd or cursor keys and rotate with the mouse.
	/// Based on Quaternions to perform all the rotations required.
	/// Note: Handles also some game logic, which has to do with the
	/// rocket controlling in HandlePlayerInput.
	/// </summary>
	public class SpaceCamera
	{
		#region Variables
		/// <summary>
		/// Default camera up vector (used for Matrix.LookAt calls)
		/// </summary>
		private static readonly Vector3 DefaultCameraUp =
			new Vector3(0.0f, 0.0f, -1.0f);

		/// <summary>
		/// Current camera position.
		/// </summary>
		private Vector3 pos;

		/// <summary>
		/// Move directions for controling the camera.
		/// X is for moving left/right, Y moves up/down and Z goes in/out.
		/// </summary>
		private enum MoveDirections
		{
			/// <summary>
			/// X direction (move left/right)
			/// </summary>
			X,
			/// <summary>
			/// Y direction (move up/down)
			/// </summary>
			Y,
			/// <summary>
			/// Z direction (move in/out)
			/// </summary>
			Z,
		} // enum MoveDirections

		/// <summary>
		/// Rotation axis for controling the camera.
		/// Pitch is for moving head up/down,
		/// Roll rotates around nose
		/// and Yaw is the rotation to left/right.
		/// </summary>
		private enum RotationAxis
		{
			/// <summary>
			/// Pitch is for moving head up/down.
			/// </summary>
			Pitch,
			/// <summary>
			/// Roll rotates around nose.
			/// </summary>
			Roll,
			/// <summary>
			/// Yaw is the rotation to left/right.
			/// </summary>
			Yaw
		} // enum RotationAxis

		/// <summary>
		/// Quaternion used for rotation.
		/// </summary>
		private Quaternion quaternion = Quaternion.Identity;

		/// <summary>
		/// Camera modes
		/// </summary>
		public enum CameraModes
		{
			/// <summary>
			/// Default mode for menu, just rotating around.
			/// </summary>
			MenuMode,
			/// <summary>
			/// In game using the rocket controls (always flying forwards).
			/// </summary>
			InGame,
			/// <summary>
			/// Free camera mode, allows to freely rotate and move around,
			/// much cooler than the InGame mode for testing and stuff.
			/// </summary>
			FreeCamera,
		} // enum CameraModes

		/// <summary>
		/// Current camera mode.
		/// </summary>
		private CameraModes cameraMode = CameraModes.FreeCamera;//.MenuMode;

		/// <summary>
		/// Keys for moving around. Assigned from settings!
		/// </summary>
		private Key moveLeftKey,
			moveRightKey,
			moveForwardKey,
			moveBackwardKey,
			rollLeftKey,
			rollRightKey;

		/// <summary>
		/// Rotation matrix, used in UpdateViewMatrix.
		/// </summary>
		private Matrix rotMatrix = Matrix.Identity;
		/// <summary>
		/// Rotation matrix
		/// </summary>
		/// <returns>Matrix</returns>
		public Matrix RotationMatrix
		{
			get
			{
				return rotMatrix;
			} // get
		} // RotationMatrix

		#endregion

		#region Properties
		/// <summary>
		/// Quaternion rotation
		/// </summary>
		/// <returns>Quaternion</returns>
		public Quaternion QuaternionRotation
		{
			get
			{
				return quaternion;
			} // get
			set
			{
				quaternion = value;
			} // set
		} // QuaternionRotation

		/// <summary>
		/// Get current x axis with help of the current view matrix.
		/// </summary>
		/// <returns>Vector 3</returns>
		static public Vector3 XAxis
		{
			get
			{
				// Get x column
				return new Vector3(
					GraphicForm.ViewMatrix.M11,
					GraphicForm.ViewMatrix.M21,
					GraphicForm.ViewMatrix.M31);
			} // get
		} // XAxis

		/// <summary>
		/// Get current y axis with help of the current view matrix.
		/// </summary>
		/// <returns>Vector 3</returns>
		static public Vector3 YAxis
		{
			get
			{
				// Get y column
				return new Vector3(
					GraphicForm.ViewMatrix.M12,
					GraphicForm.ViewMatrix.M22,
					GraphicForm.ViewMatrix.M32);
			} // get
		} // YAxis

		/// <summary>
		/// Get current z axis with help of the current view matrix.
		/// </summary>
		/// <returns>Vector 3</returns>
		static public Vector3 ZAxis
		{
			get
			{
				// Get z column
				return new Vector3(
					GraphicForm.ViewMatrix.M13,
					GraphicForm.ViewMatrix.M23,
					GraphicForm.ViewMatrix.M33);
			} // get
		} // ZAxis

		/// <summary>
		/// Set if we are in game or not. Write only property.
		/// </summary>
		/// <returns>Bool</returns>
		public bool InGame
		{
			get
			{
				return cameraMode == CameraModes.InGame;
			} // get
			set
			{
				if (value)
					cameraMode = CameraModes.InGame;
				else
					cameraMode = CameraModes.MenuMode;
			} // set
		} // InGame

		/// <summary>
		/// Free camera
		/// </summary>
		/// <returns>Bool</returns>
		public bool FreeCamera
		{
			get
			{
				return cameraMode == CameraModes.FreeCamera;
			} // get
			set
			{
				if (value == true)
					cameraMode = CameraModes.FreeCamera;
				else
					cameraMode = CameraModes.InGame;
			} // set
		} // FreeCamera
		#endregion

		#region Constructor
		/// <summary>
		/// Create space camera
		/// </summary>
		/// <param name="setCameraPos">Set camera pos</param>
		public SpaceCamera(Vector3 setCameraPos)
		{
			pos = setCameraPos;

			// Assign keys. Warning: This is VERY slow, never use it
			// inside any render loop (getting Settings, etc.)!
			moveLeftKey = (Key)EnumHelper.SearchEnumerator(
				typeof(Key), Settings.Default.MoveLeftKey);
			moveRightKey = (Key)EnumHelper.SearchEnumerator(
				typeof(Key), Settings.Default.MoveRightKey);
			moveForwardKey = (Key)EnumHelper.SearchEnumerator(
				typeof(Key), Settings.Default.MoveForwardKey);
			moveBackwardKey = (Key)EnumHelper.SearchEnumerator(
				typeof(Key), Settings.Default.MoveBackwardKey);
			rollLeftKey = (Key)EnumHelper.SearchEnumerator(
				typeof(Key), Settings.Default.RollLeftKey);
			rollRightKey = (Key)EnumHelper.SearchEnumerator(
				typeof(Key), Settings.Default.RollRightKey);
		} // SpaceCamera(setCameraPos)

		/// <summary>
		/// Create space camera
		/// </summary>
		/// <param name="setCameraPos">Set camera pos</param>
		/// <param name="setLookPos">Set look pos</param>
		public SpaceCamera(Vector3 setCameraPos, Vector3 setLookPos)
			: this(setCameraPos)
		{
			// Calculate rotation quaternion from look pos
			SetLookAt(setCameraPos, setLookPos, DefaultCameraUp);
		} // SpaceCamera(setCameraPos, setLookPos)

		/// <summary>
		/// Create space camera
		/// </summary>
		/// <param name="setCameraPos">Set camera pos</param>
		/// <param name="setRotationQuaternion">Set rotation quaternion</param>
		public SpaceCamera(Vector3 setCameraPos, Quaternion setRotationQuaternion)
			: this(setCameraPos)
		{
			quaternion = setRotationQuaternion;
		} // SpaceCamera(setCameraPos, setRotationQuaternion)
		#endregion

		#region Set position
		/// <summary>
		/// Set position
		/// </summary>
		/// <param name="setCameraPos">Set camera position</param>
		public void SetPosition(Vector3 setCameraPos)
		{
			pos = setCameraPos;
		} // SetPosition(setCameraPos)
		#endregion

		#region Helper method to get quaternion from LookAt matrix (private)
		/// <summary>
		/// Helper method to get quaternion from LookAt matrix.
		/// </summary>
		/// <param name="setCamPos">Set cam pos</param>
		/// <param name="setLookPos">Set look pos</param>
		/// <param name="setUpVector">Set up vector</param>
		private void SetLookAt(
			Vector3 setCamPos, Vector3 setLookPos, Vector3 setUpVector)
		{
			pos = setCamPos;

			// Build look at matrix and get the quaternion from that
			quaternion = Quaternion.RotationMatrix(
				Matrix.LookAtLH(pos, setLookPos, setUpVector));
		} // SetLookAt(setCamPos, setLookPos, setUpVector)
		#endregion

		#region Rotate methods
		/// <summary>
		/// Rotate around pitch, roll or yaw axis.
		/// </summary>
		/// <param name="axis">Axis</param>
		/// <param name="angle">Angle</param>
		private void Rotate(RotationAxis axis, float angle)
		{
			Rotate(
				axis == RotationAxis.Pitch ? new Vector3(1.0f, 0.0f, 0.0f) :
				// Our world is xz based for some reason ^^ this way it works
				axis == RotationAxis.Roll ? new Vector3(0.0f, 0.0f, 1.0f) :
				new Vector3(0.0f, 1.0f, 0.0f), angle);
		} // Rotate(axis, angle)

		/// <summary>
		/// Rotate
		/// </summary>
		/// <param name="axis">Axis</param>
		/// <param name="angle">Angle</param>
		private void Rotate(Vector3 axis, float angle)
		{
			quaternion *= Quaternion.RotationAxis(axis, angle);
		} // Rotate(axis, angle)
		#endregion

		#region Translate method
		/// <summary>
		/// Translate into x, y or z axis with a specfic amount.
		/// </summary>
		/// <param name="amount">Amount</param>
		/// <param name="direction">Direction</param>
		private void Translate(float amount, MoveDirections direction)
		{
			Vector3 dir =
				direction == MoveDirections.X ? XAxis :
				direction == MoveDirections.Y ? YAxis : ZAxis;
			pos += dir * amount;
		} // Translate(amount, direction)
		#endregion

		#region Handle free camera
		/// <summary>
		/// Handle free camera
		/// </summary>
		private void HandleFreeCamera()
		{
			float moveFactor = ElapsedTime.MoveFactorPerSecond * 250.0f;//100.0f;
			float slideFactor = moveFactor;// / 10.0f;
			float rotationFactor = 0.015f;//0.02f;

			// Always change camera rotation when mouse is moved,
			// this is how we control our rocket.
			if (GameForm.Mouse.RightButtonPressed)
			{
				if (GameForm.Mouse.XMovement != 0.0f ||
					GameForm.Mouse.YMovement != 0.0f)
				{
					Rotate(RotationAxis.Roll, rotationFactor *
						(GameForm.Mouse.XMovement - GameForm.Mouse.YMovement));
				} // if (GameForm.Mouse.YMovement)
			} // if (GameForm.Mouse.RightButtonPressed)
			else if (GameForm.Mouse.XMovement != 0.0f ||
				GameForm.Mouse.YMovement != 0.0f)
			{
				Rotate(RotationAxis.Yaw,
					-GameForm.Mouse.XMovement * rotationFactor);
				Rotate(RotationAxis.Pitch,
					-GameForm.Mouse.YMovement * rotationFactor);
			} // if (Mouse.left.Pressed)

			// Use asdw (qwerty keyboard), aoew (dvorak keyboard) or
			// cursor keys (all keyboards?) to move around.
			// Note: If you want to change this keys, you should implement
			// a config file and an options screen.
			if (GameForm.Keyboard.State[moveLeftKey] ||
				GameForm.Keyboard.State[Key.Left] ||
				GameForm.Keyboard.State[Key.NumPad4])
				Translate(-slideFactor, MoveDirections.X);
			if (GameForm.Keyboard.State[moveRightKey] ||
				GameForm.Keyboard.State[Key.Right] ||
				GameForm.Keyboard.State[Key.NumPad6])
				Translate(+slideFactor, MoveDirections.X);
			if (GameForm.Keyboard.State[moveForwardKey] ||
				GameForm.Keyboard.State[Key.Up] ||
				GameForm.Keyboard.State[Key.NumPad8])
				Translate(+moveFactor, MoveDirections.Z);
			if (GameForm.Keyboard.State[moveBackwardKey] ||
				GameForm.Keyboard.State[Key.Down] ||
				GameForm.Keyboard.State[Key.NumPad2])
				Translate(-moveFactor, MoveDirections.Z);

			// Use Q and E (or _ and P) to roll
			if (GameForm.Keyboard.State[rollLeftKey])
				Rotate(RotationAxis.Roll, +ElapsedTime.MoveFactorPerSecond * 2);
			if (GameForm.Keyboard.State[rollRightKey])
				Rotate(RotationAxis.Roll, -ElapsedTime.MoveFactorPerSecond * 2);
		} // HandleFreeCamera()
		#endregion

		#region Handle player input
		/// <summary>
		/// Handle player input for the game. This is where all the input happens.
		/// </summary>
		private void HandlePlayerInput()
		{
			if (Player.lifeTimeMs < Player.LifeTimeZoomAndAccelerateMs)
			{
				float speedPercentage =
					Player.lifeTimeMs < 3000 ? 0 :
					(Player.lifeTimeMs - 3000) / 2000.0f;

				// Use quadradric product for better speed up effect
				Player.SetStartingSpeed(speedPercentage * speedPercentage);

				// Always move forward
				Translate(Player.Speed *
					ElapsedTime.MoveFactorPerSecond *
					Player.MovementSpeedPerSecond, MoveDirections.Z);

				if (Player.gameTimeMs < 3000)
				{
					quaternion = Quaternion.Identity;
					pos = Vector3.Empty;
				} // if
			} // if
			
			if (Player.CanControlRocket)
			{
				// Consume some fuel
				Player.fuel -= ElapsedTime.MoveFactorPerSecond /
					Player.FuelRefillTime;

				float speedScoreFactor = 0.75f * Player.Speed;
				if (Player.speedItemTimeout > 0)
					speedScoreFactor *= 2.0f;

				// Increase score
				if (ElapsedTime.TotalFrames % 10 == 0)
					Player.score += (int)(speedScoreFactor * ElapsedTime.MsLastFrame);
			} // if

			// Don't allow any movement if still counting down or exploded
			if (Player.CanControlRocket == false)
			{
				if (Player.GameOver)
				{
					// Just rotate around the rocket.
					quaternion.RotateAxis(new Vector3(0, 1, 0),
						ElapsedTime.MsTotal / 3592.0f);
				} // if

				return;
			} // if

			float maxMoveFactor = ElapsedTime.MoveFactorPerSecond *
				Player.MovementSpeedPerSecond;
			float maxSlideFactor = maxMoveFactor * Player.SlideFactor;
			float rotationFactor = Player.RotationSpeedPerMouseMovement;

			#region Mouse/keyboard support
			// Right mouse button lets us roll around our axis
			if (GameForm.Mouse.RightButtonPressed)
			{
				if (GameForm.Mouse.XMovement != 0.0f ||
					GameForm.Mouse.YMovement != 0.0f)
				{
					Rotate(RotationAxis.Roll, rotationFactor *
						(GameForm.Mouse.XMovement - GameForm.Mouse.YMovement));
				} // if (GameForm.Mouse.YMovement)
			} // if (GameForm.Mouse.RightButtonPressed)
			// Always change camera rotation when mouse is moved,
			// this is how we control our rocket.
			else if (GameForm.Mouse.XMovement != 0.0f ||
				GameForm.Mouse.YMovement != 0.0f)
			{
				// Limit x/y movement to max. 150 units per frame
				float xMovement = GameForm.Mouse.XMovement;
				float yMovement = GameForm.Mouse.YMovement;
				Rotate(RotationAxis.Yaw,
					-xMovement * rotationFactor);
				Rotate(RotationAxis.Pitch,
					-yMovement * rotationFactor);
			} // if (Mouse.left.Pressed)

			bool consumedAdditionalFuel = false;

			// Use asdw (qwerty keyboard), aoew (dvorak keyboard) or
			// cursor keys (all keyboards?) to move around.
			// Note: If you want to change any keys, use Settings!
			if (GameForm.Keyboard.State[moveForwardKey] ||
				GameForm.Keyboard.State[Key.Up] ||
				GameForm.Keyboard.State[Key.NumPad8])
			{
				float oldPlayerSpeed = Player.Speed;
				Player.Speed += 0.75f * ElapsedTime.MoveFactorPerSecond;

				// Only decrease fuel if change happened
				if (oldPlayerSpeed != Player.Speed)
					consumedAdditionalFuel = true;
			} // if
			if (GameForm.Keyboard.State[moveBackwardKey] ||
				GameForm.Keyboard.State[Key.Down] ||
				GameForm.Keyboard.State[Key.NumPad2])
			{
				float oldPlayerSpeed = Player.Speed;
				Player.Speed -= 0.75f * ElapsedTime.MoveFactorPerSecond;

				// Only decrease fuel if change happened
				if (oldPlayerSpeed != Player.Speed)
					consumedAdditionalFuel = true;
			} // if

			if (Player.speedItemTimeout > 0)
			{
				Player.speedItemTimeout -= ElapsedTime.MsLastFrame;
				if (Player.speedItemTimeout < 0)
				{
					Player.speedItemTimeout = 0;
					// Reduce to max. possible speed
					if (Player.Speed > Player.MaxSpeedWithoutItem)
						Player.Speed = Player.MaxSpeedWithoutItem;
				} // if
			} // if

			// Adjust current speed by the current player speed.
			float moveFactor = Player.Speed * maxMoveFactor;
			float slideFactor = maxSlideFactor;

			// Always move forward
			Translate(+moveFactor, MoveDirections.Z);

			// Slide
			if (GameForm.Keyboard.State[moveLeftKey] ||
				GameForm.Keyboard.State[Key.Left] ||
				GameForm.Keyboard.State[Key.NumPad4])
			{
				consumedAdditionalFuel = true;
				Translate(-slideFactor, MoveDirections.X);
			} // if
			if (GameForm.Keyboard.State[moveRightKey] ||
				GameForm.Keyboard.State[Key.Right] ||
				GameForm.Keyboard.State[Key.NumPad6])
			{
				consumedAdditionalFuel = true;
				Translate(+slideFactor, MoveDirections.X);
			} // if

            // Up/down
            if (GameForm.Keyboard.State[Key.F])
            {
                Translate(+slideFactor, MoveDirections.Y);
            } // if
            if (GameForm.Keyboard.State[Key.V])
            {
                Translate(-slideFactor, MoveDirections.Y);
            } // if

			// Use Q and E (or _ and P) to roll
			if (GameForm.Keyboard.State[rollLeftKey])
				Rotate(RotationAxis.Roll,
					+ElapsedTime.MoveFactorPerSecond * 2.0f);
			if (GameForm.Keyboard.State[rollRightKey])
				Rotate(RotationAxis.Roll,
					-ElapsedTime.MoveFactorPerSecond * 2.0f);
			#endregion

			#region XInput support for the XBox360 controller
			// 2006-03-09: Added XInput support
			rotationFactor = 3.0f * ElapsedTime.MoveFactorPerSecond;

			// Left and right triggers lets us roll around our axis
			if (XInput.LeftTrigger != 0 ||
				XInput.RightTrigger != 0)
			{
				Rotate(RotationAxis.Roll, rotationFactor *
					(-XInput.LeftTrigger + XInput.RightTrigger));
			} // if (XInput.LeftTrigger)
			// Change camera rotation when right thumb is used.
			if (XInput.XRotation != 0.0f ||
				XInput.YRotation != 0.0f)
			{
				// Limit x/y movement to max. 150 units per frame
				float xMovement = XInput.XRotation;
				float yMovement = XInput.YRotation;
				Rotate(RotationAxis.Yaw,
					-xMovement * rotationFactor);
				Rotate(RotationAxis.Pitch,
					yMovement * rotationFactor);
			} // if (Mouse.left.Pressed)

			// Use left thumb for moving around
			if (XInput.YMovement != 0)
			{
				float oldPlayerSpeed = Player.Speed;
				Player.Speed += 0.75f * XInput.YMovement *
					ElapsedTime.MoveFactorPerSecond;

				// Only decrease fuel if change happened
				if (oldPlayerSpeed != Player.Speed)
					consumedAdditionalFuel = true;
			} // if

			// Slide
			if (XInput.XMovement != 0)
			{
				consumedAdditionalFuel = true;
				Translate(slideFactor * XInput.XMovement * 2, MoveDirections.X);
			} // if
			#endregion

			if (consumedAdditionalFuel)
			{
				// Decrease additional fuel
				Player.fuel -= (ElapsedTime.MoveFactorPerSecond /
					Player.FuelRefillTime) / 2.0f;

				// Increase score, but only 50% of the amount above.
				// Note: This happens only if we change speed, that is not often.
				if (ElapsedTime.TotalFrames % 20 == 0)
					Player.score += ElapsedTime.MsLastFrame / 2;
			} // if
		} // HandlePlayerInput()
		#endregion

		#region Randomly rotate around
		/// <summary>
		/// Randomly rotate around
		/// </summary>
		private void RandomlyRotateAround()
		{
			float moveFactor = ElapsedTime.MoveFactorPerSecond * 27.5f;
			float rotationFactor = ElapsedTime.MoveFactorPerSecond * 0.125f;

			// Rotate slightly around
			Rotate(RotationAxis.Yaw,
				(0.4f + 0.25f * (float)Math.Sin(ElapsedTime.MsTotal / 15040)) *
				rotationFactor);
			Rotate(RotationAxis.Pitch,
				(0.35f + 0.212f * (float)Math.Cos(ElapsedTime.MsTotal / 38040)) *
				rotationFactor);

			// Just move forward
			Translate(+moveFactor, MoveDirections.Z);
		} // RandomlyRotateAround()
		#endregion

		#region Update view matrix
		/// <summary>
		/// Update view matrix
		/// </summary>
		private void UpdateViewMatrix()
		{
			// Rotate world by negative of quaternion and translate everything
			// by camera position.
			quaternion.Normalize();
			rotMatrix = Matrix.RotationQuaternion(quaternion);

			// Add camera shake if camera wobbel effect is on
			if (Player.cameraWobbelTimeoutMs > 0 &&
				// But only if not zooming in and if in game.
				Player.lifeTimeMs > Player.LifeTimeZoomAndAccelerateMs &&
				InGame)
			{
				float effectStrength = 8.0f *
					Player.cameraWobbelFactor *
					(Player.cameraWobbelTimeoutMs /
					(float)Player.MaxCameraWobbelTimeoutMs);
				rotMatrix *= Matrix.Translation(
					RandomHelper.GetRandomVector3(
					-effectStrength, +effectStrength));
			} // if

			// Just set view matrix
			GameForm.ViewMatrix = Matrix.Translation(-pos) * rotMatrix;
		} // UpdateViewMatrix()
		#endregion

		#region Update
		/// <summary>
		/// Update camera, should be called every frame to handle all the input.
		/// </summary>
		public void Update()
		{
			if (cameraMode == CameraModes.FreeCamera)
				HandleFreeCamera();
			else if (cameraMode == CameraModes.InGame)
				HandlePlayerInput();
			else // for menu
				RandomlyRotateAround();
			
			UpdateViewMatrix();
		} // Update()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test space camera
		/// </summary>
		[Test]
		public static void TestSpaceCamera()
		{
			Material mat = null;
			Mesh teapot = null;

			GameForm.StartTest("TestSpaceCamera",
				delegate // Init
				{
					mat = new Material("Test\\rock.dds");
					teapot = Mesh.Teapot(GraphicForm.DirectXDevice);
				},
				delegate // Render loop
				{
					// Just render teapot in the middle of the scene.
					mat.SelectWithoutShader();
					GameForm.WorldMatrix = Matrix.Scaling(25, 25, 25);
					teapot.DrawSubset(0);

					GraphicForm.SmallFont.WriteText(1, 20,
						"MoveFactorPerSecond=" + ElapsedTime.MoveFactorPerSecond);
				});
		} // TestSpaceCamera()
#endif
		#endregion
	} // class SpaceCamera
} // namespace RocketCommander.Game
