// Project: Rocket Commander, File: Mission.cs
// Namespace: RocketCommander.GameScreens, Class: Mission
// Path: C:\code\Rocket Commander\GameScreens, Author: Abi
// Code lines: 46, Size of file: 857 Bytes
// Creation date: 01.11.2005 23:55
// Last modified: 08.12.2005 16:14
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using RocketCommander.Graphics;
using Font = RocketCommander.Graphics.Font;
using Texture = RocketCommander.Graphics.Texture;
using RocketCommander.Helpers;
using RocketCommander.Properties;
using Microsoft.DirectX.DirectInput;
using RocketCommander.Game;
#endregion

namespace RocketCommander.GameScreens
{
	/// <summary>
	/// Mission, just manages the on screen display for the game.
	/// Controlling is done in SpaceCamera class.
	/// Most graphical stuff is done in AsteroidManager.
	/// </summary>
	class Mission : IGameScreen
	{
		#region Variables
		/// <summary>
		/// Fonts for the game
		/// </summary>
		private Font smallFont = null, bigFont = null;

		/// <summary>
		/// Link to form.hudTexture and inGameTexture, the texture itself is not
		/// loaded here, but in the main form. This are just references.
		/// </summary>
		private Texture hudTexture, inGameTexture, levelTexture;

		/// <summary>
		/// Rectangles for InGame.png.
		/// Used to display on screen hud.
		/// </summary>
		private readonly Rectangle
			HudRect = new Rectangle(0, 0, 256, 157),
			ArrowRightRect = new Rectangle(0, 0, 53, 57),
			ArrowLeftRect = new Rectangle(53, 0, 53, 57),
			ArrowUpRect = new Rectangle(0, 57, 53, 57),
			ArrowDownRect = new Rectangle(53, 57, 53, 57),
			TargetIconRect = new Rectangle(106, 49, 61, 61),
			RocketLifeRect = new Rectangle(106, 0, 37, 49),
			MiniMapRocketRect = new Rectangle(143, 21, 24, 24),
			MiniMapBackgroundRect = new Rectangle(168, 0, 88, 115),
			FuelBarRect = new Rectangle(0, 115, 256, 81),
			HealthBarRect = new Rectangle(0, 196, 256, 29),
			SpeedBarRect = new Rectangle(0, 225, 256, 31),
			// Those 2 are not on inGameTexture, but on hudTexture:
			MiniMapFieldOfViewRect = new Rectangle(1, 256 - 80, 80, 80),
			ScreenBorderEffectRect = new Rectangle(164, 164, 90, 90);

		/// <summary>
		/// Game text and hud colors
		/// </summary>
		private static readonly Color
			GameTextColor = Color.Green,//.DarkGreen,
			HudColor = Color.LightGreen,
			HudRedColor = Color.Red;

		/// <summary>
		/// Usable width for fuel, health and speed bars!
		/// </summary>
		private const int FuelHealthAndSpeedWidth = 208;

		/// <summary>
		/// Distance for hud to screen borders.
		/// </summary>
		private const int HudDistanceFromBorder = 8;

		/// <summary>
		/// Hud background scale factor, make hud a little bit bigger than
		/// </summary>
		private const float HudBackgroundScaleFactor = 1.125f;
		#endregion
		
		#region Properties
		/// <summary>
		/// Name of this game screen
		/// </summary>
		/// <returns>String</returns>
		public string Name
		{
			get
			{
				return "Mission";
			} // get
		} // Name

		private bool quit = false;
		/// <summary>
		/// Returns true if we want to quit this screen and return to the
		/// previous screen. If no more screens are left the game is exited.
		/// </summary>
		/// <returns>Bool</returns>
		public bool Quit
		{
			get
			{
				return quit;
			} // get
		} // Quit
		#endregion

		#region Constructor
		/// <summary>
		/// Create mission
		/// </summary>
		public Mission(Level level, GameAsteroidManager asteroidManager)
		{
			// Set level for asteroid manager.
			asteroidManager.CurrentLevel = level;

			// Start new game.
			Player.Reset(level.Name);

			// Load fonts
			smallFont = new Font("Arial", 12);
			bigFont = new Font("Verdana",
				Math.Max(14, 28 * GraphicForm.Resolution.Width / 1024));
		} // Mission()
		#endregion

		#region Zoom into rocket
		float rocketEndGameScale = 0.0f;
		/// <summary>
		/// Zoom into rocket
		/// </summary>
		/// <param name="camera">Camera</param>
		private void ZoomIntoRocket(SpaceCamera camera, Model rocketModel)
		{
			if (Player.GameOver)
			{
				// Only start showing rocket after all explosions are nearly over!
				// Only 400ms left.
				if (Player.explosionTimeoutMs < 400 &&
					Player.explosionTimeoutMs2 < 400 &&
					Player.explosionTimeoutMs3 < 400)
				{
					// Make sure z buffer is on
					GraphicForm.DirectXDevice.RenderState.ZBufferEnable = true;

					// Scale in rocket (cool zoom effect)
					rocketEndGameScale += ElapsedTime.MoveFactorPerSecond * 5.0f;
					if (rocketEndGameScale > 10.0f)
						rocketEndGameScale = 10.0f;
					float scale = rocketEndGameScale;

					// Show rocket in middle of screen.
					Vector3 inFrontOfCameraPos =
						new Vector3(0, -0.3f, 1.75f) * 10.0f;
					inFrontOfCameraPos.TransformNormal(GraphicForm.InverseViewMatrix);
					Matrix startMatrix =
						Matrix.Scaling(scale, scale, scale) *
						Matrix.RotationX((float)Math.PI / 2.0f) *
						Matrix.RotationY(ElapsedTime.MsTotal / 2293.0f) *
						Matrix.RotationZ((float)Math.PI / 4.0f) *
						Matrix.Translation(inFrontOfCameraPos + GameForm.CameraPos);
					rocketModel.Render(startMatrix);

					// Disable z buffer, now only 2d content is rendered.
					GraphicForm.DirectXDevice.RenderState.ZBufferEnable = false;
				} // if
				else
					rocketEndGameScale = 0.0f;
			} // if

			// Zoom in when starting or a new life was started.
			else if (Player.explosionTimeoutMs < 0 &&
				Player.lifeTimeMs < 3000)
			{
				// Make sure z buffer is on
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = true;

				// Slowly zoom into rocket for the first 3 seconds
				Vector3 inFrontOfCameraPos =
					new Vector3(0, -1.5f, 3.0f) * 2.5f;
				inFrontOfCameraPos.TransformNormal(GraphicForm.InverseViewMatrix);
				Matrix startMatrix =
					Matrix.RotationX((float)Math.PI / 2.0f) *
					Matrix.RotationZ((float)Math.PI / 4.0f) *
					Matrix.Translation(new Vector3(0, -0.475f, -0.4f)) *
					Matrix.Invert(camera.RotationMatrix) *
					Matrix.Translation(inFrontOfCameraPos *
					(1.0f - (Player.lifeTimeMs / 3000.0f))) *
					Matrix.Translation(GameForm.CameraPos);
				rocketModel.Render(startMatrix);

				// Disable z buffer, now only 2d content is rendered.
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = false;
			} // if
		} // ZoomIntoRocket(camera)
		#endregion

		#region Show target
		/// <summary>
		/// Return angle between two vectors. Used for visbility testing.
		/// </summary>
		/// <param name="vec1">Vector 1</param>
		/// <param name="vec2">Vector 2</param>
		/// <returns>Float</returns>
		public float GetAngleBetweenVectors(Vector3 vec1, Vector3 vec2)
		{
			// See http://en.wikipedia.org/wiki/Vector_(spatial)
			// for help and check out the Dot Product section ^^
			// Both vectors are normalized so we can save deviding through the
			// lengths.
			return (float)Math.Acos(Vector3.Dot(vec1, vec2));
		} // Angle(vec1, vec2)

		/// <summary>
		/// Show arrow to target position on screen.
		/// Will show the target itself if it is on screen, else a arrow
		/// pointing to the nearest direction is shown at the screen border.
		/// Will also display the distance to the target below the arrow or target.
		/// </summary>
		/// <param name="levelLength">Level length for target position</param>
		private void ShowTarget(int levelLength)
		{
			// First find out where the target is.
			Vector3 targetPosition = GameAsteroidManager.TargetPosition;
			float targetDistance = (targetPosition - GameForm.CameraPos).Length();
			Point screenPos =
				GraphicForm.Convert3DPointTo2D(targetPosition);
			bool isVisible =
				GraphicForm.IsInFrontOfCamera(targetPosition);

			//smallFont.WriteText(0, 120, "screenPos=" + screenPos + "\n" +
			//	"isVisible=" + isVisible);

			if (isVisible &&
				screenPos.X >= 0 && screenPos.X < GraphicForm.Resolution.Width &&
				screenPos.Y >= 0 && screenPos.Y < GraphicForm.Resolution.Height)
			{
				// Don't show if in middle (background is already shown
				// and we introduce the problem of z buffer sorting with the
				// alpha of the asteroids). Only show if getting close to the borders.
				Vector2 distanceVector = new Vector2(
					screenPos.X - GameForm.Resolution.Width / 2,
					screenPos.Y - GameForm.Resolution.Height / 2);
				float borderFactor = distanceVector.Length() /
					(float)(GameForm.Resolution.Height / 2);
				if (borderFactor > 1.0f)
					borderFactor = 1.0f;
				inGameTexture.RenderOnScreen(new Rectangle(
					screenPos.X - TargetIconRect.Width / 2,
					screenPos.Y - TargetIconRect.Height / 2,
					TargetIconRect.Width, TargetIconRect.Height),
					TargetIconRect,
					Color.White, borderFactor);

				GameAsteroidManager.AddDistanceToBeDisplayed(screenPos, targetDistance,
					0.25f + 0.75f * (1.0f -
					(targetDistance / (levelLength * GameAsteroidManager.SectorDepth))),
					GameAsteroidManager.ProductLogo.None);
			} // if (isVisible)
			else
			{
				// We have to normalize the target position for angle calculations.
				targetPosition.Normalize();

				// Find out direction of the target by calculating 4 angels
				// for all 4 directions and then choosing the closest one!
				Vector3 leftRotation = new Vector3(-1, 0, 0);
				leftRotation.TransformNormal(GraphicForm.InverseViewMatrix);
				float leftAngle =
					GetAngleBetweenVectors(leftRotation, targetPosition);

				Vector3 rightRotation = new Vector3(+1, 0, 0);
				rightRotation.TransformNormal(GraphicForm.InverseViewMatrix);
				float rightAngle =
					GetAngleBetweenVectors(rightRotation, targetPosition);

				Vector3 upRotation = new Vector3(0, +1, 0);
				upRotation.TransformNormal(GraphicForm.InverseViewMatrix);
				float upAngle =
					GetAngleBetweenVectors(upRotation, targetPosition);

				Vector3 downRotation = new Vector3(0, -1, 0);
				downRotation.TransformNormal(GraphicForm.InverseViewMatrix);
				float downAngle =
					GetAngleBetweenVectors(downRotation, targetPosition);

				// Find out direction and screen coordinates on the screen borders.
				float xPosPercent = 0.5f + (leftAngle - (float)Math.PI / 2.0f) * 0.6f;
				float yPosPercent = 0.5f + (upAngle - (float)Math.PI / 2.0f) * 0.85f;
				int xPos = (int)(xPosPercent * GraphicForm.Resolution.Width) -
					ArrowLeftRect.Width / 2;
				int yPos = (int)(yPosPercent * GraphicForm.Resolution.Height) -
					ArrowLeftRect.Height / 2;

				// Make sure arrows are fully visible!
				if (xPos < 0)
					xPos = 0;
				if (xPos >= GraphicForm.Resolution.Width - ArrowLeftRect.Width)
					xPos = GraphicForm.Resolution.Width - ArrowLeftRect.Width - 1;
				if (yPos < 0)
					yPos = 0;
				if (yPos >= GraphicForm.Resolution.Height - ArrowLeftRect.Height)
					yPos = GraphicForm.Resolution.Height - ArrowLeftRect.Height - 1;

				// First try, just get the closest angel
				if (leftAngle < rightAngle &&
					leftAngle < upAngle &&
					leftAngle < downAngle)
				{
					// Left border
					inGameTexture.RenderOnScreen(new Rectangle(
						0, yPos, ArrowLeftRect.Width, ArrowLeftRect.Height),
						ArrowLeftRect);
					GameAsteroidManager.AddDistanceToBeDisplayed(new Point(
						ArrowLeftRect.Width - 7, yPos + ArrowLeftRect.Height / 2),
						targetDistance, 0.85f,
						GameAsteroidManager.ProductLogo.None);
				} // if (leftAngle)
				else if (rightAngle < upAngle &&
					rightAngle < downAngle)
				{
					// Right border
					inGameTexture.RenderOnScreen(new Rectangle(
						GraphicForm.Resolution.Width - ArrowRightRect.Width, yPos,
						ArrowRightRect.Width, ArrowRightRect.Height),
						ArrowRightRect);
					GameAsteroidManager.AddDistanceToBeDisplayed(new Point(
						GraphicForm.Resolution.Width - ArrowRightRect.Width,
						yPos + ArrowLeftRect.Height / 2),
						targetDistance, 0.85f,
						GameAsteroidManager.ProductLogo.None);
				} // else if
				else if (upAngle < downAngle)
				{
					// Top border
					inGameTexture.RenderOnScreen(new Rectangle(
						xPos, 0, ArrowUpRect.Width, ArrowUpRect.Height),
						ArrowUpRect);
					GameAsteroidManager.AddDistanceToBeDisplayed(new Point(
						xPos + ArrowRightRect.Width / 2, ArrowRightRect.Height / 2 - 4),
						targetDistance, 0.85f,
						GameAsteroidManager.ProductLogo.None);
				} // else if
				else
				{
					// Bottom border
					inGameTexture.RenderOnScreen(new Rectangle(
						xPos, GraphicForm.Resolution.Height - ArrowDownRect.Height,
						ArrowDownRect.Width, ArrowDownRect.Height),
						ArrowDownRect);

					// We can't add the text below the arrow because there is
					// no space left. We have to display it on top!
					GameAsteroidManager.AddDistanceToBeDisplayed(new Point(
						xPos + ArrowRightRect.Width / 2,
						GraphicForm.Resolution.Height - ArrowDownRect.Height * 2 + 5),
						targetDistance, 0.85f,
						GameAsteroidManager.ProductLogo.None);
				} // else
			} // else
		} // ShowTarget(levelLength)
		#endregion

		#region Show on screen effects
		/// <summary>
		/// Show on screen effects
		/// </summary>
		private void ShowOnScreenEffects(AnimatedTexture explosionTexture)
		{
			Size res = GameForm.Resolution;

			// Show explosion if we just died
			if (Player.explosionTimeoutMs > 0)
			{
				int screenSize = Math.Max(res.Width * 2 / 3, res.Height * 2 / 3);
				float explosionTimePercent = 1.0f -
					(Player.explosionTimeoutMs / (float)Player.MaxExplosionTimeoutMs);
				explosionTexture.Select(
					(int)(explosionTexture.AnimationLength * explosionTimePercent));
				explosionTexture.RenderOnScreen(new Rectangle(
					res.Width / 2 - screenSize / 2,
					res.Height / 2 - screenSize / 2,
					screenSize, screenSize));

				// Just rumble very heavy
				XInput.Rumble(0.8f, 0.9f);
			} // if

			if (Player.explosionTimeoutMs2 > 0 &&
				Player.explosionTimeoutMs2 < Player.MaxExplosionTimeoutMs)
			{
				int screenSize = Math.Max(res.Width * 2 / 3, res.Height * 2 / 3);
				float explosionTimePercent = 1.0f -
					(Player.explosionTimeoutMs2 / (float)Player.MaxExplosionTimeoutMs);
				explosionTexture.Select(
					(int)(explosionTexture.AnimationLength * explosionTimePercent));
				explosionTexture.RenderOnScreen(new Rectangle(
					res.Width / 2 - screenSize / 2 + res.Width / 7,
					res.Height / 2 - screenSize / 2 + res.Height / 12,
					screenSize, screenSize));
			} // if

			if (Player.explosionTimeoutMs3 > 0 &&
				Player.explosionTimeoutMs3 < Player.MaxExplosionTimeoutMs)
			{
				int screenSize = Math.Max(res.Width * 2 / 3, res.Height * 2 / 3);
				float explosionTimePercent = 1.0f -
					(Player.explosionTimeoutMs3 / (float)Player.MaxExplosionTimeoutMs);
				explosionTexture.Select(
					(int)(explosionTexture.AnimationLength * explosionTimePercent));
				explosionTexture.RenderOnScreen(new Rectangle(
					res.Width / 2 - screenSize / 2 + res.Width / 9,
					res.Height / 2 - screenSize / 2 - res.Height / 7,
					screenSize, screenSize));
			} // if

			const float BorderColorStrength = 0.6f;

			// Show blue glow border in middle if speed item is active
			if (Player.speedItemTimeout > 0)
			{
				float alpha = BorderColorStrength;
				if (Player.speedItemTimeout < 1000)
					alpha *= Player.speedItemTimeout / 1000.0f;

				hudTexture.RenderOnScreen(
					GameForm.ResolutionRect,
					ScreenBorderEffectRect,
					Level.SpeedItemColor, alpha);
			} // if
			else if (Player.itemMessageTimeoutMs >
				Player.MaxItemMessageTimeoutMs - 1000 &&
				Player.lastCollectedItem != Level.SpeedItemType &&
				Player.lastCollectedItem != Level.BombItemType)
			{
				float alpha = BorderColorStrength;
				int time = Player.itemMessageTimeoutMs -
					(Player.MaxItemMessageTimeoutMs - 1000);
				alpha *= time / 1000.0f;

				hudTexture.RenderOnScreen(
					GameForm.ResolutionRect,
					ScreenBorderEffectRect,
					Player.lastCollectedItem == Level.FuelItemType ?
					Level.FuelItemColor :
					Player.lastCollectedItem == Level.HealthItemType ?
					Level.HealthItemColor :
					Level.ExtraLifeItemColor, alpha);
			} // if
			// Show red glow border in middle if bomb item is active
			else if (Player.numberOfBombItems > 0)
			{
				hudTexture.RenderOnScreen(
					GameForm.ResolutionRect,
					ScreenBorderEffectRect,
					Level.BombItemColor,
					// Increase border strength the more bombs we have collected.
					Math.Min(1.0f, Player.numberOfBombItems * 0.3f));
			} // if
		} // ShowOnScreenEffects()
		#endregion

		#region Show minimap
		#region Rotation helpers
		/// <summary>
		/// Get camera rotation y coordinate
		/// </summary>
		/// <returns>Float</returns>
		private float GetCameraRotationY()
		{
			Vector3 cameraRotation = new Vector3(0, 1, 0);
			cameraRotation.TransformNormal(GraphicForm.InverseViewMatrix);
			return (float)Math.PI / 2.0f +
				(float)Math.Atan2(cameraRotation.Y, cameraRotation.X);
		} // GetCameraRotationY()

		/// <summary>
		/// Get camera rotation z
		/// </summary>
		/// <returns>Float</returns>
		private float GetCameraRotationZ()
		{
			Vector3 cameraRotation = new Vector3(0, 0, 1);
			cameraRotation.TransformNormal(GraphicForm.InverseViewMatrix);
			return (float)Math.PI -
				(float)Math.Atan2(cameraRotation.Z, cameraRotation.X);
		} // GetCameraRotationZ()

		/// <summary>
		/// Get rotated point. Position is rotated around center by rotation.
		/// </summary>
		/// <param name="pos">Position</param>
		/// <param name="rotation">Rotation</param>
		/// <returns>Point</returns>
		private Point GetRotatedPoint(Point offset, Point pos, float rotation)
		{
			// See Texture.RenderOnScreenWithRotation for more help.
			float sinRotation = (float)Math.Sin(rotation);
			float cosRotation = (float)Math.Cos(rotation);
			return new Point(
				offset.X + (int)(-cosRotation * pos.X - sinRotation * pos.Y),
				offset.Y + (int)(-sinRotation * pos.X + cosRotation * pos.Y));
		} // GetRotatedPoint(pos, rotation)
		#endregion

		/// <summary>
		/// Show minimap
		/// </summary>
		private void ShowMinimap(Point pos)
		{
			// Init helpers
			Size res = GraphicForm.Resolution;
			Vector3 camPos = GraphicForm.CameraPos;

			// Use a little bigger height than given in the texture
			int height = MiniMapBackgroundRect.Height * 6 / 5;
			Rectangle bgRect = new Rectangle(pos.X-1, pos.Y-1,
				MiniMapBackgroundRect.Width, height);

			// Render background
			inGameTexture.RenderOnScreen(bgRect, MiniMapBackgroundRect);

			// Show minimap on top of that
			Rectangle minimapRect = new Rectangle(
				bgRect.X + 3, bgRect.Y + 3, bgRect.Width - 8, height - 10);
			int minimapDisplayHeight = minimapRect.Height / 2;
			int minimapPos = levelTexture.TextureSize.Height -
				(int)(camPos.Z / GameAsteroidManager.SectorDepth) -
				minimapDisplayHeight * 3 / 4;
			if (minimapPos < 0)
				minimapPos = 0;
			if (minimapPos > levelTexture.TextureSize.Height -
				minimapDisplayHeight - 1)
				minimapPos = levelTexture.TextureSize.Height -
					minimapDisplayHeight - 1;

			// And render
			levelTexture.RenderOnScreen(minimapRect,
				new Rectangle(0, minimapPos, 40, minimapDisplayHeight));

			// Show rocket on top of map
			Point rocketPos = new Point(
				minimapRect.X + minimapRect.Width / 2 +
				(int)(4 * camPos.X / GameAsteroidManager.SectorWidth),
				minimapRect.Bottom -
				(int)(camPos.Z / GameAsteroidManager.SectorDepth) +
				((levelTexture.TextureSize.Height - minimapDisplayHeight) -
				minimapPos - MiniMapRocketRect.Height * 2 / 3));

			// Limit rocket pos to somewhere on the minimap
			if (rocketPos.X < minimapRect.X)
				rocketPos.X = minimapRect.X;
			if (rocketPos.Y < minimapRect.Y)
				rocketPos.Y = minimapRect.Y;
			if (rocketPos.X > minimapRect.Right)
				rocketPos.X = minimapRect.Right;
			if (rocketPos.Y > minimapRect.Bottom)
				rocketPos.Y = minimapRect.Bottom;

			// Get rotation for our rocket
			float rocketRotation = GetCameraRotationZ();

			// Render field of view
			hudTexture.RenderOnScreenWithRotation(
				rocketPos, MiniMapFieldOfViewRect.Width / 2, rocketRotation,
				MiniMapFieldOfViewRect);

			// And finally render
			inGameTexture.RenderOnScreenWithRotation(
				rocketPos, MiniMapRocketRect.Width / 3, rocketRotation,
				MiniMapRocketRect);
		} // ShowMinimap()
		#endregion

		#region Show hud controls
		float visionOccluded = 0.0f;

		/// <summary>
		/// Show hud controls
		/// </summary>
		private void ShowHudControls()
		{
			// Calculate hud areas
			int hudWidth = (int)(HudRect.Width * HudBackgroundScaleFactor);
			int hudHeight = (int)(HudRect.Height * HudBackgroundScaleFactor);
			Rectangle leftHud = new Rectangle(
				HudDistanceFromBorder,
				GraphicForm.Resolution.Height - (hudHeight + HudDistanceFromBorder),
				hudWidth, hudHeight);
			Rectangle rightHud = new Rectangle(
				GraphicForm.Resolution.Width - (hudWidth + HudDistanceFromBorder),
				leftHud.Y,
				hudWidth, hudHeight);

			// First show hud background
			hudTexture.RenderOnScreen(leftHud, HudRect);

			// And mirror hud on the right side!
			hudTexture.RenderOnScreen(rightHud, new Rectangle(
				HudRect.Right, HudRect.Y, -HudRect.Width, HudRect.Height));

			// Fuel, health and speed
			int fuelPixels =
				(int)(FuelHealthAndSpeedWidth * (1.0f - Player.fuel));
			inGameTexture.RenderOnScreen(
				new Rectangle(leftHud.X + 20 + fuelPixels, leftHud.Y + 28,
				FuelBarRect.Width - fuelPixels, FuelBarRect.Height),
				new Rectangle(FuelBarRect.X + fuelPixels, FuelBarRect.Y,
				FuelBarRect.Width - fuelPixels, FuelBarRect.Height));

			int healthPixels =
				(int)(FuelHealthAndSpeedWidth * (1.0f - Player.health));
			inGameTexture.RenderOnScreen(
				new Rectangle(leftHud.X + 20 + healthPixels,
				leftHud.Y + 28 + FuelBarRect.Height - 1,
				HealthBarRect.Width - healthPixels, HealthBarRect.Height),
				new Rectangle(HealthBarRect.X + healthPixels, HealthBarRect.Y,
				HealthBarRect.Width - healthPixels, HealthBarRect.Height));

			int speedPixels =
				(int)(FuelHealthAndSpeedWidth * (1.0f - Player.Speed));
			inGameTexture.RenderOnScreen(
				new Rectangle(leftHud.X + 20 + speedPixels,
				leftHud.Y + 28 + FuelBarRect.Height + HealthBarRect.Height - 2,
				SpeedBarRect.Width - speedPixels, SpeedBarRect.Height),
				new Rectangle(SpeedBarRect.X + speedPixels, SpeedBarRect.Y,
				SpeedBarRect.Width - speedPixels, SpeedBarRect.Height));

			// Show minimap on right hud
			ShowMinimap(new Point(rightHud.X + 18, rightHud.Y + 32));

			// Show number of lifes we have left
			for (int i = 0; i < Player.lifes; i++)
			{
				inGameTexture.RenderOnScreen(new Rectangle(
					(GraphicForm.Resolution.Width - 6) -
					(2 + (RocketLifeRect.Width - 2) * (i + 1)),
					2, RocketLifeRect.Width, RocketLifeRect.Height),
					RocketLifeRect);
			} // for

			// Show hud only if in game and alive.
			if (Player.GameOver == false)
			{
				// Show on screen altitude meter and balance
				float hudRotation = GetCameraRotationY();
				Point middle = new Point(
					GraphicForm.Resolution.Width / 2,
					GraphicForm.Resolution.Height / 2);
				int hudSize = GraphicForm.Resolution.Width / 8;
				int hudLineDistance = GraphicForm.Resolution.Width / 40;
				float hudAltitude =
					GraphicForm.CameraPos.Y / GameAsteroidManager.SectorDepth;
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = false;

				// Limit hud altitude to -20 up to +20
				if (hudAltitude > 20)
					hudAltitude = 20;
				if (hudAltitude < -20)
					hudAltitude = -20;

				// Render all hud lines
				for (int num = -25; num <= +25; num++)
				{
					int yPos = (int)((num + hudAltitude) * hudLineDistance);

					// Only show inner -5 to +5 hud lines
					if (yPos >= -5 * hudLineDistance &&
						yPos <= +5 * hudLineDistance)
					{
						// Use big line for center, medium for -5 and +5
						int innerXPos =
							num == 0 ? hudSize * 5 / 9 ://hudSize * 16 / 32 :
							num == 5 || num == -5 ? hudSize * 25 / 32 :
							num == 10 || num == -10 ? hudSize * 27 / 32 :
							num % 5 == 0 ? hudSize * 29 / 32 :
							hudSize * 30 / 32;

						// Change hud color to red if altitude is more than 5 units
						// away from center line.
						float awayFactor = 0.0f;
						if (Math.Abs(num) > 5)
						{
							awayFactor = (float)(Math.Abs(num) - 5) / 10.0f;
							if (awayFactor > 1.0f)
								awayFactor = 1.0f;
						} // if (Math.Abs)
						Color col = ColorHelper.InterpolateColor(
							HudColor, HudRedColor, awayFactor);

						// Left part
						GameForm.LineManager.AddLineWithShadow(
							new Point(middle.X - hudSize, middle.Y + yPos),
							new Point(middle.X - innerXPos, middle.Y + yPos),
							col);
						// Right part
						GameForm.LineManager.AddLineWithShadow(
							new Point(middle.X + hudSize, middle.Y + yPos),
							new Point(middle.X + innerXPos, middle.Y + yPos),
							col);

						// Show height at markers
						if (num % 5 == 0)
						{
							// Left text
							smallFont.WriteSmallTextCentered(
								middle.X - hudSize - 15, middle.Y + yPos - 2,
								(-num).ToString(),
								col, 0.75f);
							// Right text
							smallFont.WriteSmallTextCentered(
								middle.X + hudSize + 10, middle.Y + yPos - 2,
								(-num).ToString(),
								col, 0.75f);
						} // if (num)
					} // if (yPos)
				} // for (num)

				// Do collision test with occulusion culling like in lens flare class.
				float thisVisionOccluded =
					GraphicForm.OcclusionIntensity(hudTexture, middle, 12);

				// Interpolate for a nicer transission effect.
				visionOccluded = thisVisionOccluded * 0.2f + visionOccluded * 0.8f;
				Color crosshairColor = ColorHelper.InterpolateColor(
					HudRedColor, HudColor, visionOccluded);

				// Render crosshair, rotate by camera rotation y
				GameForm.LineManager.AddLineWithShadow(
					GetRotatedPoint(middle, new Point(-hudSize * 4 / 9, 0), hudRotation),
					GetRotatedPoint(middle, new Point(-hudSize / 12, 0), hudRotation),
					crosshairColor);
				GameForm.LineManager.AddLineWithShadow(
					GetRotatedPoint(middle, new Point(+hudSize * 4 / 9, 0), hudRotation),
					GetRotatedPoint(middle, new Point(+hudSize / 12, 0), hudRotation),
					crosshairColor);
				GameForm.LineManager.AddLineWithShadow(
					GetRotatedPoint(middle, new Point(0, -hudSize / 5), hudRotation),
					GetRotatedPoint(middle, new Point(0, -hudSize / 12), hudRotation),
					crosshairColor);
				GameForm.LineManager.AddLineWithShadow(
					GetRotatedPoint(middle, new Point(0, +hudSize / 7), hudRotation),
					GetRotatedPoint(middle, new Point(0, +hudSize / 12), hudRotation),
					crosshairColor);
			} // if

			// Show level position, altitude, score+rank and game time.
			smallFont.WriteText(rightHud.X + 116, rightHud.Y + 80,
				Texts.LevelPos, GameTextColor);
			smallFont.WriteText(rightHud.X + 116 + 74, rightHud.Y + 80,
				// Min. 0, max. 100, get level position.
				Math.Min(100, Math.Max(0, (int)(101 * GraphicForm.CameraPos.Z /
				(levelTexture.TextureSize.Height * GameAsteroidManager.SectorDepth)))) +
				" %", GameTextColor);

			smallFont.WriteText(rightHud.X + 116, rightHud.Y + 100,
				Texts.Altitude, GameTextColor);
			smallFont.WriteText(rightHud.X + 116 + 74, rightHud.Y + 100,
				((int)GraphicForm.CameraPos.Y /
				GameAsteroidManager.SectorDepth).ToString(), GameTextColor);

			smallFont.WriteText(rightHud.X + 116, rightHud.Y + 100 + 20,
				Texts.Score, GameTextColor);
			smallFont.WriteText(rightHud.X + 116 + 74, rightHud.Y + 100 + 20,
				Player.score.ToString() + " " +
				"(#" + (Highscores.GetRankFromCurrentScore(Player.score) + 1) + ")",
				GameTextColor);

			smallFont.WriteText(rightHud.X + 116, rightHud.Y + 100 + 20 + 20,
				Texts.Time, GameTextColor);
			smallFont.WriteText(rightHud.X + 116 + 74, rightHud.Y + 100 + 20 + 20,
				((Player.gameTimeMs / 1000) / 60).ToString("00") + ":" +
				((Player.gameTimeMs / 1000) % 60).ToString("00"), GameTextColor);
		} // ShowHudControls()
		#endregion

		#region Show screen messages
		/// <summary>
		/// Show screen messages
		/// </summary>
		private void ShowScreenMessages(GameAsteroidManager asteroidManager)
		{
			Size res = GraphicForm.Resolution;
			Vector3 pos = GraphicForm.CameraPos;
			Point messagePos = new Point(res.Width / 2, res.Height / 5);

			// If game is over, show end screen and wait until user clicks
			// or presses space.
			if (Player.GameOver)
			{
				bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
					Player.victory ? Texts.Victory : Texts.GameOver);
				bigFont.WriteTextCentered(messagePos.X, messagePos.Y +
					bigFont.Height * 4 / 3,
					Texts.YourHighscore + " " + Player.score + " " +
					"(#" + (Highscores.GetRankFromCurrentScore(Player.score) + 1) + ")");
			} // if

			// If we just lost a life, display message and let user continue
			// by pressing the left mouse button.
			else if (Player.explosionTimeoutMs >= 0)
			{
				bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
					Texts.YouLostALife);
				bigFont.WriteTextCentered(messagePos.X, messagePos.Y +
					bigFont.Height * 4 / 3,
					Texts.ClickToContinue);

				if (GameForm.Mouse.LeftButtonPressed ||
					XInput.AButton ||
					XInput.BButton ||
					XInput.XButton ||
					XInput.YButton ||
					XInput.StartButton)
					Player.explosionTimeoutMs = -1;
			} // if

			// Show countdown if a new life started
			else if (Player.lifeTimeMs < Player.LifeTimeZoomAndAccelerateMs)
			{
				float alpha = 1.0f - ((Player.lifeTimeMs % 1000) / 1000.0f);
				alpha *= 2.0f;
				if (alpha > 1.0f)
					alpha = 1.0f;

				// Show "Get ready" for 3 seconds with countdown,
				// then display "Go!".
				if (Player.lifeTimeMs < 3000)
				{
					bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
						Texts.GetReady);
					bigFont.WriteTextCentered(messagePos.X, messagePos.Y +
						bigFont.Height * 4 / 3,
						(3 - (Player.lifeTimeMs / 1000)).ToString(),
						Color.White, alpha);
				} // if
				else
					bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
						Texts.Go, Color.White, alpha);
			} // if

			// Show item helper messages
			else if (Player.itemMessageTimeoutMs > 0 &&
				String.IsNullOrEmpty(Player.currentItemMessage) == false)
			{
				Player.itemMessageTimeoutMs -= ElapsedTime.MsLastFrame;
				if (Player.itemMessageTimeoutMs < 0)
					Player.itemMessageTimeoutMs = 0;

				float alpha = 1.0f;
				if (Player.itemMessageTimeoutMs < 1000)
					alpha = Player.itemMessageTimeoutMs / 1000.0f;

				bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
					Player.currentItemMessage, Color.LightGray, alpha);
			} // if

			// Warning if nearly dead (only if that just happened)
			else if (Player.showHealthWarningTimeoutMs > 0)
			{
				float alpha = 1.0f;
				if (Player.showHealthWarningTimeoutMs < 1000)
					alpha = Player.showHealthWarningTimeoutMs / 1000.0f;

				bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
					Texts.WarningLowHealth, Color.Red, 1.0f);
			} // if

			// Warning if going out of fuel (less than 10%)
			else if (Player.fuel < 0.1f)
			{
				bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
					Texts.WarningOutOfFuel, Color.Red, 1.0f);
			} // if

			// Warning if out of playable area!
			else if (pos.X / GameAsteroidManager.SectorWidth <
				-asteroidManager.CurrentLevel.Width / 2 ||
				pos.X / GameAsteroidManager.SectorWidth >
				+asteroidManager.CurrentLevel.Width / 2 ||
				pos.Y / GameAsteroidManager.SectorHeight <
				-asteroidManager.CurrentLevel.Width / 2 ||
				pos.Y / GameAsteroidManager.SectorHeight >
				+asteroidManager.CurrentLevel.Width / 2 ||
				pos.Z < 0)
			{
				bigFont.WriteTextCentered(messagePos.X, messagePos.Y,
					Texts.WarningOutOfLevel, Color.Red, 1.0f);
			} // if
		} // ShowScreenMessages()
		#endregion

		#region Run
		/// <summary>
		/// Run game screen. Called each frame.
		/// </summary>
		/// <param name="form">Form for access to asteroid manager and co</param>
		public void Run(RocketCommanderForm form)
		{
			// Make sure the textures are linked correctly
			hudTexture = form.hudTexture;
			inGameTexture = form.inGameTexture;
			levelTexture = form.asteroidManager.CurrentLevel.Texture;

			// Enable alpha blending for all controls
			GraphicForm.EnableAlphaBlending();

			// Zoom into rocket for countdown time
			ZoomIntoRocket(form.camera, form.rocketModel);

			// Show target in z direction, it is very far away.
			ShowTarget(form.asteroidManager.CurrentLevel.Length);

			// Show on screen effects like explosion or item screen border colors.
			ShowOnScreenEffects(form.explosionTexture);

			// Always show hit direction effect if it is active (similar to screen
			// effects always on top).
			form.asteroidManager.ShowHitDirectionEffect();

			// Show all hud controls
			// Don't display them in low resolution, we can't display any useful
			// information anyway.
			if (GraphicForm.Resolution.Width >= 640)
				ShowHudControls();

			// Handle game stuff
			Player.HandleGameLogic(form.asteroidManager);

			// Show on screen helper messages
			ShowScreenMessages(form.asteroidManager);

			// End game if escape was pressed or game is over and space or mouse
			// button was pressed.
			if (GameForm.Keyboard.EscapeJustPressed ||
				XInput.BackButton ||
				(Player.GameOver &&
				(GameForm.Keyboard.State[Key.Space] ||
				XInput.AButton ||
				XInput.BButton ||
				XInput.XButton ||
				XInput.YButton ||
				GameForm.Mouse.LeftButtonPressed)))
			{
				// Upload new highscore (as we currently are in game,
				// no bonus or anything will be added, this score is low!)
				Player.SetGameOverAndUploadHighscore();

				// Stop rocket sound if it still playing
				form.asteroidManager.StopRocketMotorSound();

				// Reset camera to origin and notify we are not longer in game mode
				form.camera.SetPosition(Vector3.Empty);
				form.camera.InGame = false;

				// Quit to the main menu
				quit = true;
			} // if
		} // Run(form)
		#endregion
	} // class Mission
} // namespace RocketCommander.GameScreens
