// Project: Rocket Commander, File: Options.cs
// Namespace: RocketCommander.GameScreens, Class: Options
// Path: C:\code\Rocket Commander\GameScreens, Author: Abi
// Code lines: 361, Size of file: 12,36 KB
// Creation date: 24.12.2005 05:01
// Last modified: 24.12.2005 06:50
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using RocketCommander.Graphics;
using Font = RocketCommander.Graphics.Font;
using System.Drawing;
using RocketCommander.Game;
using RocketCommander.Properties;
using RocketCommander.Helpers;
using System.Windows.Forms;
using System.Globalization;
#endregion

namespace RocketCommander.GameScreens
{
	/// <summary>
	/// Options
	/// </summary>
	class Options : IGameScreen
	{
		#region Variables
		/// <summary>
		/// Current player name, copied from the settings file.
		/// Can be changed in this screen and will be saved to the settings file.
		/// </summary>
		public static string currentPlayerName;

		/// <summary>
		/// Fonts for options
		/// </summary>
		private Font smallFont = null,
			mediumFont = null,
			bigFont = null;
		#endregion

		#region Properties
		/// <summary>
		/// Name of this game screen
		/// </summary>
		/// <returns>String</returns>
		public string Name
		{
			get
			{
				return "Options";
			} // get
		} // Name

		/// <summary>
		/// Quit
		/// </summary>
		private bool quit = false;
		/// <summary>
		/// Returns true if we want to quit this screen and return to the
		/// previous screen. If no more screens are left the game is exited.
		/// </summary>
		/// <returns>Bool</returns>
		public bool Quit
		{
			get
			{
				return quit;
			} // get
		} // Quit
		#endregion

		#region Static constructor
		/// <summary>
		/// Create options class, will just load the player name.
		/// </summary>
		static Options()
		{
			// Load player name
			currentPlayerName = Settings.Default.PlayerName;

			// Get player name from windows computer/user name if not found yet.
			if (String.IsNullOrEmpty(currentPlayerName))
				currentPlayerName = WindowsHelper.GetDefaultPlayerName();
		} // Options()
		#endregion

		#region Constructor
		/// <summary>
		/// Create options
		/// </summary>
		public Options()
		{
			smallFont = new Font("Arial", 12);
			mediumFont = new Font("Verdana", 18);
			bigFont = new Font("Verdana", 28);

			// Clear any pressed keys
			GameForm.lastPressedKeys = "";
			GameForm.deletePressedKeys = 0;
		} // Options()
		#endregion

		#region Run
		/// <summary>
		/// Is music on
		/// </summary>
		bool isMusicOn = Settings.Default.MusicOn;
		/// <summary>
		/// Is vsync on
		/// </summary>
		bool isVsyncOn = Settings.Default.WaitForVSync;
		/// <summary>
		/// Resolution width
		/// </summary>
		int resolutionWidth = Settings.Default.ResolutionWidth;
    /// <summary>
    /// Resolution height
    /// </summary>
    int resolutionHeight = Settings.Default.ResolutionHeight;

		/// <summary>
		/// Run game screen. Called each frame.
		/// </summary>
		/// <param name="form">Form for access to asteroid manager and co</param>
		public void Run(RocketCommanderForm form)
		{
			// Render background
			form.RenderMenuBackground();

			// Write player name
			int xPos = 100 * GraphicForm.Resolution.Width / 1024,
				yPos = 180 * GraphicForm.Resolution.Height / 768;
			bigFont.WriteText(xPos, yPos,
				Texts.Options);

			// Edit player name
			yPos = 250 * GraphicForm.Resolution.Height / 768;
			mediumFont.WriteText(xPos, yPos,
				Texts.PlayerName + ": " + currentPlayerName +
				((ElapsedTime.MsTotal / 200) % 2 == 0 ? "|" : ""));

			// Allow editing with help of GameForm key press event
			if (GameForm.lastPressedKeys.Length > 0)
			{
				// , and : are not allowed for the player name!
				StringHelper.RemoveCharacter(ref GameForm.lastPressedKeys, ',');
				StringHelper.RemoveCharacter(ref GameForm.lastPressedKeys, ':');

				currentPlayerName += GameForm.lastPressedKeys;
				Settings.Default.PlayerName = currentPlayerName;
				GameForm.lastPressedKeys = "";
			} // if (GameForm.lastPressedKeys.Length)
			if (GameForm.deletePressedKeys > 0)
			{
				if (currentPlayerName.Length > 0)
				{
					int deleteChars =
						Math.Min(GameForm.deletePressedKeys, currentPlayerName.Length);
					currentPlayerName = currentPlayerName.Substring(0,
						currentPlayerName.Length - deleteChars);
					Settings.Default.PlayerName = currentPlayerName;
				} // if (currentPlayerName.Length)
				GameForm.deletePressedKeys = 0;
			} // if (GameForm.deletePressedKeys)

			// Show all available languages and allow changing
			yPos = 300 * GraphicForm.Resolution.Height / 768;
			mediumFont.WriteText(xPos, yPos, Texts.Language + ": ");

			mediumFont.WriteText(xPos + 150, yPos, Texts.English,
				Texts.Culture == CultureInfo.InvariantCulture ?
				Color.Red : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				Texts.Culture != CultureInfo.InvariantCulture &&
				form.MouseInBox(new Rectangle(xPos + 150, yPos, 100, 30)))
			{
				Texts.Culture = Application.CurrentCulture =
					CultureInfo.InvariantCulture;
				Settings.Default.Language = "Invariant";
			} // if (GameForm.Mouse.LeftButtonPressed)

			mediumFont.WriteText(xPos + 300, yPos, Texts.German,
				Texts.Culture.Name == "de-DE" ? Color.Red : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				Texts.Culture.Name != "de-DE" &&
				form.MouseInBox(new Rectangle(xPos + 300, yPos, 100, 30)))
			{
				Texts.Culture = Application.CurrentCulture =
					new CultureInfo("de-DE");
				Settings.Default.Language = Texts.Culture.Name;
			} // if (GameForm.Mouse.LeftButtonPressed)
			//TODO: more languages

			// Allow to toggle music on/off
			yPos = 350 * GraphicForm.Resolution.Height / 768;
			bool overMusicOnOff = form.MouseInBox(new Rectangle(
				xPos, yPos, 200, 30));
			mediumFont.WriteText(
				xPos, yPos,
				Texts.Music+":",
				overMusicOnOff ? Color.Red : Color.LightGray);
			mediumFont.WriteText(
				xPos + 150, yPos,
				isMusicOn ? Texts.On : Texts.Off,
				overMusicOnOff ? Color.Red : Color.LightGray);
			if (overMusicOnOff &&
				GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false)
			{
				form.ToggleMusicOnOff();
				isMusicOn = Settings.Default.MusicOn;
			} // if (overMusicOnOff)

			// Allow changing the graphic quality
			yPos = 400 * GraphicForm.Resolution.Height / 768;
			mediumFont.WriteText(xPos, yPos, Texts.Performance + ":");

			smallFont.WriteText(xPos, yPos+32, Texts.PerformanceFastest,
				GraphicForm.GraphicsPerformanceSetting ==
				GraphicForm.PerformanceSetting.Fastest ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos, yPos + 32, 250, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.MouseInBox(new Rectangle(xPos, yPos + 32, 250, 30)))
			{
				GraphicForm.GraphicsPerformanceSetting =
					GraphicForm.PerformanceSetting.Fastest;
			} // if (GameForm.Mouse.LeftButtonPressed)

			smallFont.WriteText(xPos + 250, yPos + 32, Texts.PerformanceMedium,
				GraphicForm.GraphicsPerformanceSetting ==
				GraphicForm.PerformanceSetting.Medium ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos + 250, yPos + 32, 300, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 250, yPos + 32, 300, 30)))
			{
				GraphicForm.GraphicsPerformanceSetting =
					GraphicForm.PerformanceSetting.Medium;
			} // if (GameForm.Mouse.LeftButtonPressed)

			smallFont.WriteText(xPos + 550, yPos + 32, Texts.PerformanceQuality,
				GraphicForm.GraphicsPerformanceSetting ==
				GraphicForm.PerformanceSetting.Quality ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos + 550, yPos + 32, 300, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 550, yPos + 32, 300, 30)))
			{
				GraphicForm.GraphicsPerformanceSetting =
					GraphicForm.PerformanceSetting.Quality;
			} // if (GameForm.Mouse.LeftButtonPressed)

			// Allow to select from some of the supported resolutions
			yPos = 500 * GraphicForm.Resolution.Height / 768;
			mediumFont.WriteText(xPos, yPos, Texts.ChangeResolution + ":");

			smallFont.WriteText(xPos, yPos + 32, "Autodetect Best",
				resolutionWidth == 0 &&
				resolutionHeight == 0 ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 0;
				Settings.Default.ResolutionHeight = resolutionHeight = 0;
			} // if (GameForm.Mouse.LeftButtonPressed)

			/*obs
			smallFont.WriteText(xPos, yPos + 32, "320x240",
				resolutionWidth == 320 &&
				resolutionHeight == 240 ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 320;
				Settings.Default.ResolutionHeight = resolutionHeight = 240;
			} // if (GameForm.Mouse.LeftButtonPressed)
			 */

			smallFont.WriteText(xPos+150, yPos + 32, "640x480",
				resolutionWidth == 640 &&
				resolutionHeight == 480 ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos + 150, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 150, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 640;
				Settings.Default.ResolutionHeight = resolutionHeight = 480;
			} // if (GameForm.Mouse.LeftButtonPressed)

			smallFont.WriteText(xPos + 300, yPos + 32, "800x600",
				resolutionWidth == 800 &&
				resolutionHeight == 600 ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos + 300, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 300, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 800;
				Settings.Default.ResolutionHeight = resolutionHeight = 600;
			} // if (GameForm.Mouse.LeftButtonPressed)

			smallFont.WriteText(xPos + 450, yPos + 32, "1024x768",
				resolutionWidth == 1024 &&
				resolutionHeight == 768 ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos + 450, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 450, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 1024;
				Settings.Default.ResolutionHeight = resolutionHeight = 768;
			} // if (GameForm.Mouse.LeftButtonPressed)

			smallFont.WriteText(xPos + 600, yPos + 32, "1280x1024",
				resolutionWidth == 1280 &&
				resolutionHeight == 1024 ?
				Color.Red :
				form.MouseInBox(new Rectangle(xPos + 600, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 600, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 1280;
				Settings.Default.ResolutionHeight = resolutionHeight = 1024;
			} // if (GameForm.Mouse.LeftButtonPressed)

			smallFont.WriteText(xPos + 750, yPos + 32, "1600x1200",
				resolutionWidth == 1600 &&
				resolutionHeight == 1200 ? Color.Red :
				form.MouseInBox(new Rectangle(xPos + 750, yPos + 32, 150, 30)) ?
				Color.LightSalmon : Color.White);
			if (GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false &&
				form.MouseInBox(new Rectangle(xPos + 750, yPos + 32, 150, 30)))
			{
				Settings.Default.ResolutionWidth = resolutionWidth = 1600;
				Settings.Default.ResolutionHeight = resolutionHeight = 1200;
			} // if (GameForm.Mouse.LeftButtonPressed)

			// Allow to toggle vsync on/off
			yPos += 32 + 16 + 5;
			bool overVsyncOnOff = form.MouseInBox(new Rectangle(
				xPos, yPos, 200, 20));
			smallFont.WriteText(
				xPos, yPos,
				"VSync" + ":",
				overVsyncOnOff ? Color.Red : Color.LightGray);
			smallFont.WriteText(
				xPos + 150, yPos,
				isVsyncOn ? Texts.On : Texts.Off,
				overVsyncOnOff ? Color.Red : Color.LightGray);
			if (overVsyncOnOff &&
				GameForm.Mouse.LeftButtonPressed &&
				form.wasMouseButtonPressedLastFrame == false)
			{
				// Toggle state and set to both settings and local variable
				isVsyncOn = Settings.Default.WaitForVSync = !isVsyncOn;
			} // if (overVsyncOnOff)

			if (form.RenderMenuButton(MenuButton.Back,
				new Point(1024 - 210, 768 - 150)) ||
				GameForm.Keyboard.EscapeJustPressed)
			{
				// Save options (e.g. player name or language) in case they were
				// changed.
				Settings.Default.Save();

				quit = true;
			} // if (form.RenderMenuButton)
		} // Run(form)
		#endregion
	} // class Options
} // namespace RocketCommander.GameScreens
