// Project: Rocket Commander, File: AnimatedModel.cs
// Namespace: RocketCommander.Graphics, Class: AnimatedModel
// Creation date: 28.11.2005 20:30
// Last modified: 07.12.2005 06:30
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections;
using System.Text;
using System.IO;
using System.Runtime.Serialization;
using System.Drawing;
using System.Threading;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Direct3D = Microsoft.DirectX.Direct3D;
using Material = RocketCommander.Graphics.Material;
using RocketCommander.Helpers;
using RocketCommander.Game;
using RocketCommander.Shaders;
#endregion

namespace RocketCommander.Graphics
{
	/// <summary>
	/// Animated model, derived from static model class.
	/// Animated means we can have animations and are using a tree structure
	/// for all frames and meshes. Bones are not supported yet, but if they
	/// would be implemented we could also do more advanced animations.
	/// X files do support skinning, see SkinnedAnimation sample of MDX.
	/// </summary>
	internal class AnimatedModel : Model
	{
		#region Variables
		/// <summary>
		/// Container for all frames
		/// </summary>
		private AnimationRootFrame rootFrame;
		#endregion

		#region Properties
		/// <summary>
		/// Checks if model is valid to render, if this is false,
		/// nothing will be rendered, check Error for detailed error description.
		/// </summary>
		public override bool Valid
		{
			get
			{
				return rootFrame.FrameHierarchy != null ||
					this.D3DMesh != null;
			} // get
		} // Valid

		/// <summary>
		/// Get first material if there is any (there should be)
		/// </summary>
		public override Material FirstMaterial
		{
			get
			{
				if (rootFrame.FrameHierarchy == null)
				{
					Log.Write("Can't get first material, frame is not loaded!");
					return null;
				} // if (rootFrame.FrameHierarchy)

				Frame firstFrame = rootFrame.FrameHierarchy;
				if (firstFrame != null)
				{
					CustomMeshContainer meshes =
						firstFrame.MeshContainer as CustomMeshContainer;
					while (meshes == null &&
						firstFrame.FrameFirstChild != null)
					{
						meshes = firstFrame.FrameFirstChild.MeshContainer as
							CustomMeshContainer;
						firstFrame = firstFrame.FrameFirstChild;
					} // while

					if (meshes != null &&
						meshes.Materials != null &&
						meshes.Materials.Length > 0)
					{
						return meshes.Materials[0];
					} // if (meshes)
				} // if (firstFrame)
				return null;
			} // get
		} // FirstMaterial

		/// <summary>
		/// First frame
		/// </summary>
		/// <returns>Custom frame</returns>
		public Frame FirstFrame
		{
			get
			{
				return rootFrame.FrameHierarchy;
			} // get
		} // FirstFrame

		/// <summary>
		/// Get first custom mesh container of root frame.
		/// </summary>
		/// <returns>Custom mesh container</returns>
		public CustomMeshContainer FirstCustomMeshContainer
		{
			get
			{
				return FirstFrame.MeshContainer as CustomMeshContainer;
			} // get
		} // FirstCustomMeshContainer
		#endregion

		#region CustomFrame
		/// <summary>
		/// Custom frame class for animation.
		/// </summary>
		public class CustomFrame : Frame
		{
			public CustomFrame()
				: base()
			{
			} // CustomFrame()

			public CustomFrame(string setName) : base()
			{
				this.Name = setName;
			} // CustomFrame(name)
		} // class CustomFrame
		#endregion

		#region CustomAllocateHierarchy
		/// <summary>
		/// Custom allocate hierarchy
		/// </summary>
		public class CustomAllocateHierarchy : AllocateHierarchy
		{
			/// <summary>
			/// Create custom allocate hierarchy
			/// </summary>
			public CustomAllocateHierarchy()
				: base()
			{
			} // CustomAllocateHierarchy()

			/// <summary>
			/// Create frame
			/// </summary>
			public override Frame CreateFrame(string name)
			{
				return new CustomFrame(name);
			} // CreateFrame(name)

			/// <summary>
			/// Create mesh container
			/// </summary>
			public override MeshContainer CreateMeshContainer(
				string name,
				MeshData meshData,
				ExtendedMaterial[] materials,
				EffectInstance[] effectInstances,
				GraphicsStream adjacency,
				SkinInformation skinInfo)
			{
				return new CustomMeshContainer(
					name, meshData, materials,
					effectInstances, adjacency, skinInfo);
			} // CreateMeshContainer(name, meshData, materials)
		} // class CustomAllocateHierarchy
		#endregion

		#region CustomMeshContainer (required for all the loading)
		/// <summary>
		/// Just a simple variable used only for the loading process.
		/// </summary>
		private static string currentLoadingModelName = "";

		/// <summary>
		/// Custom mesh container
		/// </summary>
		public class CustomMeshContainer : MeshContainer
		{
			private Material[] materials;
			private Frame[] frameMatrices = null;

			private int numberOfAttributes = 0;

			/// <summary>
			/// Create custom mesh container
			/// </summary>
			public CustomMeshContainer(
				string setName,
				MeshData setMeshData,
				ExtendedMaterial[] setMaterials,
				EffectInstance[] setEffectInstances,
				GraphicsStream setAdjacency,
				SkinInformation setSkinInfo)
				: base()
			{
				// Ignore non meshes or invalid vertex format
				if (setMeshData.Mesh == null ||
					setMeshData.Mesh.VertexFormat == VertexFormats.None)
					return;

				try
				{
					this.Name = setName;
					this.SetMaterials(setMaterials);
					this.SetEffectInstances(setEffectInstances);
					this.SetAdjacency(setAdjacency);
					this.SkinInformation = setSkinInfo;

					// Load the material textures
					this.materials = new Material[setMaterials.Length];
					for (int matNum = 0; matNum < setMaterials.Length; matNum++)
					{
						string textureFilename = StringHelper.ExtractFilename(
							setMaterials[matNum].TextureFilename, false);

						// Search for texture filename in shader effect if present.
						if (setEffectInstances != null &&
							matNum < setEffectInstances.Length &&
							String.IsNullOrEmpty(
							setEffectInstances[matNum].EffectFilename) == false)
						{
							EffectDefault[] parameters =
								setEffectInstances[matNum].GetDefaults();
							foreach (EffectDefault param in parameters)
								if (StringHelper.Compare(param.ParameterName,
									"diffuseTexture"))
								{
									object ret = param.Data;
									if (ret != null &&
										ret.GetType() == typeof(string))
										textureFilename =
											StringHelper.ExtractFilename((string)ret, false);
								} // if
						} // if

						// Material already loaded?
						// Note: In this game we usually only load 1 material per model,
						// more isn't used anyway and some models use the same material
						// on several meshes! Sometimes the same material is loaded
						// for other models too, for that reason alreadyLoadedMaterials
						// should be static!
						// Note: In case we need the same textures, but other material
						// colors and settings, this won't work, you should implement
						// a texture manager class instead.
						foreach (Material loadedMaterial in alreadyLoadedMaterials)
						{
							if (loadedMaterial.diffuseTexture != null &&
								StringHelper.Compare(StringHelper.ExtractFilename(
								loadedMaterial.diffuseTexture.Filename, false),
								textureFilename))
							{
								// Reuse existing material
								materials[matNum] = loadedMaterial;
								break;
							} // if
						} // foreach  (loadedMaterial)

						// Skip this material if already assigned!
						if (materials[matNum] != null)
							continue;

						// Can we use shaders with effect parameters?
						if (setEffectInstances != null &&
							matNum < setEffectInstances.Length &&
							String.IsNullOrEmpty(
							setEffectInstances[matNum].EffectFilename) == false)
							materials[matNum] = new Material(
								setEffectInstances[matNum],
								setMaterials[matNum]);

						// In case no shader was found or shader is not set,
						// load normal material (will render without shader stuff).
						if (materials[matNum] == null)
							materials[matNum] = new Material(setMaterials[matNum]);

						// Make sure we don't load this material again!
						alreadyLoadedMaterials.Add(materials[matNum]);
					} // for (matNum, <, ++)

					// Check if mesh has normals and tangents, if not, generate them!
					Mesh currentMesh = setMeshData.Mesh;
					GenerateNormalsAndTangentsIfNotPresent(ref currentMesh,
						IsRocketMode(currentLoadingModelName) == false);
					setMeshData.Mesh = currentMesh;

					// Set mesh
					this.MeshData = setMeshData;

					// Note: If object uses skinning, we should update all skin
					// information and bone data here (influcenes, etc.).
					// Not used or supported for this game!
				} // try
				catch (Exception ex)
				{
					Log.Write("Failed to load the animated model meshes: " +
						ex.ToString());
				} // catch
			} // CustomMeshContainer(setName, setMeshData, setMaterials)

			/// <summary>
			/// Get materials
			/// </summary>
			public Material[] Materials
			{
				get
				{
					return materials;
				} // get
			} // Materials

			/// <summary>
			/// Retrieve the animation frames used for this container
			/// </summary>
			public Frame[] FrameMatrices
			{
				get
				{
					return frameMatrices;
				} // get
				set
				{
					frameMatrices = value;
				} // set
			} // FrameMatrices

			/// <summary>
			/// Total number of attributes this mesh container contains
			/// </summary>
			public int NumberOfAttributes
			{
				get
				{
					return numberOfAttributes;
				} // get
			} // NumberOfAttributes
		} // class CustomMeshContainer
		#endregion

		#region Load animated model
		/// <summary>
		/// Load animated model.
		/// </summary>
		/// <param name="setFilename">Model filename</param>
		public AnimatedModel(string setFilename)
		{
			// Add .x extension if it was not given.
			if (StringHelper.GetExtension(setFilename).Length == 0)
				setFilename += "." + Extension;

			modelFilename = setFilename;
			string fullFilename =
				Directories.ModelsDirectory + "\\" + modelFilename;
			if (File.Exists(fullFilename) == false)
			{
				error = "File " + modelFilename + " does not exists!";
				Log.Write("Failed to load animated model: " + error);
				return;
			} // if (File.Exists)

			try
			{
				currentLoadingModelName = modelFilename;
				rootFrame = Mesh.LoadHierarchyFromFile(
					fullFilename,
					MeshFlags.Managed,
					GraphicForm.DirectXDevice,
					new CustomAllocateHierarchy(),
					null);

				// Calculate the center and radius of a bounding sphere
				objectRadius = Frame.CalculateBoundingSphere(
					rootFrame.FrameHierarchy,
					out objectCenter);
				if (objectRadius == 0.0f)
					objectRadius = 0.01f;

				CalcObjectMatrix();
			} // try
			catch (Exception ex)
			{
				error = ex.Message;
				Log.Write("Failed to load animated model " + modelFilename +
					", can't display model! Error: " + ex.ToString());
			} // catch (ex)
		} // AnimatedModel(setFilename)
		#endregion

		#region Dispose
		/// <summary>
		/// Dispose animated mesh
		/// </summary>
		public override void Dispose()
		{
			if (D3DMesh != null)
				base.Dispose();
			else
			{
				if (rootFrame.FrameHierarchy != null)
					Frame.Destroy(rootFrame.FrameHierarchy,
						new CustomAllocateHierarchy());
				if (rootFrame.AnimationController != null)
					rootFrame.AnimationController.Dispose();
			} // else
		} // Dispose()
		#endregion

		#region Render model
		/// <summary>
		/// Draw all frames and all child and sibling frames
		/// </summary>
		/// <param name="frame">Frame</param>
		/// <param name="parentMatrix">Parent matrix</param>
		private void DrawFrameRecursively(Frame frame, Matrix parentMatrix)
		{
			CustomMeshContainer mesh = frame.MeshContainer as CustomMeshContainer;
			
			// Go through all meshes
			while (mesh != null)
			{
				Matrix combinedTransformationMatrix =
					frame.TransformationMatrix * parentMatrix;

				// Note: Skinning is not supported here, we could use
				// mesh.SkinInformation if implemented in CustomMeshContainer here!

				if (mesh.Materials != null)
				{
					// Render for all materials
					for (int num = 0; num < mesh.Materials.Length; num++)
						if (mesh.Materials[num] != null)
						{
							GraphicForm.WorldMatrix = combinedTransformationMatrix;
							mesh.Materials[num].SelectWithoutShader();
							mesh.MeshData.Mesh.DrawSubset(num);
						} // for if
				} // if

				// Select next mesh container
				mesh = mesh.NextContainer as CustomMeshContainer;
			} // while (mesh)

			if (frame.FrameSibling != null)
				DrawFrameRecursively(
					frame.FrameSibling,
					parentMatrix);

			if (frame.FrameFirstChild != null)
				DrawFrameRecursively(
					frame.FrameFirstChild,
					frame.TransformationMatrix * parentMatrix);
		} // DrawFrameRecursively(frame)

		float currentAnimationTime = 0.0f;
		/// <summary>
		/// Render model with specified matrix.
		/// </summary>
		/// <param name="renderMatrix">Render matrix</param>
		public override void Render(Matrix renderMatrix)
		{
			// Use static model rendering if not using animated mesh.
			if (D3DMesh != null)
			{
				base.Render(renderMatrix);
				return;
			} // if (D3DMesh)

			Animate();

			// Render everything
			DrawFrameRecursively(
				rootFrame.FrameHierarchy,
				objectMatrix * renderMatrix);
		} // Render(renderMatrix)

		/// <summary>
		/// Animate
		/// </summary>
		public void Animate()
		{
			if (rootFrame.AnimationController != null)
			{
				currentAnimationTime += ElapsedTime.MoveFactorPerSecond;
				// Reached to big number? Then reset.
				if (currentAnimationTime > 1.0e15f)
				{
					currentAnimationTime = 0.0f;
					rootFrame.AnimationController.ResetTime();
				} // if (currentAnimationTime)
				double advanceTime =
					currentAnimationTime - rootFrame.AnimationController.Time;
				// The advanceTime parameter should not be negative, DirectX
				// expects 0 or a positive value.
				if (advanceTime < 0)
					advanceTime = 0.0f;
				rootFrame.AnimationController.AdvanceTime(advanceTime);
			} // if (rootFrame.AnimationController)
		} // Animate()

		/// <summary>
		/// Draw a frame and all child and sibling frames.
		/// This version renderes everything to a initialized shader!
		/// </summary>
		private void DrawFrameRecursivelyWithShader(
			Frame frame, Matrix parentMatrix, ParallaxShader shader)
		{
			CustomMeshContainer mesh = frame.MeshContainer as CustomMeshContainer;

			// Go through all meshes
			while (mesh != null)
			{
				shader.WorldMatrix = frame.TransformationMatrix * parentMatrix;

				// Apply all shader changes
				shader.D3DEffect.CommitChanges();

				// Note: Skinning is not supported here, we could use
				// mesh.SkinInformation if implemented in CustomMeshContainer here!

				if (mesh.Materials != null)
				{
					// Render for all materials
					for (int num = 0; num < mesh.Materials.Length; num++)
						if (mesh.Materials[num] != null)
						{
							mesh.MeshData.Mesh.DrawSubset(num);
						} // for if
				} // if

				// Select next mesh container
				mesh = mesh.NextContainer as CustomMeshContainer;
			} // while (mesh)

			if (frame.FrameSibling != null)
				DrawFrameRecursivelyWithShader(
					frame.FrameSibling,
					parentMatrix,
					shader);

			if (frame.FrameFirstChild != null)
				DrawFrameRecursivelyWithShader(
					frame.FrameFirstChild,
					frame.TransformationMatrix * parentMatrix,
					shader);
		} // DrawFrameRecursivelyWithShader(frame, parentMatrix, shader)

		/// <summary>
		/// Render with shader
		/// </summary>
		/// <param name="parentMatrix">Parent matrix</param>
		/// <param name="shader">Shader</param>
		public void RenderWithShader(Matrix parentMatrix, ParallaxShader shader)
		{
			// Just render with shader!
			DrawFrameRecursivelyWithShader(
				FirstFrame, objectMatrix * parentMatrix, shader);
		} // RenderWithShader(parentMatrix, shader)

		/// <summary>
		/// Draw a frame and all child and sibling frames.
		/// This version renderes everything to a initialized shader!
		/// </summary>
		private void DrawFrameRecursivelyWithoutShader(
			Frame frame, Matrix parentMatrix)
		{
			CustomMeshContainer mesh = frame.MeshContainer as CustomMeshContainer;

			// Go through all meshes
			while (mesh != null)
			{
				GraphicForm.WorldMatrix = frame.TransformationMatrix * parentMatrix;

				// Note: Skinning is not supported here, we could use
				// mesh.SkinInformation if implemented in CustomMeshContainer here!

				if (mesh.Materials != null)
				{
					// Render for all materials
					for (int num = 0; num < mesh.Materials.Length; num++)
						if (mesh.Materials[num] != null)
						{
							mesh.MeshData.Mesh.DrawSubset(num);
						} // for if
				} // if

				// Select next mesh container
				mesh = mesh.NextContainer as CustomMeshContainer;
			} // while (mesh)

			if (frame.FrameSibling != null)
				DrawFrameRecursivelyWithoutShader(
					frame.FrameSibling,
					parentMatrix);

			if (frame.FrameFirstChild != null)
				DrawFrameRecursivelyWithoutShader(
					frame.FrameFirstChild,
					frame.TransformationMatrix * parentMatrix);
		} // DrawFrameRecursivelyWithoutShader(frame, parentMatrix)

		/// <summary>
		/// Render without shader
		/// </summary>
		/// <param name="parentMatrix">Parent matrix</param>
		public void RenderWithoutShader(Matrix parentMatrix)
		{
			// Just render with shader!
			DrawFrameRecursivelyWithoutShader(
				FirstFrame, objectMatrix * parentMatrix);
		} // RenderWithoutShader(parentMatrix)
		#endregion

		#region Unit Testing
#if DEBUG
		#region Test animated model
		/// <summary>
		/// Test animated model
		/// </summary>
		[Test]
		public static void TestAnimatedModel()
		{
			Model testModel = null;
			int lastMs = 0;
			float rotation = 0.0f;
			float zoomFactor = 50.0f;// 1.0f;

			GameForm.StartTest("TestAnimatedModel",
				delegate // Init
				{
					testModel = new AnimatedModel("ExtraLife.x");//"asteroid2-lod1.X");
				},
				delegate // Render loop
				{
					GameForm.TestGameForm.camera.FreeCamera = true;

					// If mouse is pressed, hold rotation!
					if (GameForm.Mouse.LeftButtonPressed == false)
						rotation += (Environment.TickCount - lastMs) / 1500.0f;
					lastMs = Environment.TickCount;
					GameForm.WorldMatrix =
						Matrix.Scaling(zoomFactor, zoomFactor, zoomFactor) *
						Matrix.RotationAxis(new Vector3(0, 1, 0), rotation);

					GameForm.DirectXDevice.RenderState.ZBufferEnable = true;
					GameForm.DirectXDevice.RenderState.Lighting = true;
					GameForm.LightDir = new Vector3(0, 0, 1);

					if (GameForm.Mouse.RightButtonPressed)
						GameForm.LightColor = Color.Red;
					else
						GameForm.LightColor = Color.White;

					testModel.Render(GameForm.WorldMatrix);

					// Disable Z-Buffer (UI doesn't need z-buffer)
					GameForm.DirectXDevice.RenderState.ZBufferEnable = false;
					// Reset default alpha blending (for ui)
					GameForm.DirectXDevice.RenderState.AlphaBlendEnable = false;
					// Disable lighting
					GameForm.DirectXDevice.RenderState.Lighting = false;
				});
		} // TestAnimatedModel()
		#endregion

		#region Test animated model with shader
		/// <summary>
		/// Test animated model with shader
		/// </summary>
		[Test]
		public static void TestAnimatedModelWithShader()
		{
			AnimatedModel testModel = null;
			int lastMs = 0;
			float rotation = 0.0f;
			float zoomFactor = 50.0f;// 1.0f;

			GameForm.StartTest("TestAnimatedModel",
				delegate // Init
				{
					testModel = new AnimatedModel("Bomb.x");//"ExtraLife.x");
				},
				delegate // Render loop
				{
					GameForm.TestGameForm.camera.FreeCamera = true;
					GameForm.DirectXDevice.RenderState.ZBufferEnable = true;
					GameForm.DirectXDevice.RenderState.Lighting = true;

					Frame frame = testModel.FirstFrame;

					ParallaxShader shader = GameForm.ParallaxShader;
					shader.UpdateMatricesAndEffectParameters();
					shader.UpdateMaterialParameters(testModel.FirstMaterial);

					// If mouse is pressed, hold rotation!
					if (GameForm.Mouse.LeftButtonPressed == false)
						rotation += (Environment.TickCount - lastMs) / 1500.0f;
					lastMs = Environment.TickCount;

					if (GameForm.Mouse.RightButtonPressed)
						shader.DiffuseColorValue = Color.Red;
					else
						shader.DiffuseColorValue = Color.White;

					Effect effect = shader.D3DEffect;
					effect.Begin(FX.None);

					// We only support 1 pass, more isn't used anyway.
					effect.BeginPass(0);

					//normal rendering:
					testModel.RenderWithShader(
						Matrix.Scaling(zoomFactor, zoomFactor, zoomFactor) *
						Matrix.RotationAxis(new Vector3(0, 1, 0), rotation), shader);

					effect.EndPass();
					effect.End();

					// Disable Z-Buffer (UI doesn't need z-buffer)
					GameForm.DirectXDevice.RenderState.ZBufferEnable = false;
					// Reset default alpha blending (for ui)
					GameForm.DirectXDevice.RenderState.AlphaBlendEnable = false;
					// Disable lighting
					GameForm.DirectXDevice.RenderState.Lighting = false;
				});
		} // TestAnimatedModelWithShader()
		#endregion
#endif
		#endregion
	} // class AnimatedModel
} // namespace Abi.Graphic
