// Project: Rocket Commander, File: AnimatedTexture.cs
// Namespace: RocketCommander.Graphics, Class: AnimatedTexture
// Path: C:\code\Rocket Commander\Graphics, Author: Abi
// Code lines: 211, Size of file: 6,08 KB
// Creation date: 07.12.2005 04:55
// Last modified: 08.12.2005 05:16
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Direct3D = Microsoft.DirectX.Direct3D;
using RocketCommander.Helpers;
using Microsoft.DirectX.Direct3D;
using System.Drawing;
using RocketCommander.Game;
using RocketCommander.Sounds;
#endregion

namespace RocketCommander.Graphics
{
	/// <summary>
	/// Animated texture
	/// </summary>
	public class AnimatedTexture : Texture
	{
		#region Variables
		/// <summary>
		/// Textures for the animation.
		/// You can still use d3dTexture, it will just hold current animation
		/// texture! See Select(.).
		/// </summary>
		private Direct3D.Texture[] d3dTextures = null;

		/// <summary>
		/// Get animation length (how many textures are used inside of this
		/// animated texture). Can be used for the Select(.) method.
		/// </summary>
		/// <returns>Int</returns>
		public int AnimationLength
		{
			get
			{
				return d3dTextures != null ?
					d3dTextures.Length : 1;
			} // get
		} // AnimationLength
		#endregion

		#region Constructor
		/// <summary>
		/// Create animated texture, don't allow empty constructor from outside.
		/// </summary>
		protected AnimatedTexture()
		{
		} // AnimatedTexture()

		/// <summary>
		/// Create animated texture from given filename.
		/// Will go through all filenames with textureFilename + "0001", "0002",
		/// etc. until the filename is not longer found.
		/// </summary>
		/// <param name="setFilename">Set filename, must be relative and
		/// without the 0001.dds, 0002.dds, etc. extension.</param>
		public AnimatedTexture(string setFilename)
		{
			if (String.IsNullOrEmpty(setFilename))
				throw new ArgumentNullException("setFilename",
					"Unable to create texture without valid filename.");

			// Set filename
			texFilename = setFilename;

			// Try to load texture
			try
			{
				string filenameFirstPart =
					Directories.TexturesDirectory + "\\" + texFilename;

				// Build full filename
				string fullFilename = filenameFirstPart +
					"0001." + AddExtensionIfNoneGiven;

				// Check if file exists, else we can't continue loading!
				if (File.Exists(fullFilename) == false)
					throw new FileNotFoundException(fullFilename);

				ImageInformation imageInfo =
					TextureLoader.ImageInformationFromFile(fullFilename);
				size = new Size(imageInfo.Width, imageInfo.Height);

				// Size of 0 is not allowed!
				if (size.Width == 0 || size.Height == 0)
					throw new InvalidOperationException(
						"Image size="+size+" is invalid, unable to init texture.");
				CalcHalfPixelSize();

				// We will use alpha for Dxt3, Dxt5 or any format starting with "A",
				// there are a lot of those (A8R8G8B8, A4R4G4B4, A8B8G8R8, etc.)
				hasAlpha =
					imageInfo.Format == Format.Dxt5 ||
					imageInfo.Format == Format.Dxt3 ||
					imageInfo.Format.ToString().StartsWith("A");

				// Finally load texture
				direct3dTexture = TextureLoader.FromFile(
					GraphicForm.DirectXDevice,
					fullFilename);
				loaded = true;

				// Ok, now load all other animated textures
				List<Direct3D.Texture> animatedTextures =
					new List<Direct3D.Texture>();
				animatedTextures.Add(direct3dTexture);
				int texNumber = 2;
				while (File.Exists(filenameFirstPart +
					texNumber.ToString("0000") + "." + AddExtensionIfNoneGiven))
				{
					animatedTextures.Add(
						TextureLoader.FromFile(
						GraphicForm.DirectXDevice,
						filenameFirstPart +
						texNumber.ToString("0000") + "." + AddExtensionIfNoneGiven));
					texNumber++;
				} // while (File.Exists)
				d3dTextures = animatedTextures.ToArray();
				 
				GraphicForm.AddGraphicObject(this);
			} // try
			catch (Exception ex)
			{
				// Failed to load
				loaded = false;
				Log.Write("Failed to load animated texture " + texFilename +
					", will use empty texture! Error: " + ex.ToString());
			} // catch (ex)
		} // AnimatedTexture(setFilename)
		#endregion

		#region Disposing
		/// <summary>
		/// Dispose
		/// </summary>
		public override void Dispose()
		{
			if (d3dTextures != null)
			{
				foreach (Direct3D.Texture tex in d3dTextures)
					tex.Dispose();
				d3dTextures = null;
			} // if (d3dTextures)
			direct3dTexture = null;
			loaded = false;
		} // Dispose()
		#endregion

		#region Select texture as texture stage
		/// <summary>
		/// Select this texture as texture stage 0
		/// </summary>
		public void Select(int animationNumber)
		{
			if (d3dTextures != null &&
				d3dTextures.Length > 0)
			{
				// Select new animation number
				direct3dTexture = d3dTextures[animationNumber % d3dTextures.Length];
				if (direct3dTexture != null)
					GraphicForm.DirectXDevice.SetTexture(0, direct3dTexture);
			} // if (d3dTextures)
		} // Select(animationNumber)
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test textures
		/// </summary>
		[Test]
		public static void TestAnimatedTexture()
		{
			AnimatedTexture explosionTexture = null;
			Sample explosionSample = null;
			GameForm.StartTest("TestAnimatedTexture",
				delegate
				{
					explosionTexture = new AnimatedTexture(
						Directories.TextureEffectsSubDirectory + "\\" +
						"Explosion");
					explosionSample = new Sample("Explosion.wav");
				},
				delegate
				{
					GameForm.DirectXDevice.RenderState.AlphaBlendEnable = true;
					GameForm.DirectXDevice.RenderState.SourceBlend = Blend.SourceAlpha;
					GameForm.DirectXDevice.RenderState.DestinationBlend =
						Blend.InvSourceAlpha;
					explosionTexture.Select(ElapsedTime.MsTotal / 50);
					Size res = GameForm.Resolution;
					int screenSize = Math.Max(res.Width * 2 / 3, res.Height * 2 / 3);
					explosionTexture.RenderOnScreen(new Rectangle(
						res.Width / 2 - screenSize / 2,
						res.Height / 2 - screenSize / 2,
						screenSize, screenSize));

					if (GameForm.Mouse.LeftButtonPressed)
						explosionSample.Play();
				});
		} // TestAnimatedTexture()
#endif
		#endregion
	} // class AnimatedTexture
} // namespace RocketCommander.Graphics
