// Project: Rocket Commander, File: Font.cs
// Namespace: RocketCommander.Graphics, Class: Font
// Path: C:\code\Rocket Commander\Graphics, Author: Abi
// Code lines: 1084, Size of file: 33,47 KB
// Creation date: 09.11.2005 02:45
// Last modified: 27.02.2006 18:34
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
#if DEBUG
using NUnit.Framework;
#endif
using Direct3D = Microsoft.DirectX.Direct3D;
using RocketCommander.Helpers;
#endregion

namespace RocketCommander.Graphics
{
	/// <summary>
	/// Font helper class to simplify writing text on screen. Basically you
	/// can just use the font class, but it is a hassle to always call
	/// Sprite.Begin and Sprite.End before and after each font call.
	/// </summary>
	public class Font : IGraphicObject
	{
		#region Variables
		/// <summary>
		/// The windows font object
		/// </summary>
		System.Drawing.Font windowsFont = null;
		/// <summary>
		/// The d3d font object.
		/// </summary>
		Direct3D.Font d3dFont = null;
		/// <summary>
		/// Sprite to optimize rendering and to cache stuff.
		/// </summary>
		Direct3D.Sprite textSprite = null;
		/// <summary>
		/// The font family name (Arial, Times, Verdana, etc.)
		/// </summary>
		string familyName = "";
		/// <summary>
		/// Height of the font
		/// </summary>
		int height = 10;

		/// <summary>
		/// Get height of font
		/// </summary>
		public int Height
		{
			get
			{
				return height;
			} // get
		} // Height
		#endregion

		#region Constructors
		/// <summary>
		/// Load font.
		/// </summary>
		public Font(string setFamilyName, int setHeight)
		{
			familyName = setFamilyName;
			height = setHeight + 5; // +5 because of line distance!
			try
			{
				windowsFont = new System.Drawing.Font(
					familyName, setHeight, FontStyle.Regular);
			} // try
			catch
			{
				// Ignore error and select just a more standard font: "Arial"
				windowsFont = new System.Drawing.Font(
					"Arial", setHeight, FontStyle.Regular);
			} // catch

			d3dFont = new Direct3D.Font(GraphicForm.DirectXDevice, windowsFont);
			textSprite = new Direct3D.Sprite(GraphicForm.DirectXDevice);

			GraphicForm.AddGraphicObject(this);
		} // Font(setFamilyName, setHeight)
		#endregion

		#region Disposing
		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			if (d3dFont != null)
				d3dFont.Dispose();
			d3dFont = null;
			if (textSprite != null)
				textSprite.Dispose();
			textSprite = null;
		} // Dispose()
		#endregion

		#region IGraphicObject Members
		/// <summary>
		/// Called when the device is reset
		/// </summary>
		public void OnResetDevice()
		{
			if (d3dFont != null)
				d3dFont.OnResetDevice();
		} // OnResetDevice()

		/// <summary>
		/// Clear any resources that need to be lost
		/// </summary>
		public void OnLostDevice()
		{
			if (d3dFont != null)
				d3dFont.OnLostDevice();
		} // OnLostDevice()
		#endregion

		#region Write text
		/// <summary>
		/// Write text
		/// </summary>
		/// <param name="xPos">X Position</param>
		/// <param name="yPos">Y Position</param>
		/// <param name="text">Text</param>
		/// <param name="color">Color</param>
		public void WriteText(int xPos, int yPos, string text, Color color)
		{
			if (textSprite == null ||
				d3dFont == null ||
				String.IsNullOrEmpty(text))
				return;

			try
			{
				try
				{
					textSprite.Begin(
						Direct3D.SpriteFlags.AlphaBlend |
						Direct3D.SpriteFlags.SortTexture);

					d3dFont.DrawText(textSprite, text, xPos, yPos, color);
				} // try
				finally
				{
					textSprite.End();
				} // finally
			} // try
			catch (Exception ex)
			{
				Log.Write("DirectX error while drawing text: " + ex.ToString());
			} // finally
		} // WriteText(xPos, yPos, text)

		/// <summary>
		/// Write text
		/// </summary>
		/// <param name="xPos">X Position</param>
		/// <param name="yPos">Y Position</param>
		/// <param name="text">Text</param>
		public void WriteText(int xPos, int yPos, string text)
		{
			WriteText(xPos, yPos, text, Color.White);
		} // WriteText(xPos, yPos, text)

		/// <summary>
		/// Write text centered
		/// </summary>
		/// <param name="xPos">X Position</param>
		/// <param name="yPos">Y Position</param>
		/// <param name="text">Text</param>
		public void WriteTextCentered(int xPos, int yPos, string text)
		{
			// Render centered with help of WriteText.
			// Note: This is not optimal, normally we would have to determinate
			// the width of each letter and add them up, but this is too much work
			// for this simple class yet, maybe someone will implement that later.

			// First show shadow
			WriteText(xPos - text.Length * height * 3 / 10 + 3, yPos -
				height / 2 + 3,
				text, Color.Black);

			// And then the real text
			WriteText(xPos - text.Length * height * 3 / 10, yPos - height / 2, text);
		} // WriteTextCentered(xPos, yPos, text)

		/// <summary>
		/// Write text centered
		/// </summary>
		/// <param name="xPos">X Position</param>
		/// <param name="yPos">Y Position</param>
		/// <param name="text">Text</param>
		/// <param name="alpha">Alpha</param>
		public void WriteTextCentered(int xPos, int yPos, string text,
			Color col, float alpha)
		{
			// First show shadow
			WriteText(xPos - text.Length * height * 3 / 10 + 3, yPos -
				height / 2 + 3,
				text, Color.FromArgb((byte)(alpha * 200), Color.Black));

			// And then the real text
			WriteText(xPos - text.Length * height * 3 / 10, yPos - height / 2, text,
				Color.FromArgb((byte)(alpha * 255), col));
		} // WriteTextCentered(xPos, yPos, text)

		/// <summary>
		/// Write text centered
		/// </summary>
		/// <param name="xPos">X Position</param>
		/// <param name="yPos">Y Position</param>
		/// <param name="text">Text</param>
		public void WriteSmallTextCentered(int xPos, int yPos, string text,
			Color col, float alpha)
		{
			// Render centered with help of WriteText.
			// Note: This is not optimal, normally we would have to determinate
			// the width of each letter and add them up, but this is too much work
			// for this simple class yet, maybe someone will implement that later.

			// First show shadow (a little bit weaker than the real text)
			WriteText(xPos - text.Length * height * 3 / 10 + 3, yPos -
				height / 2 + 1,
				text, Color.FromArgb((byte)(alpha * 200), Color.Black));

			// And then the real text
			WriteText(xPos - text.Length * height * 3 / 10 + 2, yPos - height / 2,
				text, Color.FromArgb((byte)(alpha * 255), col));
		} // WriteTextCentered(xPos, yPos, text)

		/// <summary>
		/// Write text centered
		/// </summary>
		/// <param name="xPos">X Position</param>
		/// <param name="yPos">Y Position</param>
		/// <param name="text">Text</param>
		public void WriteSmallTextCentered(int xPos, int yPos, string text,
			float alpha)
		{
			// Render centered with help of WriteText.
			// Note: This is not optimal, normally we would have to determinate
			// the width of each letter and add them up, but this is too much work
			// for this simple class yet, maybe someone will implement that later.

			// First show shadow (a little bit weaker than the real text)
			WriteText(xPos - text.Length * height * 3 / 10 + 3, yPos - 
				height / 2 + 1,
				text, Color.FromArgb((byte)(alpha * 200), Color.Black));

			// And then the real text
			WriteText(xPos - text.Length * height * 3 / 10 + 2, yPos - height / 2,
				text, Color.FromArgb((byte)(alpha * 255), Color.White));
		} // WriteTextCentered(xPos, yPos, text)
		#endregion
	}	// class Font
} // namespace RocketCommander.Graphics
