// Project: Rocket Commander, File: Graphic.cs
// Namespace: RocketCommander.Graphics, Class: Graphic
// Path: C:\code\Rocket Commander\Graphics, Author: Abi
// Code lines: 11, Size of file: 195 Bytes
// Creation date: 02.11.2005 00:08
// Last modified: 20.01.2006 00:17
// Generated with Commenter by abi.exDream.com

// Helper to log all device events (enable for heavy debugging and testing)
//#define LOG_STUFF

#region Using directives
using System;
using System.Collections.Generic;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Threading;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;
using RocketCommander.Shaders;
using RocketCommander.Helpers;
using RocketCommander.Game;
using System.ComponentModel;
using RocketCommander.Properties;
using System.Management;
#endregion

namespace RocketCommander.Graphics
{
	/// <summary>
	/// Helper class to manage DirectX graphics.
	/// Used to initialize DirectX and provides some helper methods.
	/// This is a static class, but it is derived from form, this way
	/// we can use it to derive from for our game form, but still use all
	/// the static graphics methods from everywhere, e.g. access to the
	/// DirectX device, matrices, shaders and lightDir.
	/// For bigger projects you should derive this class to support more
	/// features and instantiate it instead of using static methods.
	/// </summary>
	public class GraphicForm : Form
	{
		#region Variables
		/// <summary>
		/// Render device, only used to load textures and displaying the sphere.
		/// </summary>
		protected static Device device = null;

		/// <summary>
		/// DirectX device
		/// </summary>
		/// <returns>Device</returns>
		public static Device DirectXDevice
		{
			get
			{
				return device;
			} // get
		} // DirectXDevice

		/// <summary>
		/// Use a standard field of view of 90 degrees, and use 1 for near plane
		/// and 1000 for far plane. Field of view might be changed for speed items.
		/// </summary>
		private static float fieldOfView = (float)Math.PI / 2.0f,
			nearPlane = GameAsteroidManager.MinViewDepth,//0.5f,
			farPlane = GameAsteroidManager.MaxViewDepth;//1000.0f;

		/// <summary>
		/// Resolution, set in constructor and in OnSizeChanged.
		/// </summary>
		private static Size resolution;

		/// <summary>
		/// Internal list of IGraphicObjects used for disposing everything
		/// before Direct3D device gets disposed. See OnDisposingDevice.
		/// </summary>
		private static List<IGraphicObject> graphicObjects =
			new List<IGraphicObject>();

		/// <summary>
		/// Line manager for rendering lines.
		/// </summary>
		private static LineManager lineManager = null;

		/// <summary>
		/// Default light dir.
		/// </summary>
		public static readonly Vector3 DefaultLightDir = new Vector3(-5, 0, 3);
		/// <summary>
		/// Light dir used for the lighting. For more advanced lighting you
		/// should write an extra class. Points lights are not supported here yet.
		/// </summary>
		private static Vector3 lightDir = Vector3.Empty;

		/// <summary>
		/// Use this color to colorize the whole scene, light will affect
		/// everything, this way we can easily lighten up everything or turn
		/// the color from light blue to dark red. Use the LightColor property!
		/// </summary>
		private static Color lightColor = Color.White;

		/// <summary>
		/// Remember the lens flare color and sky background color.
		/// </summary>
		protected static Color remLensFlareColor = Color.White,
			remSkyBackgroundColor = Color.White;
		
		/// <summary>
		/// Can we use Pixelshader 1.1 or even Pixelshader 2.0?
		/// This is required for rendering, else we can only display
		/// simple 3D fixed function pipeline rendering.
		/// </summary>
		protected static bool canDoPS11 = false, canDoPS20 = false;

		/// <summary>
		/// Performance settings
		/// </summary>
		public enum PerformanceSetting
		{
			/// <summary>
			/// Fastest performance, no shaders will be used at all.
			/// </summary>
			Fastest,
			/// <summary>
			/// Medium performance, using only pixelshader 1.1, even if
			/// ps2.0 is possible.
			/// </summary>
			Medium,
			/// <summary>
			/// High quality setting. Use always the best possible pixelshader.
			/// Can also be used if we don't have pixelshader 2.0, then the next
			/// best thing (ps1.1 if possible, else no shaders) will be used.
			/// </summary>
			Quality,
		} // enum PerformanceSettings
		/// <summary>
		/// Performance setting, used from the settings file.
		/// Determinates which pixelshader we use if any.
		/// </summary>
		private static PerformanceSetting performanceSetting =
			PerformanceSetting.Quality;

		/// <summary>
		/// World, view and projection martix, also used for the shaders!
		/// </summary>
		private static Matrix worldMatrix, viewMatrix, projectionMatrix;

		#region Build projection matrix
		/// <summary>
		/// Build projection matrix
		/// </summary>
		private void BuildProjectionMatrix()
		{
			float aspectRatio = 1.0f;
			if (this.Height != 0)
				aspectRatio = (float)this.Width / (float)this.Height;
			ProjectionMatrix = Matrix.PerspectiveFovLH(
				fieldOfView, aspectRatio, nearPlane, farPlane);
		} // BuildProjectionMatrix()
		#endregion

		#region Is form active?
		private bool isActive = true;
		/// <summary>
		/// Is active
		/// </summary>
		public bool IsActive
		{
			get
			{
				return isActive;
			} // get
		} // IsActive

		/// <summary>
		/// On activated
		/// </summary>
		protected override void OnActivated(EventArgs e)
		{
			isActive = true;
			if (this.WindowState == FormWindowState.Minimized)
				this.WindowState = FormWindowState.Normal;
		} // OnActivated(e)

		/// <summary>
		/// On deactivate
		/// </summary>
		protected override void OnDeactivate(EventArgs e)
		{
			isActive = false;
		} // OnDeactivate(e)
		#endregion

		/// <summary>
		/// Precalculated view projection matrix for the Convert3DPointTo2D,
		/// IsInFrontOfCamera and IsVisible methods. Calculated in ViewMatrix.
		/// </summary>
		private static Matrix viewProjMatrix;

		/// <summary>
		/// Small font for description, debugging and information texts.
		/// Not used for the main menu, would be far to small ^^
		/// </summary>
		private static Font smallFont = null;

		/// <summary>
		/// Parallax shader, this is the only shader we use for our objects.
		/// This simplifies a lot of things :)
		/// </summary>
		private static ParallaxShader parallaxShader = null;

		/// <summary>
		/// Sky cube map class using the PreScreenSkyCubeMapping shader.
		/// </summary>
		protected static PreScreenSkyCubeMapping skyCubeMap = null;

		/// <summary>
		/// Lens flare class.
		/// </summary>
		protected static LensFlare lensFlare = null;

		/// <summary>
		/// Remember back buffer surface to restore SetRenderTarget calls.
		/// </summary>
		private static Surface remBackSufferSurface = null,
			remDepthStencilSurface = null;

		/// <summary>
		/// Reset render target
		/// </summary>
		public static void ResetRenderTarget()
		{
			device.SetRenderTarget(0, remBackSufferSurface);
			if (RenderToTexture.zBufferSurface != null)
				device.DepthStencilSurface = remDepthStencilSurface;
		} // ResetRenderTarget()
		#endregion

		#region Properties
		/// <summary>
		/// Use PS20, true if canDoPS20 is set and settings is at quality level.
		/// </summary>
		/// <returns>Bool</returns>
		public static bool UsePS20
		{
			get
			{
				return canDoPS20 &&
					performanceSetting == PerformanceSetting.Quality;
			} // get
		} // UsePS20

		/// <summary>
		/// Use PS1.1 or PS20, true if canDoPS20 OR canDoPS11 is set and
		/// settings is at quality or medium level.
		/// </summary>
		/// <returns>Bool</returns>
		public static bool UsePS
		{
			get
			{
				return (canDoPS20 || canDoPS11) &&
					(performanceSetting == PerformanceSetting.Quality ||
					performanceSetting == PerformanceSetting.Medium);
			} // get
		} // UsePS

		/// <summary>
		/// Performance setting
		/// </summary>
		public static PerformanceSetting GraphicsPerformanceSetting
		{
			get
			{
				return performanceSetting;
			} // get
			set
			{
				performanceSetting = value;
			} // set
		} // GraphicsPerformanceSetting

		/// <summary>
		/// Line manager
		/// </summary>
		/// <returns>Line manager</returns>
		public static LineManager LineManager
		{
			get
			{
				return lineManager;
			} // get
		} // LineManager

		/// <summary>
		/// Resolution
		/// </summary>
		/// <returns>Size</returns>
		public static Size Resolution
		{
			get
			{
				return resolution;
			} // get
		} // Resolution

		/// <summary>
		/// Resolution rectangle
		/// </summary>
		/// <returns>Rectangle</returns>
		public static Rectangle ResolutionRect
		{
			get
			{
				return new Rectangle(Point.Empty, resolution);
			} // get
		} // ResolutionRect

		/// <summary>
		/// Back buffer surface
		/// </summary>
		/// <returns>Surface</returns>
		public static Surface BackBufferSurface
		{
			get
			{
				return remBackSufferSurface;
			} // get
		} // BackBufferSurface

		private static DepthFormat backBufferDepthFormat;
		/// <summary>
		/// Depth format
		/// </summary>
		public static DepthFormat BackBufferDepthFormat
		{
			get
			{
				return backBufferDepthFormat;
			} // get
		} // BackBufferDepthFormat

		private static MultiSampleType multisamplingFormat;
		/// <summary>
		/// Using multisampling
		/// </summary>
		/// <returns>Bool</returns>
		public static bool UsingMultisampling
		{
			get
			{
				return multisamplingFormat != MultiSampleType.None;
			} // get
		} // UsingMultisampling

		/// <summary>
		/// Field of view
		/// </summary>
		/// <returns>Float</returns>
		public static float FieldOfView
		{
			get
			{
				return fieldOfView;
			} // get
		} // FieldOfView

		/// <summary>
		/// World matrix
		/// </summary>
		/// <returns>Matrix</returns>
		public static Matrix WorldMatrix
		{
			get
			{
				return worldMatrix;
			} // get
			set
			{
				// Only set if changed
				if (worldMatrix != value)
				{
					worldMatrix = value;
					device.Transform.World = worldMatrix;
				} // if (worldMatrix)
			} // set
		} // WorldMatrix

		/// <summary>
		/// View matrix
		/// </summary>
		public static Matrix ViewMatrix
		{
			get
			{
				return viewMatrix;
			} // get
			set
			{
				// Always set
				viewMatrix = value;
				device.Transform.View = viewMatrix;

				// Precalculate viewProjMatrix for 3d helper methods below.
				viewProjMatrix = viewMatrix * projectionMatrix;
			} // set
		} // ViewMatrix

		/// <summary>
		/// Inverse view matrix
		/// </summary>
		/// <returns>Matrix</returns>
		public static Matrix InverseViewMatrix
		{
			get
			{
				return Matrix.Invert(viewMatrix);
			} // get
		} // InverseViewMatrix

		/// <summary>
		/// Get camera position from inverse view matrix. Similar to method
		/// used in shader. Works only if ViewMatrix is correctly set.
		/// </summary>
		/// <returns>Vector 3</returns>
		public static Vector3 CameraPos
		{
			get
			{
				Matrix invView = InverseViewMatrix;
				return new Vector3(invView.M41, invView.M42, invView.M43);
			} // get
		} // CameraPos

		/// <summary>
		/// Projection matrix (for shaders, etc.)
		/// </summary>
		public static Matrix ProjectionMatrix
		{
			get
			{
				return projectionMatrix;
			} // get
			set
			{
				projectionMatrix = value;
				device.Transform.Projection = projectionMatrix;
			} // set
		} // ProjectionMatrix

		/// <summary>
		/// Light color, used for shader color multiplication, for
		/// non shader lights and for the lens flare effect.
		/// </summary>
		/// <returns>Color</returns>
		public static Color LightColor
		{
			get
			{
				return lightColor;
			} // get
			set
			{
				lightColor = value;
				device.Lights[0].Ambient = lightColor;
				device.Lights[0].Diffuse = lightColor;
				device.Lights[0].Specular = lightColor;
			} // set
		} // LightColor

		/// <summary>
		/// Lens flare color
		/// </summary>
		/// <returns>Color</returns>
		public static Color LensFlareColor
		{
			set
			{
				remLensFlareColor = value;
			} // set
		} // LensFlareColor

		/// <summary>
		/// Sky background color
		/// </summary>
		/// <returns>Color</returns>
		public static Color SkyBackgroundColor
		{
			set
			{
				remSkyBackgroundColor = value;
			} // set
		} // SkyBackgroundColor

		/// <summary>
		/// Light dir
		/// </summary>
		/// <returns>Vector 3</returns>
		public static Vector3 LightDir
		{
			get
			{
				return lightDir;
			} // get
			set
			{
				if (lightDir != value)
				{
					lightDir = value;
					// ALWAYS normalize the light direction!
					// Else the pixel shader will really mess up stuff!
					lightDir.Normalize();
					device.Lights[0].Direction = lightDir;
					device.Lights[0].Update();
				} // if (lightDir)
			} // set
		} // LightDir

		/// <summary>
		/// Small font
		/// </summary>
		/// <returns>Font</returns>
		public static Font SmallFont
		{
			get
			{
				return smallFont;
			} // get
		} // SmallFont

		/// <summary>
		/// Parallax shader, this is the only shader we use for our objects.
		/// This simplifies a lot of things :)
		/// </summary>
		/// <returns>Parallax shader</returns>
		public static ParallaxShader ParallaxShader
		{
			get
			{
				return parallaxShader;
			} // get
		} // ParallaxShader

		protected bool quit = false;
		/// <summary>
		/// Quit application? When set to true the application will quit!
		/// </summary>
		public bool QuitApplication
		{
			get
			{
				return quit;
			} // get
		} // QuitApplication
		#endregion

		#region Helper methods for 3d-calculations
		/// <summary>
		/// Convert 3D vector to 2D vector, this is kinda the oposite of
		/// GetScreenPlaneVector (not shown here). This can be useful for user
		/// input/output, because we will often need the actual position on screen
		/// of an object in 3D space from the users view to handle it the right
		/// way. Used for lens flare and asteroid optimizations.
		/// </summary>
		/// <param name="point">3D world position</param>
		/// <return>Resulting 2D screen position</return>
		public static Point Convert3DPointTo2D(Vector3 point)
		{
			Vector3 result = Vector3.TransformCoordinate(point, viewProjMatrix);

			// Output result from 3D to 2D
			return new Point(
				(int)Math.Round(+result.X * (resolution.Width / 2)) +
				(resolution.Width / 2),
				(int)Math.Round(-result.Y * (resolution.Height / 2)) +
				(resolution.Height / 2));
		} // Convert3DPointTo2D(point)

		/// <summary>
		/// Is point in front of camera?
		/// </summary>
		/// <param name="point">Position to check.</param>
		/// <returns>Bool</returns>
		public static bool IsInFrontOfCamera(Vector3 point)
		{
			Vector4 result = Vector4.Transform(
				new Vector4(point.X, point.Y, point.Z, 1),
				viewProjMatrix);
			
			// Is result in front?
			return result.Z > result.W - nearPlane;
		} // IsInFrontOfCamera(point)

		/// <summary>
		/// Helper to check if a 3d-point is visible on the screen.
		/// Will basically do the same as IsInFrontOfCamera and Convert3DPointTo2D,
		/// but requires less code and is faster. Also returns just an bool.
		/// Will return true if point is visble on screen, false otherwise.
		/// Use the offset parameter to include points into the screen that are
		/// only a couple of pixel outside of it.
		/// </summary>
		/// <param name="point">Point</param>
		/// <param name="checkOffset">Check offset in percent of total
		/// screen</param>
		/// <returns>Bool</returns>
		public static bool IsVisible(Vector3 point, float checkOffset)
		{
			Vector4 result = Vector4.Transform(
				new Vector4(point.X, point.Y, point.Z, 1),
				viewProjMatrix);

			// Point must be in front of camera, else just skip everything.
			if (result.Z > result.W - nearPlane)
			{
				Vector2 screenPoint = new Vector2(
					result.X / result.W, result.Y / result.W);

				// Change checkOffset depending on how depth we are into the scene
				// for very near objects (z < 5) pass all tests!
				// for very far objects (z >> 5) only pass if near to +- 1.0f
				float zDist = Math.Abs(result.Z);
				if (zDist < 5.0f)
					return true;
				checkOffset = 1.0f + (checkOffset / zDist);
			
				return
					screenPoint.X >= -checkOffset && screenPoint.X <= +checkOffset &&
					screenPoint.Y >= -checkOffset && screenPoint.Y <= +checkOffset;
			} // if (result.z)

			// Point is not in front of camera, return false.
			return false;
		} // IsVisible(point)
		#endregion

		#region Occulision intensity
		static bool failedToDoOcclusionTest = false;
		/// <summary>
		/// Check if sun is occluded by 3d-data (in our case asteroids).
		/// First try was to get z buffer depth from back surface, which does
		/// the same as glReadPixels with help of locking the surface and directly
		/// accessing the pixels. This is very slow and should not be used
		/// and has other cavecats (depth buffer must be D16F_LOCKABLE).
		/// 
		/// The second approach is often to do a intersection test with a ray
		/// and the geometry in the scene, but this would be far to slow for our
		/// immense mass of asteroids we could have in a scene.
		/// 
		/// And finally there is Occlusion testing in DirectX9, which is exactly
		/// what we need for our test if the sun is occluded. We will test
		/// a small area around the sun position and see how much pixels are
		/// occluded or not. Based on that we return the itensity for the
		/// lensflare. If the itensity is 0 we don't need to render the lensflare
		/// at all (fully occluded).
		/// 
		/// Occlusion testing help:
		/// http://www.gamedev.net/reference/programming/features/occlusionculling/
		/// http://www.ati.com/developer/gdc/D3DTutorial0_Intro_And_Features.pdf
		/// </summary>
		/// <param name="sunTestTexture">Sun test texture for this test</param>
		/// <param name="point">Point</param>
		/// <param name="size">Size how many pixels gets tested (size*size)</param>
		/// <returns>Float value between 0.0 and 1.0</returns>
		public static float OcclusionIntensity(
			Texture sunTestTexture, Point point, int size)
		{
			// Can't do occlusion test, just always return full itensity.
			if (failedToDoOcclusionTest ||
				// If device is not ok, don't execute this either.
				device == null ||
				device.Disposed ||
				// Size must also be valid, else test can't work.
				size <= 0)
				return 1.0f;

			// If point is outside of visible area return 0.
			if (point.X < -size / 2 ||
				point.X >= GraphicForm.Resolution.Width + size / 2 ||
				point.Y < -size / 2 ||
				point.Y >= GraphicForm.Resolution.Height + size / 2)
				return 0.0f;

			try
			{
				// Enable z buffer test for the occlusion test.
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = true;

				// Note: Creating and disposing the Query object each frame
				// has no performance penaltity at all, if you want to optimize
				// see the Note at GetData below!
				using (Query occlusionTest =
					new Query(GraphicForm.DirectXDevice, QueryType.Occlusion))
				{
					// Start occlusion test
					occlusionTest.Issue(IssueFlags.Begin);

					// Draw the sun on a size*size rectangle with z buffer testing on.
					sunTestTexture.RenderOnScreenWithZBuffer(new Rectangle(
						point.X - size / 2, point.Y - size / 2, size, size),
						// Use only first pixel rect (at 1, 1).
						new Rectangle(1, 1, 1, 1),
						// Use 1.0f for the farest z buffer value possible.
						1.0f);

					// Occlusion test is finished
					occlusionTest.Issue(IssueFlags.End);

					// Immediatly get data. GetData returns the number of pixels actually
					// drawn into the color buffer. If anything was already rendered there
					// with a lower z buffer value, it would be occluded and skipped.
					// Note: This is not efficent, hardware might need some time until the
					// results are available and we should try to do other work during
					// that time.
					uint pixelsRendered = (uint)occlusionTest.GetData(typeof(uint), true);

					// Get percentage of pixels rendered to the max. pixels
					// of the rendered rect (num of pixels where actually rendered)!
					float ret = (float)pixelsRendered / (float)(size * size);

					// Make sure we never return values bigger than 1.0
					if (ret > 1.0f)
						ret = 1.0f;
					return ret;
				} // using (occlusionTest)
			} // try
			catch (DirectXException)
			{
				// Ignore, most likely only the device was lost, we don't care!
				return 1.0f;
			} // catch (DirectXException)
			catch (Exception ex)
			{
				failedToDoOcclusionTest = true;
				Log.Write("Failed to to occlusion test for sun lens flare, " +
					"disabling occulsion test now: " + ex.ToString());
				return 1.0f;
			} // catch
			finally
			{
				// Disable z buffer test again
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = false;
			} // finally
		} // OcclusionIntensity(sunTestTexture, point, size)
		#endregion

		#region Enable or disable alpha blending
		/// <summary>
		/// Enable alpha blending with default SourceAlpha - InvSourceAlpha
		/// </summary>
		public static void EnableAlphaBlending()
		{
			device.RenderState.AlphaBlendEnable = true;
			device.RenderState.SourceBlend = Blend.SourceAlpha;
			device.RenderState.DestinationBlend = Blend.InvSourceAlpha;
		} // EnableAlphaBlending()

		/// <summary>
		/// Disable alpha blending
		/// </summary>
		public static void DisableAlphaBlending()
		{
			device.RenderState.AlphaBlendEnable = false;
		} // DisableAlphaBlending()
		#endregion

		#region Enable or disable linear texture filtering
		/// <summary>
		/// Enable linear texture filtering
		/// </summary>
		public static void EnableLinearTextureFiltering()
		{
			device.SamplerState[0].MinFilter = TextureFilter.Linear;
			device.SamplerState[0].MagFilter = TextureFilter.Linear;
			device.SamplerState[0].MipFilter = TextureFilter.Linear;
		} // EnableLinearTextureFiltering()

		/// <summary>
		/// Disable linear texture filtering
		/// </summary>
		public static void DisableLinearTextureFiltering()
		{
			device.SamplerState[0].MinFilter = TextureFilter.Point;
			device.SamplerState[0].MagFilter = TextureFilter.Point;
			device.SamplerState[0].MipFilter = TextureFilter.Point;
		} // DisableLinearTextureFiltering()
		#endregion

		#region Initialize DirectX, shaders, events, etc.
		protected bool fullscreen = true;
		PresentParameters presentParams = null;
		/// <summary>
		/// Initialize DirectX
		/// </summary>
		protected void InitializeDirectX(Form form,
			Size setResolution,
			bool setFullscreen)
		{
			try
			{
				resolution = setResolution;
				fullscreen = setFullscreen;
				// In windowed mode we better use the real client size
				if (setFullscreen == false)
					resolution = this.ClientSize;

				// Setup DirectX device
				presentParams = new PresentParameters();

				// Always use windowed mode for debugging and fullscreen for.
				presentParams.Windowed = setFullscreen == false;

				presentParams.PresentationInterval = PresentInterval.Default;
				presentParams.BackBufferFormat = Format.X8R8G8B8;
				presentParams.BackBufferWidth = resolution.Width;
				presentParams.BackBufferHeight = resolution.Height;

				// Default to triple buffering for performance gain,
				// if we are low on video memory and use multisampling, 1 is ok too.
				presentParams.BackBufferCount = 2;

				// Discard back buffer when swapping, its faster
				presentParams.SwapEffect = SwapEffect.Discard;

				// No multisampling yet ...
				multisamplingFormat =
					presentParams.MultiSample =
					MultiSampleType.None;
					// Doesn't work perfectly in fullscreen (often out of memory
					// exceptions).
					// Note: Doesn't work on GeForce4 either, don't support it.
					// If you want to use multi sampling you should enumerate all
					// adapters first!
					// Doesn't makes much of a difference anyway because the
					// post screen blur reduces any aliasing anyway.
					//MultiSampleType.TwoSamples;
				presentParams.MultiSampleQuality = 0;

				// Use a Z-Buffer with 32 bit if possible
				presentParams.EnableAutoDepthStencil = true;
				presentParams.AutoDepthStencilFormat = DepthFormat.D24X8;//D32;

				// For windowed, default to PresentInterval.Immediate which will
				// wait not for the vertical retrace period to prevent tearing,
				// but may introduce tearing. For full screen, default to
				// PresentInterval.Default which will wait for the vertical retrace
				// period to prevent tearing.
				if (presentParams.Windowed)
					presentParams.PresentationInterval =
						PresentInterval.Immediate;
				else
					presentParams.PresentationInterval =
						Settings.Default.WaitForVSync ?
						PresentInterval.Default :
						PresentInterval.Immediate;

				// Create device and set some render states.
				// We require a pure hardware device. This can be done much nicer,
				// see the DirectX SDK and DXUtil classes, but this would be to
				// complex for this simple game IMO.
				try
				{
					device = new Device(0, DeviceType.Hardware,
						form,
						CreateFlags.HardwareVertexProcessing |
						CreateFlags.PureDevice, presentParams);
				} // try
				catch (Exception ex)
				{
					// Not default resolution? Then reset and try again.
					if (Settings.Default.ResolutionWidth != 1024 ||
						Settings.Default.ResolutionHeight != 768)
					{
						// Make sure that we reset to the default resolution to fix this
						// problem. That should help in most of the cases.
						Settings.Default.ResolutionWidth = 1024;
						Settings.Default.ResolutionHeight = 768;
						Settings.Default.Save();

						form.Size = new Size(1024, 768);
						presentParams.BackBufferWidth = 1024;
						presentParams.BackBufferHeight = 768;

						// And initialize again!
						device = new Device(0, DeviceType.Hardware,
							form,
							CreateFlags.HardwareVertexProcessing |
							CreateFlags.PureDevice, presentParams);
					} // if (Settings.Default.ResolutionWidth)
					else
						// Rethrow, not error handled here
						throw ex;
				} // catch

				// Copy used depth format in case we need it for creating
				// zBufferSurface in RenderToTexture.
				backBufferDepthFormat = presentParams.AutoDepthStencilFormat;

				// It is very important for DirectX to dispose the objects in the
				// right order. We can't dispose the device first and then the
				// models and textures, DirectX will throw exceptions when disposing
				// meshes, shaders or textures. All the other events are important too.
				device.DeviceReset += new System.EventHandler(OnResetDevice);
				device.DeviceLost += new EventHandler(OnLostDevice);
				device.DeviceResizing += new CancelEventHandler(OnResizingDevice);
				device.Disposing += new EventHandler(OnDisposingDevice);

				// Initialize device settings for the first time.
				OnResetDevice(device, null);

				// Get device caps
				Caps caps = Manager.GetDeviceCaps(0, DeviceType.Hardware);

				// We need at least ps1.1, ps2.0 is even better, if both are
				// not found, don't use shaders at all (fallback is supported!)
				canDoPS11 = caps.PixelShaderVersion.Major >= 1;
				canDoPS20 = caps.PixelShaderVersion.Major >= 2;

				#region Get graphic card name for finding out best performance setting
				// Find out which quality setting we have set, -1 means we have
				// no setting set, find out which setting is the best for us.
				if (Settings.Default.PerformanceSetting == -1)
				{
					// Default to medium setting
					performanceSetting = PerformanceSetting.Medium;

					string graphicCardRenderer = "";
					// Try get renderer string
					try
					{
						// Now use WMI
						string stringMachineName = "localhost";
						// Connect to the remote computer
						ConnectionOptions co = new ConnectionOptions();
						// Point to machine
						ManagementScope ms = new ManagementScope(
							"\\\\" + stringMachineName + "\\root\\cimv2", co);
						// Query remote computer across the connection
						System.Management.ObjectQuery oq =
							new System.Management.ObjectQuery(
							"SELECT * FROM Win32_VideoController");
						ManagementObjectSearcher query =
							new ManagementObjectSearcher(ms, oq);
						ManagementObjectCollection queryCollection = query.Get();
						foreach (ManagementObject mo in queryCollection)
						{
							graphicCardRenderer = (string)mo["Name"];
							break;
						} // foreach 
					} // try
					catch (Exception e)
					{
						Log.Write("Failed to get renderer (graphic card name): " +
							e.ToString());
					} // catch (.)

					// If we got a NVidia or Ati card, that might be some good one.
					if (StringHelper.Contains(graphicCardRenderer,
						new string[] { "nvidia", "geforce", "gforce", "detonator", "nv4",
													 "ati", "radeon", "catalyst", "x700", "x800"}))
					{
						// Check out if this is a low spec version, only the MX version,
						// SE or LE version or even a laptop "Go" version.
						// Don't go to quality mode then!
						if (StringHelper.Contains(graphicCardRenderer,
							new string[] { "SE ", "LE ", "Go ", "5200", "6200", "Go!" }) ==
							false)
							// If it is not a cheap version, we should be able to use it!
							performanceSetting = PerformanceSetting.Quality;
					} // if
					// else its a "all-in-wonder", "riva", "tnt",
					// "matrox", "g200", "g300", "g400",
					// "Savage", "SG", "s3", "savage", "trinity", etc.
					// Stay in default mode, if card has ps1.1, use it, 
					// else render without shaders.

					// Save performance settings
					Settings.Default.PerformanceSetting =
						performanceSetting == PerformanceSetting.Fastest ? 0 :
						performanceSetting == PerformanceSetting.Medium ? 1 : 2;
					Settings.Default.Save();
				} // if
				else
				{
					performanceSetting =
						Settings.Default.PerformanceSetting == 0 ?
						PerformanceSetting.Fastest :
						Settings.Default.PerformanceSetting == 1 ?
						PerformanceSetting.Medium :
						PerformanceSetting.Quality;
				} // else
				#endregion
			} // try
			catch (DirectXException ex)
			{
				// Pass error 1 level up to the main program.
				throw new Exception(
					"Unable to create DirectX device for this game (software " +
					"rendering is not supported, it is way to slow).\n" +
					"Your graphic card does most likely do not support DirectX 7, 8 " +
					"or 9 (the game works with ps2.0, ps1.1 or no shaders at all).\n" +
					"Further information can only be determinated in the debug " +
					"mode.\n" + "Error: " + ex.Message, ex);
			} // catch (DirectXException)

			try
			{
				lineManager = new LineManager();
				parallaxShader = new ParallaxShader();
				skyCubeMap = new PreScreenSkyCubeMapping();
				lensFlare = new LensFlare(LensFlare.DefaultSunPos);
				smallFont = new Font("Arial", 10);
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to load advanced shader " + ex.ToString());
				MessageBox.Show("Failed to load advanced shader " + ex.ToString(),
					"Error");
			} // catch
		} // InitializeDirectX()
		#endregion

		#region On reset device
		/// <summary>
		/// On reset device
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">E</param>
		private void OnResetDevice(object sender, EventArgs e)
		{
#if LOG_STUFF
			Log.Write("OnResetDevice");
#endif
			if (device == null ||
				device.Disposed)
				return;

			// Make sure we use Z-Buffer
			device.RenderState.ZBufferEnable = true;
			// Use standard counter clockwise culling (lefthand system)
			device.RenderState.CullMode = Cull.CounterClockwise;
			// And lighting
			device.RenderState.Lighting = true;

			// Everything is solid
			device.RenderState.FillMode = FillMode.Solid;

			// Enable linear filtering
			GraphicForm.EnableLinearTextureFiltering();

			// Modulate texture alpha with diffuse color alpha we use!
			// Used in lens flare class and for blending out objects.
			GraphicForm.DirectXDevice.TextureState[0].AlphaOperation =
				TextureOperation.Modulate;
			GraphicForm.DirectXDevice.TextureState[0].AlphaArgument1 =
				TextureArgument.TextureColor;
			GraphicForm.DirectXDevice.TextureState[0].AlphaArgument2 =
				TextureArgument.Diffuse;

			// Leave world matrix to the standard identity
			WorldMatrix = Matrix.Identity;
			// Create view and projection matrices (default stuff)
			ViewMatrix = Matrix.LookAtLH(
				new Vector3(0.0f, 0.0f, -4.0f),
				new Vector3(0.0f, 0.0f, 0.0f),
				new Vector3(0.0f, 1.0f, 0.0f));

			BuildProjectionMatrix();

			// Create ambient light as material using our default colors.
			Material mat = new Material();
			device.Material = mat.D3DMaterial;

			// Create and enable directional light
			device.Lights[0].Type = LightType.Directional;
			device.Lights[0].Enabled = true;
			LightColor = Color.White;
			LensFlareColor = Color.White;
			SkyBackgroundColor = Color.White;
			//LightDir = DefaultLightDir;
			LightDir = -LensFlare.DefaultSunPos;

			// Normalize everything because x models are not normalized and
			// scaling the matrices would mess them up anyways :(
			// Well, this is still required for normal rendering without shaders,
			// will eat up maybe 1% performance, not so bad, only cpu anyway.
			// For shaders, this setting does not affect anything at all.
			device.RenderState.NormalizeNormals = true;

			// Accept ambient, diffuse and specular color from material
			device.RenderState.AmbientMaterialSource = ColorSource.Material;
			device.RenderState.DiffuseMaterialSource = ColorSource.Material;
			device.RenderState.SpecularMaterialSource = ColorSource.Material;
			device.RenderState.SpecularEnable = true;

			remBackSufferSurface = device.GetBackBuffer(0, 0, BackBufferType.Mono);
			remDepthStencilSurface = device.DepthStencilSurface;

			// Call all our IGraphicObjects and reset them too
			foreach (IGraphicObject graphicObj in graphicObjects)
				graphicObj.OnResetDevice();
		} // OnResetDevice(sender, e)
		#endregion

		#region On lost device
		protected bool isDeviceLost = false;
		/// <summary>
		/// Fired when the device is lost
		/// </summary>
		private void OnLostDevice(object sender, EventArgs e)
		{
#if LOG_STUFF
			Log.Write("OnLostDevice");
#endif
			try
			{
				// Oh oh, device is lost, we have to restore it in HandleApplication()
				isDeviceLost = true;

				// Call all our IGraphicObjects and reset them too
				foreach (IGraphicObject graphicObj in graphicObjects)
					graphicObj.OnLostDevice();
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to release device objects in OnLostDevice: "+
					ex.ToString());
			} // catch
		} // OnLostDevice(sender, e)
		#endregion

		#region On disposing device
		/// <summary>
		/// Add graphic object for automatic disposal in OnDisposingDevice.
		/// </summary>
		/// <param name="graphicObject">Graphic object</param>
		public static void AddGraphicObject(IGraphicObject graphicObject)
		{
			graphicObjects.Add(graphicObject);
		} // AddGraphicObject(graphicObject)

		/// <summary>
		/// Fired when device is disposing.
		/// This event is very important to first release all textures,
		/// meshes and shader effects before finally disposing the device.
		/// </summary>
		private void OnDisposingDevice(object sender, EventArgs e)
		{
#if LOG_STUFF
			Log.Write("OnDisposingDevice");
#endif
			if (device == null ||
				device.Disposed)
				return;

			try
			{
				// Kill all textures, shaders, etc.
				// Note: if we don't do that, DirectX will throw exceptions
				// while going through all Dispose methods of Shaders, Textures, etc.
				foreach (IGraphicObject graphicObject in graphicObjects)
					graphicObject.Dispose();
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to dispose device objects: " + ex.ToString());
			} // catch
			finally
			{
				// Kill device pointer
				device = null;

				// Make sure we quitting
				quit = true;
			} // finally
		} // OnDisposingDevice(sender, e)
		#endregion

		#region On resizing device
		bool remToBuildProjectionMatrix = false;
		/// <summary>
		/// On resizing device
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">E</param>
		private void OnResizingDevice(object sender, CancelEventArgs e)
		{
#if LOG_STUFF
			Log.Write("OnResizingDevice");
#endif
			if (device == null)
				return;

			try
			{
				// If the device is lost, the app is minimized or
				// not acitve, then do nothing
				if (device.CheckCooperativeLevel() == false ||
					this.WindowState == FormWindowState.Minimized ||
					this.isActive == false)
				{
					e.Cancel = true;
				} // if (device.CheckCooperativeLevel)

				// Remember we are resizing
				//currentlyResizing = true;

				if (WindowState != FormWindowState.Minimized)
				{
					// Minimum size is 320x200
					if (this.Size.Width < 320 ||
						this.Size.Height < 200)
						this.Size = new Size(320, 200);
					/*obs
					// Don't allow much more than 4/3 format (1.333f)
					else if ((float)this.Size.Width / (float)this.Size.Height > 1.4f)
						this.Size = new Size(this.Size.Width, (int)(this.Size.Width / 1.4f));
					// Don't allow much less than 5/4 format (1.25f)
					else if ((float)this.Size.Width / (float)this.Size.Height < 1.2f)
						this.Size = new Size(this.Size.Width, (int)(this.Size.Width / 1.2f));
					*/

					remToBuildProjectionMatrix = true;
				} // if (WindowState)
			} // try
			catch (Exception ex)
			{
				Log.Write("OnDeviceResizing failed: " + ex.ToString());
			} // catch (ex)
		} // OnResizingDevice(sender, e)
		#endregion

		#region On closed
		/// <summary>
		/// On closed
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnClosed(EventArgs e)
		{
#if LOG_STUFF
			Log.Write("OnClosed");
#endif
			quit = true;
			base.OnClosed(e);
		} // OnClosed()
		#endregion

		#region On size changed
		/// <summary>
		/// On size changed
		/// </summary>
		/// <param name="e">E</param>
		protected override void OnSizeChanged(EventArgs e)
		{
#if LOG_STUFF
			Log.Write("OnSizeChanged");
#endif
			base.OnSizeChanged(e);
			
			resolution = this.ClientSize;
		} // OnSizeChanged()
		#endregion

		#region On key up
		bool remToMakeScreenshot = false;
		/// <summary>
		/// Key Up?
		/// </summary>
		protected override void OnKeyUp(KeyEventArgs e)
		{
			base.OnKeyUp(e);
			switch (e.KeyCode)
			{
				case Keys.PrintScreen:
					remToMakeScreenshot = true;
					e.Handled = true;
					break;
			} // switch (KeyCode)
		} // OnKeyUp(e)
		#endregion

		#region Render
		/// <summary>
		/// Render method, handles everthing that has to be done each frame.
		/// Called by Update each frame to render everything.
		/// </summary>
		protected virtual void Render()
		{
			// Override it!
		} // Render()
		#endregion

		#region Handle device
		/// <summary>
		/// Handle device, makes sure everything is good and prepared for
		/// rendering. E.g. if the device was lost and we have to wait until
		/// we can restore it again, this method returns false and let the
		/// application know we can't render right now. It will also handle
		/// all restoring and building the projection matrix when required.
		/// </summary>
		/// <returns>True if we are good to render on the device, false otherwise.
		/// </returns>
		protected bool HandleDevice()
		{
			// We only check if form was disposed and we can quit now,
			// but quitting can be done also by setting quit to true.
			// Check if form is being disposed
			if (this.IsDisposed ||
				this.Disposing)
			{
				// Then make sure app is quiting!
				quit = true;
				return false;
			} // if (form.IsDisposed)

			bool isRenderingPaused =
				this.WindowState == FormWindowState.Minimized ||
				(isActive == false && fullscreen);

			// If not active, wait for a short while to give other apps some time.
			if (isDeviceLost ||
				isActive == false ||
				isRenderingPaused)
				Thread.Sleep(20);

			// Restore device if we are going active again!
			if (isDeviceLost &&
				isRenderingPaused == false &&
				quit == false)
			{
#if LOG_STUFF
				Log.Write("Device was lost, trying to restore");
#endif
				// No longer lost, well, at least we are thinking that right now.
				isDeviceLost = false;

				int result;
				// Check the cooperative level to see if it's ok to render
				if (device.CheckCooperativeLevel(out result) == false)
				{
					if (result == (int)ResultCode.DeviceLost)
					{
#if LOG_STUFF
						Log.Write(
							"The device has been lost but cannot be reset at this time.");
#endif
						// The device has been lost but cannot be reset at this time.  
						// So wait until it can be reset.
						Thread.Sleep(80);

						// Still lost, quit and try again next frame!
						isDeviceLost = true;
						return false;
					} // if (result)

					// Try to reset the device
					try
					{
						OnLostDevice(null, EventArgs.Empty);
						this.FormBorderStyle = fullscreen ?
							FormBorderStyle.None :
							FormBorderStyle.Sizable;
						isDeviceLost = false;
						device.Reset(presentParams);
						//do we need this? OnResetDevice(null, EventArgs.Empty);
					} // try
					catch (DeviceLostException)
					{
						// The device was lost again,
						// so continue waiting until it can be reset.
#if LOG_STUFF
						Log.Write("Device lost again while trying to reset.");
#endif
						Thread.Sleep(100);

						// Still lost, try again next frame.
						isDeviceLost = true;
						return false;
					} // catch (DeviceLostException)
					catch
					{
						// Still lost, rethrow error to find out why.
						isDeviceLost = true;
						throw;
					} // catch
				} // if (device.CheckCooperativeLevel)
			} // if (isDeviceLost)

			// When device is still lost or anything else isn't right, don't render!
			if (device == null ||
				device.Disposed ||
				isRenderingPaused ||
				isDeviceLost)
			{
				// Skip rendering
				return false;
			} // if (device)

			// Do we have to build a projection matrix?
			if (remToBuildProjectionMatrix)
			{
				remToBuildProjectionMatrix = false;

				// Build projection matrix with new size settings.
				BuildProjectionMatrix();
			} // if (remToBuildProjectionMatrix)

			// Everything is good to render (which is true most of the time).
			return true;
		} // HandleDevice()
		#endregion

		#region Update
		/// <summary>
		/// Update, called by the main Program class for each frame.
		/// Handles not only graphic, but can do all the work in derived classes
		/// required each frame (input and sound update, etc.).
		/// 
		/// Note: This hides inherited member from Control.Update, but we don't
		/// need that for our own classes.
		/// </summary>
		public new virtual void Update()
		{
			// Make a screenshot?
			if (remToMakeScreenshot)
			{
				remToMakeScreenshot = false;
				MakeScreenshot();
			} // if (remToMakeScreenshot)
		} // Update()
		#endregion

		#region Make screenshot
		#region Screenshot name builder
		/// <summary>
		/// Screenshot name builder
		/// </summary>
		/// <param name="num">Num</param>
		/// <returns>String</returns>
		static private string ScreenshotNameBuilder(int num)
		{
			return Directories.ScreenshotsDirectory + "\\" +
				Application.ProductName + " Screenshot " +
				num.ToString("0000") + ".jpg";
		} // ScreenshotNameBuilder(num)
		#endregion

		#region Get current screenshot num
		/// <summary>
		/// Get current screenshot num
		/// </summary>
		/// <returns>Int</returns>
		static private int GetCurrentScreenshotNum()
		{
			// We must search for last screenshot we can found in list using own
			// fast filesearch
			int i = 0, j = 0, k = 0, l = -1;
			// First check if at least 1 screenshot exist
			if (File.Exists(ScreenshotNameBuilder(0)) == true)
			{
				// First scan for screenshot num/1000
				for (i = 1; i < 10; i++)
				{
					if (File.Exists(ScreenshotNameBuilder(i * 1000)) == false)
						break;
				} // for (i)

				// This i*1000 does not exist, continue scan next level
				// screenshotnr/100
				i--;
				for (j = 1; j < 10; j++)
				{
					if (File.Exists(ScreenshotNameBuilder(i * 1000 + j * 100)) == false)
						break;
				} // for (j)

				// This i*1000+j*100 does not exist, continue scan next level
				// screenshotnr/10
				j--;
				for (k = 1; k < 10; k++)
				{
					if (File.Exists(ScreenshotNameBuilder(
						i * 1000 + j * 100 + k * 10)) == false)
						break;
				} // for (k)

				// This i*1000+j*100+k*10 does not exist, continue scan next level
				// screenshotnr/1
				k--;
				for (l = 1; l < 10; l++)
				{
					if (File.Exists(ScreenshotNameBuilder(
						i * 1000 + j * 100 + k * 10 + l)) == false)
						break;
				} // for (l)

				// This i*1000+j*100+k*10+l does not exist, we have now last
				// screenshot nr!!!
				l--;
			} // if (File.Exists)

			return i * 1000 + j * 100 + k * 10 + l;
		} // GetCurrentScreenshotNum()

		/// <summary>
		/// Internal screenshot number (will increase by one each screenshot)
		/// </summary>
		private int screenshotNr = GetCurrentScreenshotNum();
		#endregion

		#region Get encoder info
		/// <summary>
		/// Get encoder info from mime type.
		/// </summary>
		public static ImageCodecInfo GetEncoderInfo(String mimeType)
		{
			int j;
			ImageCodecInfo[] encoders;
			encoders = ImageCodecInfo.GetImageEncoders();
			for (j = 0; j < encoders.Length; ++j)
			{
				if (encoders[j].MimeType == mimeType)
					return encoders[j];
			} // for (=, <, ++)
			return null;
		} // GetEncoderInfo(mimeType)
		#endregion

		/// <summary>
		/// Make screenshot
		/// </summary>
		protected void MakeScreenshot()
		{
			try
			{
				//NOTE: This doesn't always work on all cards, especially if
				// desktop mode switches in fullscreen mode!

				screenshotNr++;
				// Make sure screenshots directory exists
				if (Directory.Exists(Directories.ScreenshotsDirectory) == false)
					Directory.CreateDirectory(Directories.ScreenshotsDirectory);
				
				// Create bitmap
				Bitmap bitmap = new Bitmap(
					this.ClientSize.Width, this.ClientSize.Height);
				// Create a graphics object from the bitmap
				System.Drawing.Graphics gfxBitmap =
					System.Drawing.Graphics.FromImage(bitmap);
				// Get a device context for the bitmap
				IntPtr hdcBitmap = gfxBitmap.GetHdc();

				// Get a device context for our current window
				IntPtr hWnd = this.Handle;
				IntPtr hdcWindow = WindowsHelper.NativeMethods.GetWindowDC(hWnd);

				// For windowed mode only save the window content

				// Substract location from start render location,
				// sadly .NET doesn't support the minus operator for Points.
				Point screenPoint = this.PointToScreen(new System.Drawing.Point(0, 0));
				screenPoint.X -= this.Location.X;
				screenPoint.Y -= this.Location.Y;

				// Use BitBlt to copy the window content to the bitmap
				WindowsHelper.NativeMethods.BitBlt(hdcBitmap, 0, 0,
					this.ClientSize.Width, this.ClientSize.Height,
					hdcWindow, screenPoint.X, screenPoint.Y,
					(int)WindowsHelper.BitBltSrcCopy);

				// Release the bitmap'str device context
				gfxBitmap.ReleaseHdc(hdcBitmap);
				WindowsHelper.NativeMethods.ReleaseDC(hWnd, hdcWindow);

				// Dispose of the bitmap'str graphics object
				gfxBitmap.Dispose();

				// Now save the bitmap (as a jpg, not as easy as it sounds,
				// we want to set 90% quality for our bitmap).
				ImageCodecInfo myImageCodecInfo;
				Encoder myEncoder;
				EncoderParameter myEncoderParameter;
				EncoderParameters myEncoderParameters;

				// Get an ImageCodecInfo object that represents the JPEG codec.
				myImageCodecInfo = GetEncoderInfo("image/jpeg");

				// Create an Encoder object based on the GUID for the Quality parameter
				// category.
				myEncoder = Encoder.Quality;
				myEncoderParameters = new EncoderParameters(1);

				// Save the bitmap as a JPEG file with quality level to qualityPercent
				// value. Use 90% for quality!
				myEncoderParameter =
					new EncoderParameter(myEncoder, (long)90);
				myEncoderParameters.Param[0] = myEncoderParameter;

				// Save file
				bitmap.Save(ScreenshotNameBuilder(screenshotNr),
					myImageCodecInfo, myEncoderParameters);
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to save Screenshot: " + ex.ToString());
			} // catch (ex)
		} // MakeScreenshot()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test is visible
		/// </summary>
		[Test]
		public static void TestIsVisible()
		{
			Mesh testMesh = null;
			Vector3 position = new Vector3(0, 0, 0);

			GameForm.StartTest("TestModel",
				delegate // Init
				{
					testMesh = Mesh.Teapot(DirectXDevice);
					GameForm.TestGameForm.camera.SetPosition(new Vector3(0, 0, -80));
				},
				delegate // Render loop
				{
					if (IsVisible(position, 1.5f))
					{
						GraphicForm.WorldMatrix = Matrix.Scaling(25, 25, 25);
						testMesh.DrawSubset(0);
					} // testMesh

					SmallFont.WriteText(1, 20,
						"Position " + position + " is visible: " +
						IsVisible(position, 1.0f));
					SmallFont.WriteText(1, 100,
						"Screen pos=" + Convert3DPointTo2D(position));
				});
		} // TestIsVisible()
#endif
		#endregion
	} // class Graphic
} // namespace RocketCommander.Graphics
