// Project: Rocket Commander, File: LensFlare.cs
// Namespace: RocketCommander.Graphics, Class: LensFlare
// Path: C:\code\RocketCommander\Graphics, Author: Abi
// code lines: 137, Size of file: 3,82 KB
// Creation date: 14.07.2005 13:25
// Last modified: 04.12.2005 03:31
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Drawing;
#if DEBUG
using NUnit.Framework;
#endif
using RocketCommander.Helpers;
using RocketCommander.Graphics;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Material = RocketCommander.Graphics.Material;
using RocketCommander.Game;
#endregion

namespace RocketCommander.Graphics
{
	/// <summary>
	/// Lens flare class for simple 2d lens flare rendering.
	/// 
	/// If you want to read more about lens flares I suggest the following
	/// articles:
	/// http://www.gamedev.net/reference/articles/article813.asp
	/// http://www.gamedev.net/reference/articles/article874.asp
	/// Game Programming Gems, Chapter 5.1
	/// Game Programming Gems 2, Chapter 5.5
	/// </summary>
	public class LensFlare : IGraphicObject
	{
		#region Variables
		/// <summary>
		/// Default sun position!
		/// </summary>
		public static Vector3 DefaultSunPos = new Vector3(0, +12500, -20000);

		/// <summary>
		/// Returns rotated sun position for the game.
		/// </summary>
		/// <param name="rotation">Rotation</param>
		/// <returns>Vector 3</returns>
		public static Vector3 RotateSun(float rotation)
		{
			Vector3 sunPos = DefaultSunPos;
			Vector2 right = new Vector2(
				+(float)Math.Cos(rotation),
				+(float)Math.Sin(rotation));
			Vector2 up = new Vector2(
				+(float)Math.Sin(rotation),
				-(float)Math.Cos(rotation));

			// Only rotate x and z
			return new Vector3(
				-right.X * sunPos.X - up.X * sunPos.Z,
				sunPos.Y,
				-right.Y * sunPos.X - up.Y * sunPos.Z);
		} // RotateSun(rotation)

		/// <summary>
		/// Lens flare origin in 3D.
		/// </summary>
		private static Vector3 lensOrigin3D;

		/// <summary>
		/// Screen flare size (resolution dependant, 250 is for 1024*768).
		/// </summary>
		private static int ScreenFlareSize = 250;

		/// <summary>
		/// Flare texture types.
		/// </summary>
		protected const int
			SunFlareType = 0,
			GlowFlareType = 1,
			LensFlareType = 2,
			StreaksType = 3,
			RingType = 4,
			HaloType = 5,
			CircleType = 6,
			NumberOfFlareTypes = 7;

		/// <summary>
		/// Flare materials
		/// </summary>
		protected Material[] flareMaterials = new Material[NumberOfFlareTypes];

		/// <summary>
		/// Flare texture names
		/// </summary>
		string[] flareTextureNames = new string[]
			{
				Directories.TextureEffectsSubDirectory+"\\Sun.dds",
				Directories.TextureEffectsSubDirectory+"\\Glow.dds",
				Directories.TextureEffectsSubDirectory+"\\Lens.dds",
				Directories.TextureEffectsSubDirectory+"\\Streaks.dds",
				Directories.TextureEffectsSubDirectory+"\\Ring.dds",
				Directories.TextureEffectsSubDirectory+"\\Halo.dds",
				Directories.TextureEffectsSubDirectory+"\\Circle.dds",
			};

		/// <summary>
		/// Load textures, should be called in constructor.
		/// </summary>
		private void LoadTextures()
		{
			for (int num = 0; num < NumberOfFlareTypes; num++)
				flareMaterials[num] = new Material(flareTextureNames[num]);
		} // LoadTextures()
		#endregion

		#region Flare data struct
		/// <summary>
		/// Flare data struct for the quick and easy flare type list below.
		/// </summary>
		protected struct FlareData
		{
			/// <summary>
			/// Type of flare, see above.
			/// </summary>
			public int type;
			/// <summary>
			/// Position of flare (1=origin, 0=center of screen, -1=other side)
			/// </summary>
			public float position;
			/// <summary>
			/// Scale of flare in relation to MaxFlareSize.
			/// </summary>
			public float scale;
			/// <summary>
			/// Color of this flare.
			/// </summary>
			public Color color;

			/// <summary>
			/// Constructor to set all values.
			/// </summary>
			/// <param name="setType">Set type</param>
			/// <param name="setPosition">Set position</param>
			/// <param name="setScale">Set scale</param>
			/// <param name="setColor">Set Color</param>
			public FlareData(int setType, float setPosition,
				float setScale, Color setColor)
			{
				type = setType;
				position = setPosition;
				scale = setScale;
				color = setColor;
			} // FlareData(setType, setPosition, setScale)
		} // struct FlareData

		/// <summary>
		/// Flare types for the lens flares.
		/// </summary>
		protected FlareData[] flareTypes = new FlareData[]
		{
			// Small red/yellow/gray halo behind sun
			new FlareData(
			CircleType, 1.2f, 0.55f, Color.FromArgb(20, 175, 175, 255)),

			// The sun, sun+streaks+glow+red ring
			new FlareData(
			SunFlareType, 1.0f, 0.6f, Color.FromArgb(255, 255, 255, 255)),
			new FlareData(
			StreaksType, 1.0f, 1.5f, Color.FromArgb(128, 255, 255, 255)),
			new FlareData(
			GlowFlareType, 1.0f, 1.7f, Color.FromArgb(100, 255, 255, 200)),
			//new FlareData(
			//RingNum, 1.0f, 0.9f, Color.FromArgb(150, 255, 120, 120)),

			// 3 blue circles at 0.5 distance
			new FlareData(
			CircleType, 0.5f, 0.12f, Color.FromArgb(35, 60, 60, 180)),
			new FlareData(
			CircleType, 0.45f, 0.46f, Color.FromArgb(60, 100, 100, 200)),
			new FlareData(
			CircleType, 0.4f, 0.17f, Color.FromArgb(40, 120, 120, 220)),

			new FlareData(
			RingType, 0.15f, 0.2f, Color.FromArgb(100, 60, 60, 255)),
			new FlareData(
			LensFlareType, -0.5f, 0.2f, Color.FromArgb(130, 255, 60, 60)),
			new FlareData(
			LensFlareType, -0.15f, 0.15f, Color.FromArgb(90, 255, 60, 60)),
			new FlareData(
			HaloType, -0.3f, 0.6f, Color.FromArgb(180, 60, 60, 255)),
			
			// 3 red halos and circles on the opposite side of the blue halos
			new FlareData(
			HaloType, -0.4f, 0.2f, Color.FromArgb(98, 220, 80, 80)),
			//new FlareData(
			//HaloNum, -0.45f, 0.6f, Color.FromArgb(95, 220, 80, 80)),
			new FlareData(
			CircleType, -0.5f, 0.1f, Color.FromArgb(85, 220, 80, 80)),

			new FlareData(
			HaloType, -0.6f, 0.5f, Color.FromArgb(80, 60, 60, 255)),
			new FlareData(
			RingType, -0.8f, 0.3f, Color.FromArgb(110, 90, 60, 255)),

			new FlareData(
			HaloType, -0.95f, 0.5f, Color.FromArgb(120, 60, 60, 255)),
			new FlareData(
			CircleType, -1.0f, 0.15f, Color.FromArgb(85, 60, 60, 255)),
		}; // flareTypes[]
		#endregion

		#region Properties
		/// <summary>
		/// Origin 3D
		/// </summary>
		/// <returns>Vector3</returns>
		public static Vector3 Origin3D
		{
			set
			{
				lensOrigin3D = value;
			} // set
			get
			{
				return lensOrigin3D;
			} // get
		} // Origin3D
		#endregion

		#region Constructors
		/// <summary>
		/// Create lens flare base
		/// </summary>
		public LensFlare(Vector3 setLensOrigin3D)
		{
			lensOrigin3D = setLensOrigin3D;
			LoadTextures();

			GraphicForm.AddGraphicObject(this);
		} // LensFlare(setLensOrigin3D)
		#endregion

		#region Disposing
		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			// Unused ...
		} // Dispose()
		#endregion

		#region IGraphicObject Members
		/// <summary>
		/// On reset device
		/// </summary>
		public void OnResetDevice()
		{
			// occlusionTest will be regenerated automatically.
		} // OnResetDevice()

		/// <summary>
		/// On lost device
		/// </summary>
		public void OnLostDevice()
		{
			// Only occlusionTest is disposed.
			//unused: Dispose();
		} // OnLostDevice()
		#endregion

		#region Render
		/// <summary>
		/// Current sun intensity used for our lens flare effect.
		/// </summary>
		float sunIntensity = 0.0f;

		/// <summary>
		/// Render lens flare.
		/// Checks the zbuffer if objects are occluding the lensflare sun.
		/// </summary>
		public void Render(Color sunColor)
		{
			GraphicForm.DirectXDevice.RenderState.AlphaBlendEnable = true;
			GraphicForm.DirectXDevice.RenderState.SourceBlend = Blend.SourceAlpha;
			GraphicForm.DirectXDevice.RenderState.DestinationBlend = Blend.One;

			ScreenFlareSize = 250 * GraphicForm.Resolution.Width / 1024;
			Vector3 relativeLensPos = lensOrigin3D + GraphicForm.CameraPos;

			// Only show lens flare if facing in the right direction
			if (GraphicForm.IsInFrontOfCamera(relativeLensPos) == false)
				return;
			// Convert 3D point to 2D!
			Point lensOrigin =
				GraphicForm.Convert3DPointTo2D(relativeLensPos);

			// Check sun occlusion itensity and fade it in and out!
			float thisSunIntensity = GraphicForm.OcclusionIntensity(
				flareMaterials[SunFlareType].diffuseTexture,
				lensOrigin, ScreenFlareSize / 5);
			sunIntensity = thisSunIntensity * 0.1f + sunIntensity * 0.9f;

			// We can skip rendering the sun if the itensity is to low
			if (sunIntensity < 0.01f)
				return;

			Size res = GraphicForm.Resolution;
			Point center = new Point(res.Width / 2, res.Height / 2);
			Point relOrigin = new Point(
				center.X - lensOrigin.X, center.Y - lensOrigin.Y);

			// Check if origin is on screen, fade out at borders
			float alpha = 1.0f;
			float distance = Math.Abs(Math.Max(relOrigin.X, relOrigin.Y));
			if (distance > res.Height / 1.75f)
			{
				distance -= res.Height / 1.75f;
				// If distance is more than half the resolution, don't show anything!
				if (distance > res.Height / 1.75f)
					return;
				alpha = 1.0f - (distance / ((float)res.Height / 1.75f));
				if (alpha > 1.0f)
					alpha = 1.0f;
			} // if (distance)

			// Use square of sunIntensity for lens flares because we want
			// them to get very weak if sun is not fully visible.
			alpha *= sunIntensity * sunIntensity;

			foreach (FlareData data in flareTypes)
				if (flareMaterials[data.type].diffuseTexture != null)
				{
					int size = (int)(ScreenFlareSize * data.scale);
					flareMaterials[data.type].diffuseTexture.RenderOnScreen(
						new Rectangle(
						(int)(center.X - relOrigin.X * data.position - size / 2),
						(int)(center.Y - relOrigin.Y * data.position - size / 2),
						size, size),
						ColorHelper.MultiplyColors(sunColor, data.color),
						((float)data.color.A / 255.0f) *
						// For the sun and glow flares try always to use max. intensity
						(data.type == SunFlareType || data.type == GlowFlareType ?
						sunIntensity : alpha));
				} // foreach if
			GraphicForm.DirectXDevice.RenderState.AlphaBlendEnable = false;
		} // Render(sunColor)
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test lens flare
		/// </summary>
		[Test]
		public static void TestLensFlare()
		{
			Vector3 sunPos = LensFlare.DefaultSunPos;

			GameForm.StartTest("TestModel",
				delegate // Init
				{
					//lensFlare = new LensFlare(sunPos);
				},
				delegate // Render loop
				{
					GraphicForm.SmallFont.WriteText(1, 20,
						"Sun pos to 2d=" + GraphicForm.Convert3DPointTo2D(sunPos));
					GraphicForm.SmallFont.WriteText(1, 40,
						"Sun pos in front=" + GraphicForm.IsInFrontOfCamera(sunPos));
					Vector4 result = Vector4.Transform(
						new Vector4(sunPos.X, sunPos.Y, sunPos.Z, 1),
						GraphicForm.ViewMatrix * GraphicForm.ProjectionMatrix);
					GraphicForm.SmallFont.WriteText(1, 60,
						"result=" + result);
				});
		} // TestLensFlare()
#endif
		#endregion
	} // class LensFlare
} // namespace RocketCommander.Graphics
