﻿// Project: Rocket Commander, File: ElapsedTime.cs
// Namespace: RocketCommander.Helpers, Class: ElapsedTime
// Creation date: 01.11.2005 05:00
// Last modified: 05.12.2005 03:06
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections;
using System.Text;
using System.Threading;
#endregion

namespace RocketCommander.Helpers
{
	/// <summary>
	/// Helper class to manage elapsed time.
	/// Can be accessed from everywhere to find out how much time has
	/// passed since we started up, how much time the last frame needed, etc.
	/// GraphicForm.Update will update this class every tick with Update().
	/// </summary>
	public class ElapsedTime
	{
		#region Variables
		/// <summary>
		/// For frame per seconds calculations: Start tick in ns
		/// (when start counting frames)
		/// </summary>
		private static long startTimeNs =
			WindowsHelper.ConvertToNs(WindowsHelper.GetPerformanceCounter());

		/// <summary>
		/// For time elapsed the last frame: last tick time
		/// </summary>
		private static long lastTimeNs = 0;

		/// <summary>
		/// For frame per seconds calculations: Elapsed ticks
		/// (will be updated every frame)
		/// </summary>
		private static long elapsedTimeNs = 0;

		/// <summary>
		/// Elasped time last frame in ns, is calculated in Update.
		/// Ms time is NOT just ns/1000, because we would miss a couple of
		/// ms per frame due rounding errors.
		/// Btw: Start with 1ms to help some tests avoid the devide through zero
		/// problem.
		/// </summary>
		private static int
			elapsedTimeLastFrameNs = 1000,
			elapsedTimeLastFrameMs = 1;

		/// <summary>
		/// Helper for calculating frames per second. 
		/// </summary>
		private static long
			startTimeThisSecond = 0;

		/// <summary>
		/// For more accurate frames per second calculations,
		/// just count for one second, then fpsLastSecond is updated.
		/// Btw: Start with 1 to help some tests avoid the devide through zero
		/// problem.
		/// </summary>
		private static int
			frameCountThisSecond = 0,
			totalFrameCount = 0,
			fpsLastSecond = 1;
		#endregion

		#region Public properties to get ms, ns and fps
		/// <summary>
		/// Get smoothend elapsed time in ns of the last frame
		/// </summary>
		public static long NsLastFrame
		{
			get
			{
				return elapsedTimeLastFrameNs;
			} // get
		} // NsLastFrame
		/// <summary>
		/// Get total elapsed time in ns
		/// </summary>
		public static long NsTotal
		{
			get
			{
				return elapsedTimeNs;
			} // get
		} // NsTotal

		/// <summary>
		/// Get elapsed time in ms of the last frame.
		/// Now updated with ns percision (not devided through 1000 anymore
		/// losing .9 .8 .7 floating comma parts of ms, this gives you
		/// every inaccurate timing when going over 100 up to 1000 fps)!
		/// </summary>
		public static int MsLastFrame
		{
			get
			{
				return elapsedTimeLastFrameMs;
			} // get
		} // MsLastFrame
		/// <summary>
		/// Get total elapsed time in ms
		/// </summary>
		public static int MsTotal
		{
			get
			{
				return (int)(elapsedTimeNs / 1000);
			} // get
		} // MsTotal

		/// <summary>
		/// Frames per second, will always return 1 or higher (returning 0
		/// is gay because we might want to devide through this value).
		/// </summary>
		public static int FramesPerSecond
		{
			get
			{
				if (fpsLastSecond <= 0)
					return 1;
				return fpsLastSecond;
			} // get
		} // FramesPerSecond

		/// <summary>
		/// Total number of frames this application runs.
		/// </summary>
		/// <returns>Int</returns>
		public static int TotalFrames
		{
			get
			{
				return totalFrameCount;
			} // get
		} // TotalFrames

		/// <summary>
		/// Move factor per second, when we got 1 fps, this will be 1.0f,
		/// when we got 100 fps, this will be 0.01f.
		/// Very useful for physics and movement, this way we can easily
		/// make objects move at specific speed not frame rate dependend.
		/// Note: For multithreaded physics you don't need this!
		/// </summary>
		public static float MoveFactorPerSecond
		{
			get
			{
				return elapsedTimeLastFrameNs / 1000000.0f;
			} // get
		} // MoveFactorPerSecond
		#endregion

		#region Update all values, should be called each frame
		/// <summary>
		/// Update time values, elapsed ms, etc.
		/// Will be called once per frame (at the beginning).
		/// </summary>
		public static void Update()
		{
			// Get current time in ns
			long currentTimeNs =
				(long)(WindowsHelper.GetPerformanceCounter() *
				1000000 / WindowsHelper.GetPerformanceFrequency());
			// If lastTimeNs is unused, just set it startTime.
			if (lastTimeNs == 0)
			{
				// Use start time
				lastTimeNs = startTimeNs;
				// If difference is too high cap at 100ms.
				// This is much better than to use any very high value due the
				// application startup process (which can be several seconds).
				if (currentTimeNs - lastTimeNs > 100000)
					lastTimeNs = currentTimeNs - 100000;
			} // if (lastTimeNs)

			// Real time elapsed since last tick time
			elapsedTimeLastFrameNs = (int)(currentTimeNs - lastTimeNs);

			// We never miss any ms that elapses, we use percission of ns!
			// This means even if we got updates smaller than a ms (e.g. 300ns),
			// we will maybe get some updateMs with 0, but then we get an updateMs
			// with 1. This wont happen on normal computers, but anyway
			// we need this percision for high performance anyway (e.g. update
			// with 1900ns to time with 200ns should result in 2ms update and
			// not in 1ms (=1900ns/1000)!)
			elapsedTimeLastFrameMs = (int)(
				((elapsedTimeNs + elapsedTimeLastFrameNs) / 1000) -
				(elapsedTimeNs / 1000));

			// Only increase lastTimeNs by smoothendElapsedLastFrameTimeNs
			// to make sure we never miss any ns!
			lastTimeNs += elapsedTimeLastFrameNs;

			// Same for elapsed time (should be in sync with
			// elapsed time last frame)
			elapsedTimeNs += elapsedTimeLastFrameNs;

			// Increase frame counter for FramesPerSecond
			frameCountThisSecond++;
			totalFrameCount++;

			// One second elapsed?
			if (Math.Abs((int)(currentTimeNs - startTimeThisSecond)) > 1000000)
			{
				// Calc fps
				fpsLastSecond = (int)((float)(frameCountThisSecond * 1000000) /
					(currentTimeNs - startTimeThisSecond));
				// Reset startSecondTick and repaintCountSecond
				startTimeThisSecond = currentTimeNs;
				frameCountThisSecond = 0;
			} // if (Math.Abs)
		} // Update()
		#endregion
	}	// class ElapsedTime
} // namespace RocketCommander.Helpers
