// Project: Rocket Commander, File: KeyboardInput.cs
// Namespace: RocketCommander, Class: KeyboardInput
// Path: C:\code\RocketCommander\Input, Author: Abi
// Code lines: 120, Size of file: 2,14 KB
// Creation date: 01.11.2005 04:20
// Last modified: 24.11.2005 19:10
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Windows.Forms;
using System.Threading;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.DirectInput;
using RocketCommander.Helpers;
#endregion

namespace RocketCommander.Input
{
	/// <summary>
	/// Helper class to control to keyboard, using a DirectInput device.
	/// </summary>
	public class KeyboardInput
	{
		#region Variables
		/// <summary>
		/// DirectInput keyboard device
		/// </summary>
		private Device keyboard = null;

		/// <summary>
		/// State
		/// </summary>
		private KeyboardState state;
		#endregion

		#region Properties
		/// <summary>
		/// Left cursor key pressed?
		/// </summary>
		/// <returns>Bool</returns>
		public bool Left
		{
			get
			{
				return state != null &&
					state[Key.LeftArrow];
			} // get
		} // Left

		/// <summary>
		/// Right cursor key pressed?
		/// </summary>
		/// <returns>Bool</returns>
		public bool Right
		{
			get
			{
				return state != null &&
					state[Key.RightArrow];
			} // get
		} // Right

		/// <summary>
		/// Up cursor key pressed?
		/// </summary>
		/// <returns>Bool</returns>
		public bool Up
		{
			get
			{
				return state != null &&
					state[Key.UpArrow];
			} // get
		} // Up

		/// <summary>
		/// Down cursor key pressed?
		/// </summary>
		/// <returns>Bool</returns>
		public bool Down
		{
			get
			{
				return state != null &&
					state[Key.DownArrow];
			} // get
		} // Down

		/// <summary>
		/// Escape
		/// </summary>
		/// <returns>Bool</returns>
		public bool Escape
		{
			get
			{
				return state != null &&
					state[Key.Escape];
			} // get
		} // Escape

		bool lastFrameEscapePressed = false;
		/// <summary>
		/// Escape just pressed
		/// </summary>
		/// <returns>Bool</returns>
		public bool EscapeJustPressed
		{
			get
			{
				return Escape &&
					lastFrameEscapePressed == false;
			} // get
		} // EscapeJustPressed

		/// <summary>
		/// Keyboard state, use this to retrieve more keystates.
		/// For example: keyboardInput.State[Key.F1] ...
		/// </summary>
		/// <returns>State</returns>
		public KeyboardState State
		{
			get
			{
				return state;
			} // get
		} // State
		#endregion

		#region Constructor
		/// <summary>
		/// Create keyboard input
		/// </summary>
		/// <param name="form">Form</param>
		public KeyboardInput(Form form)
		{
			// Create keyboard device
			keyboard = new Device(SystemGuid.Keyboard);

			// Allow other apps to have keyboard input as well
			keyboard.SetCooperativeLevel(form,
				CooperativeLevelFlags.Background |
				CooperativeLevelFlags.NonExclusive);

			// Acquire keyboard
			keyboard.Acquire();
		} // KeyboardInput(form)
		#endregion

		#region Update
		/// <summary>
		/// Update the state of the keyboard
		/// </summary>
		public void Update()
		{
			// Remember state from last frame
			lastFrameEscapePressed = Escape;

			// Check the keyboard state
			state = keyboard.GetCurrentKeyboardState();
		} // Update()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test keyboard input
		/// </summary>
		[Test]
		public static void TestKeyboardInput()
		{
			TestForm testForm = new TestForm("TestKeyboardInput");

			KeyboardInput keyboardInput = new KeyboardInput(testForm);
			// Run application
			WindowsHelper.ForceForegroundWindow(testForm);
			testForm.Show();
			while (testForm.IsDisposed == false)
			{
				keyboardInput.Update();
				testForm.Text = "TestKeyboardInput: " +
					"Left=" + keyboardInput.Left + ", " +
					"Right=" + keyboardInput.Right + ", " +
					"Up=" + keyboardInput.Up + ", " +
					"Down=" + keyboardInput.Down;
				Application.DoEvents();
			} // while (testForm.IsDisposed)
		} // TestKeyboardInput()
#endif
		#endregion
	} // class KeyboardInput
} // namespace RocketCommander.Input
