// Project: Rocket Commander, File: MouseInput.cs
// Namespace: RocketCommander.Input, Class: MouseInput
// Path: C:\code\RocketCommander\Input, Author: Abi
// Code lines: 105, Size of file: 2,09 KB
// Creation date: 01.11.2005 04:20
// Last modified: 01.11.2005 05:36
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Windows.Forms;
using System.Threading;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.DirectInput;
using RocketCommander.Helpers;
#endregion

namespace RocketCommander.Input
{
	/// <summary>
	/// Helper class to get mouse user input, uses a DirectInput device.
	/// </summary>
	public class MouseInput
	{
		#region Variables
		/// <summary>
		/// The DirectInput mouse device
		/// </summary>
		private Device mouse = null;

		/// <summary>
		/// For x and y movement, remember the virtual movement
		/// and interpolate for xMovement and yMovement to get better
		/// results (else movement is way to blocky!)
		/// </summary>
		private float virtualMovementX = 0,
			virtualMovementY = 0,
			smoothendMovementX = 0,
			smoothendMovementY = 0;
		/// <summary>
		/// Current x, y movement and wheel delta
		/// </summary>
		private float xMovement = 0.0f,
			yMovement = 0.0f,
			wheelDelta = 0.0f;
		/// <summary>
		/// Is left or right mouse button pressed?
		/// </summary>
		private bool leftButtonPressed = false,
			rightButtonPressed = false;
		#endregion

		#region Properties
		/// <summary>
		/// Get mouse x movement.
		/// </summary>
		/// <returns>Float</returns>
		public float XMovement
		{
			get
			{
				return xMovement;
			} // get
		} // XMovement

		/// <summary>
		/// Get mouse y movement.
		/// </summary>
		/// <returns>Float</returns>
		public float YMovement
		{
			get
			{
				return yMovement;
			} // get
		} // YMovement

		/// <summary>
		/// Get mouse wheel delta.
		/// </summary>
		/// <returns>Float</returns>
		public float WheelDelta
		{
			get
			{
				return wheelDelta;
			} // get
		} // WheelDelta

		/// <summary>
		/// Is left mouse button pressed?
		/// </summary>
		/// <returns>Bool</returns>
		public bool LeftButtonPressed
		{
			get
			{
				return leftButtonPressed;
			} // get
		} // LeftButtonPressed

		/// <summary>
		/// Is right mouse button pressed?
		/// </summary>
		/// <returns>Bool</returns>
		public bool RightButtonPressed
		{
			get
			{
				return rightButtonPressed;
			} // get
		} // RightButtonPressed
		#endregion

		#region Constructor
		/// <summary>
		/// Create mouse input
		/// </summary>
		/// <param name="form">Form</param>
		public MouseInput(Form form)
		{
			// Create mouse device
			mouse = new Device(SystemGuid.Mouse);

			// Allow other apps to use the mouse too, everything else would be crazy.
			mouse.SetCooperativeLevel(form,
				CooperativeLevelFlags.Background |
				CooperativeLevelFlags.NonExclusive);

			// Set the data format to the mouse2 pre-defined format.
			mouse.SetDataFormat(DeviceDataFormat.Mouse);

			// Use relative axis mode to get moving differences, for absolut mouse
			// position use the Mouse events in GameForm (using Windows events).
			mouse.Properties.AxisModeAbsolute = false;

			// Acquire mouse
			mouse.Acquire();

			// Perform the first update
			Update();
		} // MouseInput(form)
		#endregion

		#region Update
		/// <summary>
		/// Update method, call it every frame.
		/// Gets the current position and button states.
		/// </summary>
		public void Update()
		{
			// Check the mouse state
			MouseState state = mouse.CurrentMouseState;

			// Copy over data
			virtualMovementX += state.X;
			virtualMovementY += state.Y;
			wheelDelta = state.Z;

			// Slowly interpolate from current movement to virtual movement
			xMovement = xMovement * 0.75f +
				(virtualMovementX - smoothendMovementX) * 0.25f;
			yMovement = yMovement * 0.75f +
				(virtualMovementY - smoothendMovementY) * 0.25f;
			smoothendMovementX = virtualMovementX;
			smoothendMovementY = virtualMovementY;

			// Check if left or right button is pressed
			byte[] buttons = state.GetMouseButtons();
			leftButtonPressed =
				buttons.Length > 0 &&
				buttons[0] != 0;
			rightButtonPressed =
				buttons.Length > 1 &&
				buttons[1] != 0;
		} // Update()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test mouse input
		/// </summary>
		[Test]
		public static void TestMouseInput()
		{
			TestForm testForm = new TestForm("TestMouseInput");

			MouseInput mouseInput = new MouseInput(testForm);
			// Run application
			WindowsHelper.ForceForegroundWindow(testForm);
			testForm.Show();
			while (testForm.IsDisposed == false)
			{
				mouseInput.Update();
				testForm.Text = "TestMouseInput: " +
					"X=" + mouseInput.XMovement + ", " +
					"Y=" + mouseInput.XMovement + ", " +
					"Wheel=" + mouseInput.WheelDelta + ", " +
					"LeftButton=" + mouseInput.LeftButtonPressed + ", " +
					"RightButton=" + mouseInput.RightButtonPressed;
				Application.DoEvents();
			} // while (testForm.IsDisposed)
		} // TestMouseInput()
#endif
		#endregion
	} // class MouseInput
} // namespace RocketCommander.Input
