// Project: Rocket Commander, File: XInput.cs
// Namespace: RocketCommander.Helpers, Class: XInput
// Path: C:\code\Rocket Commander\Input, Author: Abi
// Code lines: 916, Size of file: 28,38 KB
// Creation date: 09.03.2006 05:36
// Last modified: 11.03.2006 03:01
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
#endregion

namespace RocketCommander.Helpers
{
	/// <summary>
	/// XInput helpr class, rufly based on XIpnut by mceXInput by Casey Chesnut
	/// check out http://www.brains-n-brawn.com/default.aspx?vDir=mcexinput
	/// for more information.
	/// This class manages all the input states we need for Rocket Commander
	/// and allows the application just to check if any of these values are
	/// used. If no XBox 360 Controller is attached, all methods will always
	/// return 0 or false depending on their return type.
	/// </summary>
	public class XInput
	{
		#region Constants
		/// <summary>
		/// Max. number of controllers supported, usually 4 like for the XBox 360.
		/// </summary>
		public const int MaxNumberOfControllers = 4;

		/// <summary>
		/// Thumb ranges
		/// </summary>
		private const short MinThumbRange = -32768,
			MaxThumbRange = 32767;

		/// <summary>
		/// Gamepad thresholds
		/// </summary>
		private const uint GamePadTriggerThreshold = 30,
			GamePadLeftThumbDeadzone = 7849,
			GamePadRightThumbDeadzone = 8689;

		/// <summary>
		/// Trigger ranges
		/// </summary>
		private const byte MinTrigger = 0,
			MaxTrigger = 255;

		/// <summary>
		/// Vibration motor speeds
		/// </summary>
		private const ushort MinEngineSpeed = 0,
			MaxEngineSpeed = 65535;

		/// <summary>
		/// Flags to pass to XInputGetCapabilities
		/// </summary>
		private const uint FlagGamePad = 0x00000001;

		/// <summary>
		/// Device types available in XInput Capabilities
		/// </summary>
		private const uint DeviceTypeGamePad = 0x01;

		/// <summary>
		/// Device subtypes available in XInput Capabilities
		/// </summary>
		private const uint DeviceSubTypeGamePad = 0x01;

		/// <summary>
		/// Flags for XInput Capabilities
		/// </summary>
		private const uint CapsVoiceSupported = 0x0004;

		/// <summary>
		/// Use dead zone thumb?
		/// </summary>
		private static bool deadZoneThumb = true;
		/// <summary>
		/// Use trigger threshold to ignore minimal changes.
		/// </summary>
		private static bool triggerThreshold = true;
		/// <summary>
		/// Return eight dirs or just four?
		/// </summary>
		private static bool eightDirsJoy = false;//true;
		#endregion

		#region Enums
		/// <summary>
		/// Joystick direction if we just need up/left/upright, etc. values.
		/// </summary>
		public enum JoystickDir
		{
			Center,
			Up,
			Down,
			Left,
			Right,
			UpLeft,
			UpRight,
			DownLeft,
			DownRight,
		} // enum JoystickDir

		/// <summary>
		/// Game pad button constant values for checking button states.
		/// </summary>
		[Flags]
		public enum GamePadButtons
		{
			// Fields
			Up = 1,
			Down = 2,
			Left = 4,
			Right = 8,
			Start = 0x10,
			Back = 0x20,
			LeftThumb = 0x40,
			RightThumb = 0x80,
			LeftShoulder = 0x100,
			RightShouler = 0x200,
			A = 0x1000,
			B = 0x2000,
			X = 0x4000,
			Y = 0x8000,
		} // enum GamePadButtons
		#endregion

		#region Variables
		/// <summary>
		/// Is XInput available and can we use Controller 1?
		/// The controller must be plugged in and only if initializing worked
		/// we can use it, else all properties will return 0 or false.
		/// </summary>
		static bool canUseController1 = false;

		/// <summary>
		/// Helper values for controller 1, we usually only have 1 player
		/// and 1 controller for most pc games, no need to make things complicated.
		/// </summary>
		static bool aButtonPressed = false,
			aButtonPressedLastFrame = false,
			bButtonPressed = false,
			xButtonPressed = false,
			yButtonPressed = false,
			startButtonPressed = false,
			backButtonPressed = false,
			leftTriggerButtonPressed = false,
			rightTriggerButtonPressed = false;
		/// <summary>
		/// Relative movement and rotation values, all values are between 0 and 1.
		/// Most of the time all these values will be 0 or one of them is 1.
		/// </summary>
		static float xMovement = 0, yMovement = 0,
			xRotation = 0, yRotation = 0,
			leftTrigger = 0, rightTrigger = 0;

		/// <summary>
		/// Joystick dir (left cursor thingy)
		/// </summary>
		static JoystickDir joystick = JoystickDir.Center,
			joystickLastFrame = JoystickDir.Center;
		#endregion

		#region Properties
		/// <summary>
		/// XMovement
		/// </summary>
		/// <returns>Float</returns>
		public static float XMovement
		{
			get
			{
				return xMovement;
			} // get
		} // XMovement

		/// <summary>
		/// YMovement
		/// </summary>
		/// <returns>Float</returns>
		public static float YMovement
		{
			get
			{
				return yMovement;
			} // get
		} // YMovement

		/// <summary>
		/// Left trigger
		/// </summary>
		/// <returns>Float</returns>
		public static float LeftTrigger
		{
			get
			{
				return leftTrigger;
			} // get
		} // LeftTrigger

		/// <summary>
		/// Right trigger
		/// </summary>
		/// <returns>Float</returns>
		public static float RightTrigger
		{
			get
			{
				/// <summary>
				/// Right trigger
				/// </summary>
				return rightTrigger;
			} // get
		} // RightTrigger

		/// <summary>
		/// XRotation
		/// </summary>
		/// <returns>Float</returns>
		public static float XRotation
		{
			get
			{
				return xRotation;
			} // get
		} // XRotation

		/// <summary>
		/// YRotation
		/// </summary>
		/// <returns>Float</returns>
		public static float YRotation
		{
			get
			{
				/// <summary>
				/// Y coordinate rotation
				/// </summary>
				return yRotation;
			} // get
		} // YRotation

		/// <summary>
		/// Start button
		/// </summary>
		/// <returns>Bool</returns>
		public static bool StartButton
		{
			get
			{
				return startButtonPressed;
			} // get
		} // StartButton

		/// <summary>
		/// Back button
		/// </summary>
		/// <returns>Bool</returns>
		public static bool BackButton
		{
			get
			{
				return backButtonPressed;
			} // get
		} // BackButton

		/// <summary>
		/// AButton
		/// </summary>
		/// <returns>Bool</returns>
		public static bool AButton
		{
			get
			{
				return aButtonPressed;
			} // get
		} // AButton

		/// <summary>
		/// Was a button pressed last frame
		/// </summary>
		/// <returns>Bool</returns>
		public static bool WasAButtonPressedLastFrame
		{
			get
			{
				return aButtonPressedLastFrame;
			} // get
		} // WasAButtonPressedLastFrame

		/// <summary>
		/// BButton
		/// </summary>
		/// <returns>Bool</returns>
		public static bool BButton
		{
			get
			{
				return bButtonPressed;
			} // get
		} // BButton

		/// <summary>
		/// XButton
		/// </summary>
		/// <returns>Bool</returns>
		public static bool XButton
		{
			get
			{
				return xButtonPressed;
			} // get
		} // XButton

		/// <summary>
		/// YButton
		/// </summary>
		/// <returns>Bool</returns>
		public static bool YButton
		{
			get
			{
				return yButtonPressed;
			} // get
		} // YButton

		/// <summary>
		/// Left trigger button
		/// </summary>
		/// <returns>Bool</returns>
		public static bool LeftTriggerButton
		{
			get
			{
				return leftTriggerButtonPressed;
			} // get
		} // LeftTriggerButton

		/// <summary>
		/// Right trigger button
		/// </summary>
		/// <returns>Bool</returns>
		public static bool RightTriggerButton
		{
			get
			{
				return rightTriggerButtonPressed;
			} // get
		} // RightTriggerButton

		/// <summary>
		/// Joystick
		/// </summary>
		/// <returns>Joystick dir</returns>
		public static JoystickDir Joystick
		{
			get
			{
				return joystick;
			} // get
		} // Joystick

		public static JoystickDir JoystickLastFrame
		{
			get
			{
				return joystickLastFrame;
			} // get
		} // Joystick

		/*unused old stuff
		private static bool deadZoneThumb = true;
		public static bool DeadZoneThumb
		{
			get { return deadZoneThumb; }
			set { deadZoneThumb = value; }
		}

		private static bool triggerThreshold = true;
		public static bool TriggerThreshold
		{
			get { return triggerThreshold; }
			set { triggerThreshold = value; }
		}

		private static bool eightDirsJoy = true;
		public static bool EightDirsJoy
		{
			get { return eightDirsJoy; }
			set { eightDirsJoy = value; }
		}

		/// <summary>
		/// Gamepad thresholds
		/// </summary>
		private static uint GAMEPAD_TRIGGER_THRESHOLD =
			GamePadTriggerThreshold;
		public static uint TRIGGER_THRESHOLD
		{
			get { return GAMEPAD_TRIGGER_THRESHOLD; }
			set { GAMEPAD_TRIGGER_THRESHOLD = value; }
		}

		private static uint GAMEPAD_LEFT_THUMB_DEADZONE =
			GamePadLeftThumbDeadzone;
		public static uint LEFT_THUMB_DEADZONE
		{
			get { return GAMEPAD_LEFT_THUMB_DEADZONE; }
			set { GAMEPAD_LEFT_THUMB_DEADZONE = value; }
		}

		private static uint GAMEPAD_RIGHT_THUMB_DEADZONE =
			GamePadRightThumbDeadzone;
		public static uint RIGHT_THUMB_DEADZONE
		{
			get { return GAMEPAD_RIGHT_THUMB_DEADZONE; }
			set { GAMEPAD_RIGHT_THUMB_DEADZONE = value; }
		}
		 */
		#endregion

		#region NativeMethods helper class (to access XInput from .NET)
		/// <summary>
		/// Native methods
		/// </summary>
		internal class NativeMethods
		{
			/// <summary>
			/// Don't allow instantiating this class.
			/// </summary>
			private NativeMethods()
			{
			} // NativeMethods()

			/// <summary>
			/// Retrieves the capabilities and features of a connected controller.
			/// </summary>
			/// <param name="dwUserIndex">[in] Index of the user's controller.
			/// Can be a value in the range 03. For information about how this
			/// value is determined and how the value maps to indicators on the
			/// controller, see Multiple Controllers.</param>
			/// <param name="dwFlags">[in] Input flags that identify the controller
			/// type. If this value is 0, then the capabilities of all controllers
			/// connected to the system are returned. Currently, only one value is
			/// supported: XINPUT_FLAG_GAMEPAD Limit query to devices of Xbox 360
			/// Controller type. Any value of dwflags other than the above or 0 is
			/// illegal and will result in an error break when debugging.</param>
			/// <param name="pCapabilities">[out] Pointer to an XINPUT_CAPABILITIES
			/// structure that receives the controller capabilities.</param>
			/// <returns>
			/// If the function succeeds, the return value is ERROR_SUCCESS.
			/// If the controller is not connected, the return value is
			/// ERROR_DEVICE_NOT_CONNECTED. If the function fails, the return
			/// value is an error code defined in Winerror.h. The function does not
			/// use SetLastError to set the calling thread's last-error code.
			/// </returns>
			[DllImport("xinput9_1_0.dll",
				EntryPoint = "XInputGetCapabilities",
				SetLastError = true)]
			internal static extern uint XInputGetCapabilities(
				uint dwUserIndex, uint dwFlags, out Capabilities pCapabilities);

			/// <summary>
			/// Retrieves the current state of the specified controller.
			/// </summary>
			/// <param name="dwUserIndex">[in] Index of the user's controller.
			/// Can be a value from 0 to 3. For information about how this value is
			/// determined and how the value maps to indicators on the controller,
			/// see Multiple Controllers.</param>
			/// <param name="pState">[out] Pointer to an XINPUT_STATE structure that
			/// receives the current state of the controller.</param>
			/// <returns>
			/// If the function succeeds, the return value is ERROR_SUCCESS. If the
			/// controller is not connected, the return value is
			/// ERROR_DEVICE_NOT_CONNECTED. If the function fails, the return value
			/// is an error code defined in Winerror.h. The function does not use
			/// SetLastError to set the calling thread's last-error code.
			/// </returns>
			[DllImport("xinput9_1_0.dll",
				EntryPoint = "XInputGetState",
				SetLastError = true)]
			internal static extern uint XInputGetState(
				uint dwUserIndex, out State pState);

			/// <summary>
			/// Sends data to a connected controller. This function is used to
			/// activate the vibration function of a controller.
			/// </summary>
			/// <param name="dwUserIndex">[in] Index of the user's controller.
			/// Can be a value from 0 to 3. For information about how this value is
			/// determined and how the value maps to indicators on the controller, 
			/// see Multiple Controllers.</param>
			/// <param name="pVibration">[in, out] Pointer to an XINPUT_VIBRATION
			/// structure containing the vibration information to send to the
			/// controller.</param>
			/// <returns>
			/// If the function succeeds, the return value is ERROR_SUCCESS.
			/// If the controller is not connected, the return value is
			/// ERROR_DEVICE_NOT_CONNECTED. If the function fails, the return
			/// value is an error code defined in Winerror.h. The function does not
			/// use SetLastError to set the calling thread's last-error code.
			/// </returns>
			[DllImport("xinput9_1_0.dll",
				EntryPoint = "XInputSetState",
				SetLastError = true)]
			internal static extern uint XInputSetState(
				uint dwUserIndex, ref Vibration pVibration);

			/// <summary>
			/// Retrieves the sound rendering and sound capture device GUIDs 
			/// that are associated with the headset connected to the specified
			/// controller.
			/// </summary>
			/// <param name="dwUserIndex">[in] Index of the user's controller.
			/// Can be a value in the range 03. For information about how this
			/// value is determined and how the value maps to indicators on the
			/// controller, see Multiple Controllers.</param>
			/// <param name="pDSoundRenderGuid">[out] Pointer that receives the GUID
			/// of the headset sound rendering device.</param>
			/// <param name="pDSoundCaptureGuid">[out] Pointer that receives the GUID
			/// of the headset sound capture device.</param>
			/// <returns>If the function successfully retrieves the device IDs for
			/// render and capture, the return code is ERROR_SUCCESS. If there is no
			/// headset connected to the controller, the function will also retrieve
			/// ERROR_SUCCESS with GUID_NULL as the values for pDSoundRenderGuid and
			/// pDSoundCaptureGuid. If the controller port device is not physically
			/// connected, the function will return ERROR_DEVICE_NOT_CONNECTED. If
			/// the function fails, it will return a valid Win32 error code.
			/// </returns>
			[DllImport("xinput9_1_0.dll",
				EntryPoint = "XInputGetDSoundAudioDeviceGuids",
				SetLastError = true)]
			internal static extern uint XInputGetDSoundAudioDeviceGuids(
				uint dwUserIndex, out Guid pDSoundRenderGuid,
				out Guid pDSoundCaptureGuid);
		} // class NativeMethods
		#endregion

		#region Structures
		/// <summary>
		/// Describes the capabilities of a connected controller. 
		/// XINPUT_CAPABILITIES is returned by the XInputGetCapabilities function.
		/// </summary>
		/// <remarks>
		/// XInputGetCapabilities returns XINPUT_CAPABILITIES to indicate the
		/// characteristics and available functionality of a specified controller.
		///	XInputGetCapabilities sets the structure members to indicate which
		/// inputs the device supports. For binary state controls, such as digital
		/// buttons, the corresponding bit reflects whether or not the control is
		/// supported by the device.
		///	For proportional controls, such as thumbsticks, the value indicates
		/// the resolution for that control. Some number of the least significant
		/// bits may not be set, indicating that the control does not provide
		/// resolution to that level.																								
		/// The SubType member indicates the specific subtype of controller
		/// present. Games may detect the controller subtype and tune their
		/// handling of controller input or output based on subtypes that are well 
		/// suited to their game genre. For example, a car racing game might check
		/// for the presence of a wheel controller to provide finer control of the
		/// car being driven. However, titles must not disable or ignore a device
		/// based on its subtype  subtypes not recognized by the game or for
		/// which the game is not specifically tuned should be treated as a
		/// standard Xbox 360 Controller (XINPUT_DEVSUBTYPE_GAMEPAD).
		/// </remarks>
		internal struct Capabilities
		{
			/// <summary>
			/// Controller type. Must be one of the following values.
			/// XINPUT_DEVTYPE_GAMEPAD The device is a game controller. 
			/// </summary>
			public byte Type;
			/// <summary>
			/// Subtype of the game controller.
			/// XINPUT_DEVSUBTYPE_GAMEPAD Gamepad 
			/// </summary>
			public byte SubType;
			/// <summary>
			/// Features of the controller.
			/// XINPUT_CAPS_VOICE_SUPPORTED The device has an integrated voice
			/// device. 
			/// </summary>
			public ushort Flags;
			/// <summary>
			/// XINPUT_GAMEPAD structure that describes available controller
			/// features and control resolutions.
			/// </summary>
			public GamePad Gamepad;
			/// <summary>
			/// XINPUT_VIBRATION structure that describes available vibration
			/// functionality and resolutions.
			/// </summary>
			public Vibration Vibration;
		} // struct Capabilities

		/// <summary>
		/// Describes the current state of the Xbox 360 Controller. 
		/// </summary>
		/// <remarks>
		/// This structure is used by the XINPUT_STATE structure when polling for
		/// changes in the state of the controller. The specific mapping of button
		/// to game function varies depending on the game type.
		/// The constant XINPUT_GAMEPAD_TRIGGER_THRESHOLD may be used as the value
		/// which bLeftTrigger and bRightTrigger must be greater than to register
		/// as pressed. This is optional, but often desirable. 
		/// Xbox 360 Controller buttons do not manifest crosstalk.
		/// </remarks>
		internal struct GamePad
		{
			/// <summary>
			/// Bitmask of the device digital buttons, as follows. 
			/// A set bit indicates that the corresponding button is pressed. 
			/// </summary>
			public ushort wButtons;
			/// <summary>
			/// The current value of the left trigger analog control.
			/// The value is between 0 and 255.
			/// </summary>
			public byte bLeftTrigger;
			/// <summary>
			/// The current value of the right trigger analog control.
			/// The value is between 0 and 255.
			/// </summary>
			public byte bRightTrigger;
			/// <summary>
			/// Left thumbstick x-axis value.
			/// Each of the thumbstick axis members is a signed value between
			/// -32768 and 32767 describing the position of the thumbstick.
			/// A value of 0 is centered.
			/// Negative values signify down or to the left. 
			/// Positive values signify up or to the right. 
			/// The constants XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE or
			/// XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE can be used as a positive and
			/// negative value to filter a thumbstick input.
			/// </summary>
			public short sThumbLX;
			/// <summary>
			/// Left thumbstick y-axis value. The value is between -32768 and 32767.
			/// </summary>
			public short sThumbLY;
			/// <summary>
			/// Right thumbstick x-axis value. The value is between -32768 and 32767.
			/// </summary>
			public short sThumbRX;
			/// <summary>
			/// Right thumbstick y-axis value. The value is between -32768 and 32767.
			/// </summary>
			public short sThumbRY;
		} // struct GamePad

		/// <summary>
		/// Represents the state of a controller.
		/// </summary>
		/// <remarks>
		/// The dwPacketNumber member is incremented only if the status of the
		/// controller has changed since the controller was last polled.
		/// </remarks>
		internal struct State
		{
			/// <summary>
			/// State packet number. 
			/// The packet number indicates whether there have been any changes in
			/// the state of the controller. If the dwPacketNumber member is the
			/// same in sequentially returned XINPUT_STATE structures, the
			/// controller state has not changed.
			/// </summary>
			public uint dwPacketNumber;
			/// <summary>
			/// XINPUT_GAMEPAD structure containing the current state of an Xbox 360
			/// Controller.
			/// </summary>
			public GamePad Gamepad;
		} // struct State

		/// <summary>
		/// Specifies motor speed levels for the vibration function of a
		/// controller.
		/// </summary>
		/// <remarks>
		/// The left motor is the low-frequency rumble motor. 
		/// The right motor is the high-frequency rumble motor. 
		/// The two motors are not the same, and they create different vibration
		/// effects.
		/// </remarks>
		internal struct Vibration
		{
			/// <summary>
			/// Speed of the left motor. Valid values are in the range 0 to 65,535. 
			/// Zero signifies no motor use; 65,535 signifies 100 percent motor use.
			/// </summary>
			public ushort wLeftMotorSpeed;
			/// <summary>
			/// Speed of the right motor. Valid values are in the range 0 to 65,535. 
			/// Zero signifies no motor use; 65,535 signifies 100 percent motor use.
			/// </summary>
			public ushort wRightMotorSpeed;

			/// <summary>
			/// Create Vibration
			/// </summary>
			/// <param name="setLeftMotorSpeed">Set left motor speed</param>
			/// <param name="setRightMotorSpeed">Set right motor speed</param>
			public Vibration(ushort setLeftMotorSpeed, ushort setRightMotorSpeed)
			{
				wLeftMotorSpeed = setLeftMotorSpeed;
				wRightMotorSpeed = setRightMotorSpeed;
			} // Vibration(setLeftMotorSpeed, setRightMotorSpeed)
		} // struct Vibration
		#endregion

		#region GamePadState helper class
		/// <summary>
		/// Helper class for the current game pad state of one of the controllers.
		/// Contains a lot of buttons ^^
		/// </summary>
		internal class GamePadState
		{
			/// <summary>
			/// Packet number
			/// </summary>
			internal uint packetNumber;
			/// <summary>
			/// Left triggered
			/// </summary>
			/// <summary>
			/// Right triggered
			/// </summary>
			internal bool leftTriggered,
				rightTriggered;
			/// <summary>
			/// Left trigger
			/// </summary>
			/// <summary>
			/// Right trigger
			/// </summary>
			internal byte leftTrigger,
				rightTrigger;
			/// <summary>
			/// Left thumb x coordinate
			/// </summary>
			/// <summary>
			/// Left thumb y coordinate
			/// </summary>
			/// <summary>
			/// Right thumb x coordinate
			/// </summary>
			/// <summary>
			/// Right thumb y coordinate
			/// </summary>
			internal short leftThumbX,
				leftThumbY,
				rightThumbX,
				rightThumbY;
			/// <summary>
			/// Up button
			/// </summary>
			/// <summary>
			/// Down button
			/// </summary>
			/// <summary>
			/// Left button
			/// </summary>
			/// <summary>
			/// Right button
			/// </summary>
			/// <summary>
			/// Start button
			/// </summary>
			/// <summary>
			/// Back button
			/// </summary>
			/// <summary>
			/// Left thumb button
			/// </summary>
			/// <summary>
			/// Right thumb button
			/// </summary>
			/// <summary>
			/// Left shoulder button
			/// </summary>
			/// <summary>
			/// Right shoulder button
			/// </summary>
			/// <summary>
			/// A button
			/// </summary>
			/// <summary>
			/// B button
			/// </summary>
			/// <summary>
			/// X coordinate button
			/// </summary>
			/// <summary>
			/// Y coordinate button
			/// </summary>
			internal bool upButton,
				downButton,
				leftButton,
				rightButton,
				startButton,
				backButton,
				leftThumbButton,
				rightThumbButton,
				leftShoulderButton,
				rightShoulderButton,
				aButton,
				bButton,
				xButton,
				yButton;
			/// <summary>
			/// Left thumb dir
			/// </summary>
			/// <summary>
			/// Right thumb dir
			/// </summary>
			internal JoystickDir leftThumbDir,
				rightThumbDir;
			/// <summary>
			/// Left thumb deg
			/// </summary>
			/// <summary>
			/// Right thumb deg
			/// </summary>
			internal double leftThumbDeg,
				rightThumbDeg;

			/*tst
			public override string ToString()
			{
				string retVal = "packet " + this.packetNumber.ToString() + "\r\n" +
					"up " + this.upButton.ToString() + "\r\n" +
					"down " + this.downButton.ToString() + "\r\n" +
					"left " + this.leftButton.ToString() + "\r\n" +
					"right " + this.rightButton.ToString() + "\r\n" +
					"start " + this.startButton.ToString() + "\r\n" +
					"back " + this.backButton.ToString() + "\r\n" +
					"leftThumb " + this.leftThumbButton.ToString() + "\r\n" +
					"rightThumb " + this.rightThumbButton.ToString() + "\r\n" +
					"leftShoulder " + this.leftShoulderButton.ToString() + "\r\n" +
					"rightShoulder " + this.rightShoulderButton.ToString() + "\r\n" +
					"a " + this.aButton.ToString() + "\r\n" +
					"b " + this.bButton.ToString() + "\r\n" +
					"x " + this.xButton.ToString() + "\r\n" +
					"y " + this.yButton.ToString() + "\r\n" +
					"leftTriggered " + this.leftTriggered.ToString() + "\r\n" +
					"rightTriggered " + this.rightTriggered.ToString() + "\r\n" +
					"leftTrigger " + this.leftTrigger.ToString() + "\r\n" +
					"rightTrigger " + this.rightTrigger.ToString() + "\r\n" +
					"leftThumbX " + this.leftThumbX.ToString() + "\r\n" +
					"leftThumbY " + this.leftThumbY.ToString() + "\r\n" +
					"rightThumbX " + this.rightThumbX.ToString() + "\r\n" +
					"rightThumbY " + this.rightThumbY.ToString() + "\r\n" +
					"leftThumbDeg " + this.leftThumbDeg.ToString("###") + "\r\n" +
					"rightThumbDeg " + this.rightThumbDeg.ToString("###") + "\r\n" + 
					"leftThumbDir " + this.leftThumbDir.ToString() + "\r\n" +
					"rightThumbDir " + this.rightThumbDir.ToString() + "\r\n";
				return retVal;
			}
			 */
		} // class GamePadState
		#endregion

		#region Constructor
		/// <summary>
		/// Create x coordinate input
		/// </summary>
		public XInput()
		{
			// Check out devices and initialize.
			canUseController1 =
				IsXInputDllAvailable() &&
				IsControllerAvailable(0);
		} // XInput()
		#endregion

		#region Update, should be called every frame
		/// <summary>
		/// Update
		/// </summary>
		public void Update()
		{
			// We can't process any input if XInput is not valid or no controller
			// is attached.
			if (canUseController1 == false)
			{
				// Try to check every 10 seconds if a new controller was plugged in.
				if ((ElapsedTime.MsTotal - ElapsedTime.MsLastFrame) % 10000 !=
					ElapsedTime.MsTotal % 10000)
				{
					canUseController1 =
						IsXInputDllAvailable() &&
						IsControllerAvailable(0);
				} // if (ElapsedTime.MsTotal)
				return;
			} // if (canUseController1)

			// Update data
			GamePadState state = GetState(0);

			// Update buttons
			startButtonPressed = state.startButton;
			backButtonPressed = state.backButton;
			aButtonPressedLastFrame = aButtonPressed;
			aButtonPressed = state.aButton;
			bButtonPressed = state.bButton;
			xButtonPressed = state.xButton;
			yButtonPressed = state.yButton;
			leftTriggerButtonPressed = state.leftShoulderButton;
			rightTriggerButtonPressed = state.rightShoulderButton;

			// Update thumbs movement, rotation and roll triggers
			if (state.leftThumbX < 0)
				xMovement = (float)state.leftThumbX / (float)-MinThumbRange;
			else
				xMovement = (float)state.leftThumbX / (float)MaxThumbRange;
			if (state.leftThumbY < 0)
				yMovement = (float)state.leftThumbY / (float)-MinThumbRange;
			else
				yMovement = (float)state.leftThumbY / (float)MaxThumbRange;

			if (state.rightThumbX < 0)
				xRotation = (float)state.rightThumbX / (float)-MinThumbRange;
			else
				xRotation = (float)state.rightThumbX / (float)MaxThumbRange;
			if (state.rightThumbY < 0)
				yRotation = (float)state.rightThumbY / (float)-MinThumbRange;
			else
				yRotation = (float)state.rightThumbY / (float)MaxThumbRange;

			leftTrigger = (float)state.leftTrigger / (float)MaxTrigger;
			rightTrigger = (float)state.rightTrigger / (float)MaxTrigger;

			joystickLastFrame = joystick;
			joystick = state.upButton ? JoystickDir.Up :
				state.downButton ? JoystickDir.Down :
				state.leftButton ? JoystickDir.Left :
				state.rightButton ? JoystickDir.Right : JoystickDir.Center;
			// If no cursor was pressed use left thumb joystick
			if (joystick == JoystickDir.Center)
				joystick = state.leftThumbDir;

			// Update rumble stuff
			if (canUseController1)
			{
				if (remRumbleLeft != oldRumbleLeft ||
					remRumbleRight != oldRumbleRight)
				{
					oldRumbleLeft = remRumbleLeft;
					oldRumbleRight = remRumbleRight;
					SetVibration(0, remRumbleLeft, remRumbleRight);
				} // if (remRumbleLeft)

				// Always reduce rumbling
				remRumbleLeft -= ElapsedTime.MoveFactorPerSecond * 1.5f;
				if (remRumbleLeft < 0)
					remRumbleLeft = 0;
				remRumbleRight -= ElapsedTime.MoveFactorPerSecond * 1.5f;
				if (remRumbleRight < 0)
					remRumbleRight = 0;
			} // if (canUseController1)
		} // Update()
		#endregion

		#region Helper Methods
		/// <summary>
		/// Is x coordinate input dll available
		/// </summary>
		/// <returns>Bool</returns>
		private static bool IsXInputDllAvailable()
		{
			bool isAvail;
			try
			{
				XInput.Capabilities capabilities;
				uint retVal = NativeMethods.XInputGetCapabilities(
					0, XInput.FlagGamePad, out capabilities);
				isAvail = true;
			} // try
			catch (DllNotFoundException)
			{
				isAvail = false;
			} // catch (DllNotFoundException)
			return isAvail;
		} // IsXInputDllAvailable()

		/// <summary>
		/// Is controller available
		/// </summary>
		/// <param name="userId">User id</param>
		/// <returns>Bool</returns>
		private static bool IsControllerAvailable(int userId)
		{
			XInput.Capabilities capabilities;
			return NativeMethods.XInputGetCapabilities(
				(uint)userId, XInput.FlagGamePad, out capabilities) == 0;
		} // IsControllerAvailable(userId)

		/// <summary>
		/// Number of controllers available
		/// </summary>
		/// <returns>Int</returns>
		private static int NumberOfControllersAvailable()
		{
			int count = 0;
			for (int i = 0; i < MaxNumberOfControllers; i++)
			{
				if (IsControllerAvailable(i) == true)
				{
					count++;
				} // if (IsControllerAvailable)
			} // for (int)
			return count;
		} // NumberOfControllersAvailable()

		/// <summary>
		/// Remember rumble factors we want to set.
		/// </summary>
		static float remRumbleLeft = 0,
			remRumbleRight = 0;
		/// <summary>
		/// Old rumble factors, used to compare if anything changed.
		/// </summary>
		static float oldRumbleLeft = 0,
			oldRumbleRight = 0;

		/// <summary>
		/// Rumble
		/// </summary>
		/// <param name="leftPercent">Left percent</param>
		/// <param name="rightPercent">Right percent</param>
		public static void Rumble(float leftPercent, float rightPercent)
		{
			if (leftPercent > remRumbleLeft)
				remRumbleLeft = leftPercent;
			if (rightPercent > remRumbleRight)
				remRumbleRight = rightPercent;
		} // Rumble(leftPercent, rightPercent)

		/// <summary>
		/// Set vibration
		/// </summary>
		/// <param name="userId">User id</param>
		/// <param name="vibrationLeftPercent">Vibration left percent</param>
		/// <param name="vibrationRightPercent">Vibration right percent</param>
		private static void SetVibration(int userId,
			float vibrationLeftPercent, float vibrationRightPercent)
		{
			ushort left = (ushort)(MaxEngineSpeed * vibrationLeftPercent);
			ushort right = (ushort)(MaxEngineSpeed * vibrationRightPercent);
			if (left < MinEngineSpeed)
				left = MinEngineSpeed;
			if (right < MinEngineSpeed)
				right = MinEngineSpeed;
			if (left > MaxEngineSpeed)
				left = MaxEngineSpeed;
			if (right > MaxEngineSpeed)
				right = MaxEngineSpeed;
			XInput.Vibration vibration = new XInput.Vibration();
			vibration.wLeftMotorSpeed = left;
			vibration.wRightMotorSpeed = right;
			uint retVal = NativeMethods.XInputSetState((uint)userId, ref vibration);
		} // SetVibration(userId, vibrationLeftPercent, vibrationRightPercent)

		/// <summary>
		/// Get state
		/// </summary>
		/// <param name="userId">User id</param>
		/// <returns>Game pad state</returns>
		private static GamePadState GetState(int userId)
		{
			XInput.State state;
			uint retVal = NativeMethods.XInputGetState((uint)userId, out state);
			GamePadState gps = new GamePadState();
			if (retVal != 0)
			{
				// Failed? Then don't use the controller anymore!
				canUseController1 = false;
				return gps;
			} // if (retVal)

			gps.packetNumber = state.dwPacketNumber;

			// Handle buttons
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Up) != 0)
				gps.upButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Down) != 0)
				gps.downButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Left) != 0)
				gps.leftButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Right) != 0)
				gps.rightButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Start) != 0)
				gps.startButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Back) != 0)
				gps.backButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.LeftThumb) != 0)
				gps.leftThumbButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.RightThumb) != 0)
				gps.rightThumbButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.LeftShoulder) != 0)
				gps.leftShoulderButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.RightShouler) != 0)
				gps.rightShoulderButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.A) != 0)
				gps.aButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.B) != 0)
				gps.bButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.X) != 0)
				gps.xButton = true;
			if ((state.Gamepad.wButtons & (ushort)GamePadButtons.Y) != 0)
				gps.yButton = true;

			// Handle triggers
			if (triggerThreshold == true)
			{
				if (state.Gamepad.bLeftTrigger < GamePadTriggerThreshold)
				{
					state.Gamepad.bLeftTrigger = 0;
				} // if (state.Gamepad.bLeftTrigger)
				if (state.Gamepad.bRightTrigger < GamePadTriggerThreshold)
				{
					state.Gamepad.bRightTrigger = 0;
				} // if (state.Gamepad.bRightTrigger)
			} // if (triggerThreshold)
			if (state.Gamepad.bLeftTrigger > 0)
			{
				gps.leftTriggered = true;
			} // if (state.Gamepad.bLeftTrigger)
			if (state.Gamepad.bRightTrigger > 0)
			{
				gps.rightTriggered = true;
			} // if (state.Gamepad.bRightTrigger)
			gps.leftTrigger = state.Gamepad.bLeftTrigger;
			gps.rightTrigger = state.Gamepad.bRightTrigger;

			// Handle Joysticks
			if (deadZoneThumb == true)
			{
				if ((state.Gamepad.sThumbLX < GamePadLeftThumbDeadzone &&
					state.Gamepad.sThumbLX > -GamePadLeftThumbDeadzone) &&
					(state.Gamepad.sThumbLY < GamePadLeftThumbDeadzone &&
					state.Gamepad.sThumbLY > -GamePadLeftThumbDeadzone))
				{
					state.Gamepad.sThumbLX = 0;
					state.Gamepad.sThumbLY = 0;
				} // if (state.Gamepad.sThumbLX)

				if ((state.Gamepad.sThumbRX < GamePadRightThumbDeadzone &&
					state.Gamepad.sThumbRX > -GamePadRightThumbDeadzone) &&
					(state.Gamepad.sThumbRY < GamePadRightThumbDeadzone &&
					state.Gamepad.sThumbRY > -GamePadRightThumbDeadzone))
				{
					state.Gamepad.sThumbRX = 0;
					state.Gamepad.sThumbRY = 0;
				} // if (state.Gamepad.sThumbRX)
			} // if (deadZoneThumb)
			gps.leftThumbX = state.Gamepad.sThumbLX;
			gps.leftThumbY = state.Gamepad.sThumbLY;
			gps.rightThumbX = state.Gamepad.sThumbRX;
			gps.rightThumbY = state.Gamepad.sThumbRY;

			double degree;
			gps.leftThumbDir = GetJoystickDir(
				state.Gamepad.sThumbLX, state.Gamepad.sThumbLY,
				eightDirsJoy, out degree);
			gps.leftThumbDeg = degree;
			gps.rightThumbDir = GetJoystickDir(
				state.Gamepad.sThumbRX, state.Gamepad.sThumbRY,
				eightDirsJoy, out degree);
			gps.rightThumbDeg = degree;

			return gps;
		} // GetState(userId)

		/// <summary>
		/// Get joystick dir
		/// </summary>
		/// <param name="x">X coordinate</param>
		/// <param name="y">Y coordinate</param>
		/// <param name="eightDirs">Eight dirs</param>
		/// <param name="degrees">Degrees</param>
		/// <returns>Joystick dir</returns>
		private static JoystickDir GetJoystickDir(short x, short y,
			bool eightDirs, out double degrees)
		{
			degrees = 0;
			JoystickDir jd = JoystickDir.Center;
			if (x == 0 && y == 0)
				return jd;

			double atan2 = Math.Atan2(y, x);
			degrees = atan2 * (180 / Math.PI); //-180 to 180
			degrees = degrees + 180; //0 to 360

			if (eightDirs == true)
			{
				// 360 and 0 = left
				// 270 = up
				// 180 = right
				// 90 = down
				// 8 slices = 45 degrees each
				// 22.5 degree offset
				if (degrees >= 337.5 || degrees < 22.5)
				{
					jd = JoystickDir.Left;
				} // if (degrees)
				else if (degrees >= 22.5 && degrees < 67.5)
				{
					jd = JoystickDir.DownLeft;
				} // else if
				else if (degrees >= 67.5 && degrees < 112.5)
				{
					jd = JoystickDir.Down;
				} // else if
				else if (degrees >= 112.5 && degrees < 157.5)
				{
					jd = JoystickDir.DownRight;
				} // else if
				else if (degrees >= 157.5 && degrees < 202.5)
				{
					jd = JoystickDir.Right;
				} // else if
				else if (degrees >= 202.5 && degrees < 247.5)
				{
					jd = JoystickDir.UpRight;
				} // else if
				else if (degrees >= 247.5 && degrees < 292.5)
				{
					jd = JoystickDir.Up;
				} // else if
				else if (degrees >= 292.5 && degrees < 337.5)
				{
					jd = JoystickDir.UpLeft;
				} // else if
			} // if (eightDirs)
			// 4 dirs
			else
			{
				int absX = Math.Abs((int)x);
				int absY = Math.Abs((int)y);
				// up-down
				if (absY > absX)
				{
					if (y > 0)
						jd = JoystickDir.Up;
					else
						jd = JoystickDir.Down;
				} // if (absY)
				// left-right
				else
				{
					if (x < 0)
						jd = JoystickDir.Left;
					else
						jd = JoystickDir.Right;
				} // else
			} // else
			return jd;
		} // GetJoystickDir(x, y, eightDirs)

		/*unused: voice support!
		/// <summary>
		/// can the joystick support voice
		/// </summary>
		public static bool IsVoiceCapabile(uint userId)
		{
			XInput.Capabilities capabilities;
			uint retVal = NativeMethods.XInputGetCapabilities(
				userId, XInput.FlagGamePad, out capabilities);
			if(capabilities.Flags == XInput.CapsVoiceSupported)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// is the headphone plugged into the joystick
		/// </summary>
		public static bool IsVoiceAvailable(uint userId)
		{
			Guid render;
			Guid capture;
			uint retVal = NativeMethods.XInputGetDSoundAudioDeviceGuids(
				0, out render, out capture);
			if(render == Guid.Empty || capture == Guid.Empty)
			{
				return false;
			}
			else
			{
				return true;
			}
		}
		 */
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test XInput (use XBox 360 controller to test!)
		/// </summary>
		[Test]
		public static void TestMouseInput()
		{
			TestForm testForm = new TestForm("TestXInput");

			XInput xInput = new XInput();
			// Run application
			WindowsHelper.ForceForegroundWindow(testForm);
			testForm.Show();
			while (testForm.IsDisposed == false)
			{
				xInput.Update();

				// Write text into caption
				string text = "TestXInput: " +
					"X=" + XInput.XMovement + ", " +
					"Y=" + XInput.YMovement + ", " +
					"Left=" + XInput.LeftTrigger + ", " +
					"Right=" + XInput.RightTrigger + ", " +
					"XRot=" + XInput.XRotation + ", " +
					"YRot=" + XInput.YRotation + ", " +
					"Buttons=";
				if (XInput.StartButton)
					text += "Start ";
				if (XInput.BackButton)
					text += "Back ";
				if (XInput.AButton)
					text += "A ";
				if (XInput.BButton)
					text += "B ";
				if (XInput.XButton)
					text += "X ";
				if (XInput.YButton)
					text += "Y ";
				if (XInput.LeftTriggerButton)
					text += "LeftTrigger ";
				if (XInput.RightTriggerButton)
					text += "RightTrigger ";
				testForm.Text = text;

				// Rumble with trigger buttons
				XInput.Rumble(XInput.LeftTrigger, XInput.RightTrigger);

				Application.DoEvents();
			} // while (testForm.IsDisposed)
		} // TestMouseInput()
#endif
		#endregion
	} // class XInput
} // namespace RocketCommander.Helpers
