// Project: Rocket Commander, File: Program.cs
// Namespace: RocketCommander, Class: Program
// Path: C:\code\Rocket Commander, Author: Abi
// Code lines: 153, Size of file: 5.8 KB
// Creation date: 01.11.2005 07:12
// Last modified: 12.03.2006 12:51
// Generated with Commenter by abi.exDream.com

#region Using directives
#if DEBUG
using NUnit.Framework;
#endif
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using RocketCommander.Graphics;
using RocketCommander.Game;
using RocketCommander.Helpers;
using RocketCommander.Shaders;
using System.Globalization;
using RocketCommander.Properties;
using System.IO;
using System.IO.Compression;
using System.Diagnostics;
using Font = System.Drawing.Font;
using System.Threading;
#endregion

namespace RocketCommander
{
	/// <summary>
	/// Program class, will just start the game
	/// (using RocketCommanderForm).
	/// </summary>
	static class Program
	{
		#region Variables
		/// <summary>
		/// Version number for this program. Used to check for updates
		/// and also displayed in the main menu.
		/// </summary>
		public static int versionHigh = 0,
			versionLow = 0;
		#endregion

		#region Main
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			// We do we do this? Well, to make sure all dlls and stuff are installed
			// and can be used correctly. This happens especially if Managed DirectX
			// dlls are not found or not the correct version.
			// The MessageBox is very useful to find this stuff out.
			try
			{
				Application.EnableVisualStyles();
				Application.SetCompatibleTextRenderingDefault(false);
#if DEBUG
				UnitTesting.StartTest();
#else
				StartApplication();
#endif
			} // try
			catch (System.IO.FileNotFoundException ex)
			{
				string filename = ex.FileName != null ?
					ex.FileName.Split(new char[] { ',' })[0] : "<no file specified>";
				if (filename.EndsWith(".dll") == false &&
					filename.EndsWith(".exe") == false &&
					filename != StringHelper.CutExtension(filename))
					filename += ".dll";
				Log.Write("Important file not found (" + filename +
					"), unable to execute.\nError: " + ex.ToString());
				MessageBox.Show("Important file not found (" + filename +
					"), unable to execute.\nError: " + ex.Message,
					"Fatal Error");
			} // catch (ex)
			catch (Exception ex)
			{
				Log.Write("Fatal application error: " + ex.ToString());
				MessageBox.Show("Fatal application error: " + ex.ToString(),
					"Fatal error");
			} // catch (ex)
		} // Main()
		#endregion

		#region Start application (will call RocketCommanderForm)
		/// <summary>
		/// Extra function to init context of RocketCommanderForm, this will throw
		/// an exception if something could not be loaded yet.
		/// </summary>
		static void StartApplication()
		{
			// This is how normal forms are used, but we want more control over it!
			//old: Application.Run(new RocketCommanderForm());

			// Make sure the settings are loaded
			Settings.Default.Reload();

			// Make sure we got the correct culture set for Texts.
			if (Settings.Default.Language.Length > 0)
			{
				try
				{
					Texts.Culture =
						Settings.Default.Language == "Invariant" ?
						CultureInfo.InvariantCulture :
						new CultureInfo(Settings.Default.Language);
				} // try
				catch { } // ignore errors
			} // if (Settings.Default.Language.Length)

			// Check for updates
			BigScreenMessage bigLoadingMessage = new BigScreenMessage(
				Texts.CheckingForUpdates);
			bigLoadingMessage.Show();
			bigLoadingMessage.Update();

			// Get own file version
			FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(
				Application.ExecutablePath);
			versionHigh = fvi.FileMajorPart;
			versionLow = fvi.FileMinorPart;

			try
			{
				// Get server version
				int serverVersionHigh = 0,
					serverVersionLow = 0;
				new www.rocketcommander.com.
					RocketCommanderService().GetLatestRCVersion(
					"RocketCommander", out serverVersionHigh, out serverVersionLow);

				// Is server version better than ours?
				if (serverVersionHigh > versionHigh ||
					serverVersionLow > versionLow)
				{
					// Quit and start updater!
					Process.Start("RCAutoUpdater.exe");
					return;
				} // if (serverVersionHigh)
			} // try
			catch
			{
				// Failed? Ignore .. maybe we are just not in the internet ...
			} // catch

			// Start game
			bigLoadingMessage.SetMessage(Texts.StartingRocketCommander);
			bigLoadingMessage.Update();

			// We however want full control over every frame and are using
			// a polling (updating each frame) instead of waiting for events.
			using (GraphicForm form = new RocketCommanderForm())
			{
				bigLoadingMessage.Close();
				form.Show();
				while (form.QuitApplication == false)
				{
					// Handle everything for our game
					form.Update();

					// Still handle default events
					Application.DoEvents();
				} // while (form.Quit)
			} // using (form)
		} // StartApplication()
		#endregion

		#region Unit Testing
#if DEBUG
		#region Test start game
		/// <summary>
		/// Test start game
		/// </summary>
		[Test]
		public static void TestStartGame()
		{
			// Make sure the settings are loaded
			Settings.Default.Reload();

			using (GraphicForm form = new RocketCommanderForm())
			{
				form.Show();
				WindowsHelper.ForceForegroundWindow(form);
				while (form.QuitApplication == false)
				{
					// Handle everything for our game
					form.Update();

					// Still handle default events
					Application.DoEvents();
				} // while (form.Quit)
			} // using (form)
		} // TestStartGame()
		#endregion

		#region Language texts tests
		/// <summary>
		/// Texts tests
		/// </summary>
		[TestFixture]
		public class TextsTests
		{
			/// <summary>
			/// Test language texts
			/// </summary>
			[Test]
			public void TestLanguageTexts()
			{
				Texts.Culture = CultureInfo.InvariantCulture;
				Assert.AreEqual("Go!", Texts.Go);
				Assert.AreEqual("Language", Texts.Language);

				Texts.Culture = CultureInfo.GetCultureInfo("de-DE");
				Assert.AreEqual("Los gehts!", Texts.Go);
				Assert.AreEqual("Sprache", Texts.Language);
			} // TestLanguageTexts()
		} // class TextsTests
		#endregion

		#region Test display window
		/// <summary>
		/// This is our first unit test, just displays an empty
		/// window, nothing special.
		/// </summary>
		[Test]
		public static void TestDisplayWindow()
		{
			Settings.Default.Fullscreen = false;
			Settings.Default.ResolutionWidth = 400;
			Settings.Default.ResolutionHeight = 300;
			RocketCommanderForm testForm = new RocketCommanderForm();
			testForm.Text = "TestDisplayWindow";
			Application.Run(testForm);
		} // TestDisplayWindow()
		#endregion

		#region Test gzip compression
		/// <summary>
		/// Test GZip compression
		/// </summary>
		public static void TestGZipCompression()
		{
			FileStream inputFile = null;
			GZipStream compressedZipStream = null;
			try
			{
				// Open and read the contents of the file
				inputFile = new FileStream("Rocket Commander.exe", FileMode.Open,
																	 FileAccess.Read, FileShare.Read);
				byte[] buffer = new byte[inputFile.Length];
				int count = inputFile.Read(buffer, 0, buffer.Length);
				inputFile.Close();

				// Compress the file contents by writing them to a memory stream
				//MemoryStream memoryBuffer = new MemoryStream();
				FileStream memoryBuffer = File.Create("Readme.zip");
				compressedZipStream = new GZipStream(memoryBuffer,
					CompressionMode.Compress, true);
				compressedZipStream.Write(buffer, 0, buffer.Length);
				compressedZipStream.Close();
				Console.WriteLine("Original Size: {0}, Compressed Size: {1}",
													buffer.Length, memoryBuffer.Length);

				// Pause so we can see the output
				Console.ReadLine();
			}
			finally
			{
				if (inputFile != null) inputFile.Close();
				if (compressedZipStream != null) compressedZipStream.Close();
			}
		} // TestGZipCompression()
		#endregion

		#region Create RC update and RC version
		/// <summary>
		/// Create RC update and RC version
		/// </summary>
		[Test]
		public static void CreateRCUpdateAndRCVersion()
		{
			const string RCUpdateFilename = "RocketCommander.RCUpdate";
			const string RCVersionFilename = "RocketCommander.RCVersion";
			// Make sure old RCUpdate and RCVersion are deleted.
			if (File.Exists(RCUpdateFilename))
				File.Delete(RCUpdateFilename);
			if (File.Exists(RCVersionFilename))
				File.Delete(RCVersionFilename);

			// Put all required Rocket Commander files into .RCUpdate
			/*tst
			string[] files = Directory.GetFiles(
				Directory.GetCurrentDirectory(),
				"Rocket Commander.*");
			 */
			string[] files = new string[]
			{
				"Rocket Commander.exe",
			};
			Console.WriteLine(StringHelper.WriteArrayData(files));

			// Save zipped file into RCUpdate
			FileStream updateFile = File.Create(RCUpdateFilename);
			GZipStream zipStream = new GZipStream(
				updateFile, CompressionMode.Compress, true);
			
			MemoryStream headerStream = new MemoryStream();
			BinaryWriter headerWriter = new BinaryWriter(headerStream);

			headerWriter.Write("Rocket Commander Update File");
			headerWriter.Write(files.Length);
			for (int num = 0; num < files.Length; num++)
			{
				FileStream file = File.Open(
					//"Rocket Commander.exe", FileMode.Open,
					"..\\Release\\"+files[num], FileMode.Open,
					FileAccess.Read, FileShare.ReadWrite);
				// Write file length and name into header
				headerWriter.Write((int)file.Length);
				headerWriter.Write(files[num]);
				file.Close();
			} // for (num)

			headerWriter.Flush();
			byte[] headerData = headerStream.ToArray();
			zipStream.Write(headerData, 0, headerData.Length);

			for (int num = 0; num < files.Length; num++)
			{
				// Open file
				FileStream file = File.Open(
					//"Rocket Commander.exe", FileMode.Open,
					"..\\Release\\"+files[num], FileMode.Open,
					FileAccess.Read, FileShare.ReadWrite);
				
				// Write data block
				byte[] fileData = new byte[file.Length];
				file.Read(fileData, 0, fileData.Length);
				file.Close();

				zipStream.Write(fileData, 0, fileData.Length);
			} // for (num)

			Console.WriteLine(
				"headerStream.Length=" + headerStream.Length +
				", files=" + files.Length +
				", updateFile.Length=" + updateFile.Length);

			zipStream.Close();

			// Save version info into RCVersion
			FileVersionInfo fvi =
				FileVersionInfo.GetVersionInfo("Rocket Commander.exe");
			int versionHigh = fvi.FileMajorPart;
			int versionLow = fvi.FileMinorPart;
			FileStream versionFile = File.Create(RCVersionFilename);
			BinaryWriter versionWriter = new BinaryWriter(versionFile);
			versionWriter.Write(versionHigh);
			versionWriter.Write(versionLow);
			versionFile.Close();
		} // CreateRCUpdateAndRCVersion()
		#endregion

		#region Load RC update
		/// <summary>
		/// Load RC update
		/// </summary>
		[Test]
		public static void LoadRCUpdate()
		{
			if (File.Exists("RocketCommander.RCUpdate") == false)
				throw new FileNotFoundException("RocketCommander.RCUpdate was not " +
					"found, please call CreateRCUpdateAndRCVersion() first!");

			FileStream updateFile = File.Open("RocketCommander.RCUpdate",
				FileMode.Open, FileAccess.Read, FileShare.Read);
			GZipStream zipStream = new GZipStream(
				updateFile, CompressionMode.Decompress);
			MemoryStream memStream = new MemoryStream();
			BinaryWriter memWriter = new BinaryWriter(memStream);
			int offset = 0;
			int totalBytes = 0;
			byte[] smallBuffer = new byte[1024];
			while (true)
			{
				int bytesRead = zipStream.Read(smallBuffer, 0, 1024);
				if (bytesRead == 0)
					break;
				memWriter.Write(smallBuffer, 0, bytesRead);
				offset += bytesRead;
				totalBytes += bytesRead;
			} // while (true)
			updateFile.Close();

			// Ok, read header first
			memStream.Seek(0, SeekOrigin.Begin);
			BinaryReader reader = new BinaryReader(memStream);

			string rcUpdateId = reader.ReadString();
			Assert.AreEqual("Rocket Commander Update File", rcUpdateId);

			int numOfFiles = reader.ReadInt32();
			//worked: Assert.AreEqual(4, numOfFiles);
			Assert.AreEqual(1, numOfFiles);

			int[] fileLengths = new int[numOfFiles];
			string[] fileNames = new string[numOfFiles];
			for (int num = 0; num < numOfFiles; num++)
			{
				fileLengths[num] = reader.ReadInt32();
				fileNames[num] = reader.ReadString();
				Console.WriteLine("fileNames[" + num + "]=" + fileNames[num]);
				Console.WriteLine("fileLengths[" + num + "]=" + fileLengths[num]);
			} // for (num)

			// Now read all files
			for (int num = 0; num < numOfFiles; num++)
			{
				// Read data
				byte[] data = reader.ReadBytes(fileLengths[num]);

				// Write to test file
				FileStream exportFile = File.Create("Exported " + fileNames[num]);
				exportFile.Write(data, 0, data.Length);
				exportFile.Close();
			} // for (num)
		} // LoadRCUpdate()
		#endregion

		#region Load RC version
		/// <summary>
		/// Load RC version
		/// </summary>
		[Test]
		public static void LoadRCVersion()
		{
			FileStream versionFile = File.Open(
				"RocketCommander.RCVersion", FileMode.Open,
				FileAccess.Read, FileShare.Read);
			BinaryReader versionReader = new BinaryReader(versionFile);
			// High version number
			Assert.AreEqual(1, versionReader.ReadInt32());
			// Low version number
			Assert.AreEqual(1, versionReader.ReadInt32());
			versionFile.Close();
		} // LoadRCVersion()
		#endregion
#endif
		#endregion
	} // class Program
} // namespace RocketCommander
