// Project: Rocket Commander, File: RocketCommanderForm.cs
// Namespace: RocketCommander, Class: RocketCommanderForm
// Path: C:\code\Rocket Commander, Author: Abi
// Code lines: 19, Size of file: 424 Bytes
// Creation date: 01.11.2005 07:12
// Last modified: 12.12.2005 08:33
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using RocketCommander.Graphics;
using RocketCommander.Sounds;
using RocketCommander.Input;
using RocketCommander.Helpers;
using RocketCommander.Game;
using MainMenu = RocketCommander.GameScreens.MainMenu;
using Texture = RocketCommander.Graphics.Texture;
using System.Diagnostics;
using RocketCommander.Properties;
using System.Globalization;
using Microsoft.DirectX.DirectInput;
using RocketCommander.GameScreens;
using System.Threading;
#endregion

namespace RocketCommander
{
	#region Button types
	/// <summary>
	/// Menu button types
	/// </summary>
	public enum MenuButton
	{
		Missions,
		Highscore,
		Credits,
		// Extra buttons
		Help,
		Options,
		// Exit, put at bottom of all main menu buttons
		Exit,
		// Back button, not used in main menu
		Back,
	} // enum MenuButtons

	/// <summary>
	/// Bottom link buttons
	/// </summary>
	public enum BottomLinkButton
	{
		ExDream,
		RocketCommander,
		Microsoft,
	} // enum BottomLinkButton
	#endregion

	/// <summary>
	/// Rocket commander form
	/// </summary>
	public partial class RocketCommanderForm : GameForm 
	{
		#region Variables
		/// <summary>
		/// Game screens stack. We can easily add and remove game screens
		/// and they follow the game logic automatically. Very cool.
		/// </summary>
		private Stack<IGameScreen> gameScreens = new Stack<IGameScreen>();

		/// <summary>
		/// All available levels are preloaded and set to the asteroidManager
		/// when player wants to play them.
		/// </summary>
		public Level[] levels = null;

		/// <summary>
		/// Asteroid manager, always used. Even in the main menu for the
		/// background.
		/// </summary>
		public GameAsteroidManager asteroidManager = null;

		/// <summary>
		/// Rocket model, only used to zoom in at the beginning of each game
		/// and life in the game.
		/// </summary>
		public Model rocketModel = null;

		/// <summary>
		/// Load the main menu and mouse textures at game start, this way
		/// we have access in all game screens and don't ever have to reload
		/// any of these textures.
		/// Update 2005-12-24: Added helper button texture and help screen texture!
		/// </summary>
		public Texture mainMenuTexture = null,
			helperTexture = null,
			helpScreenTexture = null,
			mouseCursorTexture = null;

		/// <summary>
		/// Same logic for menu sounds, load them once and use them everywhere.
		/// </summary>
		public Sample menuClick = null,
			menuHighlight = null;

		/// <summary>
		/// Also preload the in game textures for all in game controls and stuff.
		/// Note: These Hud textures are saved in png format to save disk space.
		/// Light effect texture is used to show items from a far distance.
		/// </summary>
		public Texture hudTexture = null,
			inGameTexture = null,
			lightEffectTexture = null;

		/// <summary>
		/// Explosion texture, displayed when rocket is exploding.
		/// </summary>
		public AnimatedTexture explosionTexture = null;
		#endregion

		#region Version
		/// <summary>
		/// Version info for the title.
		/// </summary>
		/// <returns>String</returns>
		public static string VersionInfo
		{
			get
			{
				string[] splittedVersion =
					Application.ProductVersion.Split(new char[] { '.' });
				return "v" + splittedVersion[0] + "." + splittedVersion[1];
			} // get
		} // VersionInfo
		#endregion

		#region Constructor
		/// <summary>
		/// Create rocket commander form
		/// </summary>
		public RocketCommanderForm()
			: base(Application.ProductName + " " + VersionInfo,
			new Size(Settings.Default.ResolutionWidth,
			Settings.Default.ResolutionHeight),
			Settings.Default.Fullscreen)
		{
			// Make sure our resource file uses the same culture as
			// we specified in settings. If none is set, use the current culture.
			if (Settings.Default.Language.Length > 0)
			{
				try
				{
					Application.CurrentCulture =
						Settings.Default.Language == "Invariant" ?
						CultureInfo.InvariantCulture :
						new CultureInfo(Settings.Default.Language);
				} // try
				catch (Exception)
				{
					Log.Write("Failed to set culture to: " + Settings.Default.Language +
						". Default culture " + Application.CurrentCulture.Name +
						" is used now!");
				} // catch
			} // if
			Texts.Culture = Application.CurrentCulture;

			// Disable mouse, we use our own mouse texture in the menu
			// and don't use any mouse cursor in the game anyway.
			Cursor.Hide();

			// Load all available levels
			levels = Level.LoadAllLevels();

			// Initialize asteroidManager and use last avialable level.
			asteroidManager = new GameAsteroidManager(levels[levels.Length-1]);
			rocketModel = new Model("Rocket");

			// Make ambient color gray instead of black.
			Microsoft.DirectX.Direct3D.Material mat =
				rocketModel.FirstMaterial.D3DMaterial;
			mat.Ambient = Color.FromArgb(80, 80, 80);
			rocketModel.FirstMaterial.D3DMaterial = mat;

			mainMenuTexture = new Texture("MainMenu.png");
			helperTexture = new Texture("ExtraButtons.png");
			helpScreenTexture = new Texture("HelpScreen.png");
			mouseCursorTexture = new Texture("MouseCursor.dds");

			hudTexture = new Texture("Hud.png");
			inGameTexture = new Texture("InGame.png");

			lightEffectTexture = new Texture(
				Directories.TextureEffectsSubDirectory + "\\LightEffect.dds");

			explosionTexture = new AnimatedTexture(
				Directories.TextureEffectsSubDirectory + "\\Explosion");

			// Init sound and load samples
			menuClick = new Sample("Click.wav");
			menuHighlight = new Sample("Highlight.wav");

			// Create main menu screen
			gameScreens.Push(new MainMenu());
		} // RocketCommanderForm()
		#endregion

		#region Toggle music on/off
		/// <summary>
		/// Toggle music on off
		/// </summary>
		public void ToggleMusicOnOff()
		{
			if (Settings.Default.MusicOn)
				music.Stop();
			else
				music.Start();
		} // ToggleMusicOnOff()
		#endregion

		#region Add game screen
		/// <summary>
		/// Add game screen, which will be used until we quit it or add
		/// another game screen on top of it.
		/// </summary>
		/// <param name="newGameScreen">New game screen</param>
		public void AddGameScreen(IGameScreen newGameScreen)
		{
			gameScreens.Push(newGameScreen);

			bool inGame = newGameScreen.GetType() == typeof(Mission);
			camera.InGame = inGame;
			music.CurrentMusicMode = inGame;
		} // AddGameScreen(newGameScreen)
		#endregion

		#region Remove current game screen
		/// <summary>
		/// Remove current game screen
		/// </summary>
		public void RemoveCurrentGameScreen()
		{
			gameScreens.Pop();

			bool inGame = gameScreens.Count > 0 &&
				gameScreens.Peek().GetType() == typeof(Mission);
			camera.InGame = inGame;
			music.CurrentMusicMode = inGame;
		} // RemoveCurrentGameScreen()
		#endregion

		#region Render background
		/// <summary>
		/// Render menu background
		/// </summary>
		public void RenderMenuBackground()
		{
			// Make sure alpha blending is enabled.
			GraphicForm.EnableAlphaBlending();

			Rectangle upperRect = new Rectangle(
				GraphicForm.ResolutionRect.X,
				GraphicForm.ResolutionRect.Y,
				GraphicForm.ResolutionRect.Width,
				GraphicForm.ResolutionRect.Height -
				(int)Math.Round(70 * Resolution.Height / 768.0f));
			mainMenuTexture.RenderOnScreen(
				upperRect,
				// Skip last 70 pixels, done when rendering bottom link buttons
				new Rectangle(0, 0, 1024, 768-70));

			// Show all bottom link buttons (they are part of the background)
			foreach (BottomLinkButton button in
				EnumHelper.GetEnumerator(typeof(BottomLinkButton)))
				if (RenderBottomLinkButton(button))
				{
					if (button == BottomLinkButton.ExDream)
					{
						Process.Start("http://www.exDream.com");
						// Give some time to os, won't help much, but anyway ...
						Thread.Sleep(100);
					} // if
					else if (button == BottomLinkButton.RocketCommander)
					{
						Process.Start("http://www.RocketCommander.com");
						Thread.Sleep(100);
					} // if
					else if (button == BottomLinkButton.Microsoft)
					{
						Process.Start("http://www.Microsoft.com");
						Thread.Sleep(100);
					} // if
				} // if
		} // RenderMenuBackground()
		#endregion

		#region Render button
		/// <summary>
		/// Mouse in box
		/// </summary>
		/// <param name="rect">Rectangle</param>
		/// <returns>Bool</returns>
		public bool MouseInBox(Rectangle rect)
		{
			return mousePos.X >= rect.X && mousePos.X < rect.Right &&
				mousePos.Y >= rect.Y && mousePos.Y < rect.Bottom;
		} // MouseInBox(rect)

		/// <summary>
		/// Mouse was not in rectangle last frame
		/// </summary>
		/// <param name="rect">Rectangle</param>
		public bool MouseWasNotInRectLastFrame(Rectangle rect)
		{
			// Check the opposite of MouseInBox.
			return !(lastFrameMousePos.X >= rect.X &&
				lastFrameMousePos.X < rect.Right &&
				lastFrameMousePos.Y >= rect.Y &&
				lastFrameMousePos.Y < rect.Bottom);
		} // MouseWasNotInRectLastFrame(rect)

		/// <summary>
		/// Render button
		/// </summary>
		/// <param name="buttonType">Button type</param>
		/// <param name="rect">Rectangle</param>
		public bool RenderMenuButton(
			MenuButton buttonType,
			//bool highlight,
			Point pos)
		{
			// Calc screen rect for rendering (recalculate relative screen position
			// from 1024x768 to actual screen resolution, just in case ^^).
			Rectangle rect = new Rectangle(
				pos.X * Resolution.Width / 1024,
				pos.Y * Resolution.Height / 768,
				200 * Resolution.Width / 1024,
				77 * Resolution.Height / 768);

			// Is button highlighted?
			bool highlight = MouseInBox(rect);

			// Was not highlighted last frame?
			if (highlight &&
				MouseWasNotInRectLastFrame(rect))
				menuHighlight.Play(0.385f);

			// See MainMenu.dds for pixel locations
			bool useHelperTexture =
				buttonType == MenuButton.Help ||
				buttonType == MenuButton.Options;
			int buttonNum = (int)buttonType;

			// Correct last 2 button numbers (exit and back)
			if (buttonNum >= (int)MenuButton.Exit)
				buttonNum -= 2;

			Rectangle pixelRect = useHelperTexture ?
				new Rectangle(0 + 204 * ((int)buttonType - (int)MenuButton.Help),
				0 + 80 * (highlight ? 1 : 0), 200, 77) :
				new Rectangle(3 + 204 * buttonNum,
				840 + 80 * (highlight ? 1 : 0), 200, 77);

			// Render
			(useHelperTexture ? helperTexture : mainMenuTexture).RenderOnScreen(
				rect, pixelRect);

			// Play click sound if button was just clicked
			bool ret =
				(mouseInput.LeftButtonPressed &&
				wasMouseButtonPressedLastFrame == false ||
				XInput.AButton &&
				XInput.WasAButtonPressedLastFrame == false) &&
				this.IsActive &&
				highlight;

			if (buttonType == MenuButton.Back &&
				XInput.BackButton)
				ret = true;
			if (buttonType == MenuButton.Missions &&
				XInput.StartButton)
				ret = true;

			if (ret == true)
				menuClick.Play(0.9f);

			// Return true if button was pressed, false otherwise
			return ret;
		} // RenderButton(buttonType, rect)
		#endregion

		#region Render bottom link buttons
		/// <summary>
		/// Render bottom link button
		/// </summary>
		/// <param name="buttonType">Button type</param>
		/// <returns>Bool</returns>
		public bool RenderBottomLinkButton(BottomLinkButton buttonType)
		{
			// Calc screen rect for rendering
			Rectangle pixelRect =
				buttonType == BottomLinkButton.ExDream ?
				new Rectangle(0, 698, 195+60, 70) :
				buttonType == BottomLinkButton.RocketCommander ?
				new Rectangle(195 + 60, 698, 264 + 392 - (195 + 60), 70) :
				new Rectangle(656, 698, 1024-656, 70);
			Rectangle rect = new Rectangle(
				(int)Math.Round(pixelRect.X * Resolution.Width / 1024.0f),
				(int)Math.Round(pixelRect.Y * Resolution.Height / 768.0f),
				(int)Math.Round(pixelRect.Width * Resolution.Width / 1024.0f),
				(int)Math.Round(pixelRect.Height * Resolution.Height / 768.0f));
			rect.Height = Resolution.Height - rect.Y + 1;

			// Is button highlighted?
			// Update, don't highlight if out of screen.
			bool highlight = MouseInBox(new Rectangle(
				rect.X + 10, rect.Y + 2, rect.Width - 15, rect.Height - 14));

			// Was not highlighted last frame?
			if (highlight &&
				MouseWasNotInRectLastFrame(rect))
				menuHighlight.Play(0.385f);

			// Render
			if (highlight)
				pixelRect.Y += 70;
			mainMenuTexture.RenderOnScreen(rect, pixelRect);

			// Play click sound if button was just clicked
			bool ret = mouseInput.LeftButtonPressed &&
				wasMouseButtonPressedLastFrame == false &&
				this.IsActive &&
				highlight;

			if (ret == true)
				menuClick.Play(0.9f);

			// Return true if button was pressed, false otherwise
			return ret;
		} // RenderBottomLinkButton(buttonType)
		#endregion

		#region Render background
		/// <summary>
		/// Render mouse cursor
		/// </summary>
		public void RenderMouseCursor()
		{
			// We got 4 animation steps, rotate them by the current time
			int mouseAnimationStep = (ElapsedTime.MsTotal / 100) % 4;

			// And render mouse on screen.
			mouseCursorTexture.RenderOnScreen(
				new Rectangle(mousePos.X, mousePos.Y, 60, 60),
				new Rectangle(64 * mouseAnimationStep, 0, 60, 60));
		} // RenderMouseCursor()
		#endregion

		#region Update
		/// <summary>
		/// Update
		/// </summary>
		public override void Update()
		{
			// If that game screen should be quitted, remove it from stack!
			if (gameScreens.Count > 0 &&
				gameScreens.Peek().Quit)
				RemoveCurrentGameScreen();

			// If no more game screens are left, it is time to quit!
			if (gameScreens.Count == 0 ||
				device == null ||
				device.Disposed)
				quit = true;

			base.Update();
		} // Update()
		#endregion

		#region Render (called each frame)
		Point lastFrameMousePos = new Point(0, 0);
		public bool wasMouseButtonPressedLastFrame = false;
		/// <summary>
		/// Render, only method used here, called from Update each frame.
		/// We don't need to implement any more methods in this class.
		/// </summary>
		protected override void Render()
		{
			try
			{
				// Always render asteroidManager first!
				asteroidManager.Render(
					// Use the inGameTexture for the hud display.
					camera.InGame ? inGameTexture : null,
					// And the lightEffectTexture for all item glows
					lightEffectTexture);
			} // try
			catch (Exception ex)
			{
				Log.Write("Fatal error, rendering of asteroid field failed: " +
					ex.ToString());
			} // catch

			// Render lens flare on top of 3d stuff
			lensFlare.Render(remLensFlareColor);

			// Add scene glow on top of everything
			if (showGlow)
				glowShader.Show();

			// Render rocket in front of view in menu mode
			// Render after post screen effects to really see anything ^^
			if (camera.InGame == false)
			{
				// Make sure z buffer is on
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = true;

				Vector3 inFrontOfCameraPos =
					new Vector3(0, -1.33f, 2.5f);
				inFrontOfCameraPos.TransformCoordinate(InverseViewMatrix);
				rocketModel.Render(
					Matrix.RotationX((float)Math.PI / 2.2f) *
					Matrix.RotationZ(ElapsedTime.MsTotal / 8400.0f) *
					Matrix.Invert(camera.RotationMatrix) *
					Matrix.Translation(inFrontOfCameraPos));
			} // if

			// Disable z buffer, now only 2d content is rendered.
			GraphicForm.DirectXDevice.RenderState.ZBufferEnable = false;

			try
			{
				// Execute the game screen on top.
				if (gameScreens.Count > 0)
					gameScreens.Peek().Run(this);
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to execute " + gameScreens.Peek().Name +
					"\nError: " + ex.ToString());
				Cursor.Show();
				if (MessageBox.Show("Failed to execute " + gameScreens.Peek().Name +
					": " + ex.Message + "\n\n" +
					"Do you want to quit this part of the game?", "Error",
					MessageBoxButtons.YesNo) == DialogResult.Yes)
					RemoveCurrentGameScreen();
				Cursor.Hide();
			} // catch

			// Show distances (if in game)
			if (camera.InGame)
				asteroidManager.ShowAllDistances(SmallFont);
			
			// Don't forget to render all lines
			LineManager.Render();

			// Show mouse cursor (in all modes except in the game)
			if (camera.InGame == false &&
				gameScreens.Count > 0)
				RenderMouseCursor();

			lastFrameMousePos = mousePos;
			wasMouseButtonPressedLastFrame = mouseInput.LeftButtonPressed;
		} // Render()
		#endregion
	} // class RocketCommanderForm
} // namespace RocketCommander
