﻿// Project: Rocket Commander, File: ParallaxShader.cs
// Namespace: RocketCommander.Shaders, Class: ParallaxShader
// Creation date: 01.11.2005 17:26
// Last modified: 14.11.2005 06:22
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;
using Direct3D = Microsoft.DirectX.Direct3D;
using DirectXTexture = Microsoft.DirectX.Direct3D.Texture;
using RocketCommander.Graphics;
using RocketCommander.Helpers;
using Texture = RocketCommander.Graphics.Texture;
using Material = RocketCommander.Graphics.Material;
#endregion

namespace RocketCommander.Shaders
{
	/// <summary>
	/// Parallax shader class for all parallax shaders (most used shader here),
	/// will just derive from BaseShader, not much special stuff in here.
	/// </summary>
	public class ParallaxShader : IGraphicObject
	{
		#region Variables
		/// <summary>
		/// Shader effect filename.
		/// </summary>
		public const string Filename = "ParallaxMapping.fx";

		/// <summary>
		/// Technique name
		/// </summary>
		/// <returns>String</returns>
		private string TechniqueName
		{
			get
			{
				return GraphicForm.UsePS20 ? "Specular20" : "Specular";
			} // get
		} // TechniqueName

		/// <summary>
		/// The shader effect
		/// </summary>
		protected Effect effect = null;

		/// <summary>
		/// Effect handles for ParallaxShader.fx.
		/// If you have more than 1 shader, you should write a BaseShader class
		/// and reuse the existing program logic to simplify all shader classes.
		/// </summary>
		protected EffectHandle viewProj,
			world,
			viewInverse,
			lightDir,
			ambientColor,
			diffuseColor,
			specularColor,
			shininess,
			diffuseTexture,
			normalTexture,
			heightTexture,
			parallaxAmount;
		#endregion

		#region Properties
		/// <summary>
		/// Get Direct3D effect, can be null when effect is not valid.
		/// We have to render with normal methods then
		/// (fallback if shader is not supported!)
		/// </summary>
		public Effect D3DEffect
		{
			get
			{
				return effect;
			} // get
		} // D3DEffect

		/// <summary>
		/// Is shader valid? No error while loading?
		/// When this returns true the shader can be used.
		/// Will also check if we want to use shaders, either
		/// Graphics.CanDoPS11
		/// </summary>
		public bool Valid
		{
			get
			{
				return effect != null &&
					GraphicForm.UsePS;
			} // get
		} // Valid

		/// <summary>
		/// Set effect invalid, used when an exception happened and we
		/// can't continue to use this shader anymore.
		/// </summary>
		public void SetEffectInvalid()
		{
			effect = null;
		} // SetEffectInvalid()

		/// <summary>
		/// Last used world matrix
		/// </summary>
		private Matrix lastUsedWorldMatrix = Matrix.Zero;
		/// <summary>
		/// Set world matrix
		/// </summary>
		public Matrix WorldMatrix
		{
			set
			{
				if (effect != null &&
					world != null &&
					lastUsedWorldMatrix != value)
				{
					lastUsedWorldMatrix = value;
					effect.SetValue(world, value);
				} // if (effect)
			} // set
		} // WorldMatrix

		/// <summary>
		/// Last used inverse view matrix
		/// </summary>
		private Matrix lastUsedInverseViewMatrix = Matrix.Zero;
		/// <summary>
		/// Set view inverse matrix
		/// </summary>
		protected Matrix InverseViewMatrix
		{
			set
			{
				if (effect != null &&
					viewInverse != null &&
					lastUsedInverseViewMatrix != value)
				{
					lastUsedInverseViewMatrix = value;
					effect.SetValue(viewInverse, value);
				} // if (effect)
			} // set
		} // InverseViewMatrix

		/// <summary>
		/// Last used view proj matrix
		/// </summary>
		private Matrix lastUsedViewProjMatrix = Matrix.Zero;
		/// <summary>
		/// Set view proj matrix
		/// </summary>
		protected Matrix ViewProjMatrix
		{
			set
			{
				if (effect != null &&
					viewProj != null &&
					lastUsedViewProjMatrix != value)
				{
					lastUsedViewProjMatrix = value;
					effect.SetValue(viewProj, value);
				} // if (effect.D3DEffect)
			} // set
		} // ViewProjMatrix

		/// <summary>
		/// Last used light dir
		/// </summary>
		private Vector3 lastUsedLightDir = Vector3.Empty;
		/// <summary>
		/// Set light direction
		/// </summary>
		protected Vector3 LightDirection
		{
			set
			{
				if (effect != null &&
					lightDir != null &&
					lastUsedLightDir != value)
				{
					// Make sure lightDir is normalized (fx files are optimized
					// to work with a normalized lightDir vector)
					value.Normalize();
					lastUsedLightDir = value;
					// Set negative value, shader is optimized not to negate dir!
					Vector3 invertedLight = -value;
					effect.SetValue(lightDir, new Vector4(
						invertedLight.X, invertedLight.Y, invertedLight.Z, 0.0f));
				} // if (effect.D3DEffect)
			} // set
		} // LightDirection

		/// <summary>
		/// Last used ambient color
		/// </summary>
		private Color lastUsedAmbientColor = Color.Empty;//.FromArgb(0, 0, 0, 0);
		/// <summary>
		/// Ambient color
		/// </summary>
		public Color AmbientColorValue
		{
			set
			{
				if (effect != null &&
					ambientColor != null &&
					lastUsedAmbientColor != value)
				{
					lastUsedAmbientColor = value;
					effect.SetValue(ambientColor,
						ColorHelper.ConvertColorToVector4(value));
				} // if (effect.D3DEffect)
			} // set
		} // AmbientColorValue

		/// <summary>
		/// Last used diffuse color
		/// </summary>
		private Color lastUsedDiffuseColor = Color.Empty;//.FromArgb(0, 0, 0, 0);
		/// <summary>
		/// Diffuse color
		/// </summary>
		public Color DiffuseColorValue
		{
			set
			{
				if (effect != null &&
					diffuseColor != null &&
					lastUsedDiffuseColor != value)
				{
					lastUsedDiffuseColor = value;
					effect.SetValue(diffuseColor,
						ColorHelper.ConvertColorToVector4(value));
				} // if (effect.D3DEffect)
			} // set
		} // DiffuseColorValue

		/// <summary>
		/// Last used specular color
		/// </summary>
		private Color lastUsedSpecularColor = Color.Empty;//.FromArgb(0, 0, 0, 0);
		/// <summary>
		/// Specular color
		/// </summary>
		public Color SpecularColorValue
		{
			set
			{
				if (effect != null &&
					specularColor != null &&
					lastUsedSpecularColor != value)
				{
					lastUsedSpecularColor = value;
					effect.SetValue(specularColor,
						ColorHelper.ConvertColorToVector4(value));
				} // if (effect.D3DEffect)
			} // set
		} // SpecularColorValue

		/// <summary>
		/// Last used shininess
		/// </summary>
		private float lastUsedShininess = 0;
		/// <summary>
		/// Shininess for specular color
		/// </summary>
		public float ShininessValue
		{
			set
			{
				if (effect != null &&
					shininess != null &&
					lastUsedShininess != value)
				{
					lastUsedShininess = value;
					effect.SetValue(shininess, value);
				} // if (effect.D3DEffect)
			} // set
		} // ShininessValue

		/// <summary>
		/// Last used diffuse texture
		/// </summary>
		private DirectXTexture lastUsedDiffuseTexture = null;
		/// <summary>
		/// Set diffuse texture
		/// </summary>
		internal Texture DiffuseTextureValue
		{
			set
			{
				if (effect != null &&
					diffuseTexture != null &&
					value != null &&
					lastUsedDiffuseTexture != value.D3DTexture)
				{
					lastUsedDiffuseTexture = value.D3DTexture;
					effect.SetValue(diffuseTexture, value.D3DTexture);
				} // if (effect.D3DEffect)
			} // set
		} // DiffuseTextureValue

		/// <summary>
		/// Last used normal texture
		/// </summary>
		private DirectXTexture lastUsedNormalTexture = null;
		/// <summary>
		/// Set normal texture for normal mapping
		/// </summary>
		protected Texture NormalTextureValue
		{
			set
			{
				if (effect != null &&
					normalTexture != null &&
					value != null &&
					lastUsedNormalTexture != value.D3DTexture)
				{
					lastUsedNormalTexture = value.D3DTexture;
					effect.SetValue(normalTexture, value.D3DTexture);
				} // if (effect.D3DEffect)
			} // set
		} // NormalTextureValue

		/// <summary>
		/// Last used height texture
		/// </summary>
		private DirectXTexture lastUsedHeightTexture = null;
		/// <summary>
		/// Set height texture for parallax mapping
		/// </summary>
		protected Texture HeightTextureValue
		{
			set
			{
				if (effect != null &&
					heightTexture != null &&
					value != null &&
					lastUsedHeightTexture != value.D3DTexture)
				{
					lastUsedHeightTexture = value.D3DTexture;
					effect.SetValue(heightTexture, value.D3DTexture);
				} // if (effect.D3DEffect)
			} // set
		} // HeightTextureValue

		/// <summary>
		/// Last used parallax amount
		/// </summary>
		private float lastUsedParallaxAmount = 0;
		/// <summary>
		/// Parallax amount for parallax and offset shaders.
		/// </summary>
		public float ParallaxAmountValue
		{
			set
			{
				if (effect != null &&
					parallaxAmount != null &&
					lastUsedParallaxAmount != value)
				{
					lastUsedParallaxAmount = value;
					effect.SetValue(parallaxAmount, value);
				} // if (effect.D3DEffect)
			} // set
		} // ParallaxAmountValue
		#endregion

		#region Constructor
		/// <summary>
		/// Create parallax shader
		/// </summary>
		public ParallaxShader()
		{
			Reload();
		} // ParallaxShader()
		#endregion

		#region Dispose
		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			// Dispose old shader
			if (effect != null)
			{
				try
				{
					effect.Dispose();
				} // try
				catch (Exception ex)
				{
					Log.Write("Failed to dispose shader " + Filename + ": " +
						ex.ToString());
				} // catch
			} // if (effect)
			effect = null;
		} // Dispose()
		#endregion

		#region IGraphicObject Members
		/// <summary>
		/// On reset device
		/// </summary>
		public void OnResetDevice()
		{
		} // OnResetDevice()

		/// <summary>
		/// On lost device
		/// </summary>
		public void OnLostDevice()
		{
		} // OnLostDevice()
		#endregion

		#region Reload effect
		/// <summary>
		/// Reload
		/// </summary>
		public void Reload()
		{
			// Dispose old shader
			Dispose();

			string fullFilename = Directories.ShadersDirectory + "\\" + Filename;
			if (File.Exists(fullFilename) == false)
				throw new FileNotFoundException(
					"Can't load shader without valid shader filename",
					fullFilename);

			// Load it
			string effectError = "";
			try
			{
				effect = Effect.FromFile(GraphicForm.DirectXDevice,
					// Just load, no special stuff used.
					fullFilename, null, null, ShaderFlags.None, null, out effectError);
			} // try
			catch (Exception ex)
			{
				// Only use exception text if no effectError was logged
				if (String.IsNullOrEmpty(effectError))
					effectError = ex.ToString();
			} // catch

			// Error happened and no exception was thrown yet?
			if (String.IsNullOrEmpty(effectError) == false)
				throw new Exception(
					"Failed to load shader " + this.ToString() + ": " + effectError);

			// Reset and get all avialable parameters.
			ResetParameters();
			GetParameters();

			GraphicForm.AddGraphicObject(this);
		} // Reload()
		#endregion

		#region Reset parameters
		/// <summary>
		/// Reset parameters
		/// </summary>
		protected void ResetParameters()
		{
			lastUsedWorldMatrix = Matrix.Zero;
			lastUsedInverseViewMatrix = Matrix.Zero;
			lastUsedViewProjMatrix = Matrix.Zero;
			lastUsedLightDir = Vector3.Empty;
			lastUsedAmbientColor = Color.Empty;
			lastUsedDiffuseColor = Color.Empty;
			lastUsedSpecularColor = Color.Empty;
			lastUsedShininess = 0;
			lastUsedDiffuseTexture = null;
			lastUsedNormalTexture = null;
			lastUsedHeightTexture = null;
			lastUsedParallaxAmount = 0;
		} // ResetParameters()
		#endregion

		#region GetParameters (happens only once when initializing)
		/// <summary>
		/// Get parameters
		/// </summary>
		protected virtual void GetParameters()
		{
			// Can't get parameters if loading failed!
			if (effect == null)
				return;

			try
			{
				world =
					effect.GetParameter(null, "world");
				viewInverse =
					effect.GetParameter(null, "viewInverse");
				viewProj =
					effect.GetParameter(null, "viewProj");
				lightDir =
					effect.GetParameter(null, "lightDir");
				ambientColor =
					effect.GetParameter(null, "ambientColor");
				diffuseColor =
					effect.GetParameter(null, "diffuseColor");
				specularColor =
					effect.GetParameter(null, "specularColor");
				shininess =
					effect.GetParameter(null, "shininess");

				diffuseTexture =
					effect.GetParameter(null, "diffuseTexture");
				normalTexture =
					effect.GetParameter(null, "normalTexture");
				heightTexture =
					effect.GetParameter(null, "heightTexture");
				parallaxAmount =
					effect.GetParameter(null, "parallaxAmount");

				// We only got 2 techniques, select the best one we can use.
				effect.Technique = TechniqueName;

				// For PS1.1 we need additionally the NormalizeCubeTexture
				//always load, we can can switch modes now:
				//if (GraphicForm.CanDoPS20 == false)
				{
					EffectHandle normalizeCubeTexture =
						effect.GetParameter(null, "NormalizeCubeTexture");
					if (normalizeCubeTexture != null)
					{
						CubeTexture normalizationMap = null;
						if (File.Exists(
							Directories.ShadersDirectory + "\\" + "NormalizeCubeMap.dds"))
						{
							// Load normalization cube map (only used for ps1.1)
							normalizationMap =
								TextureLoader.FromCubeFile(GraphicForm.DirectXDevice,
								Directories.ShadersDirectory + "\\" + "NormalizeCubeMap.dds");
							effect.SetValue(normalizeCubeTexture, normalizationMap);
						} // if (File.Exists)
						else
						{
							Log.Write("WARNING: " + Directories.ShadersDirectory + "\\" +
								"NormalizeCubeMap.dds does not exist, unable to load it for " +
								"shader " + Filename);
						} // else
					} // if (normalizeCubeTexture)
				} // if (GraphicForm.CanDoPS20)
			} // try
			catch (Direct3DXException ex)
			{
				Log.Write("GetParameters failed. " + ex.ToString());
				//TODO, could be improved with, but is a lot of work:
				//DirectXHelper.GetError(ex.ErrorCode);

				// Kill effect
				effect.Dispose();
				effect = null;
			} // catch
		} // GetParameters()
		#endregion

		#region Update shader parameters
		/// <summary>
		/// Update matrices and effect parameters
		/// </summary>
		public void UpdateMatricesAndEffectParameters()
		{
			// World matrix is set in UpdateWorldMatrix(.)
			ViewProjMatrix = GraphicForm.ViewMatrix *
				GraphicForm.ProjectionMatrix;
			InverseViewMatrix = GraphicForm.InverseViewMatrix;
			LightDirection = GraphicForm.LightDir;
		} // UpdateMatricesAndEffectParameters()

		/// <summary>
		/// Update parameters, the matrices and light parameters come from
		/// GraphicForm. The material (colors and textures) are updated with mat.
		/// </summary>
		/// <param name="mat">Material</param>
		public void UpdateMaterialParameters(Material mat)
		{
			// We can't update without valid material
			if (mat == null)
				return;

			AmbientColorValue = mat.D3DMaterial.Ambient;
			DiffuseColorValue = ColorHelper.MultiplyColors(
				mat.D3DMaterial.Diffuse, GraphicForm.LightColor);
			SpecularColorValue = ColorHelper.MultiplyColors(
				mat.D3DMaterial.Specular, GraphicForm.LightColor);
			ShininessValue = mat.D3DMaterial.SpecularSharpness;

			DiffuseTextureValue = mat.diffuseTexture;
			NormalTextureValue = mat.normalTexture;
			HeightTextureValue = mat.heightTexture;

			ParallaxAmountValue = mat.parallaxAmount;
		} // UpdateMaterialParameters(mat)
		#endregion
	} // class ParallaxShader
} // namespace RocketCommander.Shaders
