﻿// Project: Rocket Commander, File: PostScreenGlow.cs
// Namespace: RocketCommander.Shaders, Class: PostScreenGlow
// Path: C:\code\RocketCommander\Shaders, Author: Abi
// Code lines: 618, Size of file: 19,89 KB
// Creation date: 10.11.2005 07:46
// Last modified: 24.12.2005 01:40
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections;
using System.Text;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;
using Direct3D = Microsoft.DirectX.Direct3D;
using RocketCommander.Graphics;
using RocketCommander.Helpers;
using RocketCommander.Game;
using Texture = RocketCommander.Graphics.Texture;
#endregion

namespace RocketCommander.Shaders
{
	/// <summary>
	/// Post screen glow shader based on PostScreenGlow.fx
	/// </summary>
	public class PostScreenGlow : IGraphicObject
	{
		#region Variables
		/// <summary>
		/// The shader effect filename for this shader.
		/// </summary>
		private const string Filename = "PostScreenGlow.fx";

		/// <summary>
		/// Helper texture for the ScreenGlow20 shader.
		/// </summary>
		private const string GlowShaderLookupTexture = "GlowShaderLookup.dds";

		/// <summary>
		/// Effect
		/// </summary>
		private Effect effect = null;
		/// <summary>
		/// Effect handles for window size and scene map.
		/// </summary>
		private EffectHandle windowSize,
			sceneMap,
			downsampleMap,
			blurMap1,
			blurMap2,
			radialSceneMap,
			radialBlurScaleFactor,
			screenBorderFadeoutMap;

		/// <summary>
		/// Links to the passTextures, easier to write code this way.
		/// This are just reference copies.
		/// </summary>
		private RenderToTexture sceneMapTexture,
			downsampleMapTexture,
			blurMap1Texture,
			blurMap2Texture,
			radialSceneMapTexture;

		/// <summary>
		/// Helper texture for the screen border (darken the borders).
		/// </summary>
		private Texture screenBorderFadeoutMapTexture = null;
		#endregion

		#region Properties
		/// <summary>
		/// Valid
		/// </summary>
		/// <returns>Bool</returns>
		public bool Valid
		{
			get
			{
				return effect != null &&
					GraphicForm.UsePS;
			} // get
		} // Valid

		/// <summary>
		/// Last used radial blur scale factor
		/// </summary>
		private float lastUsedRadialBlurScaleFactor = 0;
		/// <summary>
		/// Radial blur scale factor
		/// </summary>
		public float RadialBlurScaleFactor
		{
			set
			{
				if (effect != null &&
					radialBlurScaleFactor != null &&
					lastUsedRadialBlurScaleFactor != value)
				{
					lastUsedRadialBlurScaleFactor = value;
					effect.SetValue(radialBlurScaleFactor, value);
				} // if (effect.D3DEffect)
			} // set
		} // RadialBlurScaleFactor
		#endregion

		#region Constructor
		/// <summary>
		/// Create post screen glow
		/// </summary>
		public PostScreenGlow()
		{
			Reload();

			// Scene map texture
			sceneMapTexture = new RenderToTexture(
				RenderToTexture.SizeType.FullScreen);
			// Downsample map texture (to 1/4 of the screen)
			downsampleMapTexture = new RenderToTexture(
				RenderToTexture.SizeType.QuarterScreen);

			// Blur map texture
			blurMap1Texture = new RenderToTexture(
				RenderToTexture.SizeType.QuarterScreen);
			// Blur map texture
			blurMap2Texture = new RenderToTexture(
				RenderToTexture.SizeType.QuarterScreen);

			// Final map for glow, used to perform radial blur next step
			radialSceneMapTexture = new RenderToTexture(
				RenderToTexture.SizeType.FullScreen);
		} // PostScreenGlow()
		#endregion

		#region Dispose
		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			// Dispose old shader
			if (effect != null)
			{
				try
				{
					effect.Dispose();
				} // try
				catch (Exception ex)
				{
					Log.Write("Failed to dispose shader " + Filename + ": " +
						ex.ToString());
				} // catch
			} // if (effect)
			effect = null;
		} // Dispose()
		#endregion

		#region IGraphicObject Members
		/// <summary>
		/// On reset device
		/// </summary>
		public void OnResetDevice()
		{
		} // OnResetDevice()

		/// <summary>
		/// On lost device
		/// </summary>
		public void OnLostDevice()
		{
		} // OnLostDevice()
		#endregion

		#region Reload
		/// <summary>
		/// Reload
		/// </summary>
		public void Reload()
		{
			// Dispose old shader
			Dispose();

			string fullFilename = Directories.ShadersDirectory + "\\" + Filename;
			if (File.Exists(fullFilename) == false)
				throw new FileNotFoundException(
					"Can't load shader without valid shader filename",
					fullFilename);

			// Load it
			string effectError = "";
			try
			{
				effect = Effect.FromFile(GraphicForm.DirectXDevice,
					// Just load, no special stuff used.
					fullFilename, null, null, ShaderFlags.None, null, out effectError);
			} // try
			catch (Exception ex)
			{
				// Only use exception text if no effectError was logged
				if (String.IsNullOrEmpty(effectError))
					effectError = ex.ToString();
			} // catch

			// Error happened and no exception was thrown yet?
			if (String.IsNullOrEmpty(effectError) == false)
				throw new Exception(
					"Failed to load shader " + this.ToString() + ": " + effectError);

			GetParameters();

			GraphicForm.AddGraphicObject(this);
		} // Reload()
		#endregion

		#region Get parameters
		/// <summary>
		/// Reload
		/// </summary>
		protected void GetParameters()
		{
			// Can't get parameters if loading failed!
			if (effect == null)
				return;

			windowSize =
				effect.GetParameter(null, "windowSize");
			sceneMap =
				effect.GetParameter(null, "sceneMap");

			// We need both windowSize and sceneMap.
			if (windowSize == null ||
				sceneMap == null)
				throw new NotSupportedException("windowSize and sceneMap must be " +
					"valid in PostScreenShader=" + Filename);

			// Init additional stuff
			downsampleMap =
				effect.GetParameter(null, "downsampleMap");
			blurMap1 =
				effect.GetParameter(null, "blurMap1");
			blurMap2 =
				effect.GetParameter(null, "blurMap2");
			radialSceneMap =
				effect.GetParameter(null, "radialSceneMap");

			// Load screen border texture
			screenBorderFadeoutMap =
				effect.GetParameter(null, "screenBorderFadeoutMap");
			screenBorderFadeoutMapTexture =
				new Texture("ScreenBorderFadeout.dds");
			// Set texture
			effect.SetValue(screenBorderFadeoutMap,
				screenBorderFadeoutMapTexture.D3DTexture);

			// Glow shader lookup texture must exists (used for ps2.0)
			if (File.Exists(Directories.ShadersDirectory + "\\GlowShaderLookup.dds"))
			{
				radialBlurScaleFactor =
					effect.GetParameter(null, "radialBlurScaleFactor");

				// Load GlowShader helper texture, only used for ps_2_0
				EffectHandle glowShaderLookup =
					effect.GetParameter(null, "GlowShaderLookup");
				if (glowShaderLookup != null)
				{
					Direct3D.Texture glowShaderLookupMap = TextureLoader.FromFile(
						GraphicForm.DirectXDevice,
						Directories.ShadersDirectory + "\\" + "GlowShaderLookup.dds");
					effect.SetValue(glowShaderLookup, glowShaderLookupMap);
				} // if (normalizeCubeTexture)
			} // if (Glow shader lookup texture must exists)
		} // GetParameters()
		#endregion

		#region Show
		/// <summary>
		/// Execute shaders and show result on screen, Start(..) must have been
		/// called before and the scene should be rendered to sceneMapTexture.
		/// </summary>
		public void Show()
		{
			// Only apply post screen glow if texture is valid and effect is valid 
			if (sceneMapTexture == null ||
				Valid == false)
				return;

			try
			{
				// Copy back buffer to sceneMap render target.
				// Note: We could also use SetRenderTarget, render everything
				// and call ResetRenderTarget instead of this copy, but this
				// won't work for a multisample back buffer!
				GraphicForm.DirectXDevice.StretchRectangle(
					GraphicForm.BackBufferSurface,
					// Use full back buffer surface
					new System.Drawing.Rectangle(
					0, 0, GraphicForm.Resolution.Width, GraphicForm.Resolution.Height),
					sceneMapTexture.TextureSurface,
					// Use full render target surface
					new System.Drawing.Rectangle(
					0, 0, sceneMapTexture.Width, sceneMapTexture.Height),
					// Let DirectX decide which filter mode is the best.
					Microsoft.DirectX.Direct3D.TextureFilter.None);
			} // try
			catch (Exception ex)
			{
				throw new Exception("Failed to copy rectangle from back buffer " +
					"surface to render target. Error: " + ex.Message);
			} // catch

			try
			{
				// Don't use or write to the z buffer
				Device device = GraphicForm.DirectXDevice;
				device.RenderState.ZBufferEnable = false;
				device.RenderState.ZBufferWriteEnable = false;
				// Also don't use any kind of blending.
				device.RenderState.AlphaBlendEnable = false;
				device.RenderState.Lighting = false;

				if (windowSize != null)
					effect.SetValue(windowSize, new float[]
						//{ Graphics.Resolution.width, Graphics.Resolution.height });
						{ sceneMapTexture.Width, sceneMapTexture.Height });
				if (sceneMap != null)
					effect.SetValue(sceneMap,
						sceneMapTexture.D3DTexture);
				if (downsampleMap != null)
					effect.SetValue(downsampleMap,
						downsampleMapTexture.D3DTexture);
				if (blurMap1 != null)
					effect.SetValue(blurMap1,
						blurMap1Texture.D3DTexture);
				if (blurMap2 != null)
					effect.SetValue(blurMap2,
						blurMap2Texture.D3DTexture);
				if (radialSceneMap != null)
					effect.SetValue(radialSceneMap,
						radialSceneMapTexture.D3DTexture);

				RadialBlurScaleFactor = //-0.0125f;
					// Warning: To big values will make the motion blur look to
					// stepy (we see each step and thats not good). -0.02 should be max.
					-(0.005f + Player.Speed * 0.0225f);

				if (GraphicForm.UsePS20)
					effect.Technique = "ScreenGlow20";
				else
					effect.Technique = "ScreenGlow";

				int passes = effect.Begin(0);
				for (int pass = 0; pass < passes; pass++)
				{
					if (pass == 0)
						radialSceneMapTexture.SetRenderTarget();
					else if (pass == 1)
						downsampleMapTexture.SetRenderTarget();
					else if (pass == 2)
						blurMap1Texture.SetRenderTarget();
					else if (pass == 3)
						blurMap2Texture.SetRenderTarget();
					else
						GraphicForm.ResetRenderTarget();

					effect.BeginPass(pass);
					// For first effect we use radial blur, draw it with a grid
					// to get cooler results (more blur at borders than in middle).
					if (pass == 0)
						VBScreenHelper.Render10x10Grid();
					else
						VBScreenHelper.Render();
					effect.EndPass();
				} // for (pass, <, ++)
			} // try
			finally
			{
				effect.End();

				// Restore z buffer state
				GraphicForm.DirectXDevice.RenderState.ZBufferEnable = true;
				GraphicForm.DirectXDevice.RenderState.ZBufferWriteEnable = true;
			} // finally
		} // Show()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test post screen glow
		/// </summary>
		[Test]
		public static void TestPostScreenGlow()
		{
			Model testModel = null;
			PostScreenGlow glowShader = null;

			GameForm.StartTest("TestPostScreenGlow",
				delegate
				{
					testModel = new Model("Asteroid2.x");
					glowShader = new PostScreenGlow();
				},
				delegate
				{
					//Thread.Sleep(10);

					testModel.Render(Vector3.Empty);

					if (GameForm.Keyboard.State[
						Microsoft.DirectX.DirectInput.Key.Space] == false)
						glowShader.Show();
				});
		} // TestPostScreenGlow()
#endif
		#endregion
	} // class PostScreenGlow
} // namespace RocketCommander.Shaders
