﻿// Project: Rocket Commander, File: RenderToTexture.cs
// Namespace: RocketCommander.Shaders, Class: RenderToTexture
// Path: C:\code\RocketCommander\Shaders, Author: Abi
// Code lines: 477, Size of file: 15,95 KB
// Creation date: 10.11.2005 08:01
// Last modified: 10.11.2005 08:12
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections;
using System.Text;
using System.Drawing;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Direct3D = Microsoft.DirectX.Direct3D;
using RocketCommander.Graphics;
using RocketCommander.Helpers;
using Texture = RocketCommander.Graphics.Texture;
#endregion

namespace RocketCommander.Shaders
{
	/// <summary>
	/// Render to texture helper class based on the Texture class.
	/// This class allows to render stuff onto textures, if thats not
	/// supported, it will just not work and report an engine log message.
	/// This class is required for most PostScreenShaders.
	/// </summary>
	public class RenderToTexture : Texture
	{
		#region Variables
		/// <summary>
		/// Texture surface for SetRenderTarget. Uses d3dTexture (inherited).
		/// </summary>
		Surface textureSurface = null;
		/// <summary>
		/// If we are using any kind of multisampling we have to use an
		/// extra zBuffer for our textureSurface, because we can't use
		/// multisampling when rendering to texture.
		/// We only need one zBufferSurface for all RenderToTexture instances!
		/// </summary>
		internal static Surface zBufferSurface = null;

		/// <summary>
		/// Posible size types for creating a RenderToTexture object.
		/// </summary>
		public enum SizeType
		{
			/// <summary>
			/// Uses the full screen size for this texture
			/// </summary>
			FullScreen,
			/// <summary>
			/// Uses half the full screen size, e.g. 800x600 becomes 400x300.
			/// </summary>
			HalfScreen,
			/// <summary>
			/// Uses a quarter of the full screen size, e.g. 800x600 becomes 200x150.
			/// </summary>
			QuarterScreen,
		} // enum SizeTypes
		private SizeType sizeType;

		/// <summary>
		/// Calc size
		/// </summary>
		private void CalcSize()
		{
			switch (sizeType)
			{
				case SizeType.FullScreen:
					size = GraphicForm.Resolution;
					break;
				case SizeType.HalfScreen:
					size = new Size(GraphicForm.Resolution.Width / 2,
						GraphicForm.Resolution.Height / 2);
					break;
				case SizeType.QuarterScreen:
					size = new Size(GraphicForm.Resolution.Width / 4,
						GraphicForm.Resolution.Height / 4);
					break;
			} // switch
			CalcHalfPixelSize();
		} // CalcSize()
		#endregion

		#region Properties
		/// <summary>
		/// Get the used texture surface we want to render on.
		/// </summary>
		public Surface TextureSurface
		{
			get
			{
				return textureSurface;
			} // get
		} // TextureSurface
		#endregion

		#region Constructors
		/// <summary>
		/// Id for each created RenderToTexture for the generated filename.
		/// </summary>
		private static int RenderToTextureGlobalInstanceId = 0;
		/// <summary>
		/// Creates an offscreen texture with the specified size which
		/// can be used for render to texture.
		/// </summary>
		public RenderToTexture(SizeType setSizeType)
		{
			sizeType = setSizeType;
			CalcSize();

			texFilename = "RenderToTexture instance " +
				RenderToTextureGlobalInstanceId++;

			Create();

			if (Valid)
				GraphicForm.AddGraphicObject(this);
		} // RenderToTexture(setSizeType)
		#endregion

		#region Create
		/// <summary>
		/// Create
		/// </summary>
		private void Create()
		{
			Dispose();

			try
			{
				// Create the texture first, so we can get access to it's surface
				direct3dTexture = new Direct3D.Texture(
					GraphicForm.DirectXDevice,
					size.Width, size.Height, 1,
					Usage.RenderTarget,
					Format.A8R8G8B8,
					Pool.Default);

				// Get surface for this texture (we need it to set render target)
				textureSurface = direct3dTexture.GetSurfaceLevel(0);

				// Note: Might also be required for no multisampling, or clear z buffer
				// after using render target if more 3d content has to be rendered
				// this frame.
				if (GraphicForm.UsingMultisampling &&
					zBufferSurface == null)
					zBufferSurface =
						GraphicForm.DirectXDevice.CreateDepthStencilSurface(
						GraphicForm.Resolution.Width, GraphicForm.Resolution.Height,
						// Lets use the same stuff as the back buffer.
						GraphicForm.BackBufferDepthFormat,
						// Don't use multisampling, render target does not support that.
						MultiSampleType.None, 0, true);

				loaded = true;
			} // try
			catch (Exception ex)
			{
				Log.Write("Creating RenderToTexture surface failed: " + ex.ToString());
				textureSurface = null;
				loaded = false;
			} // catch
		} // Create()
		#endregion

		#region Dispose
		/// <summary>
		/// Dispose
		/// </summary>
		public override void Dispose()
		{
			if (textureSurface != null)
				textureSurface.Dispose();
			textureSurface = null;

			base.Dispose();
		} // Dispose()
		#endregion

		#region Clear
		/// <summary>
		/// Clear render target (call SetRenderTarget first)
		/// </summary>
		public void Clear(Color clearColor)
		{
			if (loaded == false ||
				textureSurface == null)
				return;

			GraphicForm.DirectXDevice.Clear(
				ClearFlags.Target | ClearFlags.ZBuffer,
				clearColor.ToArgb(), 1.0f, 0);
		} // Clear(clearColor)
		#endregion

		#region Set render target
		/// <summary>
		/// Set render target to this texture to render stuff on it.
		/// </summary>
		public bool SetRenderTarget()
		{
			if (loaded == false ||
				textureSurface == null)
				return false;

			GraphicForm.DirectXDevice.SetRenderTarget(0, textureSurface);
			if (zBufferSurface != null)
				GraphicForm.DirectXDevice.DepthStencilSurface = zBufferSurface;
			return true;
		} // SetRenderTarget()
		#endregion

		#region Device events (lost and reset device for resizing, etc.)
		/// <summary>
		/// Called when the device is reset
		/// </summary>
		public override void OnResetDevice()
		{
			// Recalc size
			CalcSize();

			// Re-create everything.
			Create();
		} // OnResetDevice()

		/// <summary>
		/// Clear any resources that need to be lost, called when device is lost.
		/// </summary>
		public override void OnLostDevice()
		{
			Dispose();
		} // OnLostDevice()
		#endregion
	} // class RenderToTexture
} // namespace Abi.Graphic
