// Project: Rocket Commander, File: VBScreenHelper.cs
// Namespace: RocketCommander.Shaders, Class: VBScreen
// Path: C:\code\Rocket Commander\Shaders, Author: Abi
// Code lines: 298, Size of file: 17,12 KB
// Creation date: 07.11.2005 01:57
// Last modified: 14.11.2005 21:45
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;
using Direct3D = Microsoft.DirectX.Direct3D;
using RocketCommander.Graphics;
using System.Drawing;
#endregion

namespace RocketCommander.Shaders
{
	/// <summary>
	/// The VBScreenHelper class helps you to create and use vertex buffers for
	/// on screen rendering used in PreScreen and PostScreen shaders.
	/// You don't have to create a VB for every shader anymore, just call
	/// VBScreenHelper.Render() and everything will be created for you and
	/// reused if same parameters are requested again!
	/// Supports also Grid screen rendering required for radial motion blur.
	/// </summary>
	public class VBScreenHelper
	{
		#region VBScreen helper class
		/// <summary>
		/// VBScreen holds all data for the vbScreens list to reuse existing
		/// VBScreens. Handles also the VB, creation and rendering.
		/// </summary>
		private class VBScreen
		{
			#region Variables
			/// <summary>
			/// Vertex buffer to render stuff on screen.
			/// </summary>
			private VertexBuffer vbScreen;
			#endregion

			#region Constructor
			/// <summary>
			/// Create VB screen
			/// </summary>
			public VBScreen()
			{
				vbScreen = new VertexBuffer(
					typeof(CustomVertex.PositionTextured),
					4,
					GraphicForm.DirectXDevice,
					Usage.WriteOnly,
					CustomVertex.PositionTextured.Format,
					Pool.Managed);
				vbScreen.Created +=
					new System.EventHandler(OnCreateVBScreen);
				OnCreateVBScreen(vbScreen, null);

				// Recreate vb if device was lost and is reseted again
				GraphicForm.DirectXDevice.DeviceReset +=
					new System.EventHandler(OnCreateVBScreen);
			} // VBScreen()
			#endregion

			#region Create VB
			/// <summary>
			/// On create VB screen
			/// </summary>
			/// <param name="sender">Sender</param>
			/// <param name="e">E</param>
			private void OnCreateVBScreen(object sender, EventArgs e)
			{
				GraphicsStream gfxStream = vbScreen.Lock(0, 0, 0);
				CustomVertex.PositionTextured[] vertices =
					new CustomVertex.PositionTextured[]
				{
					new CustomVertex.PositionTextured(
					new Vector3(-1.0f, -1.0f, 0.5f),
					0, 1),
					new CustomVertex.PositionTextured(
					new Vector3(-1.0f, 1.0f, 0.5f),
					0, 0),
					new CustomVertex.PositionTextured(
					new Vector3(1.0f, -1.0f, 0.5f),
					1, 1),
					new CustomVertex.PositionTextured(
					new Vector3(1.0f, 1.0f, 0.5f),
					1, 0),
				};

				gfxStream.Write(vertices);
				vbScreen.Unlock();
			} // OnCreateVBScreen(sender, e)
			#endregion

			#region Render
			/// <summary>
			/// Render
			/// </summary>
			public void Render()
			{
				// Rendering is pretty straight forward
				GraphicForm.DirectXDevice.VertexFormat =
					CustomVertex.PositionTextured.Format;
				GraphicForm.DirectXDevice.SetStreamSource(
					0, vbScreen,
					0, CustomVertex.PositionTextured.StrideSize);
				GraphicForm.DirectXDevice.DrawPrimitives(
					PrimitiveType.TriangleStrip, 0, 2);
			} // Render()
			#endregion
		} // class VBScreen
		#endregion

		#region Render
		/// <summary>
		/// Vb screen instance
		/// </summary>
		static VBScreen vbScreenInstance = null;
		/// <summary>
		/// Just render a vertex buffer with the screen coordinates.
		/// No subTexelSize stuff is performed, do that in the fx file.
		/// </summary>
		public static void Render()
		{
			if (vbScreenInstance == null)
				vbScreenInstance = new VBScreen();

			vbScreenInstance.Render();
		} // Render()
		#endregion

		#region GridScreen helper class
		/// <summary>
		/// Another vertex and index buffer for a screen grid, basically
		/// used for the same purpose as VBScreen, but allows us to create
		/// a grid (e.g. 10x10), very useful for advanced post screen shaders.
		/// </summary>
		private class GridScreen
		{
			#region Variables
			Size gridDimension;
			IndexBuffer indexBuffer = null;
			VertexBuffer vertexBuffer = null;
			#endregion

			#region Constructor
			/// <summary>
			/// Create grid screen
			/// </summary>
			/// <param name="setGridDimension">Set grid dimension</param>
			public GridScreen(Size setGridDimension)
			{
				if (setGridDimension.Width < 2 ||
					setGridDimension.Height < 2)
					throw new ArgumentException("setGridDimension="+setGridDimension,
						"Grid size must be at least (2, 2).");
				gridDimension = setGridDimension;

				indexBuffer = new IndexBuffer(
					typeof(ushort),
					(gridDimension.Width-1)*(gridDimension.Height-1)*2*3,
					GraphicForm.DirectXDevice,
					Usage.WriteOnly,
					Pool.Managed);
				vertexBuffer = new VertexBuffer(
					typeof(CustomVertex.PositionTextured),
					gridDimension.Width * gridDimension.Height,
					GraphicForm.DirectXDevice,
					Usage.WriteOnly,
					CustomVertex.PositionTextured.Format,
					Pool.Managed);//Pool.SystemMemory);

				indexBuffer.Created += new System.EventHandler(OnCreateIB);
				OnCreateIB(indexBuffer, null);
				GraphicForm.DirectXDevice.DeviceReset +=
					new System.EventHandler(OnCreateIB);

				vertexBuffer.Created += new System.EventHandler(OnCreateVB);
				OnCreateVB(vertexBuffer, null);
				GraphicForm.DirectXDevice.DeviceReset +=
					new System.EventHandler(OnCreateVB);
			} // GridScreen(setGridDimension)
			#endregion

			#region On create IB
			/// <summary>
			/// On create IB
			/// </summary>
			/// <param name="sender">Sender</param>
			/// <param name="e">E</param>
			private void OnCreateIB(object sender, EventArgs e)
			{
				ushort[] indices = (ushort[])indexBuffer.Lock(
					0, typeof(ushort), LockFlags.None,
					(gridDimension.Width-1)*(gridDimension.Height-1)*3*2);
				// Just simply create all indices of the grid
				int num = 0;
				for (int x = 0; x < gridDimension.Width-1; x++)
					for (int y = 0; y < gridDimension.Height-1; y++)
					{
						ushort index1 = (ushort)(x + y * gridDimension.Width);
						ushort index2 = (ushort)((x + 1) + y * gridDimension.Width);
						ushort index3 = (ushort)((x + 1) + (y + 1) * gridDimension.Width);
						ushort index4 = (ushort)(x + (y + 1) * gridDimension.Width);

						indices[num] = index1;
						indices[num+1] = index3;
						indices[num+2] = index2;
						indices[num+3] = index1;
						indices[num+4] = index4;
						indices[num+5] = index3;

						num += 6;
					} // for for (int)

				indexBuffer.Unlock();
			} // OnCreateIB(sender, e)
			#endregion

			#region On create VB
			/// <summary>
			/// On create VB
			/// </summary>
			/// <param name="sender">Sender</param>
			/// <param name="e">E</param>
			private void OnCreateVB(object sender, EventArgs e)
			{
				GraphicsStream gfxStream = vertexBuffer.Lock(0, 0, 0);
				CustomVertex.PositionTextured[] vertices =
					new CustomVertex.PositionTextured[
					gridDimension.Width * gridDimension.Height];
				// Just simply create all vertices of the grid
				for (int x = 0; x < gridDimension.Width; x++)
					for (int y = 0; y < gridDimension.Height; y++)
					{
						vertices[x + y * gridDimension.Width] =
							new CustomVertex.PositionTextured(new Vector3(
							-1.0f+2.0f*(float)x/(float)(gridDimension.Width-1),
							-1.0f+2.0f*(float)y/(float)(gridDimension.Height-1),
							0.5f),
							(float)x/(float)(gridDimension.Width-1),
							// DirectX expect bottom up for the screen rendering.
							1.0f-((float)y/(float)(gridDimension.Height-1)));
					} // for for (int)

				gfxStream.Write(vertices);
				vertexBuffer.Unlock();
			} // OnCreateVB(sender, e)
			#endregion

			#region Render
			/// <summary>
			/// Render
			/// </summary>
			public void Render()
			{
				GraphicForm.DirectXDevice.VertexFormat =
					CustomVertex.PositionTextured.Format;
				GraphicForm.DirectXDevice.SetStreamSource(
					0, vertexBuffer,
					0, CustomVertex.PositionTextured.StrideSize);
				GraphicForm.DirectXDevice.Indices = indexBuffer;

				GraphicForm.DirectXDevice.DrawIndexedPrimitives(
					PrimitiveType.TriangleList,
					0, 0, gridDimension.Width * gridDimension.Height,
					0, (gridDimension.Width - 1) * (gridDimension.Height - 1) * 2);
			} // Render()
			#endregion
		} // class VBGridScreen
		#endregion

		#region Render 10x10 screen grid
		static GridScreen gridScreen10x10Instance = null;
		/// <summary>
		/// Just render a 10x10 grid with help of GridScreen on the screen.
		/// No subTexelSize stuff is performed, do that in the fx file.
		/// </summary>
		public static void Render10x10Grid()
		{
			if (gridScreen10x10Instance == null)
				gridScreen10x10Instance = new GridScreen(
					new Size(10, 10));

			gridScreen10x10Instance.Render();
		} // Render10x10Grid()
		#endregion
	} // class VBScreenHelper
} // namespace RocketCommander.Shaders
