// Project: Rocket Commander, File: MusicItem.cs
// Namespace: RocketCommander.Sound, Class: MusicItem
// Creation date: 13.12.2004 03:28
// Last modified: 11.12.2005 17:57
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
using Microsoft.DirectX.AudioVideoPlayback;
using RocketCommander.Helpers;
#endregion

namespace RocketCommander.Sounds
{
	/// <summary>
	/// Single Music Item, consists of filename, length and position.
	/// Will be updated when playing music item (else length defaults to 0)
	/// </summary>
	public class MusicItem
	{
		#region Variables
		/// <summary>
		/// Filename for this music item.
		/// </summary>
		public string filename;
		/// <summary>
		/// Current length and position in ms of the item.
		/// </summary>
		public int lengthInMs, positionInMs;
		/// <summary>
		/// Graphic Audio file we use underneath.
		/// </summary>
		private Audio audioFile = null;

		/// <summary>
		/// Failed to load
		/// </summary>
		private bool failedToLoad = false;

		/// <summary>
		/// Was music paused?
		/// </summary>
		private bool paused = false;

		/// <summary>
		/// Paused
		/// </summary>
		/// <returns>Bool</returns>
		public bool Paused
		{
			get
			{
				return paused;
			} // get
		} // Paused

		/// <summary>
		/// Is music file loaded?
		/// </summary>
		public bool Loaded
		{
			get
			{
				return audioFile != null;
			} // get
		} // Valid
		#endregion

		#region Constructor
		/// <summary>
		/// Create music item
		/// </summary>
		public MusicItem(string setFilename)
		{
			filename = setFilename;
			lengthInMs = 0;
			positionInMs = 0;
			audioFile = null;
		} // MusicItem(setFilename)
		#endregion

		#region Properties
		/// <summary>
		/// Get position percent
		/// </summary>
		public float PositionPercent
		{
			get
			{
				if (lengthInMs > 0)
					return (float)positionInMs / (float)lengthInMs;
				return 0.0f;
			} // get
		} // PositionPercent

		/// <summary>
		/// Get position time string
		/// </summary>
		public string PositionTimeText
		{
			get
			{
				return (positionInMs / (60 * 1000)).ToString("00") + ":" +
					((positionInMs / 1000) % 60).ToString("00");
			} // get
		} // PositionTimeText

		/// <summary>
		/// Get length time string
		/// </summary>
		public string LengthTimeText
		{
			get
			{
				return (lengthInMs / (60 * 1000)).ToString("00") + ":" +
					((lengthInMs / 1000) % 60).ToString("00");
			} // get
		} // LengthTimeText
		#endregion

		#region Methods
		/// <summary>
		/// Load and play
		/// </summary>
		public bool LoadAndPlay()
		{
			// Failed before, don't try again
			if (failedToLoad)
				return false;

			try
			{
				audioFile = new Audio(
					Directories.MusicDirectory + "\\" + filename, true);
				lengthInMs = (int)(audioFile.Duration * 1000);
				SetVolume(MusicManager.MusicVolume);
			} // try
			catch (Exception ex)
			{
				Log.Write("Failed to load music file: "+ex.ToString());
				// Remember that this item failed. Don't play it again!
				failedToLoad = true;
				audioFile = null;
			} // catch
			return audioFile != null;
		} // LoadAndPlay()

		/// <summary>
		/// Update position
		/// </summary>
		public void UpdatePosition()
		{
			if (audioFile != null)
				positionInMs = (int)(audioFile.CurrentPosition * 1000.0f);
		} // UpdatePosition()

		/// <summary>
		/// Is at end
		/// </summary>
		public bool IsAtEnd()
		{
			return positionInMs + 100 >= lengthInMs;
		} // IsAtEnd()

		/// <summary>
		/// Close
		/// </summary>
		public void Close()
		{
			if (audioFile != null)
			{
				audioFile.Dispose();
				audioFile = null;
			} // if (audioFile)
		} // Close()

		/// <summary>
		/// Pause
		/// </summary>
		public void Pause()
		{
			if (audioFile == null)
				return;
			positionInMs = (int)(audioFile.CurrentPosition * 1000);
			audioFile.Pause();
			paused = true;
		} // Pause()

		/// <summary>
		/// Restart
		/// </summary>
		public void Restart()
		{
			// Not initializied?
			if (audioFile == null)
				// Then try to init and play
				LoadAndPlay();
			else
				// Just resume if already loaded and was just paused!
				audioFile.Play();
		} // Restart()

		/// <summary>
		/// Set volume
		/// </summary>
		public void SetVolume(float volume)
		{
			if (audioFile == null)
				return;
			// See Sample.cs
			int helper = //650 + volume * 7 / 2;
				650 + (int)(350 * volume);
			if (helper < 0)
				helper = 0;
			if (helper > 1000)
				helper = 1000;
			audioFile.Volume = (helper - 1000) * 10; // -10000 - 0
		} // SetVolume(volume)

		/// <summary>
		/// Seek
		/// </summary>
		public void Seek(float percent)
		{
			if (audioFile == null)
				return;
			audioFile.SeekCurrentPosition(
				percent * audioFile.Duration,
				SeekPositionFlags.AbsolutePositioning);
		} // Seek(percent)
		#endregion

		#region To string
		/// <summary>
		/// To string
		/// </summary>
		public override string ToString()
		{
			return "(name=" + filename +
				", length=" + lengthInMs +
				", position=" + positionInMs +
				", loaded=" + (audioFile != null) + ")";
		} // ToString()
		#endregion
	}	// class MusicItem
} // namespace RocketCommander.Sound
