// Project: Rocket Commander, File: MusicManager.cs
// Namespace: RocketCommander.Sound, Class: MusicManager
// Creation date: 13.12.2004 03:28
// Last modified: 12.12.2005 02:16
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
#if DEBUG
using NUnit.Framework;
#endif
using RocketCommander.Helpers;
using RocketCommander.Properties;
#endregion

namespace RocketCommander.Sounds
{
	/// <summary>
	/// MusicManager: Handles music files (mp3s) and plays them.
	/// Depending if we are currently in game or not the game music or the
	/// menu music will be played and repeated as long as the game runs.
	/// Be sure to call Update() each frame, else playback will stop after
	/// finishing playing current song!
	/// </summary>
	public class MusicManager : IDisposable
	{
		#region Variables
		/// <summary>
		/// Music volume
		/// </summary>
		public const float MusicVolume = 0.9f;

		/// <summary>
		/// List of MusicItems, we only support 2 musics: menu and game music.
		/// </summary>
		private List<MusicItem> list = new List<MusicItem>();

		/// <summary>
		/// Current music mode: True means we are in game and playing the game
		/// music, false means we are currently in the menu.
		/// </summary>
		private bool currentMusicMode = false;

		/// <summary>
		/// Menu and game music filenames.
		/// </summary>
		private const string
			MenuMusicFilename = "Rocket Commander Menu Music.mp3",
			GameMusicFilename = "Rocket Commander Game Music.mp3";
		#endregion

		#region Properties
		/// <summary>
		/// Current music mode
		/// </summary>
		/// <returns>Bool</returns>
		public bool CurrentMusicMode
		{
			set
			{
				bool oldMusicMode = currentMusicMode;
				currentMusicMode = value;

				if (currentMusicMode != oldMusicMode &&
					list.Count >= 2 &&
					stopMusic == false)
				{
					// Close old
					list[oldMusicMode ? 1 : 0].Close();

					// And play new
					list[currentMusicMode ? 1 : 0].LoadAndPlay();
				} // if (currentMusicMode)
			} // set
		} // CurrentMusicMode
		#endregion

		#region Methods
		bool stopMusic = false;
		/// <summary>
		/// Permanently stop music, used from the main menu to disable any music.
		/// </summary>
		public void Stop()
		{
			if (list.Count < 2 ||
				stopMusic)
				return;

			//not required here: Settings.Default.MusicOn = false;
			stopMusic = true;
			list[currentMusicMode ? 1 : 0].Pause();
		} // Stop()

		/// <summary>
		/// Start playing music, disables the stop music method.
		/// Next call to Update will let the music play again!
		/// </summary>
		public void Start()
		{
			if (stopMusic == false)
				return;

			Settings.Default.MusicOn = true;
			stopMusic = false;
		} // Start()

		/// <summary>
		/// Pause music, used when switching to another application.
		/// </summary>
		public void Pause()
		{
			if (list.Count < 2)
				return;

			list[currentMusicMode ? 1 : 0].Pause();
		} // Stop()

		/// <summary>
		/// Update current position of current MusicItem and go to next
		/// item if at end, also restart (and randomize) list if at end!
		/// </summary>
		public void Update()
		{
			if (list.Count < 2 ||
				stopMusic)
				return;

			if (list[currentMusicMode ? 1 : 0].Paused)
				list[currentMusicMode ? 1 : 0].Restart();
			list[currentMusicMode ? 1 : 0].UpdatePosition();
			if (list[currentMusicMode ? 1 : 0].IsAtEnd())
				list[currentMusicMode ? 1 : 0].LoadAndPlay();
		} // Update()
		#endregion

		#region Constructor
		/// <summary>
		/// Init MusicManager at given music directory, will just load
		/// the 2 files: menu and game music. Music will start playing
		/// when calling Update (has to be done each frame).
		/// </summary>
		public MusicManager()
		{
			list.Add(new MusicItem(MenuMusicFilename));
			list.Add(new MusicItem(GameMusicFilename));

			// Play any music if music is disabled
			stopMusic = Settings.Default.MusicOn == false;
		} // MusicManager(directory, initiallyPaused, initialVolume)
		#endregion

		#region IDisposable Members
		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			if (list.Count < 2)
				return;

			list[currentMusicMode ? 1 : 0].Close();
		} // Dispose()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Test this class
		/// </summary>
		[Test]
		public static void TestPlayMusic()
		{
			// Start playing
			new MusicManager().Update();

			// Run sound test
			Sample.SampleTests.TestPlaySamples();
		} // TestThisClass()
#endif
		#endregion
	}	// class MusicManager
} // namespace RocketCommander.Sound
