// Project: Rocket Commander, File: Sample.cs
// Namespace: RocketCommander.Sound, Class: Sample
// Creation date: 13.12.2004 03:28
// Last modified: 24.12.2005 08:21
// Generated with Commenter by abi.exDream.com

#region Using directives
using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.IO;
#if DEBUG
using NUnit.Framework;
#endif
using Microsoft.DirectX.DirectSound;
using RocketCommander.Helpers;
#endregion

namespace RocketCommander.Sounds
{
	/// <summary>
	/// Sample class, will manage sound samples (load, save, play, stop, etc.).
	/// All Samples will be managed by the Sound class (automatic disposing
	/// there at the end of the application).
	/// Note: A lot of methods inside this class with catch exceptions and
	/// ignore them if anything goes wrong, this is not good exception handling,
	/// but since this happens VERY rarely on DirectSound samples in the beta
	/// test and I was unable to get to the root of this problem.
	/// Anyway, it works fine and if for some reason some sample fails after
	/// 20 minute playing and continues to work after that, who cares?
	/// </summary>
	public class Sample : IDisposable
	{
		#region Variables
		/// <summary>
		/// Loaded
		/// </summary>
		private bool loaded = false;
		/// <summary>
		/// Is Sample successfully loaded?
		/// </summary>
		public bool Loaded
		{
			get
			{
				return loaded;
			} // get
		} // Loaded

		/// <summary>
		/// Filename
		/// </summary>
		private string filename = "";
		/// <summary>
		/// Get filename of this sound sample
		/// </summary>
		public string Filename
		{
			get
			{
				return filename;
			} // get
		} // Filename

		private const int MaxSimultaneousSounds = 3;
		/// <summary>
		/// Buffer
		/// </summary>
		private SecondaryBuffer[] buffer = null;
		//obs: new SecondaryBuffer[MaxSimultaneousSounds];
		/// <summary>
		/// Channel volume
		/// </summary>
		private float[] channelVolume = null;//obs: { 0, 0, 0 };
		/// <summary>
		/// Channel panning
		/// </summary>
		private float[] channelPanning = null;//obs: { 0, 0, 0 };
		/// <summary>
		/// Current buffer number
		/// </summary>
		private int currentBufferNum = 0;

		/// <summary>
		/// Default frequency
		/// </summary>
		private int defaultFrequency = 0;
		#endregion

		#region Constructor
		/// <summary>
		/// Only warn once about device null
		/// </summary>
		static bool onlyWarnOnceAboutDeviceNull = false;
		/// <summary>
		/// Constructor: Load sample with given filename.
		/// </summary>
		public Sample(string setFilename, int setNumberOfSimultaneousSounds)
		{
			// Can't load without valid device!
			if (Sound.Device == null)
			{
				if (onlyWarnOnceAboutDeviceNull == false)
				{
					onlyWarnOnceAboutDeviceNull = true;
					Log.Write("DirectSound device is not initialized, " +
						"unable to load sample: " + setFilename);
				} // if (onlyWarnOnceAboutDeviceNull)
				return;
			} // if (Sound.Device)

			filename = setFilename;
			try
			{
				if (File.Exists(Directories.SoundsDirectory + "\\" + filename) ==
					false)
				{
					Log.Write("Sample " + filename + " was not found!");
					loaded = false;
					return;
				} // if (File.Exists)

				BufferDescription bufferDesc = new BufferDescription();
				bufferDesc.StaticBuffer = true;
				bufferDesc.ControlPan = true;
				bufferDesc.ControlVolume = true;
				bufferDesc.ControlFrequency = true;

				buffer = new SecondaryBuffer[setNumberOfSimultaneousSounds];
				channelVolume = new float[setNumberOfSimultaneousSounds];
				channelPanning = new float[setNumberOfSimultaneousSounds];

				buffer[0] = new SecondaryBuffer(
					Directories.SoundsDirectory + "\\" + filename,
					bufferDesc,
					Sound.Device);
				defaultFrequency = buffer[0].Frequency;

				for (int i = 1; i < buffer.Length; i++)
					buffer[i] = buffer[0];

				try
				{
					for (int i = 1; i < buffer.Length; i++)
						buffer[i] = buffer[0].Clone(Sound.Device);
				} // try
				catch { } // catch

				Sound.AddSample(this);
				loaded = true;
			} // try
			catch (Exception ex)
			{
				loaded = false;
				Log.Write("Failed to load sound file " +
					filename + ". Error: " + ex.ToString());
			} // catch (ex)
		} // Sample(setFilename)

		/// <summary>
		/// Create sample
		/// </summary>
		/// <param name="setFilename">Set filename</param>
		public Sample(string setFilename)
			: this(setFilename, MaxSimultaneousSounds)
		{
		} // Sample(setFilename)
		#endregion

		#region Disposing
		/// <summary>
		/// Already disposed?
		/// </summary>
		protected bool disposed = false;
		/// <summary>
		/// Dispose
		/// </summary>
		public virtual void Dispose()
		{
			if (disposed)
				return;
			disposed = true;

			// Kill sample
			try
			{
				// Only dispose first buffer, clones instances don't have to be
				// disposed.
				if (buffer[0] != null)
					buffer[0].Dispose();
			} // try
			catch { } // ignore any error
		} // Dispose()
		#endregion

		#region Volume
		/// <summary>
		/// Stay in range, val will be set to min if less or to max when bigger.
		/// </summary>
		private float StayInRange(float val, float min, float max)
		{
			if (val < min)
				return min;
			if (val > max)
				return max;
			return val;
		} // StayInRange(val, min, max)

		/// <summary>
		/// Get/Set Volume, will convert volume from 0.0-1.0 to db format for DS.
		/// </summary>
		public float Volume
		{
			get
			{
				return channelVolume[currentBufferNum];
			} // get
			set
			{
				try
				{				
					// Update per channel
					if (channelVolume[currentBufferNum] != value)
					{
						channelVolume[currentBufferNum] =
							StayInRange(value, 0.0f, 1.0f);
						// Convert volume to db
						// (1/2=-10dB, 1/4=-20dB, 1/8=-30dB, 1/12=-35dB)
						int helper =
							650 + (int)(350 * channelVolume[currentBufferNum]);
						if (helper < 0)
							helper = 0;
						if (helper > 1000)
							helper = 1000;
						int volumeInDb = (helper - 1000) * 10; // range: -10000 - 0
						// Set to current buffer
						if (buffer[currentBufferNum] != null)
							buffer[currentBufferNum].Volume = volumeInDb;
					} // if (channelVolume[currentBufferNum])
				} // try
				catch { } // ignore if anything goes wrong here
			} // set
		} // Volume
		#endregion

		#region Panning
		/// <summary>
		/// Get/Set Panning, will convert from -100-100 to format in DS.
		/// </summary>
		private float Panning
		{
			get
			{
				return channelPanning[currentBufferNum];
			} // get
			set
			{
				if (channelPanning[currentBufferNum] != value)
				{
					channelPanning[currentBufferNum] =
						StayInRange(value, -1.0f, 1.0f);
					// Set to buffer
					if (buffer[currentBufferNum] != null)
						buffer[currentBufferNum].Pan =
							(int)(channelPanning[currentBufferNum] * 10000);
				} // if (channelPanning[currentBufferNum])
			} // set
		} // Panning
		#endregion

		#region Frequency
		/// <summary>
		/// Pitch factor for changing frequency.
		/// Changes only the 
		/// </summary>
		/// <returns>Float</returns>
		public float PitchFactor
		{
			set
			{
				try
				{
					buffer[currentBufferNum].Frequency =
						(int)(defaultFrequency * value);
				} // try
				catch { } // ignore if anything goes wrong here
			} // set
		} // Frequency
		#endregion

		#region Play looped
		/// <summary>
		/// Play looped, use Volume, Panning and Frequency to change playback.
		/// </summary>
		public void PlayLooped()
		{
			// Can't play if not loaded sucessfully
			if (loaded == false ||
				buffer == null)
				return;

			// Use default values
			Volume = 1.0f;
			Panning = 0.0f;

			// Just play
			buffer[currentBufferNum].Play(0, BufferPlayFlags.Looping);
		} // PlayLooped()
		#endregion

		#region Stop
		/// <summary>
		/// Stop
		/// </summary>
		public void Stop()
		{
			// Can't stop if not loaded sucessfully
			if (loaded == false ||
				buffer == null)
				return;
			
			try
			{
				buffer[currentBufferNum].Stop();
			} // try
			catch { } // ignore if anything goes wrong here
		} // Stop()
		#endregion

		#region Play sample
		/// <summary>
		/// Helpers to forbid same sound to be played more than twice
		/// at the same time (will cause sound system to fuck up ^^).
		/// So if we have already started 2 sounds of that type, we will
		/// wait for at least 200ms to start playing any new sound!
		/// </summary>
		private int lastPlayedTime = 0;
		/// <summary>
		/// Reset last played time
		/// </summary>
		public void ResetLastPlayedTime()
		{
			lastPlayedTime = 0;
		} // ResetLastPlayedTime()

		private const int PlayWaitTimeMs = 100;

		/// <summary>
		/// Play the sample. This overload allows to set volume and panning.
		/// </summary>
		public void Play(float setVolume, float setPanning)
		{
			// Can't play if not loaded sucessfully
			if (loaded == false ||
				buffer == null)
				return;

			// Only play one sample at the same time in a 100ms timeframe
			if (ElapsedTime.MsTotal - lastPlayedTime >= PlayWaitTimeMs)
			{
				try
				{
					lastPlayedTime = ElapsedTime.MsTotal;

					// Go to next buffer or restart at first buffer when all playing
					if (buffer[currentBufferNum].PlayPosition > 0 &&
						currentBufferNum < buffer.Length - 1)
						currentBufferNum++;
					else
						currentBufferNum = 0;

					// Now set volume and panning
					Volume = setVolume;
					Panning = setPanning;

					// Make sure to start at position 0
					buffer[currentBufferNum].SetCurrentPosition(0);
					// And play the crap
					buffer[currentBufferNum].Play(0, BufferPlayFlags.Default);
				} // try
				catch { } // ignore if anything goes wrong here
			} // if (ElapsedTime.MsTotal)
			// else don't play
		} // Play(setVolume, setPanning)

		/// <summary>
		/// Play the sample. This overload allows to set volume, panning is 0.
		/// </summary>
		public void Play(float setVolume)
		{
			Play(setVolume, 0);
		} // Play(setVolume)

		/// <summary>
		/// Play the sample. This overload uses 1 for volume and 0 for panning.
		/// </summary>
		public void Play()
		{
			Play(1, 0);
		} // Play()
		#endregion

		#region Unit Testing
#if DEBUG
		/// <summary>
		/// Sample tests
		/// </summary>
		public class SampleTests
		{
			#region Test play samples
			/// <summary>
			/// Test play samples
			/// </summary>
			[Test]
			public static void TestPlaySamples()
			{
				// Initialize and load stuff
				TestForm testForm = new TestForm("TestPlaySamples");
				Sound.Init(testForm);
				Sample sample1 = new Sample("Fuel.wav"),
					sample2 = new Sample("Speed.wav"),
					sample3 = new Sample("Whosh4.wav");

				// Add buttons for user control
				testForm.AddButton("Play sample 1 "+sample1.Filename,
					delegate { sample1.Play(); });
				testForm.AddButton("Play sample 2 "+sample2.Filename,
					delegate { sample2.Play(); });
				testForm.AddButton("Play sample 3 "+sample3.Filename,
					delegate { sample3.Play(); });
				testForm.AddButton("Change sample 1 to Health.wav",
					delegate
					{
						sample1 = new Sample("Health.wav");
						// Also update text of button 1
						testForm.Controls[0].Text = "Play sample 1 "+sample1.Filename;
					});

				// And run the thing
				testForm.Run();
			} // TestPlaySamples()
			#endregion

			#region Test pitching sounds
			/// <summary>
			/// Test pitching sounds, little test suite for
			/// playing around with sounds and changing the frequencies.
			/// </summary>
			[Test]
			public static void TestPitchingSounds()
			{
				TestForm form = new TestForm("TestPitchingSounds");
				Sound.Init(form);
				Sample testSample = new Sample("Victory.wav");

				form.AddButton("Change to Victory.wav",
					delegate { testSample = new Sample("Victory.wav"); });
				form.AddButton("Change to Health.wav",
					delegate { testSample = new Sample("Health.wav"); });
				form.AddButton("Change to Fuel.wav",
					delegate { testSample = new Sample("Fuel.wav"); });

				form.AddButton("Play normally",
					delegate { testSample.Play(); });
				form.AddButton("Play left channel",
					delegate { testSample.Play(1, -1); });
				form.AddButton("Play right channel",
					delegate { testSample.Play(1, 1); });

				form.AddButton("Play faster",
					delegate
					{
						testSample.Play();
						testSample.PitchFactor = 2.0f;
					});
				form.AddButton("Play slower",
					delegate
					{
						testSample.Play();
						testSample.PitchFactor = 0.5f;
					});

				form.Run();
			} // TestPitchingSounds()
			#endregion
		} // class SampleTests
#endif
		#endregion
	}	// class Sample
} // namespace RocketCommander.Sounds
